@@ -26,6 +26,8 @@ import org.apache.lucene.index.DocValues;
 import org.apache.lucene.index.IndexReader;
 import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.index.NumericDocValues;
+import org.apache.lucene.index.PointValues;
+import org.apache.lucene.search.comparators.DoubleComparator;
 
 /**
  * Base class for producing {@link DoubleValues}
@@ -41,7 +43,7 @@ import org.apache.lucene.index.NumericDocValues;
  * special long-to-double encoding is required.
  *
  * Scores may be used as a source for value calculations by wrapping a {@link Scorer} using
- * {@link #fromScorer(Scorer)} and passing the resulting DoubleValues to {@link #getValues(LeafReaderContext, DoubleValues)}.
+ * {@link #fromScorer(Scorable)} and passing the resulting DoubleValues to {@link #getValues(LeafReaderContext, DoubleValues)}.
  * The scores can then be accessed using the {@link #SCORES} DoubleValuesSource.
  */
 public abstract class DoubleValuesSource implements SegmentCacheable {
@@ -68,9 +70,9 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
    * @throws IOException if an {@link IOException} occurs
    */
   public Explanation explain(LeafReaderContext ctx, int docId, Explanation scoreExplanation) throws IOException {
-    DoubleValues dv = getValues(ctx, DoubleValuesSource.constant(scoreExplanation.getValue()).getValues(ctx, null));
+    DoubleValues dv = getValues(ctx, DoubleValuesSource.constant(scoreExplanation.getValue().doubleValue()).getValues(ctx, null));
     if (dv.advanceExact(docId))
-      return Explanation.match((float) dv.doubleValue(), this.toString());
+      return Explanation.match(dv.doubleValue(), this.toString());
     return Explanation.noMatch(this.toString());
   }
 
@@ -81,8 +83,8 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
    * IndexReader-independent implementations can just return {@code this}
    *
    * Queries that use DoubleValuesSource objects should call rewrite() during
-   * {@link Query#createWeight(IndexSearcher, boolean, float)} rather than during
-   * {@link Query#rewrite(IndexReader)} to avoid IndexReader reference leakage
+   * {@link Query#createWeight(IndexSearcher, ScoreMode, float)} rather than during
+   * {@link Query#rewrite(IndexReader)} to avoid IndexReader reference leakage.
    *
    * For the same reason, implementations that cache references to the IndexSearcher
    * should return a new object from this method.
@@ -268,21 +270,12 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
 
   private static class ConstantValuesSource extends DoubleValuesSource {
 
+    private final DoubleValues doubleValues;
     private final double value;
 
     private ConstantValuesSource(double value) {
       this.value = value;
-    }
-
-    @Override
-    public DoubleValuesSource rewrite(IndexSearcher searcher) {
-      return this;
-    }
-
-
-    @Override
-    public DoubleValues getValues(LeafReaderContext ctx, DoubleValues scores) throws IOException {
-      return new DoubleValues() {
+      this.doubleValues = new DoubleValues() {
         @Override
         public double doubleValue() throws IOException {
           return value;
@@ -295,6 +288,17 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
       };
     }
 
+    @Override
+    public DoubleValuesSource rewrite(IndexSearcher searcher) {
+      return this;
+    }
+
+
+    @Override
+    public DoubleValues getValues(LeafReaderContext ctx, DoubleValues scores) throws IOException {
+      return doubleValues;
+    }
+
     @Override
     public boolean needsScores() {
       return false;
@@ -308,7 +312,7 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
 
     @Override
     public Explanation explain(LeafReaderContext ctx, int docId, Explanation scoreExplanation) {
-      return Explanation.match((float) value, "constant(" + value + ")");
+      return Explanation.match(value, "constant(" + value + ")");
     }
 
     @Override
@@ -334,7 +338,7 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
   /**
    * Returns a DoubleValues instance that wraps scores returned by a Scorer
    */
-  public static DoubleValues fromScorer(Scorer scorer) {
+  public static DoubleValues fromScorer(Scorable scorer) {
     return new DoubleValues() {
       @Override
       public double doubleValue() throws IOException {
@@ -408,7 +412,7 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
     public Explanation explain(LeafReaderContext ctx, int docId, Explanation scoreExplanation) throws IOException {
       DoubleValues values = getValues(ctx, null);
       if (values.advanceExact(docId))
-        return Explanation.match((float) values.doubleValue(), this.toString());
+        return Explanation.match(values.doubleValue(), this.toString());
       else
         return Explanation.noMatch(this.toString());
     }
@@ -428,6 +432,16 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
       this.producer = producer;
     }
 
+    @Override
+    public void setMissingValue(Object missingValue) {
+      if (missingValue instanceof Number) {
+        this.missingValue = missingValue;
+        ((DoubleValuesComparatorSource) getComparatorSource()).setMissingValue(((Number) missingValue).doubleValue());
+      } else {
+        super.setMissingValue(missingValue);
+      }
+    }
+
     @Override
     public boolean needsScores() {
       return producer.needsScores();
@@ -444,8 +458,13 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
 
     @Override
     public SortField rewrite(IndexSearcher searcher) throws IOException {
-      return new DoubleValuesSortField(producer.rewrite(searcher), reverse);
+      DoubleValuesSortField rewritten = new DoubleValuesSortField(producer.rewrite(searcher), reverse);
+      if (missingValue != null) {
+        rewritten.setMissingValue(missingValue);
+      }
+      return rewritten;
     }
+
   }
 
   private static class DoubleValuesHolder {
@@ -454,28 +473,45 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
 
   private static class DoubleValuesComparatorSource extends FieldComparatorSource {
     private final DoubleValuesSource producer;
+    private double missingValue;
 
     DoubleValuesComparatorSource(DoubleValuesSource producer) {
       this.producer = producer;
+      this.missingValue = 0d;
+    }
+
+    void setMissingValue(double missingValue) {
+      this.missingValue = missingValue;
     }
 
     @Override
     public FieldComparator<Double> newComparator(String fieldname, int numHits,
                                                int sortPos, boolean reversed) {
-      return new FieldComparator.DoubleComparator(numHits, fieldname, 0.0){
-
-        LeafReaderContext ctx;
-        DoubleValuesHolder holder = new DoubleValuesHolder();
-
+      return new DoubleComparator(numHits, fieldname, missingValue, reversed, sortPos) {
         @Override
-        protected NumericDocValues getNumericDocValues(LeafReaderContext context, String field) throws IOException {
-          ctx = context;
-          return asNumericDocValues(holder, Double::doubleToLongBits);
-        }
+        public LeafFieldComparator getLeafComparator(LeafReaderContext context) throws IOException {
+          DoubleValuesHolder holder = new DoubleValuesHolder();
 
-        @Override
-        public void setScorer(Scorer scorer) throws IOException {
-          holder.values = producer.getValues(ctx, fromScorer(scorer));
+          return new DoubleComparator.DoubleLeafComparator(context) {
+            LeafReaderContext ctx;
+
+            @Override
+            protected NumericDocValues getNumericDocValues(LeafReaderContext context, String field) {
+              ctx = context;
+              return asNumericDocValues(holder, Double::doubleToLongBits);
+            }
+
+            @Override
+            protected PointValues getPointValues(LeafReaderContext context, String field) {
+              return null;
+            }
+
+            @Override
+            public void setScorer(Scorable scorer) throws IOException {
+              holder.values = producer.getValues(ctx, fromScorer(scorer));
+              super.setScorer(scorer);
+            }
+          };
         }
       };
     }
@@ -556,7 +592,7 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
 
     @Override
     public DoubleValuesSource rewrite(IndexSearcher searcher) throws IOException {
-      return new WeightDoubleValuesSource(searcher.rewrite(query).createWeight(searcher, true, 1f));
+      return new WeightDoubleValuesSource(searcher.rewrite(query).createWeight(searcher, ScoreMode.COMPLETE, 1f));
     }
 
     @Override
@@ -583,8 +619,12 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
       Scorer scorer = weight.scorer(ctx);
       if (scorer == null)
         return DoubleValues.EMPTY;
-      DocIdSetIterator it = scorer.iterator();
+
       return new DoubleValues() {
+        private final TwoPhaseIterator tpi = scorer.twoPhaseIterator();
+        private final DocIdSetIterator disi = (tpi == null) ? scorer.iterator() : tpi.approximation();
+        private Boolean tpiMatch = null; // cache tpi.matches()
+
         @Override
         public double doubleValue() throws IOException {
           return scorer.score();
@@ -592,9 +632,19 @@ public abstract class DoubleValuesSource implements SegmentCacheable {
 
         @Override
         public boolean advanceExact(int doc) throws IOException {
-          if (it.docID() > doc)
-            return false;
-          return it.docID() == doc || it.advance(doc) == doc;
+          if (disi.docID() < doc) {
+            disi.advance(doc);
+            tpiMatch = null;
+          }
+          if (disi.docID() == doc) {
+            if (tpi == null) {
+              return true;
+            } else if (tpiMatch == null) {
+              tpiMatch = tpi.matches();
+            }
+            return tpiMatch;
+          }
+          return false;
         }
       };
     }
