@@ -103,7 +103,10 @@ public class TestHierarchicalDocBuilder extends AbstractDataImportHandlerTestCas
   
   @After
   public void after() {
-    req.close();
+    if (null != req) {
+      req.close();
+      req = null;
+    }
     MockDataSource.clearCache();
   }
 
@@ -256,7 +259,7 @@ public class TestHierarchicalDocBuilder extends AbstractDataImportHandlerTestCas
     /* The limit of search queue is doubled to catch the error in case when for some reason there are more docs than expected  */
     SolrIndexSearcher searcher = req.getSearcher();
     TopDocs result = searcher.search(query, values.length * 2);
-    assertEquals(values.length, result.totalHits);
+    assertEquals(values.length, result.totalHits.value);
     List<String> actualValues = new ArrayList<String>();
     for (int index = 0; index < values.length; ++index) {
       Document doc = searcher.doc(result.scoreDocs[index].doc);
@@ -293,7 +296,7 @@ public class TestHierarchicalDocBuilder extends AbstractDataImportHandlerTestCas
    * <pre>
    * 
    * &lt;dataConfig>
-   *   <dataSource type="MockDataSource" />
+   *   &lt;dataSource type="MockDataSource" />
    *   &lt;document>
    *     &lt;entity name="parent" query="SELECT * FROM parent">
    *       &lt;field column="id" />
