@@ -17,10 +17,10 @@
 package org.apache.solr.update.processor;
 
 import java.io.IOException;
+import java.io.Reader;
 import java.lang.invoke.MethodHandles;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
@@ -41,11 +41,11 @@ import org.slf4j.LoggerFactory;
 
 
 /**
- * Identifies the language of a set of input fields.
- * Also supports mapping of field names based
- * on detected language.
  * <p>
- * See <a href="http://wiki.apache.org/solr/LanguageDetection">http://wiki.apache.org/solr/LanguageDetection</a>
+ *   Identifies the language of a set of input fields.
+ *   Also supports mapping of field names based on detected language.
+ * </p>
+ * See <a href="https://lucene.apache.org/solr/guide/7_4/detecting-languages-during-indexing.html">Detecting Languages During Indexing</a> in reference guide
  * @since 3.5
  * @lucene.experimental
  */
@@ -149,7 +149,7 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
           if(keyVal.length == 2) {
             lcMap.put(keyVal[0], keyVal[1]);
           } else {
-            log.error("Unsupported format for langid.lcmap: "+mapping+". Skipping this mapping.");
+            log.error("Unsupported format for langid.lcmap: {}. Skipping this mapping.", mapping);
           }
         }
       }
@@ -162,7 +162,7 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
           if(keyVal.length == 2) {
             mapLcMap.put(keyVal[0], keyVal[1]);
           } else {
-            log.error("Unsupported format for langid.map.lcmap: "+mapping+". Skipping this mapping.");
+            log.error("Unsupported format for langid.map.lcmap: {}. Skipping this mapping.", mapping);
           }
         }
       }
@@ -175,13 +175,15 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
       if (maxFieldValueChars > maxTotalChars) {
         if (maxTotalChars == MAX_TOTAL_CHARS_DEFAULT) {
           // If the user specified only maxFieldValueChars, make maxTotalChars the same as it
-          log.warn(MAX_FIELD_VALUE_CHARS + " (" + maxFieldValueChars + ") is less than " + MAX_TOTAL_CHARS + " ("
-              + maxTotalChars + ").  Setting " + MAX_TOTAL_CHARS + " to " + maxFieldValueChars + ".");
+          log.warn("{} ({}) is less than {} ({}).  Setting {} to {}."
+              , MAX_FIELD_VALUE_CHARS, maxFieldValueChars, MAX_TOTAL_CHARS
+              , maxTotalChars, MAX_TOTAL_CHARS, maxFieldValueChars);
           maxTotalChars = maxFieldValueChars;
         } else {
           // If the user specified maxTotalChars, make maxFieldValueChars the same as it
-          log.warn(MAX_FIELD_VALUE_CHARS + " (" + maxFieldValueChars + ") is less than " + MAX_TOTAL_CHARS + " ("
-              + maxTotalChars + ").  Setting " + MAX_FIELD_VALUE_CHARS + " to " + maxTotalChars + ".");
+          log.warn("{} ({}) is less than {} ({}).  Setting {} to {}."
+              , MAX_FIELD_VALUE_CHARS, maxFieldValueChars, MAX_TOTAL_CHARS
+              , maxTotalChars, MAX_FIELD_VALUE_CHARS, maxTotalChars );
           maxFieldValueChars = maxTotalChars;
         }
       }
@@ -207,11 +209,10 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
   }
 
   /**
-   * This is the main, testable process method called from processAdd()
-   * @param doc the SolrInputDocument to work on
-   * @return the modified SolrInputDocument
+   * This is the main process method called from processAdd()
+   * @param doc the SolrInputDocument to modify
    */
-  protected SolrInputDocument process(SolrInputDocument doc) {
+  protected void process(SolrInputDocument doc) {
     String docLang = null;
     HashSet<String> docLangs = new HashSet<>();
     String fallbackLang = getFallbackLang(doc, fallbackFields, fallbackValue);
@@ -220,10 +221,14 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
       List<DetectedLanguage> languagelist = detectLanguage(doc);
       docLang = resolveLanguage(languagelist, fallbackLang);
       docLangs.add(docLang);
-      log.debug("Detected main document language from fields "+ Arrays.toString(inputFields) +": "+docLang);
+      if (log.isDebugEnabled()) {
+        log.debug("Detected main document language from fields {}: {}", Arrays.toString(inputFields), docLang);
+      }
 
       if(doc.containsKey(langField) && overwrite) {
-        log.debug("Overwritten old value "+doc.getFieldValue(langField));
+        if (log.isDebugEnabled()) {
+          log.debug("Overwritten old value {}", doc.getFieldValue(langField));
+        }
       }
       if(langField != null && langField.length() != 0) {
         doc.setField(langField, docLang);
@@ -232,7 +237,7 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
       // langField is set, we sanity check it against whitelist and fallback
       docLang = resolveLanguage(doc.getFieldValue(langField).toString(), fallbackLang);
       docLangs.add(docLang);
-      log.debug("Field "+langField+" already contained value "+docLang+", not overwriting.");
+      log.debug("Field {} already contained value {}, not overwriting.", langField, docLang);
     }
 
     if(enableMapping) {
@@ -240,18 +245,20 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
         if(doc.containsKey(fieldName)) {
           String fieldLang;
           if(mapIndividual && mapIndividualFieldsSet.contains(fieldName)) {
-            List<DetectedLanguage> languagelist = detectLanguage(doc);
+            List<DetectedLanguage> languagelist = detectLanguage(solrDocReader(doc, new String[]{fieldName}));
             fieldLang = resolveLanguage(languagelist, docLang);
             docLangs.add(fieldLang);
-            log.debug("Mapping field "+fieldName+" using individually detected language "+fieldLang);
+            log.debug("Mapping field {} using individually detected language {}", fieldName, fieldLang);
           } else {
             fieldLang = docLang;
-            log.debug("Mapping field "+fieldName+" using document global language "+fieldLang);
+            log.debug("Mapping field {} using document global language {}", fieldName, fieldLang);
           }
           String mappedOutputField = getMappedField(fieldName, fieldLang);
 
           if (mappedOutputField != null) {
-            log.debug("Mapping field {} to {}", doc.getFieldValue(docIdField), fieldLang);
+            if (log.isDebugEnabled()) {
+              log.debug("Mapping field {} to {}", doc.getFieldValue(docIdField), fieldLang);
+            }
             SolrInputField inField = doc.getField(fieldName);
             doc.setField(mappedOutputField, inField.getValue());
             if(!mapKeepOrig) {
@@ -270,8 +277,6 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
     if(langsField != null && langsField.length() != 0) {
       doc.setField(langsField, docLangs.toArray());
     }
-
-    return doc;
   }
 
   /**
@@ -285,24 +290,33 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
     for(String field : fallbackFields) {
       if(doc.containsKey(field)) {
         lang = (String) doc.getFieldValue(field);
-        log.debug("Language fallback to field "+field);
+        log.debug("Language fallback to field {}", field);
         break;
       }
     }
     if(lang == null) {
-      log.debug("Language fallback to value "+fallbackValue);
+      log.debug("Language fallback to value {}", fallbackValue);
       lang = fallbackValue;
     }
     return lang;
   }
 
   /**
-   * Detects language(s) from a string.
-   * Classes wishing to implement their own language detection module should override this method.
-   * @param content The content to identify
+   * Detects language(s) from all configured fields
+   * @param doc The solr document
    * @return List of detected language(s) according to RFC-3066
    */
-  protected abstract List<DetectedLanguage> detectLanguage(SolrInputDocument content);
+  protected List<DetectedLanguage> detectLanguage(SolrInputDocument doc) {
+    return detectLanguage(solrDocReader(doc, inputFields));
+  }
+
+  /**
+   * Detects language(s) from a reader, typically based on some fields in SolrInputDocument
+   * Classes wishing to implement their own language detection module should override this method.
+   * @param solrDocReader A reader serving the text from the document to detect
+   * @return List of detected language(s) according to RFC-3066
+   */
+  protected abstract List<DetectedLanguage> detectLanguage(Reader solrDocReader);
 
   /**
    * Chooses a language based on the list of candidates detected
@@ -331,7 +345,9 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
       DetectedLanguage lang = languages.get(0);
       String normalizedLang = normalizeLangCode(lang.getLangCode());
       if(langWhitelist.isEmpty() || langWhitelist.contains(normalizedLang)) {
-        log.debug("Language detected {} with certainty {}", normalizedLang, lang.getCertainty());
+        if (log.isDebugEnabled()) {
+          log.debug("Language detected {} with certainty {}", normalizedLang, lang.getCertainty());
+        }
         if(lang.getCertainty() >= threshold) {
           langStr = normalizedLang;
         } else {
@@ -339,7 +355,9 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
           langStr = fallbackLang;
         }
       } else {
-        log.debug("Detected a language not in whitelist ({}), using fallback {}", lang.getLangCode(), fallbackLang);
+        if (log.isDebugEnabled()) {
+          log.debug("Detected a language not in whitelist ({}), using fallback {}", lang.getLangCode(), fallbackLang);
+        }
         langStr = fallbackLang;
       }
     }
@@ -360,7 +378,7 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
   protected String normalizeLangCode(String langCode) {
     if (lcMap.containsKey(langCode)) {
       String lc = lcMap.get(langCode);
-      log.debug("Doing langcode normalization mapping from "+langCode+" to "+lc);
+      log.debug("Doing langcode normalization mapping from {} to {}", langCode, lc);
       return lc;
     }
     return langCode;
@@ -383,7 +401,7 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
       log.warn("Unsuccessful field name mapping from {} to {}, field does not exist and enforceSchema=true; skipping mapping.", currentField, newFieldName);
       return null;
     } else {
-      log.debug("Doing mapping from "+currentField+" with language "+language+" to field "+newFieldName);
+      log.debug("Doing mapping from {} with language {} to field {}", currentField, language, newFieldName);
     }
     return newFieldName;
   }
@@ -400,67 +418,22 @@ public abstract class LanguageIdentifierUpdateProcessor extends UpdateRequestPro
     this.enabled = enabled;
   }
 
-
-
   /**
-   * Concatenates content from multiple fields
+   * Returns a reader that streams String content from fields.
+   * This is more memory efficient than building a full string buffer
+   * @param doc the solr document
+   * @param fields the field names to read
+   * @return a reader over the fields
+   */
+  protected SolrInputDocumentReader solrDocReader(SolrInputDocument doc, String[] fields) {
+    return new SolrInputDocumentReader(doc, fields, maxTotalChars, maxFieldValueChars, " ");
+  }
+  
+  /**
+   * Concatenates content from input fields defined in langid.fl.
+   * For test purposes only
    */
   protected String concatFields(SolrInputDocument doc) {
-    StringBuilder sb = new StringBuilder(getExpectedSize(doc, inputFields));
-    for (String fieldName : inputFields) {
-      log.debug("Appending field " + fieldName);
-      if (doc.containsKey(fieldName)) {
-        Collection<Object> fieldValues = doc.getFieldValues(fieldName);
-        if (fieldValues != null) {
-          for (Object content : fieldValues) {
-            if (content instanceof String) {
-              String stringContent = (String) content;
-              if (stringContent.length() > maxFieldValueChars) {
-                sb.append(stringContent.substring(0, maxFieldValueChars));
-              } else {
-                sb.append(stringContent);
-              }
-              sb.append(" ");
-              if (sb.length() > maxTotalChars) {
-                sb.setLength(maxTotalChars);
-                break;
-              }
-            } else {
-              log.warn("Field " + fieldName + " not a String value, not including in detection");
-            }
-          }
-        }
-      }
-    }
-    return sb.toString();
-  }
-
-  /**
-   * Calculate expected string size.
-   *
-   * @param doc           solr input document
-   * @param fields        fields to select
-   * @return expected size of string value
-   */
-  private int getExpectedSize(SolrInputDocument doc, String[] fields) {
-    int docSize = 0;
-    for (String field : fields) {
-      if (doc.containsKey(field)) {
-        Collection<Object> contents = doc.getFieldValues(field);
-        if (contents != null) {
-          for (Object content : contents) {
-            if (content instanceof String) {
-              docSize += Math.min(((String) content).length(), maxFieldValueChars);
-            }
-          }
-
-          if (docSize > maxTotalChars) {
-            docSize = maxTotalChars;
-            break;
-          }
-        }
-      }
-    }
-    return docSize;
+    return SolrInputDocumentReader.asString(solrDocReader(doc, inputFields));
   }
 }
