@@ -17,21 +17,27 @@
 package org.apache.solr.response;
 
 import java.io.File;
+import java.io.FilePermission;
 import java.io.IOException;
-import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.StringWriter;
 import java.io.Writer;
 import java.lang.invoke.MethodHandles;
 import java.nio.charset.StandardCharsets;
+import java.security.AccessControlContext;
+import java.security.AccessController;
+import java.security.Permissions;
+import java.security.PrivilegedActionException;
+import java.security.PrivilegedExceptionAction;
+import java.security.ProtectionDomain;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Properties;
+import java.util.PropertyPermission;
 import java.util.ResourceBundle;
 
-import org.apache.commons.lang.StringUtils;
 import org.apache.solr.client.solrj.SolrResponse;
 import org.apache.solr.client.solrj.response.QueryResponse;
 import org.apache.solr.client.solrj.response.SolrResponseBase;
@@ -46,21 +52,22 @@ import org.apache.velocity.VelocityContext;
 import org.apache.velocity.app.VelocityEngine;
 import org.apache.velocity.runtime.RuntimeConstants;
 import org.apache.velocity.runtime.resource.loader.ClasspathResourceLoader;
-import org.apache.velocity.tools.ConversionUtils;
+import org.apache.velocity.tools.generic.CollectionTool;
 import org.apache.velocity.tools.generic.ComparisonDateTool;
 import org.apache.velocity.tools.generic.DisplayTool;
 import org.apache.velocity.tools.generic.EscapeTool;
-import org.apache.velocity.tools.generic.ListTool;
 import org.apache.velocity.tools.generic.LocaleConfig;
 import org.apache.velocity.tools.generic.MathTool;
 import org.apache.velocity.tools.generic.NumberTool;
 import org.apache.velocity.tools.generic.ResourceTool;
-import org.apache.velocity.tools.generic.SortTool;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import static org.apache.solr.common.params.CommonParams.SORT;
 
+/**
+ * @deprecated since 8.4; see <a href="https://cwiki.apache.org/confluence/display/SOLR/Deprecations">Deprecations</a>
+ */
 public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAware {
   // init param names, these are _only_ loaded at init time (no per-request control of these)
   //   - multiple different named writers could be created with different init params
@@ -68,7 +75,7 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
   public static final String PROPERTIES_FILE = "init.properties.file";
 
   // System property names, these are _only_ loaded at node startup (no per-request control of these)
-  public static final String SOLR_RESOURCE_LOADER_ENABLED = "solr.resource.loader.enabled";
+  public static final String SOLR_RESOURCE_LOADER_ENABLED = "velocity.resourceloader.solr.enabled";
 
   // request param names
   public static final String TEMPLATE = "v.template";
@@ -86,23 +93,23 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
   private String initPropertiesFileName;  // used just to hold from init() to inform()
 
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
-  private static final SolrVelocityLogger velocityLogger = new SolrVelocityLogger(log);
   private Properties velocityInitProps = new Properties();
   private Map<String,String> customTools = new HashMap<String,String>();
 
   @Override
-  public void init(NamedList args) {
+  public void init(@SuppressWarnings({"rawtypes"})NamedList args) {
+    log.warn("VelocityResponseWriter is deprecated. This may be removed in future Solr releases. Please SOLR-14065.");
     fileResourceLoaderBaseDir = null;
     String templateBaseDir = (String) args.get(TEMPLATE_BASE_DIR);
 
     if (templateBaseDir != null && !templateBaseDir.isEmpty()) {
       fileResourceLoaderBaseDir = new File(templateBaseDir).getAbsoluteFile();
       if (!fileResourceLoaderBaseDir.exists()) { // "*not* exists" condition!
-        log.warn(TEMPLATE_BASE_DIR + " specified does not exist: " + fileResourceLoaderBaseDir);
+        log.warn("{} specified does not exist: {}", TEMPLATE_BASE_DIR, fileResourceLoaderBaseDir);
         fileResourceLoaderBaseDir = null;
       } else {
         if (!fileResourceLoaderBaseDir.isDirectory()) { // "*not* a directory" condition
-          log.warn(TEMPLATE_BASE_DIR + " specified is not a directory: " + fileResourceLoaderBaseDir);
+          log.warn("{} specified is not a directory: {}", TEMPLATE_BASE_DIR, fileResourceLoaderBaseDir);
           fileResourceLoaderBaseDir = null;
         }
       }
@@ -110,9 +117,11 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
 
     initPropertiesFileName = (String) args.get(PROPERTIES_FILE);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList tools = (NamedList)args.get("tools");
     if (tools != null) {
       for(Object t : tools) {
+        @SuppressWarnings({"rawtypes"})
         Map.Entry tool = (Map.Entry)t;
         customTools.put(tool.getKey().toString(), tool.getValue().toString());
       }
@@ -123,14 +132,13 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
   public void inform(SolrCore core) {
     // need to leverage SolrResourceLoader, so load init.properties.file here instead of init()
     if (initPropertiesFileName != null) {
-      InputStream is = null;
       try {
         velocityInitProps.load(new InputStreamReader(core.getResourceLoader().openResource(initPropertiesFileName), StandardCharsets.UTF_8));
       } catch (IOException e) {
-        log.warn("Error loading " + PROPERTIES_FILE + " specified property file: " + initPropertiesFileName, e);
+        log.warn("Error loading {} specified property file: {}", PROPERTIES_FILE, initPropertiesFileName, e);
       }
     }
-  }
+    }
 
   @Override
   public String getContentType(SolrQueryRequest request, SolrQueryResponse response) {
@@ -142,6 +150,39 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
 
   @Override
   public void write(Writer writer, SolrQueryRequest request, SolrQueryResponse response) throws IOException {
+    // run doWrite() with the velocity sandbox
+    try {
+      AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
+        @Override
+        public Void run() throws IOException {
+          doWrite(writer, request, response);
+          return null;
+        }
+      }, VELOCITY_SANDBOX);
+    } catch (PrivilegedActionException e) {
+      throw (IOException) e.getException();
+    }
+  }
+
+  // sandbox for velocity code
+  // TODO: we could read in a policy file instead, in case someone needs to tweak it?
+  private static final AccessControlContext VELOCITY_SANDBOX;
+  static {
+    Permissions permissions = new Permissions();
+    // TODO: restrict the scope of this! we probably only need access to classpath
+    permissions.add(new FilePermission("<<ALL FILES>>", "read,readlink"));
+    // properties needed by SolrResourceLoader (called from velocity code)
+    permissions.add(new PropertyPermission("jetty.testMode", "read"));
+    permissions.add(new PropertyPermission("solr.allow.unsafe.resourceloading", "read"));
+    // properties needed by log4j (called from velocity code)
+    permissions.add(new PropertyPermission("java.version", "read"));
+    // needed by velocity duck-typing
+    permissions.add(new RuntimePermission("accessDeclaredMembers"));
+    permissions.setReadOnly();
+    VELOCITY_SANDBOX = new AccessControlContext(new ProtectionDomain[] { new ProtectionDomain(null, permissions) });
+  }
+
+  private void doWrite(Writer writer, SolrQueryRequest request, SolrQueryResponse response) throws IOException {
     VelocityEngine engine = createEngine(request);  // TODO: have HTTP headers available for configuring engine
 
     Template template = getTemplate(engine, request);
@@ -190,11 +231,13 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
     }
   }
 
+  @SuppressWarnings({"unchecked"})
   private VelocityContext createContext(SolrQueryRequest request, SolrQueryResponse response) {
     VelocityContext context = new VelocityContext();
 
     // Register useful Velocity "tools"
     String locale = request.getParams().get(LOCALE);
+    @SuppressWarnings({"rawtypes"})
     Map toolConfig = new HashMap();
     toolConfig.put("locale", locale);
 
@@ -202,8 +245,7 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
     context.put("log", log);   // TODO: add test; TODO: should this be overridable with a custom "log" named tool?
     context.put("esc", new EscapeTool());
     context.put("date", new ComparisonDateTool());
-    context.put("list", new ListTool());
-    context.put(SORT, new SortTool());
+    context.put(SORT, new CollectionTool());
 
     MathTool mathTool = new MathTool();
     mathTool.configure(toolConfig);
@@ -222,7 +264,7 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
     resourceTool.configure(toolConfig);
     context.put("resource", resourceTool);
 
-   if (request.getCore().getCoreDescriptor().isConfigSetTrusted()) {
+    if (request.getCore().getCoreDescriptor().isConfigSetTrusted()) {
       // Load custom tools, only if in a trusted configset
 
       /*
@@ -281,10 +323,12 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
   }
 
   private VelocityEngine createEngine(SolrQueryRequest request) {
+
     boolean trustedMode = request.getCore().getCoreDescriptor().isConfigSetTrusted();
 
 
     VelocityEngine engine = new VelocityEngine();
+
     // load the built-in _macros.vm first, then load VM_global_library.vm for legacy (pre-5.0) support,
     // and finally allow macros.vm to have the final say and override anything defined in the preceding files.
     engine.setProperty(RuntimeConstants.VM_LIBRARY, "_macros.vm,VM_global_library.vm,macros.vm");
@@ -331,9 +375,11 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
     loaders.add("builtin");
     engine.setProperty("builtin.resource.loader.instance", new ClasspathResourceLoader());
 
-    engine.setProperty(RuntimeConstants.RESOURCE_LOADER, StringUtils.join(loaders,','));
+    engine.setProperty(RuntimeConstants.RESOURCE_LOADER, String.join(",", loaders));
+
 
     engine.setProperty(RuntimeConstants.INPUT_ENCODING, "UTF-8");
+    engine.setProperty(RuntimeConstants.SPACE_GOBBLING, RuntimeConstants.SpaceGobbling.LINES.toString());
 
     // install a class/package restricting uberspector
     engine.setProperty(RuntimeConstants.UBERSPECT_CLASSNAME,"org.apache.velocity.util.introspection.SecureUberspector");
@@ -354,8 +400,19 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
     engine.addProperty(RuntimeConstants.INTROSPECTOR_RESTRICT_CLASSES,"org.apache.solr.core.SolrResourceLoader");
     engine.addProperty(RuntimeConstants.INTROSPECTOR_RESTRICT_CLASSES,"org.apache.solr.core.CoreContainer");
 
-    // bring in any custom properties too
-    engine.init(velocityInitProps);
+    if (trustedMode) {
+      // Work around VELOCITY-908 with Velocity not handling locales properly
+      Object spaceGobblingInitProperty = velocityInitProps.get(RuntimeConstants.SPACE_GOBBLING);
+      if (spaceGobblingInitProperty != null) {
+        // If there is an init property, uppercase it before Velocity.
+        velocityInitProps.put(RuntimeConstants.SPACE_GOBBLING,
+            String.valueOf(spaceGobblingInitProperty).toUpperCase(Locale.ROOT));
+      }
+      // bring in any custom properties too
+      engine.setProperties(velocityInitProps);
+    }
+
+    engine.init();
 
     return engine;
   }
@@ -393,7 +450,7 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
     return "{\"result\":\"" + replaced + "\"}";
   }
 
-  // see: http://svn.apache.org/repos/asf/velocity/tools/branches/2.0.x/src/main/java/org/apache/velocity/tools/generic/ResourceTool.java
+  // see: https://github.com/apache/velocity-tools/blob/trunk/velocity-tools-generic/src/main/java/org/apache/velocity/tools/generic/ResourceTool.java
   private static class SolrVelocityResourceTool extends ResourceTool {
 
     private ClassLoader solrClassLoader;
@@ -410,17 +467,5 @@ public class VelocityResponseWriter implements QueryResponseWriter, SolrCoreAwar
           (loc == null) ? this.getLocale() : this.toLocale(loc),
           solrClassLoader);
     }
-
-    // Why did Velocity Tools make this private?  Copied from ResourceTools.java
-    private Locale toLocale(Object obj) {
-      if (obj == null) {
-        return null;
-      }
-      if (obj instanceof Locale) {
-        return (Locale) obj;
-      }
-      String s = String.valueOf(obj);
-      return ConversionUtils.toLocale(s);
-    }
   }
 }
