@@ -58,7 +58,7 @@ import org.apache.solr.common.util.ExecutorUtil;
 import org.apache.solr.common.util.IOUtils;
 import org.apache.solr.common.util.Utils;
 import org.apache.solr.core.SolrResourceLoader;
-import org.apache.solr.util.DefaultSolrThreadFactory;
+import org.apache.solr.common.util.SolrNamedThreadFactory;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
@@ -71,7 +71,9 @@ import static org.apache.solr.common.util.ExecutorUtil.awaitTermination;
 
 /**
  * Responsible for scheduling active triggers, starting and stopping them and
- * performing actions when they fire
+ * performing actions when they fire.
+ *
+ * @deprecated to be removed in Solr 9.0 (see SOLR-14656)
  */
 public class ScheduledTriggers implements Closeable {
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
@@ -82,7 +84,6 @@ public class ScheduledTriggers implements Closeable {
 
   static final Map<String, Object> DEFAULT_PROPERTIES = new HashMap<>();
 
-  // Note: values must be all in milliseconds!
   static {
     DEFAULT_PROPERTIES.put(TRIGGER_SCHEDULE_DELAY_SECONDS, DEFAULT_SCHEDULED_TRIGGER_DELAY_SECONDS);
     DEFAULT_PROPERTIES.put(TRIGGER_COOLDOWN_PERIOD_SECONDS, DEFAULT_COOLDOWN_PERIOD_SECONDS);
@@ -136,14 +137,16 @@ public class ScheduledTriggers implements Closeable {
 
   private final TriggerListeners listeners;
 
+  private final List<TriggerListener> additionalListeners = new ArrayList<>();
+
   private AutoScalingConfig autoScalingConfig;
 
   public ScheduledTriggers(SolrResourceLoader loader, SolrCloudManager cloudManager) {
     scheduledThreadPoolExecutor = (ScheduledThreadPoolExecutor) Executors.newScheduledThreadPool(DEFAULT_TRIGGER_CORE_POOL_SIZE,
-        new DefaultSolrThreadFactory("ScheduledTrigger"));
+        new SolrNamedThreadFactory("ScheduledTrigger"));
     scheduledThreadPoolExecutor.setRemoveOnCancelPolicy(true);
     scheduledThreadPoolExecutor.setExecuteExistingDelayedTasksAfterShutdownPolicy(false);
-    actionExecutor = ExecutorUtil.newMDCAwareSingleThreadExecutor(new DefaultSolrThreadFactory("AutoscalingActionExecutor"));
+    actionExecutor = ExecutorUtil.newMDCAwareSingleThreadExecutor(new SolrNamedThreadFactory("AutoscalingActionExecutor"));
     this.cloudManager = cloudManager;
     this.stateManager = cloudManager.getDistribStateManager();
     this.loader = loader;
@@ -171,7 +174,9 @@ public class ScheduledTriggers implements Closeable {
       Map<String, Object> newProps = autoScalingConfig.getProperties();
       String key = entry.getKey();
       if (newProps.containsKey(key) && !entry.getValue().equals(newProps.get(key))) {
-        log.debug("Changing value of autoscaling property: {} from: {} to: {}", key, entry.getValue(), newProps.get(key));
+        if (log.isDebugEnabled()) {
+          log.debug("Changing value of autoscaling property: {} from: {} to: {}", key, entry.getValue(), newProps.get(key));
+        }
         switch (key) {
           case TRIGGER_SCHEDULE_DELAY_SECONDS:
             triggerDelay.set(((Number) newProps.get(key)).intValue());
@@ -223,9 +228,9 @@ public class ScheduledTriggers implements Closeable {
         throw new AlreadyClosedException("ScheduledTriggers has been closed and cannot be used anymore");
       }
       if (cloudManager.isClosed()) {
-        log.error("Failed to add trigger " + newTrigger.getName() + " - closing or disconnected from data provider", e);
+        log.error("Failed to add trigger {} - closing or disconnected from data provider", newTrigger.getName(), e);
       } else {
-        log.error("Failed to add trigger " + newTrigger.getName(), e);
+        log.error("Failed to add trigger {}", newTrigger.getName(), e);
       }
       return;
     }
@@ -267,7 +272,7 @@ public class ScheduledTriggers implements Closeable {
         return false;
       }
       if (event.isIgnored())  {
-        log.debug("-------- Ignoring event: " + event);
+        log.debug("-------- Ignoring event: {}", event);
         event.getProperties().put(TriggerEvent.IGNORED, true);
         triggerListeners.fireListeners(event.getSource(), event, TriggerEventProcessorStage.IGNORED, "Event was ignored.");
         return true; // always return true for ignored events
@@ -275,12 +280,12 @@ public class ScheduledTriggers implements Closeable {
       // even though we pause all triggers during action execution there is a possibility that a trigger was already
       // running at the time and would have already created an event so we reject such events during cooldown period
       if (cooldownStart.get() + cooldownPeriod.get() > cloudManager.getTimeSource().getTimeNs()) {
-        log.debug("-------- Cooldown period - rejecting event: " + event);
+        log.debug("-------- Cooldown period - rejecting event: {}", event);
         event.getProperties().put(TriggerEvent.COOLDOWN, true);
         triggerListeners.fireListeners(event.getSource(), event, TriggerEventProcessorStage.IGNORED, "In cooldown period.");
         return false;
       } else {
-        log.debug("++++++++ Cooldown inactive - processing event: " + event);
+        log.debug("++++++++ Cooldown inactive - processing event: {}", event);
         // start cooldown here to immediately reject other events
         cooldownStart.set(cloudManager.getTimeSource().getTimeNs());
       }
@@ -310,7 +315,7 @@ public class ScheduledTriggers implements Closeable {
             assert hasPendingActions.get();
             long eventProcessingStart = cloudManager.getTimeSource().getTimeNs();
             TriggerListeners triggerListeners1 = triggerListeners.copy();
-            log.debug("-- processing actions for " + event);
+            log.debug("-- processing actions for {}", event);
             try {
               // in future, we could wait for pending tasks in a different thread and re-enqueue
               // this event so that we continue processing other events and not block this action executor
@@ -318,6 +323,7 @@ public class ScheduledTriggers implements Closeable {
 
               ActionContext actionContext = new ActionContext(cloudManager, newTrigger, new HashMap<>());
               for (TriggerAction action : actions) {
+                @SuppressWarnings({"unchecked"})
                 List<String> beforeActions = (List<String>) actionContext.getProperties().computeIfAbsent(TriggerEventProcessorStage.BEFORE_ACTION.toString(), k -> new ArrayList<String>());
                 beforeActions.add(action.getName());
                 triggerListeners1.fireListeners(event.getSource(), event, TriggerEventProcessorStage.BEFORE_ACTION, action.getName(), actionContext);
@@ -327,6 +333,7 @@ public class ScheduledTriggers implements Closeable {
                   triggerListeners1.fireListeners(event.getSource(), event, TriggerEventProcessorStage.FAILED, action.getName(), actionContext, e, null);
                   throw new TriggerActionException(event.getSource(), action.getName(), "Error processing action for trigger event: " + event, e);
                 }
+                @SuppressWarnings({"unchecked"})
                 List<String> afterActions = (List<String>) actionContext.getProperties().computeIfAbsent(TriggerEventProcessorStage.AFTER_ACTION.toString(), k -> new ArrayList<String>());
                 afterActions.add(action.getName());
                 triggerListeners1.fireListeners(event.getSource(), event, TriggerEventProcessorStage.AFTER_ACTION, action.getName(), actionContext);
@@ -348,8 +355,10 @@ public class ScheduledTriggers implements Closeable {
               // resume triggers after cool down period
               resumeTriggers(cloudManager.getTimeSource().convertDelay(TimeUnit.NANOSECONDS, cooldownPeriod.get(), TimeUnit.MILLISECONDS));
             }
-            log.debug("-- processing took {} ms for event id={}",
-                TimeUnit.NANOSECONDS.toMillis(cloudManager.getTimeSource().getTimeNs() - eventProcessingStart), event.id);
+            if (log.isDebugEnabled()) {
+              log.debug("-- processing took {} ms for event id={}",
+                  TimeUnit.NANOSECONDS.toMillis(cloudManager.getTimeSource().getTimeNs() - eventProcessingStart), event.id);
+            }
           });
         } else {
           if (enqueued) {
@@ -424,6 +433,7 @@ public class ScheduledTriggers implements Closeable {
               String path = parentPath + '/' + child;
               VersionedData data = stateManager.getData(path, null);
               if (data != null) {
+                @SuppressWarnings({"rawtypes"})
                 Map map = (Map) Utils.fromJSON(data.getData());
                 String requestid = (String) map.get("requestid");
                 try {
@@ -447,7 +457,7 @@ public class ScheduledTriggers implements Closeable {
                   if (rootCause instanceof TimeoutException && rootCause.getMessage().contains("Could not connect to ZooKeeper")) {
                     throw e;
                   }
-                  log.error("Unexpected exception while waiting for pending task with requestid: " + requestid + " to finish", e);
+                  log.error("Unexpected exception while waiting for pending task with requestid: {} to finish", requestid,  e);
                 }
               }
             }
@@ -473,7 +483,7 @@ public class ScheduledTriggers implements Closeable {
    */
   public synchronized void removeAll() {
     getScheduledTriggerNames().forEach(t -> {
-      log.info("-- removing trigger: " + t);
+      log.info("-- removing trigger: {}", t);
       remove(t);
     });
   }
@@ -496,12 +506,12 @@ public class ScheduledTriggers implements Closeable {
     try {
       stateManager.removeRecursively(statePath, true, true);
     } catch (Exception e) {
-      log.warn("Failed to remove state for removed trigger " + statePath, e);
+      log.warn("Failed to remove state for removed trigger {}", statePath, e);
     }
     try {
       stateManager.removeRecursively(eventsPath, true, true);
     } catch (Exception e) {
-      log.warn("Failed to remove events for removed trigger " + eventsPath, e);
+      log.warn("Failed to remove events for removed trigger {}", eventsPath, e);
     }
   }
 
@@ -554,6 +564,22 @@ public class ScheduledTriggers implements Closeable {
     log.debug("ScheduledTriggers closed completely");
   }
 
+  /**
+   * Add a temporary listener for internal use (tests, simulation).
+   * @param listener listener instance
+   */
+  public void addAdditionalListener(TriggerListener listener) {
+    listeners.addAdditionalListener(listener);
+  }
+
+  /**
+   * Remove a temporary listener for internal use (tests, simulation).
+   * @param listener listener instance
+   */
+  public void removeAdditionalListener(TriggerListener listener) {
+    listeners.removeAdditionalListener(listener);
+  }
+
   private class TriggerWrapper implements Runnable, Closeable {
     AutoScaling.Trigger trigger;
     ScheduledFuture<?> scheduledFuture;
@@ -612,7 +638,7 @@ public class ScheduledTriggers implements Closeable {
                 // override REPLAYING=true
                 event.getProperties().put(TriggerEvent.REPLAYING, true);
                 if (!trigger.getProcessor().process(event)) {
-                  log.error("Failed to re-play event, discarding: " + event);
+                  log.error("Failed to re-play event, discarding: {}", event);
                 }
                 queue.pollEvent(); // always remove it from queue
               }
@@ -621,14 +647,14 @@ public class ScheduledTriggers implements Closeable {
                 trigger.restoreState();
               } catch (Exception e) {
                 // log but don't throw - see below
-                log.error("Error restoring trigger state " + trigger.getName(), e);
+                log.error("Error restoring trigger state {}", trigger.getName(), e);
               }
               replay = false;
             }
           } catch (AlreadyClosedException e) {
             
           } catch (Exception e) {
-            log.error("Unexpected exception from trigger: " + trigger.getName(), e);
+            log.error("Unexpected exception from trigger: {}", trigger.getName(), e);
           }
           try {
             trigger.run();
@@ -637,7 +663,7 @@ public class ScheduledTriggers implements Closeable {
           } catch (Exception e) {
             // log but do not propagate exception because an exception thrown from a scheduled operation
             // will suppress future executions
-            log.error("Unexpected exception from trigger: " + trigger.getName(), e);
+            log.error("Unexpected exception from trigger: {}", trigger.getName(), e);
           } finally {
             // checkpoint after each run
             trigger.saveState();
@@ -659,6 +685,7 @@ public class ScheduledTriggers implements Closeable {
   private class TriggerListeners {
     Map<String, Map<TriggerEventProcessorStage, List<TriggerListener>>> listenersPerStage = new HashMap<>();
     Map<String, TriggerListener> listenersPerName = new HashMap<>();
+    List<TriggerListener> additionalListeners = new ArrayList<>();
     ReentrantLock updateLock = new ReentrantLock();
 
     public TriggerListeners() {
@@ -682,6 +709,41 @@ public class ScheduledTriggers implements Closeable {
       return new TriggerListeners(listenersPerStage, listenersPerName);
     }
 
+    public void addAdditionalListener(TriggerListener listener) {
+      updateLock.lock();
+      try {
+        AutoScalingConfig.TriggerListenerConfig config = listener.getConfig();
+        for (TriggerEventProcessorStage stage : config.stages) {
+          addPerStage(config.trigger, stage, listener);
+        }
+        // add also for beforeAction / afterAction TriggerStage
+        if (!config.beforeActions.isEmpty()) {
+          addPerStage(config.trigger, TriggerEventProcessorStage.BEFORE_ACTION, listener);
+        }
+        if (!config.afterActions.isEmpty()) {
+          addPerStage(config.trigger, TriggerEventProcessorStage.AFTER_ACTION, listener);
+        }
+        additionalListeners.add(listener);
+      } finally {
+        updateLock.unlock();
+      }
+    }
+
+    public void removeAdditionalListener(TriggerListener listener) {
+      updateLock.lock();
+      try {
+        listenersPerName.remove(listener.getConfig().name);
+        listenersPerStage.forEach((trigger, perStage) -> {
+          perStage.forEach((stage, listeners) -> {
+            listeners.remove(listener);
+          });
+        });
+        additionalListeners.remove(listener);
+      } finally {
+        updateLock.unlock();
+      }
+    }
+
     void setAutoScalingConfig(AutoScalingConfig autoScalingConfig) {
       updateLock.lock();
       // we will recreate this from scratch
@@ -699,7 +761,7 @@ public class ScheduledTriggers implements Closeable {
             try {
               listener.close();
             } catch (Exception e) {
-              log.warn("Exception closing old listener " + listener.getConfig(), e);
+              log.warn("Exception closing old listener {}", listener.getConfig(), e);
             }
             it.remove();
           }
@@ -718,7 +780,7 @@ public class ScheduledTriggers implements Closeable {
               try {
                 oldListener.close();
               } catch (Exception e) {
-                log.warn("Exception closing old listener " + oldListener.getConfig(), e);
+                log.warn("Exception closing old listener {}", oldListener.getConfig(), e);
               }
             } else {
               listener = oldListener; // reuse
@@ -729,7 +791,7 @@ public class ScheduledTriggers implements Closeable {
             try {
               listener = loader.newInstance(clazz, TriggerListener.class);
             } catch (Exception e) {
-              log.warn("Invalid TriggerListener class name '" + clazz + "', skipping...", e);
+              log.warn("Invalid TriggerListener class name '{}', skipping...", clazz, e);
             }
             if (listener != null) {
               try {
@@ -737,7 +799,7 @@ public class ScheduledTriggers implements Closeable {
                 listener.init();
                 listenersPerName.put(config.name, listener);
               } catch (Exception e) {
-                log.warn("Error initializing TriggerListener " + config, e);
+                log.warn("Error initializing TriggerListener {}", config, e);
                 IOUtils.closeQuietly(listener);
                 listener = null;
               }
@@ -758,6 +820,13 @@ public class ScheduledTriggers implements Closeable {
             addPerStage(config.trigger, TriggerEventProcessorStage.AFTER_ACTION, listener);
           }
         }
+        // re-add additional listeners
+        List<TriggerListener> additional = new ArrayList<>(additionalListeners);
+        additionalListeners.clear();
+        for (TriggerListener listener : additional) {
+          addAdditionalListener(listener);
+        }
+
       } finally {
         updateLock.unlock();
       }
@@ -836,7 +905,7 @@ public class ScheduledTriggers implements Closeable {
           try {
             listener.onEvent(event, stage, actionName, context, error, message);
           } catch (Exception e) {
-            log.warn("Exception running listener " + listener.getConfig(), e);
+            log.warn("Exception running listener {}", listener.getConfig(), e);
           }
         }
       } finally {
