@@ -24,10 +24,11 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.NoSuchElementException;
+import java.util.Objects;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
 
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.solr.client.solrj.cloud.DistribStateManager;
 import org.apache.solr.client.solrj.cloud.SolrCloudManager;
 import org.apache.solr.client.solrj.cloud.autoscaling.VersionedData;
@@ -41,15 +42,17 @@ import org.apache.solr.common.cloud.ClusterState;
 import org.apache.solr.common.cloud.DocCollection;
 import org.apache.solr.common.cloud.Replica;
 import org.apache.solr.common.cloud.Slice;
+import org.apache.solr.common.cloud.SolrZkClient;
+import org.apache.solr.common.util.Utils;
 import org.apache.solr.common.cloud.ZkNodeProps;
 import org.apache.solr.common.cloud.ZkStateReader;
-import org.apache.solr.common.util.Utils;
 import org.apache.solr.util.TestInjection;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import static org.apache.solr.cloud.overseer.CollectionMutator.checkCollectionKeyExistence;
 import static org.apache.solr.cloud.overseer.CollectionMutator.checkKeyExistence;
+import static org.apache.solr.cloud.overseer.SliceMutator.getZkClient;
 import static org.apache.solr.common.params.CommonParams.NAME;
 
 public class ReplicaMutator {
@@ -57,10 +60,12 @@ public class ReplicaMutator {
 
   protected final SolrCloudManager cloudManager;
   protected final DistribStateManager stateManager;
+  protected SolrZkClient zkClient;
 
   public ReplicaMutator(SolrCloudManager cloudManager) {
     this.cloudManager = cloudManager;
     this.stateManager = cloudManager.getDistribStateManager();
+    this.zkClient = getZkClient(cloudManager);
   }
 
   protected Replica setProperty(Replica replica, String key, String value) {
@@ -72,7 +77,7 @@ public class ReplicaMutator {
 
     Map<String, Object> replicaProps = new LinkedHashMap<>(replica.getProperties());
     replicaProps.put(key, value);
-    return new Replica(replica.getName(), replicaProps);
+    return new Replica(replica.getName(), replicaProps, replica.getCollection(), replica.getSlice());
   }
 
   protected Replica unsetProperty(Replica replica, String key) {
@@ -81,7 +86,7 @@ public class ReplicaMutator {
     if (!replica.containsKey(key)) return replica;
     Map<String, Object> replicaProps = new LinkedHashMap<>(replica.getProperties());
     replicaProps.remove(key);
-    return new Replica(replica.getName(), replicaProps);
+    return new Replica(replica.getName(), replicaProps, replica.getCollection(), replica.getSlice());
   }
 
   protected Replica setLeader(Replica replica) {
@@ -144,10 +149,11 @@ public class ReplicaMutator {
     }
     log.info("Setting property {} with value {} for collection {}", property, propVal, collectionName);
     log.debug("Full message: {}", message);
-    if (StringUtils.equalsIgnoreCase(replica.getStr(property), propVal)) return ZkStateWriter.NO_OP; // already the value we're going to set
+    if (StringUtils.equalsIgnoreCase(replica.getStr(property), propVal))
+      return ZkStateWriter.NO_OP; // already the value we're going to set
 
     // OK, there's no way we won't change the cluster state now
-    Map<String,Replica> replicas = collection.getSlice(sliceName).getReplicasCopy();
+    Map<String, Replica> replicas = collection.getSlice(sliceName).getReplicasCopy();
     if (isUnique == false) {
       replicas.get(replicaName).getProperties().put(property, propVal);
     } else { // Set prop for this replica, but remove it for all others.
@@ -159,7 +165,7 @@ public class ReplicaMutator {
         }
       }
     }
-    Slice newSlice = new Slice(sliceName, replicas, collection.getSlice(sliceName).shallowCopy());
+    Slice newSlice = new Slice(sliceName, replicas, collection.getSlice(sliceName).shallowCopy(),collectionName);
     DocCollection newCollection = CollectionMutator.updateSlice(collectionName, collection,
         newSlice);
     return new ZkWriteCommand(collectionName, newCollection);
@@ -245,12 +251,11 @@ public class ReplicaMutator {
       log.info("Failed to update state because the replica does not exist, {}", message);
       return ZkStateWriter.NO_OP;
     }
-
     if (coreNodeName == null) {
       coreNodeName = ClusterStateMutator.getAssignedCoreNodeName(collection,
           message.getStr(ZkStateReader.NODE_NAME_PROP), message.getStr(ZkStateReader.CORE_NAME_PROP));
       if (coreNodeName != null) {
-        log.debug("node=" + coreNodeName + " is already registered");
+        log.debug("node={} is already registered", coreNodeName);
       } else {
         if (!forceSetState) {
           log.info("Failed to update state because the replica does not exist, {}", message);
@@ -268,7 +273,7 @@ public class ReplicaMutator {
       //get shardId from ClusterState
       sliceName = ClusterStateMutator.getAssignedId(collection, coreNodeName);
       if (sliceName != null) {
-        log.debug("shard=" + sliceName + " is already registered");
+        log.debug("shard={} is already registered", sliceName);
       }
     }
     if (sliceName == null) {
@@ -276,17 +281,18 @@ public class ReplicaMutator {
       if (collectionExists) {
         // use existing numShards
         numShards = collection.getSlices().size();
-        log.debug("Collection already exists with " + ZkStateReader.NUM_SHARDS_PROP + "=" + numShards);
+        log.debug("Collection already exists with {} = {}", ZkStateReader.NUM_SHARDS_PROP, numShards);
       }
       sliceName = Assign.assignShard(collection, numShards);
-      log.info("Assigning new node to shard shard=" + sliceName);
+      log.info("Assigning new node to shard shard={}", sliceName);
     }
 
     Slice slice = collection != null ?  collection.getSlice(sliceName) : null;
 
+    Replica oldReplica = null;
     Map<String, Object> replicaProps = new LinkedHashMap<>(message.getProperties());
     if (slice != null) {
-      Replica oldReplica = slice.getReplica(coreNodeName);
+      oldReplica = slice.getReplica(coreNodeName);
       if (oldReplica != null) {
         if (oldReplica.containsKey(ZkStateReader.LEADER_PROP)) {
           replicaProps.put(ZkStateReader.LEADER_PROP, oldReplica.get(ZkStateReader.LEADER_PROP));
@@ -325,18 +331,25 @@ public class ReplicaMutator {
     String shardState = (String) replicaProps.remove(ZkStateReader.SHARD_STATE_PROP);
     String shardParent = (String) replicaProps.remove(ZkStateReader.SHARD_PARENT_PROP);
 
+    String nodeName = (String)replicaProps.get(ZkStateReader.NODE_NAME_PROP);
+    if (nodeName != null) {
+      String baseUrl = Utils.getBaseUrlForNodeName(nodeName, cloudManager.getClusterStateProvider().getClusterProperty(ZkStateReader.URL_SCHEME, "http"));
+      replicaProps.put(ZkStateReader.BASE_URL_PROP, baseUrl);
+    }
+    Replica replica = new Replica(coreNodeName, replicaProps, collectionName, sliceName);
 
-    Replica replica = new Replica(coreNodeName, replicaProps);
-    
     log.debug("Will update state for replica: {}", replica);
 
     Map<String, Object> sliceProps = null;
     Map<String, Replica> replicas;
 
+    boolean sliceChanged = true;
     if (slice != null) {
+      Slice.State originalState = slice.getState();
       collection = checkAndCompleteShardSplit(prevState, collection, coreNodeName, sliceName, replica);
       // get the current slice again because it may have been updated due to checkAndCompleteShardSplit method
       slice = collection.getSlice(sliceName);
+      sliceChanged = originalState != slice.getState();
       sliceProps = slice.getProperties();
       replicas = slice.getReplicasCopy();
     } else {
@@ -347,13 +360,61 @@ public class ReplicaMutator {
       sliceProps.put(Slice.PARENT, shardParent);
     }
     replicas.put(replica.getName(), replica);
-    slice = new Slice(sliceName, replicas, sliceProps);
+    slice = new Slice(sliceName, replicas, sliceProps, collectionName);
 
     DocCollection newCollection = CollectionMutator.updateSlice(collectionName, collection, slice);
     log.debug("Collection is now: {}", newCollection);
+    if (collection != null && collection.isPerReplicaState() && oldReplica != null) {
+      if (!sliceChanged && !persistStateJson(replica, oldReplica, collection)) {
+        if (log.isDebugEnabled()) {
+          log.debug(
+              "state.json is not persisted slice/replica : {}/{} \n , old : {}, \n new {}",
+              replica.getSlice(),
+              replica.getName(),
+              Utils.toJSONString(oldReplica.getProperties()),
+              Utils.toJSONString(replica.getProperties()));
+        }
+        return ZkWriteCommand.NO_OP;
+      }
+    }
     return new ZkWriteCommand(collectionName, newCollection);
   }
 
+  /** Whether it is required to persist the state.json */
+  private boolean persistStateJson(Replica newReplica, Replica oldReplica, DocCollection coll) {
+    if (!Objects.equals(newReplica.getBaseUrl(), oldReplica.getBaseUrl())) return true;
+    if (!Objects.equals(newReplica.getCoreName(), oldReplica.getCoreName())) return true;
+    if (!Objects.equals(newReplica.getNodeName(), oldReplica.getNodeName())) return true;
+    if (!Objects.equals(newReplica.getState(), oldReplica.getState())) return true;
+    if (!Objects.equals(
+        newReplica.getProperties().get(ZkStateReader.FORCE_SET_STATE_PROP),
+        oldReplica.getProperties().get(ZkStateReader.FORCE_SET_STATE_PROP))) {
+      if (log.isInfoEnabled()) {
+        log.info(
+            "{} force_set_state is changed from {} -> {}",
+            newReplica.getName(),
+            oldReplica.getProperties().get(ZkStateReader.FORCE_SET_STATE_PROP),
+            newReplica.getProperties().get(ZkStateReader.FORCE_SET_STATE_PROP));
+      }
+    return true;
+  }
+  Slice slice = coll.getSlice(newReplica.getSlice());
+  // the slice may be in recovery
+    if (slice.getState() == Slice.State.RECOVERY) {
+      if (log.isInfoEnabled()) {
+        log.info("{} slice state_is_recovery", slice.getName());
+      }
+      return true;
+    }
+    if (Objects.equals(oldReplica.getProperties().get("state"), "recovering")) {
+      if (log.isInfoEnabled()) {
+        log.info("{} state_is_recovering", newReplica.getName());
+      }
+      return true;
+    }
+    return false;
+  }
+
   /**
    * Handles non-legacy state updates
    */
@@ -394,7 +455,9 @@ public class ReplicaMutator {
           }
         }
         if (allActive) {
-          log.info("Shard: {} - all {} replicas are active. Finding status of fellow sub-shards", sliceName, slice.getReplicasMap().size());
+          if (log.isInfoEnabled()) {
+            log.info("Shard: {} - all {} replicas are active. Finding status of fellow sub-shards", sliceName, slice.getReplicasMap().size());
+          }
           // find out about other sub shards
           Map<String, Slice> allSlicesCopy = new HashMap<>(collection.getSlicesMap());
           List<Slice> subShardSlices = new ArrayList<>();
@@ -405,7 +468,9 @@ public class ReplicaMutator {
             Slice otherSlice = entry.getValue();
             if (otherSlice.getState() == Slice.State.RECOVERY) {
               if (slice.getParent() != null && slice.getParent().equals(otherSlice.getParent())) {
-                log.info("Shard: {} - Fellow sub-shard: {} found", sliceName, otherSlice.getName());
+                if (log.isInfoEnabled()) {
+                  log.info("Shard: {} - Fellow sub-shard: {} found", sliceName, otherSlice.getName());
+                }
                 // this is a fellow sub shard so check if all replicas are active
                 for (Map.Entry<String, Replica> sliceEntry : otherSlice.getReplicasMap().entrySet()) {
                   if (sliceEntry.getValue().getState() != Replica.State.ACTIVE) {
@@ -413,7 +478,9 @@ public class ReplicaMutator {
                     break outer;
                   }
                 }
-                log.info("Shard: {} - Fellow sub-shard: {} has all {} replicas active", sliceName, otherSlice.getName(), otherSlice.getReplicasMap().size());
+                if (log.isInfoEnabled()) {
+                  log.info("Shard: {} - Fellow sub-shard: {} has all {} replicas active", sliceName, otherSlice.getName(), otherSlice.getReplicasMap().size());
+                }
                 subShardSlices.add(otherSlice);
               }
             }
@@ -424,10 +491,10 @@ public class ReplicaMutator {
             String parentSliceName = (String) sliceProps.remove(Slice.PARENT);
             // now lets see if the parent leader is still the same or else there's a chance of data loss
             // see SOLR-9438 for details
-            String shardParentZkSession  = (String) sliceProps.remove("shard_parent_zk_session");
+            String shardParentZkSession = (String) sliceProps.remove("shard_parent_zk_session");
             String shardParentNode = (String) sliceProps.remove("shard_parent_node");
             boolean isLeaderSame = true;
-            if (shardParentNode != null && shardParentZkSession != null)  {
+            if (shardParentNode != null && shardParentZkSession != null) {
               log.info("Checking whether sub-shard leader node is still the same one at {} with ZK session id {}", shardParentNode, shardParentZkSession);
               try {
                 VersionedData leaderZnode = null;
@@ -437,7 +504,7 @@ public class ReplicaMutator {
                 } catch (NoSuchElementException e) {
                   // ignore
                 }
-                if (leaderZnode == null)  {
+                if (leaderZnode == null) {
                   log.error("The shard leader node: {} is not live anymore!", shardParentNode);
                   isLeaderSame = false;
                 } else if (!shardParentZkSession.equals(leaderZnode.getOwner())) {
@@ -446,8 +513,8 @@ public class ReplicaMutator {
                   isLeaderSame = false;
                 }
               } catch (Exception e) {
-                log.warn("Error occurred while checking if parent shard node is still live with the same zk session id. " +
-                    "We cannot switch shard states at this time.", e);
+                log.warn("Error occurred while checking if parent shard node is still live with the same zk session id. {}"
+                    , "We cannot switch shard states at this time.", e);
                 return collection; // we aren't going to make any changes right now
               }
             }
@@ -465,13 +532,17 @@ public class ReplicaMutator {
                 String lastTimeStr = subShardSlice.getStr(ZkStateReader.STATE_TIMESTAMP_PROP);
                 if (lastTimeStr != null) {
                   long start = Long.parseLong(lastTimeStr);
-                  log.info("TIMINGS: Sub-shard " + subShardSlice.getName() + " recovered in " +
-                      TimeUnit.MILLISECONDS.convert(now - start, TimeUnit.NANOSECONDS) + " ms");
+                  if (log.isInfoEnabled()) {
+                    log.info("TIMINGS: Sub-shard {} recovered in {} ms", subShardSlice.getName(),
+                        TimeUnit.MILLISECONDS.convert(now - start, TimeUnit.NANOSECONDS));
+                  }
                 } else {
-                  log.info("TIMINGS Sub-shard " + subShardSlice.getName() + " not available: " + subShardSlice);
+                  if (log.isInfoEnabled()) {
+                    log.info("TIMINGS Sub-shard {} not available: {}", subShardSlice.getName(), subShardSlice);
+                  }
                 }
               }
-            } else  {
+            } else {
               // we must mark the shard split as failed by switching sub-shards to recovery_failed state
               propMap.put(sliceName, Slice.State.RECOVERY_FAILED.toString());
               for (Slice subShardSlice : subShardSlices) {
@@ -482,8 +553,8 @@ public class ReplicaMutator {
             try {
               SplitShardCmd.unlockForSplit(cloudManager, collection.getName(), parentSliceName);
             } catch (Exception e) {
-              log.warn("Failed to unlock shard after " + (isLeaderSame ? "" : "un") + "successful split: {} / {}",
-                  collection.getName(), parentSliceName);
+              log.warn("Failed to unlock shard after {} successful split: {} / {}"
+                  , (isLeaderSame ? "" : "un"), collection.getName(), parentSliceName);
             }
             ZkNodeProps m = new ZkNodeProps(propMap);
             return new SliceMutator(cloudManager).updateShardState(prevState, m).collection;
@@ -494,4 +565,3 @@ public class ReplicaMutator {
     return collection;
   }
 }
-
