@@ -26,25 +26,31 @@ import java.lang.invoke.MethodHandles;
 import java.nio.charset.StandardCharsets;
 import java.nio.file.Files;
 import java.nio.file.Path;
+import java.nio.file.Paths;
 import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collections;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
 import java.util.Set;
+import java.util.stream.Collectors;
 
 import com.google.common.base.Strings;
 import org.apache.commons.io.IOUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.solr.client.solrj.impl.HttpClientUtil;
 import org.apache.solr.common.SolrException;
 import org.apache.solr.common.util.NamedList;
+import org.apache.solr.common.util.Utils;
 import org.apache.solr.logging.LogWatcherConfig;
 import org.apache.solr.metrics.reporters.SolrJmxReporter;
 import org.apache.solr.update.UpdateShardHandlerConfig;
-import org.apache.solr.util.DOMUtil;
+import org.apache.solr.common.util.DOMUtil;
 import org.apache.solr.util.JmxUtil;
-import org.apache.solr.util.PropertiesUtil;
+import org.apache.solr.common.util.PropertiesUtil;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 import org.w3c.dom.Node;
@@ -55,28 +61,61 @@ import static org.apache.solr.common.params.CommonParams.NAME;
 
 
 /**
- *
+ * Loads {@code solr.xml}.
  */
 public class SolrXmlConfig {
 
+  // TODO should these from* methods return a NodeConfigBuilder so that the caller (a test) can make further
+  //  manipulations like add properties and set the CorePropertiesLocator and "async" mode?
+
+  public final static String ZK_HOST = "zkHost";
   public final static String SOLR_XML_FILE = "solr.xml";
   public final static String SOLR_DATA_HOME = "solr.data.home";
 
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
-  public static NodeConfig fromConfig(Config config) {
+  /**
+   * Given some node Properties, checks if non-null and a 'zkHost' is alread included.  If so, the Properties are
+   * returned as is.  If not, then the returned value will be a new Properties, wrapping the original Properties, 
+   * with the 'zkHost' value set based on the value of the corispond System property (if set)
+   *
+   * In theory we only need this logic once, ideally in SolrDispatchFilter, but we put it here to re-use 
+   * redundently because of how much surface area our API has for various tests to poke at us.
+   */
+  public static Properties wrapAndSetZkHostFromSysPropIfNeeded(final Properties props) {
+    if (null != props && ! StringUtils.isEmpty(props.getProperty(ZK_HOST))) {
+      // nothing to do...
+      return props;
+    }
+    // we always wrap if we might set a property -- never mutate the original props
+    final Properties results = (null == props ? new Properties() : new Properties(props));
+    final String sysprop = System.getProperty(ZK_HOST);
+    if (! StringUtils.isEmpty(sysprop)) {
+      results.setProperty(ZK_HOST, sysprop);
+    }
+    return results;
+  }
 
+  
+  public static NodeConfig fromConfig(Path solrHome, XmlConfigFile config) {
     checkForIllegalConfig(config);
 
-    config.substituteProperties();
-
+    // Regardless of where/how we this XmlConfigFile was loaded from, if it contains a zkHost property,
+    // we're going to use that as our "default" and only *directly* check the system property if it's not specified.
+    //
+    // (checking the sys prop here is really just for tests that by-pass SolrDispatchFilter. In non-test situations,
+    // SolrDispatchFilter will check the system property if needed in order to try and load solr.xml from ZK, and
+    // should have put the sys prop value in the node properties for us)
+    final String defaultZkHost
+      = wrapAndSetZkHostFromSysPropIfNeeded(config.getSubstituteProperties()).getProperty(ZK_HOST);
+    
     CloudConfig cloudConfig = null;
     UpdateShardHandlerConfig deprecatedUpdateConfig = null;
 
     if (config.getNodeList("solr/solrcloud", false).getLength() > 0) {
       NamedList<Object> cloudSection = readNodeListAsNamedList(config, "solr/solrcloud/*[@name]", "<solrcloud>");
       deprecatedUpdateConfig = loadUpdateConfig(cloudSection, false);
-      cloudConfig = fillSolrCloudSection(cloudSection);
+      cloudConfig = fillSolrCloudSection(cloudSection, config, defaultZkHost);
     }
 
     NamedList<Object> entries = readNodeListAsNamedList(config, "solr/*[@name]", "<solr>");
@@ -96,20 +135,23 @@ public class SolrXmlConfig {
       updateConfig = deprecatedUpdateConfig;
     }
 
-    NodeConfig.NodeConfigBuilder configBuilder = new NodeConfig.NodeConfigBuilder(nodeName, config.getResourceLoader());
+    NodeConfig.NodeConfigBuilder configBuilder = new NodeConfig.NodeConfigBuilder(nodeName, solrHome);
+    configBuilder.setSolrResourceLoader(config.getResourceLoader());
     configBuilder.setUpdateShardHandlerConfig(updateConfig);
     configBuilder.setShardHandlerFactoryConfig(getShardHandlerFactoryPluginInfo(config));
     configBuilder.setSolrCoreCacheFactoryConfig(getTransientCoreCacheFactoryPluginInfo(config));
+    configBuilder.setTracerConfig(getTracerPluginInfo(config));
     configBuilder.setLogWatcherConfig(loadLogWatcherConfig(config, "solr/logging/*[@name]", "solr/logging/watcher/*[@name]"));
     configBuilder.setSolrProperties(loadProperties(config));
     if (cloudConfig != null)
       configBuilder.setCloudConfig(cloudConfig);
     configBuilder.setBackupRepositoryPlugins(getBackupRepositoryPluginInfos(config));
     configBuilder.setMetricsConfig(getMetricsConfig(config));
+    configBuilder.setDefaultZkHost(defaultZkHost);
     return fillSolrSection(configBuilder, entries);
   }
 
-  public static NodeConfig fromFile(SolrResourceLoader loader, Path configFile) {
+  public static NodeConfig fromFile(Path solrHome, Path configFile, Properties substituteProps) {
 
     log.info("Loading container configuration from {}", configFile);
 
@@ -119,7 +161,7 @@ public class SolrXmlConfig {
     }
 
     try (InputStream inputStream = Files.newInputStream(configFile)) {
-      return fromInputStream(loader, inputStream);
+      return fromInputStream(solrHome, inputStream, substituteProps);
     } catch (SolrException exc) {
       throw exc;
     } catch (Exception exc) {
@@ -128,16 +170,24 @@ public class SolrXmlConfig {
     }
   }
 
-  public static NodeConfig fromString(SolrResourceLoader loader, String xml) {
-    return fromInputStream(loader, new ByteArrayInputStream(xml.getBytes(StandardCharsets.UTF_8)));
+  /** TEST-ONLY */
+  public static NodeConfig fromString(Path solrHome, String xml) {
+    return fromInputStream(
+        solrHome,
+        new ByteArrayInputStream(xml.getBytes(StandardCharsets.UTF_8)),
+        new Properties());
   }
 
-  public static NodeConfig fromInputStream(SolrResourceLoader loader, InputStream is) {
+  public static NodeConfig fromInputStream(Path solrHome, InputStream is, Properties substituteProps) {
+    SolrResourceLoader loader = new SolrResourceLoader(solrHome);
+    if (substituteProps == null) {
+      substituteProps = new Properties();
+    }
     try {
       byte[] buf = IOUtils.toByteArray(is);
       try (ByteArrayInputStream dup = new ByteArrayInputStream(buf)) {
-        Config config = new Config(loader, null, new InputSource(dup), null, false);
-        return fromConfig(config);
+        XmlConfigFile config = new XmlConfigFile(loader, null, new InputSource(dup), null, substituteProps);
+        return fromConfig(solrHome, config);
       }
     } catch (SolrException exc) {
       throw exc;
@@ -146,16 +196,11 @@ public class SolrXmlConfig {
     }
   }
 
-  public static NodeConfig fromSolrHome(SolrResourceLoader loader, Path solrHome) {
-    return fromFile(loader, solrHome.resolve(SOLR_XML_FILE));
+  public static NodeConfig fromSolrHome(Path solrHome, Properties substituteProps) {
+    return fromFile(solrHome, solrHome.resolve(SOLR_XML_FILE), substituteProps);
   }
 
-  public static NodeConfig fromSolrHome(Path solrHome) {
-    SolrResourceLoader loader = new SolrResourceLoader(solrHome);
-    return fromSolrHome(loader, solrHome);
-  }
-
-  private static void checkForIllegalConfig(Config config) {
+  private static void checkForIllegalConfig(XmlConfigFile config) {
     failIfFound(config, "solr/@coreLoadThreads");
     failIfFound(config, "solr/@persistent");
     failIfFound(config, "solr/@sharedLib");
@@ -168,12 +213,12 @@ public class SolrXmlConfig {
     assertSingleInstance("backup", config);
   }
 
-  private static void assertSingleInstance(String section, Config config) {
+  private static void assertSingleInstance(String section, XmlConfigFile config) {
     if (config.getNodeList("/solr/" + section, false).getLength() > 1)
       throw new SolrException(SolrException.ErrorCode.SERVER_ERROR, "Multiple instances of " + section + " section found in solr.xml");
   }
 
-  private static void failIfFound(Config config, String xPath) {
+  private static void failIfFound(XmlConfigFile config, String xPath) {
 
     if (config.getVal(xPath, false) != null) {
       throw new SolrException(SolrException.ErrorCode.SERVER_ERROR, "Should not have found " + xPath +
@@ -181,12 +226,12 @@ public class SolrXmlConfig {
     }
   }
 
-  private static Properties loadProperties(Config config) {
+  private static Properties loadProperties(XmlConfigFile config) {
     try {
       Node node = ((NodeList) config.evaluate("solr", XPathConstants.NODESET)).item(0);
       XPath xpath = config.getXPath();
       NodeList props = (NodeList) xpath.evaluate("property", node, XPathConstants.NODESET);
-      Properties properties = new Properties();
+      Properties properties = new Properties(config.getSubstituteProperties());
       for (int i = 0; i < props.getLength(); i++) {
         Node prop = props.item(i);
         properties.setProperty(DOMUtil.getAttr(prop, NAME),
@@ -195,12 +240,12 @@ public class SolrXmlConfig {
       return properties;
     }
     catch (XPathExpressionException e) {
-      log.warn("Error parsing solr.xml: " + e.getMessage());
+      log.warn("Error parsing solr.xml: ", e);
       return null;
     }
   }
 
-  private static NamedList<Object> readNodeListAsNamedList(Config config, String path, String section) {
+  private static NamedList<Object> readNodeListAsNamedList(XmlConfigFile config, String path, String section) {
     NodeList nodes = config.getNodeList(path, false);
     if (nodes == null) {
       return null;
@@ -242,6 +287,9 @@ public class SolrXmlConfig {
         case "collectionsHandler":
           builder.setCollectionsAdminHandlerClass(value);
           break;
+        case "healthCheckHandler":
+          builder.setHealthCheckHandlerClass(value);
+          break;
         case "infoHandler":
           builder.setInfoHandlerClass(value);
           break;
@@ -254,12 +302,18 @@ public class SolrXmlConfig {
         case "solrDataHome":
           builder.setSolrDataHome(value);
           break;
+        case "maxBooleanClauses":
+          builder.setBooleanQueryMaxClauseCount(parseInt(name, value));
+          break;
         case "managementPath":
           builder.setManagementPath(value);
           break;
         case "sharedLib":
           builder.setSharedLibDirectory(value);
           break;
+        case "allowPaths":
+          builder.setAllowPaths(stringToPaths(value));
+          break;
         case "configSetBaseDir":
           builder.setConfigSetBaseDirectory(value);
           break;
@@ -283,6 +337,15 @@ public class SolrXmlConfig {
     return builder.build();
   }
 
+  private static Set<Path> stringToPaths(String commaSeparatedString) {
+    if (Strings.isNullOrEmpty(commaSeparatedString)) {
+      return Collections.emptySet();
+    }
+    // Parse list of paths. The special value '*' is mapped to _ALL_ to mean all paths
+    return Arrays.stream(commaSeparatedString.split(",\\s?"))
+        .map(p -> Paths.get("*".equals(p) ? "_ALL_" : p)).collect(Collectors.toSet());
+  }
+
   private static UpdateShardHandlerConfig loadUpdateConfig(NamedList<Object> nl, boolean alwaysDefine) {
 
     if (nl == null && !alwaysDefine)
@@ -357,14 +420,20 @@ public class SolrXmlConfig {
     throw new SolrException(SolrException.ErrorCode.SERVER_ERROR, section + " section missing required entry '" + key + "'");
   }
 
-  private static CloudConfig fillSolrCloudSection(NamedList<Object> nl) {
+  private static CloudConfig fillSolrCloudSection(NamedList<Object> nl, XmlConfigFile config, String defaultZkHost) {
 
-    String hostName = required("solrcloud", "host", removeValue(nl, "host"));
     int hostPort = parseInt("hostPort", required("solrcloud", "hostPort", removeValue(nl, "hostPort")));
+    if (hostPort <= 0) {
+      // Default to the port that jetty is listening on, or 8983 if that is not provided.
+      hostPort = parseInt("jetty.port", System.getProperty("jetty.port", "8983"));
+    }
+    String hostName = required("solrcloud", "host", removeValue(nl, "host"));
     String hostContext = required("solrcloud", "hostContext", removeValue(nl, "hostContext"));
 
     CloudConfig.CloudConfigBuilder builder = new CloudConfig.CloudConfigBuilder(hostName, hostPort, hostContext);
-
+    // set the defaultZkHost until/unless it's overridden in the "cloud section" (below)...
+    builder.setZkHost(defaultZkHost);
+    
     for (Map.Entry<String, Object> entry : nl) {
       String name = entry.getKey();
       if (entry.getValue() == null)
@@ -382,7 +451,7 @@ public class SolrXmlConfig {
           break;
         case "autoReplicaFailoverBadNodeExpiration": case "autoReplicaFailoverWorkLoopDelay":
           //TODO remove this in Solr 8.0
-          log.info("Configuration parameter " + name + " is ignored");
+          log.info("Configuration parameter {} is ignored", name);
           break;
         case "autoReplicaFailoverWaitAfterExpiration":
           builder.setAutoReplicaFailoverWaitAfterExpiration(parseInt(name, value));
@@ -413,7 +482,7 @@ public class SolrXmlConfig {
     return builder.build();
   }
 
-  private static LogWatcherConfig loadLogWatcherConfig(Config config, String loggingPath, String watcherPath) {
+  private static LogWatcherConfig loadLogWatcherConfig(XmlConfigFile config, String loggingPath, String watcherPath) {
 
     String loggingClass = null;
     boolean enabled = true;
@@ -450,12 +519,12 @@ public class SolrXmlConfig {
 
   }
 
-  private static PluginInfo getShardHandlerFactoryPluginInfo(Config config) {
+  private static PluginInfo getShardHandlerFactoryPluginInfo(XmlConfigFile config) {
     Node node = config.getNode("solr/shardHandlerFactory", false);
     return (node == null) ? null : new PluginInfo(node, "shardHandlerFactory", false, true);
   }
 
-  private static PluginInfo[] getBackupRepositoryPluginInfos(Config config) {
+  private static PluginInfo[] getBackupRepositoryPluginInfos(XmlConfigFile config) {
     NodeList nodes = (NodeList) config.evaluate("solr/backup/repository", XPathConstants.NODESET);
     if (nodes == null || nodes.getLength() == 0)
       return new PluginInfo[0];
@@ -466,9 +535,20 @@ public class SolrXmlConfig {
     return configs;
   }
 
-  private static MetricsConfig getMetricsConfig(Config config) {
+  private static MetricsConfig getMetricsConfig(XmlConfigFile config) {
     MetricsConfig.MetricsConfigBuilder builder = new MetricsConfig.MetricsConfigBuilder();
-    Node node = config.getNode("solr/metrics/suppliers/counter", false);
+    Node node = config.getNode("solr/metrics", false);
+    // enabled by default
+    boolean enabled = true;
+    if (node != null) {
+      enabled = Boolean.parseBoolean(DOMUtil.getAttrOrDefault(node, "enabled", "true"));
+    }
+    builder.setEnabled(enabled);
+    if (!enabled) {
+      log.info("Metrics collection is disabled.");
+      return builder.build();
+    }
+    node = config.getNode("solr/metrics/suppliers/counter", false);
     if (node != null) {
       builder = builder.setCounterSupplier(new PluginInfo(node, "counterSupplier", false, false));
     }
@@ -488,6 +568,15 @@ public class SolrXmlConfig {
     if (node != null) {
       builder = builder.setHistoryHandler(new PluginInfo(node, "history", false, false));
     }
+    node = config.getNode("solr/metrics/missingValues", false);;
+    if (node != null) {
+      NamedList<Object> missingValues = DOMUtil.childNodesToNamedList(node);
+      builder.setNullNumber(decodeNullValue(missingValues.get("nullNumber")));
+      builder.setNotANumber(decodeNullValue(missingValues.get("notANumber")));
+      builder.setNullString(decodeNullValue(missingValues.get("nullString")));
+      builder.setNullObject(decodeNullValue(missingValues.get("nullObject")));
+    }
+
     PluginInfo[] reporterPlugins = getMetricReporterPluginInfos(config);
     Set<String> hiddenSysProps = getHiddenSysProps(config);
     return builder
@@ -496,7 +585,21 @@ public class SolrXmlConfig {
         .build();
   }
 
-  private static PluginInfo[] getMetricReporterPluginInfos(Config config) {
+  private static Object decodeNullValue(Object o) {
+    if (o instanceof String) { // check if it's a JSON object
+      String str = (String) o;
+      if (!str.trim().isEmpty() && (str.startsWith("{") || str.startsWith("["))) {
+        try {
+          o = Utils.fromJSONString((String) o);
+        } catch (Exception e) {
+          // ignore
+        }
+      }
+    }
+    return o;
+  }
+
+  private static PluginInfo[] getMetricReporterPluginInfos(XmlConfigFile config) {
     NodeList nodes = (NodeList) config.evaluate("solr/metrics/reporter", XPathConstants.NODESET);
     List<PluginInfo> configs = new ArrayList<>();
     boolean hasJmxReporter = false;
@@ -514,7 +617,7 @@ public class SolrXmlConfig {
     // if there's an MBean server running but there was no JMX reporter then add a default one
     MBeanServer mBeanServer = JmxUtil.findFirstMBeanServer();
     if (mBeanServer != null && !hasJmxReporter) {
-      log.info("MBean server found: " + mBeanServer + ", but no JMX reporters were configured - adding default JMX reporter.");
+      log.info("MBean server found: {}, but no JMX reporters were configured - adding default JMX reporter.", mBeanServer);
       Map<String,Object> attributes = new HashMap<>();
       attributes.put("name", "default");
       attributes.put("class", SolrJmxReporter.class.getName());
@@ -524,7 +627,7 @@ public class SolrXmlConfig {
     return configs.toArray(new PluginInfo[configs.size()]);
   }
 
-  private static Set<String> getHiddenSysProps(Config config) {
+  private static Set<String> getHiddenSysProps(XmlConfigFile config) {
     NodeList nodes = (NodeList) config.evaluate("solr/metrics/hiddenSysProps/str", XPathConstants.NODESET);
     if (nodes == null || nodes.getLength() == 0) {
       return NodeConfig.NodeConfigBuilder.DEFAULT_HIDDEN_SYS_PROPS;
@@ -543,8 +646,13 @@ public class SolrXmlConfig {
     }
   }
 
-  private static PluginInfo getTransientCoreCacheFactoryPluginInfo(Config config) {
+  private static PluginInfo getTransientCoreCacheFactoryPluginInfo(XmlConfigFile config) {
     Node node = config.getNode("solr/transientCoreCacheFactory", false);
     return (node == null) ? null : new PluginInfo(node, "transientCoreCacheFactory", false, true);
   }
+
+  private static PluginInfo getTracerPluginInfo(XmlConfigFile config) {
+    Node node = config.getNode("solr/tracerConfig", false);
+    return (node == null) ? null : new PluginInfo(node, "tracerConfig", false, true);
+  }
 }
