@@ -25,6 +25,7 @@ import java.io.Closeable;
 import java.io.IOException;
 import java.lang.invoke.MethodHandles;
 import java.net.MalformedURLException;
+import java.net.URI;
 import java.net.URL;
 import java.util.ArrayList;
 import java.util.Arrays;
@@ -60,6 +61,7 @@ import org.apache.solr.client.solrj.SolrRequest;
 import org.apache.solr.client.solrj.cloud.NodeStateProvider;
 import org.apache.solr.client.solrj.cloud.SolrCloudManager;
 import org.apache.solr.client.solrj.cloud.autoscaling.ReplicaInfo;
+import org.apache.solr.client.solrj.cloud.autoscaling.Variable;
 import org.apache.solr.client.solrj.cloud.autoscaling.VersionedData;
 import org.apache.solr.client.solrj.impl.HttpClientUtil;
 import org.apache.solr.cloud.LeaderElector;
@@ -89,7 +91,7 @@ import org.apache.solr.request.SolrQueryRequest;
 import org.apache.solr.response.SolrQueryResponse;
 import org.apache.solr.security.AuthorizationContext;
 import org.apache.solr.security.PermissionNameProvider;
-import org.apache.solr.util.DefaultSolrThreadFactory;
+import org.apache.solr.common.util.SolrNamedThreadFactory;
 import org.apache.zookeeper.KeeperException;
 import org.rrd4j.ConsolFun;
 import org.rrd4j.DsType;
@@ -108,11 +110,13 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import static java.util.stream.Collectors.toMap;
+import static org.apache.solr.common.cloud.ZkStateReader.URL_SCHEME;
 import static org.apache.solr.common.params.CommonParams.ID;
 
 /**
  * Collects metrics from all nodes in the system on a regular basis in a background thread.
  * @since 7.4
+ * @deprecated this functionality will be removed in Solr 9.0
  */
 public class MetricsHistoryHandler extends RequestHandlerBase implements PermissionNameProvider, Closeable {
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
@@ -135,7 +139,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
 
     DEFAULT_NODE_GAUGES.add("CONTAINER.fs.coreRoot.usableSpace");
 
-    DEFAULT_CORE_GAUGES.add("INDEX.sizeInBytes");
+    DEFAULT_CORE_GAUGES.add(Variable.Type.CORE_IDX.metricsAttribute);
 
     DEFAULT_CORE_COUNTERS.add("QUERY./select.requests");
     DEFAULT_CORE_COUNTERS.add("UPDATE./update.requests");
@@ -183,18 +187,21 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
     }
     // override from ZK if available
     if (cloudManager != null) {
+      @SuppressWarnings({"unchecked"})
       Map<String, Object> props = (Map<String, Object>)cloudManager.getClusterStateProvider()
           .getClusterProperty("metrics", Collections.emptyMap())
           .getOrDefault("history", Collections.emptyMap());
       args.putAll(props);
 
-      overseerUrlScheme = cloudManager.getClusterStateProvider().getClusterProperty("urlScheme", "http");
+      overseerUrlScheme = cloudManager.getClusterStateProvider().getClusterProperty(URL_SCHEME, "http");
     } else {
       overseerUrlScheme = "http";
     }
 
     this.nodeName = nodeName;
-    this.enable = Boolean.parseBoolean(String.valueOf(args.getOrDefault(ENABLE_PROP, "true")));
+    // disable when metrics reporting is disabled
+    this.enable = Boolean.parseBoolean(String.valueOf(args.getOrDefault(ENABLE_PROP, "true")))
+        && metricsHandler.isEnabled();
     // default to false - don't collect local per-replica metrics
     this.enableReplicas = Boolean.parseBoolean(String.valueOf(args.getOrDefault(ENABLE_REPLICAS_PROP, "false")));
     this.enableNodes = Boolean.parseBoolean(String.valueOf(args.getOrDefault(ENABLE_NODES_PROP, "false")));
@@ -205,8 +212,6 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
     this.metricsHandler = metricsHandler;
     this.cloudManager = cloudManager;
     this.timeSource = cloudManager != null ? cloudManager.getTimeSource() : TimeSource.NANO_TIME;
-    factory = new SolrRrdBackendFactory(solrClient, CollectionAdminParams.SYSTEM_COLL,
-            syncPeriod, this.timeSource);
 
     counters.put(Group.core.toString(), DEFAULT_CORE_COUNTERS);
     counters.put(Group.node.toString(), Collections.emptyList());
@@ -226,8 +231,11 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
     }
 
     if (enable) {
+      factory = new SolrRrdBackendFactory(solrClient, CollectionAdminParams.SYSTEM_COLL,
+              syncPeriod, this.timeSource);
+
       collectService = (ScheduledThreadPoolExecutor) Executors.newScheduledThreadPool(1,
-          new DefaultSolrThreadFactory("MetricsHistoryHandler"));
+          new SolrNamedThreadFactory("MetricsHistoryHandler"));
       collectService.setRemoveOnCancelPolicy(true);
       collectService.setExecuteExistingDelayedTasksAfterShutdownPolicy(false);
       collectService.scheduleWithFixedDelay(() -> collectMetrics(),
@@ -235,11 +243,16 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
           timeSource.convertDelay(TimeUnit.SECONDS, collectPeriod, TimeUnit.MILLISECONDS),
           TimeUnit.MILLISECONDS);
       checkSystemCollection();
+    } else {
+      factory = null;
     }
   }
 
   // check that .system exists
   public void checkSystemCollection() {
+    if (!enable) {
+      return;
+    }
     if (cloudManager != null) {
       try {
         if (cloudManager.isClosed() || Thread.interrupted()) {
@@ -250,7 +263,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
         DocCollection systemColl = clusterState.getCollectionOrNull(CollectionAdminParams.SYSTEM_COLL);
         if (systemColl == null) {
           if (logMissingCollection) {
-            log.info("No " + CollectionAdminParams.SYSTEM_COLL + " collection, keeping metrics history in memory.");
+            log.info("No {} collection, keeping metrics history in memory.", CollectionAdminParams.SYSTEM_COLL);
             logMissingCollection = false;
           }
           factory.setPersistent(false);
@@ -264,7 +277,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
             }
           }
           if (!ready) {
-            log.debug(CollectionAdminParams.SYSTEM_COLL + "collection not ready yet, keeping metrics history in memory");
+            log.debug("{} collection not ready yet, keeping metrics history in memory", CollectionAdminParams.SYSTEM_COLL);
             factory.setPersistent(false);
             return;
           }
@@ -286,7 +299,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
         logMissingCollection = true;
       } catch (Exception e) {
         if (logMissingCollection) {
-          log.info("No " + CollectionAdminParams.SYSTEM_COLL + " collection, keeping metrics history in memory.");
+          log.info("No {} collection, keeping metrics history in memory.", CollectionAdminParams.SYSTEM_COLL);
         }
         logMissingCollection = false;
         factory.setPersistent(false);
@@ -299,9 +312,11 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
   }
 
   public void removeHistory(String registry) throws IOException {
-    registry = SolrMetricManager.overridableRegistryName(registry);
+    registry = SolrMetricManager.enforcePrefix(registry);
     knownDbs.remove(registry);
-    factory.remove(registry);
+    if (factory != null) {
+      factory.remove(registry);
+    }
   }
 
   @VisibleForTesting
@@ -321,7 +336,10 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
       if (data != null && data.getData() != null) {
         props = ZkNodeProps.load(data.getData());
       }
-    } catch (KeeperException | IOException | NoSuchElementException e) {
+    } catch (KeeperException | NoSuchElementException e) {
+      log.info("Could not obtain overseer's address, skipping.", e);
+      return null;
+    } catch (IOException e) {
       log.warn("Could not obtain overseer's address, skipping.", e);
       return null;
     } catch (InterruptedException e) {
@@ -339,7 +357,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
     try {
       nodeName = LeaderElector.getNodeName(oid);
     } catch (Exception e) {
-      log.warn("Unknown format of leader id, skipping: " + oid, e);
+      log.warn("Unknown format of leader id, skipping: {}", oid, e);
       return null;
     }
     return nodeName;
@@ -377,6 +395,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
     ExecutorUtil.setServerThreadFlag(false);
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private void collectLocalReplicaMetrics() {
     List<Group> groups = new ArrayList<>();
     if (enableNodes) {
@@ -390,7 +409,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
       if (Thread.interrupted()) {
         return;
       }
-      log.debug("--  collecting local " + group + "...");
+      log.debug("--  collecting local {}...", group);
       ModifiableSolrParams params = new ModifiableSolrParams();
       params.add(MetricsHandler.GROUP_PARAM, group.toString());
       params.add(MetricsHandler.COMPACT_PARAM, "true");
@@ -440,7 +459,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
           }
         }
       } catch (Exception e) {
-        e.printStackTrace();
+        log.warn("Exception retrieving local metrics for group {}: {}", group, e);
       }
     }
   }
@@ -586,13 +605,14 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
             s.update();
           }
         } catch (Exception e) {
+          log.warn("Exception storing sample in RrdDb for group {}: {}", group, e);
         }
       });
     });
   }
 
   private RrdDef createDef(String registry, Group group) {
-    registry = SolrMetricManager.overridableRegistryName(registry);
+    registry = SolrMetricManager.enforcePrefix(registry);
 
     // base sampling period is collectPeriod - samples more frequent than
     // that will be dropped, samples less frequent will be interpolated
@@ -638,6 +658,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
         RrdDb newDb = new RrdDb(def, factory);
         return newDb;
       } catch (IOException e) {
+        log.warn("Can't create RrdDb for registry {}, group {}: {}", registry, group, e);
         return null;
       }
     });
@@ -646,20 +667,10 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
 
   @Override
   public void close() {
-    log.debug("Closing " + hashCode());
-    if (collectService != null) {
-      boolean shutdown = false;
-      while (!shutdown) {
-        try {
-          // Wait a while for existing tasks to terminate
-          collectService.shutdownNow();
-          shutdown = collectService.awaitTermination(5, TimeUnit.SECONDS);
-        } catch (InterruptedException ie) {
-          // Preserve interrupt status
-          Thread.currentThread().interrupt();
-        }
-      }
+    if (log.isDebugEnabled()) {
+      log.debug("Closing {}", hashCode());
     }
+    ExecutorUtil.shutdownNowAndAwaitTermination(collectService);
     if (factory != null) {
       factory.close();
     }
@@ -701,6 +712,10 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
 
   @Override
   public void handleRequestBody(SolrQueryRequest req, SolrQueryResponse rsp) throws Exception {
+    if (!enable) {
+      rsp.add("error", "metrics history collection is disabled");
+      return;
+    }
     String actionStr = req.getParams().get(CommonParams.ACTION);
     if (actionStr == null) {
       throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, "'action' is a required param");
@@ -752,7 +767,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
         }
         if (factory.exists(name)) {
           // get a throwaway copy (safe to close and discard)
-          RrdDb db = new RrdDb(URI_PREFIX + name, true, factory);
+          RrdDb db = RrdDb.getBuilder().setBackendFactory(factory).setReadOnly(true).setPath(new URI(URI_PREFIX + name)).build();
           SimpleOrderedMap<Object> status = new SimpleOrderedMap<>();
           status.add("status", getDbStatus(db));
           status.add("node", nodeName);
@@ -794,6 +809,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
     rsp.getResponseHeader().add("zkConnected", cloudManager != null);
   }
 
+  @SuppressWarnings({"unchecked"})
   private NamedList<Object> handleRemoteRequest(String nodeName, SolrQueryRequest req) {
     String baseUrl = Utils.getBaseUrlForNodeName(nodeName, overseerUrlScheme);
     String url;
@@ -802,7 +818,7 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
       u = new URL(u.getProtocol(), u.getHost(), u.getPort(), "/api/cluster/metrics/history");
       url = u.toString();
     } catch (MalformedURLException e) {
-      log.warn("Invalid Overseer url '" + baseUrl + "', unable to fetch remote metrics history", e);
+      log.warn("Invalid Overseer url '{}', unable to fetch remote metrics history", baseUrl, e);
       return null;
     }
     // always use javabin
@@ -816,11 +832,12 @@ public class MetricsHistoryHandler extends RequestHandlerBase implements Permiss
         return (NamedList<Object>)codec.unmarshal(new ByteArrayInputStream(data));
       }
     } catch (IOException e) {
-      log.warn("Exception forwarding request to Overseer at " + url, e);
+      log.warn("Exception forwarding request to Overseer at {}", url, e);
       return null;
     }
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private void mergeRemoteRes(SolrQueryResponse rsp, NamedList<Object> remoteRes) {
     if (remoteRes == null || remoteRes.get("metrics") == null) {
       return;
