@@ -26,7 +26,7 @@ import java.util.Map;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
 
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.solr.cloud.LeaderElector;
 import org.apache.solr.cloud.OverseerTaskProcessor;
 import org.apache.solr.cloud.overseer.SliceMutator;
@@ -110,6 +110,7 @@ class RebalanceLeaders {
   final static String INACTIVE_PREFERREDS = "inactivePreferreds";
   final static String ALREADY_LEADERS = "alreadyLeaders";
   final static String SUMMARY = "Summary";
+  @SuppressWarnings({"rawtypes"})
   final SimpleOrderedMap results = new SimpleOrderedMap();
   final Map<String, String> pendingOps = new HashMap<>();
   private String collectionName;
@@ -122,6 +123,7 @@ class RebalanceLeaders {
     coreContainer = collectionsHandler.getCoreContainer();
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   void execute() throws KeeperException, InterruptedException {
     DocCollection dc = checkParams();
 
@@ -137,7 +139,7 @@ class RebalanceLeaders {
     for (Slice slice : dc.getSlices()) {
       ensurePreferredIsLeader(slice);
       if (asyncRequests.size() == max) {
-        log.info("Queued " + max + " leader reassignments, waiting for some to complete.");
+        log.info("Queued {} leader reassignments, waiting for some to complete.", max);
         keepGoing = waitAsyncRequests(maxWaitSecs, false);
         if (keepGoing == false) {
           break; // If we've waited longer than specified, don't continue to wait!
@@ -150,7 +152,7 @@ class RebalanceLeaders {
     if (keepGoing == true) {
       log.info("All leader reassignments completed.");
     } else {
-      log.warn("Exceeded specified timeout of ." + maxWaitSecs + "' all leaders may not have been reassigned");
+      log.warn("Exceeded specified timeout of '{}' all leaders may not have been reassigned'", maxWaitSecs);
     }
 
     checkLeaderStatus();
@@ -276,8 +278,8 @@ class RebalanceLeaders {
   // if there's only one we can't change anything.
   private boolean electionQueueInBadState(List<String> electionNodes, Slice slice, Replica replica) {
     if (electionNodes.size() < 2) { // if there's only one node in the queue, should already be leader and we shouldn't be here anyway.
-      log.warn("Rebalancing leaders and slice {} has less than two elements in the leader " +
-          "election queue, but replica {} doesn't think it's the leader.", slice.getName(), replica.getName());
+      log.warn("Rebalancing leaders and slice {} has less than two elements in the leader election queue, but replica {} doesn't think it's the leader."
+          , slice.getName(), replica.getName());
       return true;
     }
 
@@ -286,6 +288,7 @@ class RebalanceLeaders {
 
   // Provide some feedback to the user about what actually happened, or in this case where no action was
   // possible
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private void addInactiveToResults(Slice slice, Replica replica) {
     SimpleOrderedMap inactives = (SimpleOrderedMap) results.get(INACTIVE_PREFERREDS);
     if (inactives == null) {
@@ -300,6 +303,7 @@ class RebalanceLeaders {
 
   // Provide some feedback to the user about what actually happened, or in this case where no action was
   // necesary since this preferred replica was already the leader
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private void addAlreadyLeaderToResults(Slice slice, Replica replica) {
     SimpleOrderedMap alreadyLeaders = (SimpleOrderedMap) results.get(ALREADY_LEADERS);
     if (alreadyLeaders == null) {
@@ -314,7 +318,7 @@ class RebalanceLeaders {
 
   // Put the replica in at the head of the queue and send all nodes with the same sequence number to the back of the list
   // There can be "ties", i.e. replicas in the queue with the same sequence number. Sorting doesn't necessarily sort
-  // the one we most care about first. So put the node we _don't care about at the end of the election queuel
+  // the one we most care about first. So put the node we _don't care about at the end of the election queue_
 
   void makeReplicaFirstWatcher(Slice slice, Replica replica)
       throws KeeperException, InterruptedException {
@@ -405,7 +409,8 @@ class RebalanceLeaders {
     propMap.put(QUEUE_OPERATION, REBALANCELEADERS.toLower());
     propMap.put(CORE_NAME_PROP, core);
     propMap.put(CORE_NODE_NAME_PROP, replica.getName());
-    propMap.put(ZkStateReader.BASE_URL_PROP, replica.getProperties().get(ZkStateReader.BASE_URL_PROP));
+    propMap.put(ZkStateReader.NODE_NAME_PROP, replica.getNodeName());
+    propMap.put(ZkStateReader.BASE_URL_PROP, coreContainer.getZkController().getZkStateReader().getBaseUrlForNodeName(replica.getNodeName()));
     propMap.put(REJOIN_AT_HEAD_PROP, Boolean.toString(rejoinAtHead)); // Get ourselves to be first in line.
     propMap.put(ELECTION_NODE_PROP, electionNode);
     String asyncId = REBALANCELEADERS.toLower() + "_" + core + "_" + Math.abs(System.nanoTime());
@@ -458,13 +463,16 @@ class RebalanceLeaders {
   }
 
   // If we actually changed the leader, we should send that fact back in the response.
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private void addToSuccesses(Slice slice, Replica replica) {
     SimpleOrderedMap successes = (SimpleOrderedMap) results.get("successes");
     if (successes == null) {
       successes = new SimpleOrderedMap();
       results.add("successes", successes);
     }
-    log.info("Successfully changed leader of shard {} to replica {}", slice.getName(), replica.getName());
+    if (log.isInfoEnabled()) {
+      log.info("Successfully changed leader of shard {} to replica {}", slice.getName(), replica.getName());
+    }
     SimpleOrderedMap res = new SimpleOrderedMap();
     res.add("status", "success");
     res.add("msg", "Successfully changed leader of slice " + slice.getName() + " to " + replica.getName());
@@ -474,7 +482,8 @@ class RebalanceLeaders {
   // If for any reason we were supposed to change leadership, that should be recorded in changingLeaders. Any
   // time we verified that the change actually occurred, that entry should have been removed. So report anything
   // left over as a failure.
-    private void addAnyFailures() {
+  @SuppressWarnings({"unchecked", "rawtypes"})
+  private void addAnyFailures() {
     if (pendingOps.size() == 0) {
       return;
     }
@@ -482,7 +491,9 @@ class RebalanceLeaders {
     results.add("failures", fails);
 
     for (Map.Entry<String, String> ent : pendingOps.entrySet()) {
-      log.info("Failed to change leader of shard {} to replica {}", ent.getKey(), ent.getValue());
+      if (log.isInfoEnabled()) {
+        log.info("Failed to change leader of shard {} to replica {}", ent.getKey(), ent.getValue());
+      }
       SimpleOrderedMap res = new SimpleOrderedMap();
       res.add("status", "failed");
       res.add("msg", String.format(Locale.ROOT, "Could not change leder for slice %s to %s", ent.getKey(), ent.getValue()));
