@@ -31,6 +31,7 @@ import com.carrotsearch.hppc.cursors.IntObjectCursor;
 import com.carrotsearch.hppc.cursors.LongCursor;
 import com.carrotsearch.hppc.cursors.LongObjectCursor;
 import com.carrotsearch.hppc.cursors.ObjectCursor;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.lucene.index.DocValues;
 import org.apache.lucene.index.LeafReader;
 import org.apache.lucene.index.LeafReaderContext;
@@ -38,20 +39,24 @@ import org.apache.lucene.index.MultiDocValues;
 import org.apache.lucene.index.NumericDocValues;
 import org.apache.lucene.index.OrdinalMap;
 import org.apache.lucene.index.SortedDocValues;
-import org.apache.lucene.search.BooleanClause.Occur;
+import org.apache.lucene.search.BooleanClause;
 import org.apache.lucene.search.BooleanQuery;
 import org.apache.lucene.search.Collector;
 import org.apache.lucene.search.DocIdSetIterator;
 import org.apache.lucene.search.LeafCollector;
+import org.apache.lucene.search.MatchAllDocsQuery;
 import org.apache.lucene.search.Query;
+import org.apache.lucene.search.Scorable;
 import org.apache.lucene.search.ScoreDoc;
-import org.apache.lucene.search.Scorer;
+import org.apache.lucene.search.ScoreMode;
 import org.apache.lucene.search.Sort;
 import org.apache.lucene.search.TermInSetQuery;
 import org.apache.lucene.search.TopDocs;
 import org.apache.lucene.search.TopDocsCollector;
 import org.apache.lucene.search.TopFieldCollector;
 import org.apache.lucene.search.TopScoreDocCollector;
+import org.apache.lucene.search.TotalHitCountCollector;
+import org.apache.lucene.search.TotalHits;
 import org.apache.lucene.util.BitSetIterator;
 import org.apache.lucene.util.BytesRef;
 import org.apache.lucene.util.BytesRefBuilder;
@@ -59,7 +64,9 @@ import org.apache.lucene.util.CharsRefBuilder;
 import org.apache.lucene.util.FixedBitSet;
 import org.apache.lucene.util.LongValues;
 import org.apache.solr.common.SolrDocumentList;
+import org.apache.solr.common.SolrException;
 import org.apache.solr.common.params.ExpandParams;
+import org.apache.solr.common.params.GroupParams;
 import org.apache.solr.common.params.SolrParams;
 import org.apache.solr.common.util.NamedList;
 import org.apache.solr.common.util.SimpleOrderedMap;
@@ -67,7 +74,6 @@ import org.apache.solr.core.PluginInfo;
 import org.apache.solr.core.SolrCore;
 import org.apache.solr.request.SolrQueryRequest;
 import org.apache.solr.schema.FieldType;
-import org.apache.solr.schema.NumberType;
 import org.apache.solr.schema.SchemaField;
 import org.apache.solr.schema.StrField;
 import org.apache.solr.search.CollapsingQParserPlugin;
@@ -75,8 +81,11 @@ import org.apache.solr.search.DocIterator;
 import org.apache.solr.search.DocList;
 import org.apache.solr.search.DocSlice;
 import org.apache.solr.search.QParser;
+import org.apache.solr.search.QueryUtils;
+import org.apache.solr.search.ReturnFields;
 import org.apache.solr.search.SolrIndexSearcher;
 import org.apache.solr.search.SortSpecParsing;
+import org.apache.solr.search.SyntaxError;
 import org.apache.solr.util.plugin.PluginInfoInitialized;
 import org.apache.solr.util.plugin.SolrCoreAware;
 
@@ -85,6 +94,8 @@ import org.apache.solr.util.plugin.SolrCoreAware;
  * The CollapsingPostFilter collapses a result set on a field.
  * <p>
  * The ExpandComponent expands the collapsed groups for a single page.
+ * When multiple collapse groups are specified then, the field is chosen from collapse group with min cost.
+ * If the cost are equal then, the field is chosen from first collapse group.
  * <p>
  * http parameters:
  * <p>
@@ -93,7 +104,7 @@ import org.apache.solr.util.plugin.SolrCoreAware;
  * expand.sort=field asc|desc<br>
  * expand.q=*:* (optional, overrides the main query)<br>
  * expand.fq=type:child (optional, overrides the main filter queries)<br>
- * expand.field=field (mandatory if the not used with the CollapsingQParserPlugin)<br>
+ * expand.field=field (mandatory, if the not used with the CollapsingQParserPlugin. This is given higher priority when both are present)<br>
  */
 public class ExpandComponent extends SearchComponent implements PluginInfoInitialized, SolrCoreAware {
   public static final String COMPONENT_NAME = "expand";
@@ -108,6 +119,9 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
   @Override
   public void prepare(ResponseBuilder rb) throws IOException {
     if (rb.req.getParams().getBool(ExpandParams.EXPAND, false)) {
+      if (rb.req.getParams().getBool(GroupParams.GROUP, false)) {
+        throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, "Can not use expand with Grouping enabled");
+      }
       rb.doExpand = true;
     }
   }
@@ -133,18 +147,24 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
     if (field == null) {
       List<Query> filters = rb.getFilters();
       if (filters != null) {
+        int cost = Integer.MAX_VALUE;
         for (Query q : filters) {
           if (q instanceof CollapsingQParserPlugin.CollapsingPostFilter) {
             CollapsingQParserPlugin.CollapsingPostFilter cp = (CollapsingQParserPlugin.CollapsingPostFilter) q;
-            field = cp.getField();
-            hint = cp.hint;
+            // if there are multiple collapse pick the low cost one
+            // if cost are equal then first one is picked
+            if (cp.getCost() < cost) {
+              cost = cp.getCost();
+              field = cp.getField();
+              hint = cp.hint;
+            }
           }
         }
       }
     }
 
     if (field == null) {
-      throw new IOException("Expand field is null.");
+      throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, "missing expand field");
     }
 
     String sortParam = params.get(ExpandParams.EXPAND_SORT);
@@ -158,40 +178,35 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       sort = SortSpecParsing.parseSortSpec(sortParam, rb.req).getSort();
     }
 
-    Query query;
-    if (qs == null) {
-      query = rb.getQuery();
-    } else {
-      try {
+    final Query query;
+    List<Query> newFilters = new ArrayList<>();
+    try {
+      if (qs == null) {
+        query = rb.getQuery();
+      } else {
         QParser parser = QParser.getParser(qs, req);
         query = parser.getQuery();
-      } catch (Exception e) {
-        throw new IOException(e);
       }
-    }
 
-    List<Query> newFilters = new ArrayList<>();
-
-    if (fqs == null) {
-      List<Query> filters = rb.getFilters();
-      if (filters != null) {
-        for (Query q : filters) {
-          if (!(q instanceof CollapsingQParserPlugin.CollapsingPostFilter)) {
-            newFilters.add(q);
+      if (fqs == null) {
+        List<Query> filters = rb.getFilters();
+        if (filters != null) {
+          for (Query q : filters) {
+            if (!(q instanceof CollapsingQParserPlugin.CollapsingPostFilter)) {
+              newFilters.add(q);
+            }
           }
         }
-      }
-    } else {
-      try {
+      } else {
         for (String fq : fqs) {
-          if (fq != null && fq.trim().length() != 0 && !fq.equals("*:*")) {
+          if (StringUtils.isNotBlank(fq) && !fq.equals("*:*")) {
             QParser fqp = QParser.getParser(fq, req);
             newFilters.add(fqp.getQuery());
           }
         }
-      } catch (Exception e) {
-        throw new IOException(e);
       }
+    } catch (SyntaxError e) {
+      throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, e);
     }
 
     SolrIndexSearcher searcher = req.getSearcher();
@@ -201,7 +216,6 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
     FieldType fieldType = schemaField.getType();
 
     SortedDocValues values = null;
-    long nullValue = 0L;
 
     if(fieldType instanceof StrField) {
       //Get The Top Level SortedDocValues
@@ -212,24 +226,10 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       } else {
         values = DocValues.getSorted(reader, field);
       }
-    } else {
-      //Get the nullValue for the numeric collapse field
-      String defaultValue = searcher.getSchema().getField(field).getDefaultValue();
-      
-      final NumberType numType = fieldType.getNumberType();
-
-      // Since the expand component depends on the operation of the collapse component, 
-      // which validates that numeric field types are 32-bit,
-      // we don't need to handle invalid 64-bit field types here.
-      if (defaultValue != null) {
-        if (numType == NumberType.INTEGER) {
-          nullValue = Long.parseLong(defaultValue);
-        } else if (numType == NumberType.FLOAT) {
-          nullValue = Float.floatToIntBits(Float.parseFloat(defaultValue));
-        }
-      } else if (NumberType.FLOAT.equals(numType)) { // Integer case already handled by nullValue defaulting to 0
-        nullValue = Float.floatToIntBits(0.0f);
-      }
+    } else if (fieldType.getNumberType() == null) {
+      // possible if directly expand.field is specified
+      throw new SolrException(SolrException.ErrorCode.BAD_REQUEST,
+          "Expand not supported for fieldType:'" + fieldType.getTypeName() +"'");
     }
 
     FixedBitSet groupBits = null;
@@ -258,6 +258,7 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       return;
     }
 
+    boolean nullGroupOnCurrentPage = false;
     int currentContext = 0;
     int currentDocBase = contexts.get(currentContext).docBase;
     int nextDocBase = (currentContext+1)<contexts.size() ? contexts.get(currentContext+1).docBase : Integer.MAX_VALUE;
@@ -274,7 +275,6 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
         currentValues = sortedDocValues[currentContext];
         segmentOrdinalMap = ordinalMap.getGlobalOrds(currentContext);
       }
-      int count = 0;
 
       ordBytes = new IntObjectHashMap<>();
 
@@ -289,47 +289,46 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
             segmentOrdinalMap = ordinalMap.getGlobalOrds(currentContext);
           }
         }
-
+        collapsedSet.add(globalDoc);
         int contextDoc = globalDoc - currentDocBase;
         if(ordinalMap != null) {
           if (contextDoc > currentValues.docID()) {
             currentValues.advance(contextDoc);
           }
           if (contextDoc == currentValues.docID()) {
-            int ord = currentValues.ordValue();
-            ++count;
-            BytesRef ref = currentValues.lookupOrd(ord);
-            ord = (int)segmentOrdinalMap.get(ord);
-            ordBytes.put(ord, BytesRef.deepCopyOf(ref));
-            groupBits.set(ord);
-            collapsedSet.add(globalDoc);
+            int contextOrd = currentValues.ordValue();
+            int ord = (int)segmentOrdinalMap.get(contextOrd);
+            if (!groupBits.getAndSet(ord)) {
+              BytesRef ref = currentValues.lookupOrd(contextOrd);
+              ordBytes.put(ord, BytesRef.deepCopyOf(ref));
+            }
+          } else {
+            nullGroupOnCurrentPage = true;
           }
+          
         } else {
           if (globalDoc > values.docID()) {
             values.advance(globalDoc);
           }
           if (globalDoc == values.docID()) {
             int ord = values.ordValue();
-            ++count;
-            BytesRef ref = values.lookupOrd(ord);
-            ordBytes.put(ord, BytesRef.deepCopyOf(ref));
-            groupBits.set(ord);
-            collapsedSet.add(globalDoc);
+            if (!groupBits.getAndSet(ord)) {
+              BytesRef ref = values.lookupOrd(ord);
+              ordBytes.put(ord, BytesRef.deepCopyOf(ref));
+            }
+          } else {
+            nullGroupOnCurrentPage = true;
           }
         }
       }
 
+      int count = ordBytes.size();
       if(count > 0 && count < 200) {
-        try {
-          groupQuery = getGroupQuery(field, count, ordBytes);
-        } catch(Exception e) {
-          throw new IOException(e);
-        }
+        groupQuery = getGroupQuery(field, count, ordBytes);
       }
     } else {
       groupSet = new LongHashSet(docList.size());
       NumericDocValues collapseValues = contexts.get(currentContext).reader().getNumericDocValues(field);
-      int count = 0;
       for(int i=0; i<globalDocs.length; i++) {
         int globalDoc = globalDocs[i];
         while(globalDoc >= nextDocBase) {
@@ -338,24 +337,21 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
           nextDocBase = currentContext+1 < contexts.size() ? contexts.get(currentContext+1).docBase : Integer.MAX_VALUE;
           collapseValues = contexts.get(currentContext).reader().getNumericDocValues(field);
         }
+        collapsedSet.add(globalDoc);
         int contextDoc = globalDoc - currentDocBase;
         int valueDocID = collapseValues.docID();
         if (valueDocID < contextDoc) {
           valueDocID = collapseValues.advance(contextDoc);
         }
-        long value;
         if (valueDocID == contextDoc) {
-          value = collapseValues.longValue();
-        } else {
-          value = 0;
-        }
-        if(value != nullValue) {
-          ++count;
+          final long value = collapseValues.longValue();
           groupSet.add(value);
-          collapsedSet.add(globalDoc);
+        } else {
+          nullGroupOnCurrentPage = true;
         }
       }
 
+      int count = groupSet.size();
       if(count > 0 && count < 200) {
         if (fieldType.isPointField()) {
           groupQuery = getPointGroupQuery(schemaField, count, groupSet);
@@ -365,12 +361,31 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       }
     }
 
+    final boolean expandNullGroup =
+      params.getBool(ExpandParams.EXPAND_NULL, false) &&
+      // Our GroupCollector can typically ignore nulls (and the user's nullGroup param) unless the
+      // current page had any - but if expand.q was specified, current page doesn't mater: We
+      // need look for nulls if the user asked us to because we don't know what the expand.q will match
+      (nullGroupOnCurrentPage || (null != query));
+
+    
+    if (expandNullGroup && null != groupQuery) {
+      // we need to also consider docs w/o a field value 
+      final BooleanQuery.Builder inner = new BooleanQuery.Builder();
+      inner.add(fieldType.getExistenceQuery(null, schemaField), BooleanClause.Occur.MUST_NOT);
+      inner.add(new MatchAllDocsQuery(), BooleanClause.Occur.MUST);
+      final BooleanQuery.Builder outer = new BooleanQuery.Builder();
+      outer.add(inner.build(), BooleanClause.Occur.SHOULD);
+      outer.add(groupQuery, BooleanClause.Occur.SHOULD);
+      groupQuery = outer.build();
+    }
+    
     Collector collector;
     if (sort != null)
       sort = sort.rewrite(searcher);
 
 
-    Collector groupExpandCollector = null;
+    GroupCollector groupExpandCollector = null;
 
     if(values != null) {
       //Get The Top Level SortedDocValues again so we can re-iterate:
@@ -381,10 +396,14 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       } else {
         values = DocValues.getSorted(reader, field);
       }
-      
-      groupExpandCollector = new GroupExpandCollector(values, groupBits, collapsedSet, limit, sort);
+
+      groupExpandCollector = new GroupExpandCollector(limit, sort, query, expandNullGroup,
+                                                      fieldType, ordBytes,
+                                                      values, groupBits, collapsedSet);
     } else {
-      groupExpandCollector = new NumericGroupExpandCollector(field, nullValue, groupSet, collapsedSet, limit, sort);
+      groupExpandCollector = new NumericGroupExpandCollector(limit, sort, query, expandNullGroup,
+                                                             fieldType, ordBytes,
+                                                             field, groupSet, collapsedSet);
     }
 
     if(groupQuery !=  null) {
@@ -400,45 +419,9 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       collector = groupExpandCollector;
     }
 
-    if (pfilter.filter == null) {
-      searcher.search(query, collector);
-    } else {
-      Query q = new BooleanQuery.Builder()
-          .add(query, Occur.MUST)
-          .add(pfilter.filter, Occur.FILTER)
-          .build();
-      searcher.search(q, collector);
-    }
-    LongObjectMap<Collector> groups = ((GroupCollector) groupExpandCollector).getGroups();
-    NamedList outMap = new SimpleOrderedMap();
-    CharsRefBuilder charsRef = new CharsRefBuilder();
-    for (LongObjectCursor<Collector> cursor : groups) {
-      long groupValue = cursor.key;
-      TopDocsCollector<?> topDocsCollector = TopDocsCollector.class.cast(cursor.value);
-      TopDocs topDocs = topDocsCollector.topDocs();
-      ScoreDoc[] scoreDocs = topDocs.scoreDocs;
-      if (scoreDocs.length > 0) {
-        int[] docs = new int[scoreDocs.length];
-        float[] scores = new float[scoreDocs.length];
-        for (int i = 0; i < docs.length; i++) {
-          ScoreDoc scoreDoc = scoreDocs[i];
-          docs[i] = scoreDoc.doc;
-          scores[i] = scoreDoc.score;
-        }
-        DocSlice slice = new DocSlice(0, docs.length, docs, scores, topDocs.totalHits, topDocs.getMaxScore());
+    searcher.search(QueryUtils.combineQueryAndFilter(query, pfilter.filter), collector);
 
-        if(fieldType instanceof StrField) {
-          final BytesRef bytesRef = ordBytes.get((int)groupValue);
-          fieldType.indexedToReadable(bytesRef, charsRef);
-          String group = charsRef.toString();
-          outMap.add(group, slice);
-        } else {
-          outMap.add(numericToString(fieldType, groupValue), slice);
-        }
-      }
-    }
-
-    rb.rsp.add("expanded", outMap);
+    rb.rsp.add("expanded", groupExpandCollector.getGroups(searcher, rb.rsp.getReturnFields()));
   }
 
   @Override
@@ -460,8 +443,9 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
     }
   }
 
-  @SuppressWarnings("unchecked")
+
   @Override
+  @SuppressWarnings({"unchecked", "rawtypes"})
   public void handleResponses(ResponseBuilder rb, ShardRequest sreq) {
 
     if (!rb.doExpand) {
@@ -487,6 +471,7 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
     }
   }
 
+  @SuppressWarnings("rawtypes")
   @Override
   public void finishStage(ResponseBuilder rb) {
 
@@ -506,25 +491,26 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
     rb.rsp.add("expanded", expanded);
   }
 
-  private static class GroupExpandCollector implements Collector, GroupCollector {
-    private SortedDocValues docValues;
-    private OrdinalMap ordinalMap;
-    private SortedDocValues segmentValues;
-    private LongValues segmentOrdinalMap;
-    private MultiDocValues.MultiSortedDocValues multiSortedDocValues;
+  private static class GroupExpandCollector extends GroupCollector {
+    private final SortedDocValues docValues;
+    private final OrdinalMap ordinalMap;
+    private final MultiDocValues.MultiSortedDocValues multiSortedDocValues;
 
-    private LongObjectMap<Collector> groups;
-    private FixedBitSet groupBits;
-    private IntHashSet collapsedSet;
+    private final LongObjectMap<Collector> groups;
+    private final FixedBitSet groupBits;
+    private final IntHashSet collapsedSet;
 
-    public GroupExpandCollector(SortedDocValues docValues, FixedBitSet groupBits, IntHashSet collapsedSet, int limit, Sort sort) throws IOException {
-      int numGroups = collapsedSet.size();
-      groups = new LongObjectHashMap<>(numGroups);
+    public GroupExpandCollector(int limit, Sort sort, Query query, boolean expandNulls,
+                                FieldType fieldType, IntObjectHashMap<BytesRef> ordBytes,
+                                SortedDocValues docValues, FixedBitSet groupBits, IntHashSet collapsedSet) throws IOException {
+      super(limit, sort, query, expandNulls, fieldType, ordBytes);
+
+      // groupBits.cardinality() is more expensive then collapsedSet.size() which is adequate for an upper bound
+      this.groups = new LongObjectHashMap<>(collapsedSet.size());
       DocIdSetIterator iterator = new BitSetIterator(groupBits, 0); // cost is not useful here
       int group;
       while ((group = iterator.nextDoc()) != DocIdSetIterator.NO_MORE_DOCS) {
-        Collector collector = (sort == null) ? TopScoreDocCollector.create(limit) : TopFieldCollector.create(sort, limit, false, false, false, true);
-        groups.put(group, collector);
+        groups.put(group, getCollector());
       }
 
       this.collapsedSet = collapsedSet;
@@ -533,17 +519,20 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       if(docValues instanceof MultiDocValues.MultiSortedDocValues) {
         this.multiSortedDocValues = (MultiDocValues.MultiSortedDocValues)docValues;
         this.ordinalMap = multiSortedDocValues.mapping;
+      } else {
+        this.multiSortedDocValues = null;
+        this.ordinalMap = null;
       }
     }
 
     public LeafCollector getLeafCollector(LeafReaderContext context) throws IOException {
       final int docBase = context.docBase;
 
-      if(ordinalMap != null) {
-        this.segmentValues = this.multiSortedDocValues.values[context.ord];
-        this.segmentOrdinalMap = ordinalMap.getGlobalOrds(context.ord);
-      }
+      final boolean useOrdinalMapping = (null != ordinalMap);
+      final SortedDocValues segmentValues = useOrdinalMapping ? this.multiSortedDocValues.values[context.ord] : null;
+      final LongValues segmentOrdinalMap = useOrdinalMapping ? this.ordinalMap.getGlobalOrds(context.ord) : null;
 
+      final LeafCollector leafNullGroupCollector = expandNullGroup ? nullGroupCollector.getLeafCollector(context) : null;
       final LongObjectMap<LeafCollector> leafCollectors = new LongObjectHashMap<>();
       for (LongObjectCursor<Collector> entry : groups) {
         leafCollectors.put(entry.key, entry.value.getLeafCollector(context));
@@ -551,17 +540,25 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       return new LeafCollector() {
 
         @Override
-        public void setScorer(Scorer scorer) throws IOException {
+        public void setScorer(Scorable scorer) throws IOException {
           for (ObjectCursor<LeafCollector> c : leafCollectors.values()) {
             c.value.setScorer(scorer);
           }
+          if (expandNullGroup) {
+            leafNullGroupCollector.setScorer(scorer);
+          }
         }
 
         @Override
         public void collect(int docId) throws IOException {
           int globalDoc = docId + docBase;
+
+          if (collapsedSet.contains(globalDoc)) {
+            return; // this doc is already a group head
+          }
+          
           int ord = -1;
-          if(ordinalMap != null) {
+          if (useOrdinalMapping) {
             if (docId > segmentValues.docID()) {
               segmentValues.advance(docId);
             }
@@ -577,38 +574,39 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
               ord = -1;
             }
           }
-
-          if (ord > -1 && groupBits.get(ord) && !collapsedSet.contains(globalDoc)) {
-            LeafCollector c = leafCollectors.get(ord);
-            c.collect(docId);
+          
+          if (ord > -1) {
+            if (groupBits.get(ord)) {
+              LeafCollector c = leafCollectors.get(ord);
+              c.collect(docId);
+            }
+          } else if (expandNullGroup) {
+            leafNullGroupCollector.collect(docId);
           }
         }
       };
     }
 
-    public LongObjectMap<Collector> getGroups() {
+    @Override
+    protected LongObjectMap<Collector> getGroups() {
       return groups;
     }
   }
 
-  private static class NumericGroupExpandCollector implements Collector, GroupCollector {
-    private NumericDocValues docValues;
+  private static class NumericGroupExpandCollector extends GroupCollector {
 
-    private String field;
-    private LongObjectHashMap<Collector> groups;
+    private final String field;
+    private final LongObjectHashMap<Collector> groups;
+    private final IntHashSet collapsedSet;
 
-    private IntHashSet collapsedSet;
-    private long nullValue;
-
-    public NumericGroupExpandCollector(String field, long nullValue, LongHashSet groupSet, IntHashSet collapsedSet, int limit, Sort sort) throws IOException {
-      int numGroups = collapsedSet.size();
-      this.nullValue = nullValue;
-      groups = new LongObjectHashMap<>(numGroups);
-      Iterator<LongCursor> iterator = groupSet.iterator();
-      while (iterator.hasNext()) {
-        LongCursor cursor = iterator.next();
-        Collector collector = (sort == null) ? TopScoreDocCollector.create(limit) : TopFieldCollector.create(sort, limit, false, false, false, true);
-        groups.put(cursor.value, collector);
+    public NumericGroupExpandCollector(int limit, Sort sort, Query query, boolean expandNulls,
+                                       FieldType fieldType, IntObjectHashMap<BytesRef> ordBytes,
+                                       String field, LongHashSet groupSet, IntHashSet collapsedSet) throws IOException {
+      super(limit, sort, query, expandNulls, fieldType, ordBytes);
+      
+      this.groups = new LongObjectHashMap<>(groupSet.size());
+      for (LongCursor cursor : groupSet) {
+        groups.put(cursor.value, getCollector());
       }
 
       this.field = field;
@@ -617,10 +615,10 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
 
     public LeafCollector getLeafCollector(LeafReaderContext context) throws IOException {
       final int docBase = context.docBase;
-      this.docValues = context.reader().getNumericDocValues(this.field);
-
+      
+      final NumericDocValues docValues = context.reader().getNumericDocValues(this.field);
+      final LeafCollector leafNullGroupCollector = expandNullGroup ? nullGroupCollector.getLeafCollector(context) : null;
       final LongObjectHashMap<LeafCollector> leafCollectors = new LongObjectHashMap<>();
-
       for (LongObjectCursor<Collector> entry : groups) {
         leafCollectors.put(entry.key, entry.value.getLeafCollector(context));
       }
@@ -628,49 +626,143 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
       return new LeafCollector() {
 
         @Override
-        public void setScorer(Scorer scorer) throws IOException {
+        public void setScorer(Scorable scorer) throws IOException {
           for (ObjectCursor<LeafCollector> c : leafCollectors.values()) {
             c.value.setScorer(scorer);
           }
+          if (expandNullGroup) {
+            leafNullGroupCollector.setScorer(scorer);
+          }
         }
 
         @Override
         public void collect(int docId) throws IOException {
-          long value;
           if (docValues.advanceExact(docId)) {
-            value = docValues.longValue();
-          } else {
-            value = 0;
-          }
-          final int index;
-          if (value != nullValue && 
-              (index = leafCollectors.indexOf(value)) >= 0 && 
-              !collapsedSet.contains(docId + docBase)) {
-            leafCollectors.indexGet(index).collect(docId);
+            final long value = docValues.longValue();
+            final int index = leafCollectors.indexOf(value);
+            if (index >= 0 && !collapsedSet.contains(docId + docBase)) {
+              leafCollectors.indexGet(index).collect(docId);
+            }
+          } else if (expandNullGroup && !collapsedSet.contains(docId + docBase)) {
+            leafNullGroupCollector.collect(docId);
           }
         }
       };
     }
 
-    public LongObjectHashMap<Collector> getGroups() {
+    @Override
+    protected LongObjectHashMap<Collector> getGroups() {
       return groups;
     }
 
   }
 
-  //TODO lets just do simple abstract base class -- a fine use of inheritance
-  private interface GroupCollector extends Collector {
-    public LongObjectMap<Collector> getGroups();
+  private static abstract class GroupCollector implements Collector {
+    
+    protected final int limit;
+    protected final Sort sort;
+    protected final Query query;
+    protected final boolean expandNullGroup;
+    protected final FieldType fieldType;
+    protected final IntObjectHashMap<BytesRef> ordBytes;
+    
+    protected final Collector nullGroupCollector;
+    
+    public GroupCollector(int limit, Sort sort, Query query, boolean expandNullGroup,
+                          FieldType fieldType, IntObjectHashMap<BytesRef> ordBytes) throws IOException {
+      this.limit = limit;
+      this.sort = sort;
+      this.query = query;
+      this.expandNullGroup = expandNullGroup;
+      this.fieldType = fieldType;
+      this.ordBytes = ordBytes;
+      this.nullGroupCollector = expandNullGroup ? getCollector() : null;
+    }
+    
+    protected abstract LongObjectMap<Collector> getGroups();
 
+    public final SimpleOrderedMap<DocSlice> getGroups(SolrIndexSearcher searcher, ReturnFields returnFields) throws IOException {
+      
+      final SimpleOrderedMap<DocSlice> outMap = new SimpleOrderedMap<>();
+      final CharsRefBuilder charsRef = new CharsRefBuilder();
+      for (LongObjectCursor<Collector> cursor : getGroups()) {
+        final long groupValue = cursor.key;
+        final DocSlice slice = collectorToDocSlice(cursor.value, searcher, returnFields);
+        if (null != slice) {
+          addGroupSliceToOutputMap(outMap, charsRef, groupValue, slice);
+        }
+      }
+      if (expandNullGroup) {
+        assert null != nullGroupCollector;
+        final DocSlice nullGroup = collectorToDocSlice(nullGroupCollector, searcher, returnFields);
+        if (null != nullGroup) {
+          outMap.add(null, nullGroup);
+        }
+      }
+      return outMap;
+    }
+    
+    private DocSlice collectorToDocSlice(Collector groupCollector, SolrIndexSearcher searcher, ReturnFields returnFields) throws IOException {
+      if (groupCollector instanceof TopDocsCollector) {
+        TopDocsCollector<?> topDocsCollector = TopDocsCollector.class.cast(groupCollector);
+        TopDocs topDocs = topDocsCollector.topDocs();
+        ScoreDoc[] scoreDocs = topDocs.scoreDocs;
+        if (scoreDocs.length > 0) {
+          if (returnFields.wantsScore() && sort != null) {
+            TopFieldCollector.populateScores(scoreDocs, searcher, query);
+          }
+          int[] docs = new int[scoreDocs.length];
+          float[] scores = new float[scoreDocs.length];
+          for (int i = 0; i < docs.length; i++) {
+            ScoreDoc scoreDoc = scoreDocs[i];
+            docs[i] = scoreDoc.doc;
+            scores[i] = scoreDoc.score;
+          }
+          assert topDocs.totalHits.relation == TotalHits.Relation.EQUAL_TO;
+          return new DocSlice(0, docs.length, docs, scores, topDocs.totalHits.value, Float.NaN, TotalHits.Relation.EQUAL_TO);
+        }
+      } else {
+        int totalHits = ((TotalHitCountCollector) groupCollector).getTotalHits();
+        if (totalHits > 0) {
+          return new DocSlice(0, 0, null, null, totalHits, 0, TotalHits.Relation.EQUAL_TO);
+        }
+      }
+      return null;
+    }
+    
+    private void addGroupSliceToOutputMap(NamedList<DocSlice> outMap, CharsRefBuilder charsRef,
+                                          long groupValue, DocSlice slice) {
+      if(fieldType instanceof StrField) {
+        final BytesRef bytesRef = ordBytes.get((int)groupValue);
+        fieldType.indexedToReadable(bytesRef, charsRef);
+        String group = charsRef.toString();
+        outMap.add(group, slice);
+      } else {
+        outMap.add(numericToString(fieldType, groupValue), slice);
+      }
+    }
+    
     @Override
-    default boolean needsScores() {
+    public ScoreMode scoreMode() {
       final LongObjectMap<Collector> groups = getGroups();
       if (groups.isEmpty()) {
-        return true; // doesn't matter?
+        return ScoreMode.COMPLETE; // doesn't matter?
       } else {
-        return groups.iterator().next().value.needsScores(); // we assume all the collectors should have the same nature
+        return groups.iterator().next().value.scoreMode(); // we assume all the collectors should have the same nature
       }
     }
+
+    protected final Collector getCollector()  throws IOException {
+      Collector collector;
+      if (limit == 0) {
+        collector = new TotalHitCountCollector();
+      } else if (sort == null) {
+        collector = TopScoreDocCollector.create(limit, Integer.MAX_VALUE);
+      } else {
+        collector = TopFieldCollector.create(sort, limit, Integer.MAX_VALUE);
+      }
+      return collector;
+    }
   }
 
   private Query getGroupQuery(String fname,
@@ -679,9 +771,9 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
                            LongHashSet groupSet) {
 
     BytesRef[] bytesRefs = new BytesRef[size];
+    int index = -1;
     BytesRefBuilder term = new BytesRefBuilder();
     Iterator<LongCursor> it = groupSet.iterator();
-    int index = -1;
 
     while (it.hasNext()) {
       LongCursor cursor = it.next();
@@ -708,7 +800,7 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
     return sf.getType().getSetQuery(null, sf, values);
   }
 
-  private String numericToString(FieldType fieldType, long val) {
+  private static String numericToString(FieldType fieldType, long val) {
     if (fieldType.getNumberType() != null) {
       switch (fieldType.getNumberType()) {
         case INTEGER:
@@ -727,7 +819,7 @@ public class ExpandComponent extends SearchComponent implements PluginInfoInitia
 
   private Query getGroupQuery(String fname,
                               int size,
-                              IntObjectHashMap<BytesRef> ordBytes) throws Exception {
+                              IntObjectHashMap<BytesRef> ordBytes) {
     BytesRef[] bytesRefs = new BytesRef[size];
     int index = -1;
     Iterator<IntObjectCursor<BytesRef>>it = ordBytes.iterator();
