@@ -18,16 +18,19 @@ package org.apache.solr.handler.component;
 
 import java.io.IOException;
 import java.util.Arrays;
+import java.util.Comparator;
 import java.util.HashMap;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
+import java.util.Map;
 import java.util.regex.Pattern;
+
 import org.apache.lucene.index.IndexReaderContext;
 import org.apache.lucene.index.LeafReader;
 import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.index.Term;
-import org.apache.lucene.index.TermContext;
+import org.apache.lucene.index.TermStates;
 import org.apache.lucene.index.Terms;
 import org.apache.lucene.index.TermsEnum;
 import org.apache.lucene.search.Query;
@@ -47,6 +50,7 @@ import org.apache.solr.common.params.TermsParams;
 import org.apache.solr.common.util.NamedList;
 import org.apache.solr.common.util.SimpleOrderedMap;
 import org.apache.solr.common.util.StrUtils;
+import org.apache.solr.common.util.Utils;
 import org.apache.solr.handler.component.HttpShardHandlerFactory.WhitelistHostChecker;
 import org.apache.solr.request.SimpleFacets.CountPair;
 import org.apache.solr.schema.FieldType;
@@ -85,7 +89,7 @@ public class TermsComponent extends SearchComponent {
       !HttpShardHandlerFactory.doGetDisableShardsWhitelist());
 
   @Override
-  public void init( NamedList args )
+  public void init( @SuppressWarnings({"rawtypes"})NamedList args )
   {
     super.init(args);
     whitelistHostChecker = new WhitelistHostChecker(
@@ -146,9 +150,11 @@ public class TermsComponent extends SearchComponent {
     String[] fields = params.getParams(TermsParams.TERMS_FIELD);
 
     NamedList<Object> termsResult = new SimpleOrderedMap<>();
-    rb.rsp.add("terms", termsResult);
+    rb.rsp.add(TermsParams.TERMS, termsResult);
 
-    if (fields == null || fields.length==0) return;
+    if (fields == null || fields.length == 0) {
+      return;
+    }
 
     boolean termStats = params.getBool(TermsParams.TERMS_STATS, false);
 
@@ -159,8 +165,8 @@ public class TermsComponent extends SearchComponent {
     }
 
     String termList = params.get(TermsParams.TERMS_LIST);
+    boolean includeTotalTermFreq = params.getBool(TermsParams.TERMS_TTF, false);
     if (termList != null) {
-      boolean includeTotalTermFreq = params.getBool(TermsParams.TERMS_TTF, false);
       fetchTerms(rb.req.getSearcher(), fields, termList, includeTotalTermFreq, termsResult);
       return;
     }
@@ -184,26 +190,27 @@ public class TermsComponent extends SearchComponent {
 
     boolean raw = params.getBool(TermsParams.TERMS_RAW, false);
 
-
     final LeafReader indexReader = rb.req.getSearcher().getSlowAtomicReader();
 
     for (String field : fields) {
-      NamedList<Integer> fieldTerms = new NamedList<>();
+      NamedList<Object> fieldTerms = new NamedList<>();
+      termsResult.add(field, fieldTerms);
 
       Terms terms = indexReader.terms(field);
       if (terms == null) {
         // field does not exist in terms index.  Check points.
         SchemaField sf = rb.req.getSchema().getFieldOrNull(field);
         if (sf != null && sf.getType().isPointField()) {
-          if (lowerStr!=null || upperStr!=null || prefix!=null || regexp!=null) {
+          // FIXME: terms.ttf=true is not supported for pointFields
+          if (lowerStr != null || upperStr != null || prefix != null || regexp != null) {
             throw new SolrException(SolrException.ErrorCode.BAD_REQUEST,
                 String.format(Locale.ROOT, "The terms component does not support Points-based fields with sorting or with parameters %s,%s,%s,%s ", TermsParams.TERMS_LOWER, TermsParams.TERMS_UPPER, TermsParams.TERMS_PREFIX_STR, TermsParams.TERMS_REGEXP_STR));
           }
 
+          PointMerger.ValueIterator valueIterator = new PointMerger.ValueIterator(sf, rb.req.getSearcher().getRawReader().leaves());
+          MutableValue mv = valueIterator.getMutableValue();
           if (sort) {
-            PointMerger.ValueIterator valueIterator = new PointMerger.ValueIterator(sf, rb.req.getSearcher().getRawReader().leaves());
-            MutableValue mv = valueIterator.getMutableValue();
-            BoundedTreeSet<CountPair<MutableValue, Integer>> queue = (sort ? new BoundedTreeSet<>(limit) : null);
+            BoundedTreeSet<CountPair<MutableValue, Integer>> queue = new BoundedTreeSet<>(limit);
 
             for (; ; ) {
               long count = valueIterator.getNextCount();
@@ -215,56 +222,46 @@ public class TermsComponent extends SearchComponent {
             }
 
             for (CountPair<MutableValue, Integer> item : queue) {
-              fieldTerms.add(item.key.toString(), item.val);
+              fieldTerms.add(Utils.OBJECT_TO_STRING.apply(item.key.toObject()), item.val);
             }
-            termsResult.add(field, fieldTerms);
             continue;
-          }
-
-          if (!sort) {
+          } else {
             /***
             // streaming solution that is deferred until writing the response
             // TODO: we can't use the streaming solution until XML writer supports PushWriter!
             termsResult.add(field, (MapWriter) ew -> {
-              PointMerger.ValueIterator valueIterator = new PointMerger.ValueIterator(sf, rb.req.getSearcher().getRawReader().leaves());
-              MutableValue mv = valueIterator.getMutableValue();
               int num = 0;
               for(;;) {
                 long count = valueIterator.getNextCount();
                 if (count < 0) break;
                 if (count < freqmin || count > freqmax) continue;
                 if (++num > limit) break;
-                ew.put(mv.toString(), (int)count); // match the numeric type of terms
+                ew.put(Utils.OBJECT_TO_STRING.apply(mv.toObject()), (int)count); // match the numeric type of terms
               }
             });
              ***/
 
-            PointMerger.ValueIterator valueIterator = new PointMerger.ValueIterator(sf, rb.req.getSearcher().getRawReader().leaves());
-            MutableValue mv = valueIterator.getMutableValue();
             int num = 0;
             for(;;) {
               long count = valueIterator.getNextCount();
               if (count < 0) break;
               if (count < freqmin || count > freqmax) continue;
               if (++num > limit) break;
-              fieldTerms.add(mv.toString(), (int)count); // match the numeric type of terms
+              fieldTerms.add(Utils.OBJECT_TO_STRING.apply(mv.toObject()), (int)count); // match the numeric type of terms
             }
-
-            termsResult.add(field, fieldTerms);
             continue;
           }
         }
-
-        termsResult.add(field, fieldTerms);  // add empty
         continue;
       }
-      termsResult.add(field, fieldTerms);
 
       FieldType ft = raw ? null : rb.req.getSchema().getFieldTypeNoEx(field);
-      if (ft==null) ft = new StrField();
+      if (ft == null) {
+        ft = new StrField();
+      }
 
       // prefix must currently be text
-      BytesRef prefixBytes = prefix==null ? null : new BytesRef(prefix);
+      BytesRef prefixBytes = prefix == null ? null : new BytesRef(prefix);
 
       BytesRef upperBytes = null;
       if (upperStr != null) {
@@ -290,9 +287,8 @@ public class TermsComponent extends SearchComponent {
         }
       }
 
-
-     TermsEnum termsEnum = terms.iterator();
-     BytesRef term = null;
+      TermsEnum termsEnum = terms.iterator();
+      BytesRef term = null;
 
       if (lowerBytes != null) {
         if (termsEnum.seekCeil(lowerBytes) == TermsEnum.SeekStatus.END) {
@@ -310,9 +306,9 @@ public class TermsComponent extends SearchComponent {
       }
 
       int i = 0;
-      BoundedTreeSet<CountPair<BytesRef, Integer>> queue = (sort ? new BoundedTreeSet<CountPair<BytesRef, Integer>>(limit) : null);
+      BoundedTreeSet<TermsResponse.Term> queue = (sort? new BoundedTreeSet<>(limit, new TermCountComparator()): null);
       CharsRefBuilder external = new CharsRefBuilder();
-      while (term != null && (i<limit || sort)) {
+      while (term != null && (i < limit || sort)) {
         boolean externalized = false; // did we fill in "external" yet for this term?
 
         // stop if the prefix doesn't match
@@ -338,16 +334,16 @@ public class TermsComponent extends SearchComponent {
         // This is a good term in the range.  Check if mincount/maxcount conditions are satisfied.
         int docFreq = termsEnum.docFreq();
         if (docFreq >= freqmin && docFreq <= freqmax) {
+          // TODO: handle raw somehow
+          if (!externalized) {
+            ft.indexedToReadable(term, external);
+          }
           // add the term to the list
           if (sort) {
-            queue.add(new CountPair<>(BytesRef.deepCopyOf(term), docFreq));
+            queue.add(new TermsResponse.Term(external.toString(), docFreq, termsEnum.totalTermFreq()));
           } else {
-
-            // TODO: handle raw somehow
-            if (!externalized) {
-              ft.indexedToReadable(term, external);
-            }
-            fieldTerms.add(external.toString(), docFreq);
+            addTermToNamedList(fieldTerms, external.toString(),
+                docFreq, termsEnum.totalTermFreq(), includeTotalTermFreq);
             i++;
           }
         }
@@ -356,10 +352,12 @@ public class TermsComponent extends SearchComponent {
       }
 
       if (sort) {
-        for (CountPair<BytesRef, Integer> item : queue) {
-          if (i >= limit) break;
-          ft.indexedToReadable(item.key, external);
-          fieldTerms.add(external.toString(), item.val);
+        for (TermsResponse.Term item : queue) {
+          if (i >= limit) {
+            break;
+          }
+          addTermToNamedList(fieldTerms, item.getTerm(), item.getFrequency(),
+              item.getTotalTermFreq(), includeTotalTermFreq);
           i++;
         }
       }
@@ -418,6 +416,7 @@ public class TermsComponent extends SearchComponent {
         th.parse(terms);
 
 
+        @SuppressWarnings({"unchecked"})
         NamedList<Number> stats = (NamedList<Number>)srsp.getSolrResponse().getResponse().get("indexstats");
         if(stats != null) {
           th.numDocs += stats.get("numDocs").longValue();
@@ -434,33 +433,43 @@ public class TermsComponent extends SearchComponent {
     }
 
     TermsHelper ti = rb._termsHelper;
+    @SuppressWarnings({"rawtypes"})
     NamedList terms = ti.buildResponse();
 
     rb.rsp.add("terms", terms);
     if(ti.stats) {
-      NamedList<Number> stats = new SimpleOrderedMap();
-      stats.add("numDocs", Long.valueOf(ti.numDocs));
+      NamedList<Number> stats = new SimpleOrderedMap<>();
+      stats.add("numDocs", ti.numDocs);
       rb.rsp.add("indexstats", stats);
     }
     rb._termsHelper = null;
   }
 
-  private static ShardRequest createShardQuery(SolrParams params) {
+  static ShardRequest createShardQuery(SolrParams params) {
     ShardRequest sreq = new ShardRequest();
     sreq.purpose = ShardRequest.PURPOSE_GET_TERMS;
 
     // base shard request on original parameters
     sreq.params = new ModifiableSolrParams(params);
 
-    // remove any limits for shards, we want them to return all possible
-    // responses
-    // we want this so we can calculate the correct counts
-    // dont sort by count to avoid that unnecessary overhead on the shards
-    sreq.params.remove(TermsParams.TERMS_MAXCOUNT);
-    sreq.params.remove(TermsParams.TERMS_MINCOUNT);
-    sreq.params.set(TermsParams.TERMS_LIMIT, -1);
-    sreq.params.set(TermsParams.TERMS_SORT, TermsParams.TERMS_SORT_INDEX);
-
+    // if using index-order, we can send all parameters to all shards
+    // since all required data are returned within the first n rows
+    String actualSort = sreq.params.get(TermsParams.TERMS_SORT, TermsParams.TERMS_SORT_COUNT);
+    
+    boolean fast = actualSort.equals(TermsParams.TERMS_SORT_INDEX) &&
+        sreq.params.getLong(TermsParams.TERMS_MINCOUNT, 0) <= 1 &&
+        sreq.params.getLong(TermsParams.TERMS_MAXCOUNT, -1) <=0;
+    
+    if (!fast) {
+      // remove any limits for shards, we want them to return all possible
+      // responses
+      // we want this so we can calculate the correct counts
+      // dont sort by count to avoid that unnecessary overhead on the shards
+      sreq.params.remove(TermsParams.TERMS_MAXCOUNT);
+      sreq.params.remove(TermsParams.TERMS_MINCOUNT);
+      sreq.params.set(TermsParams.TERMS_LIMIT, -1);
+      sreq.params.set(TermsParams.TERMS_SORT, TermsParams.TERMS_SORT_INDEX);
+    }
     return sreq;
   }
 
@@ -488,7 +497,7 @@ public class TermsComponent extends SearchComponent {
     }
 
     public void parse(NamedList<NamedList<Object>> terms) {
-      // exit if there is no terms
+      // exit if there are no terms
       if (terms == null) {
         return;
       }
@@ -496,15 +505,16 @@ public class TermsComponent extends SearchComponent {
       TermsResponse termsResponse = new TermsResponse(terms);
 
       // loop though each field and add each term+freq to map
-      for (String key : fieldmap.keySet()) {
-        HashMap<String, TermsResponse.Term> termmap = fieldmap.get(key);
-        List<TermsResponse.Term> termlist = termsResponse.getTerms(key);
+      for (Map.Entry<String, HashMap<String, TermsResponse.Term>> entry : fieldmap.entrySet()) {
+        List<TermsResponse.Term> termlist = termsResponse.getTerms(entry.getKey());
 
         // skip this field if there are no terms
         if (termlist == null) {
           continue;
         }
 
+        HashMap<String, TermsResponse.Term> termmap = entry.getValue();
+
         // loop though each term
         for (TermsResponse.Term tc : termlist) {
           String term = tc.getTerm();
@@ -531,50 +541,34 @@ public class TermsComponent extends SearchComponent {
         sort = false;
       }
 
-      // init minimum frequency
-      long freqmin = 1;
-      String s = params.get(TermsParams.TERMS_MINCOUNT);
-      if (s != null)  freqmin = Long.parseLong(s);
+      long freqmin = params.getLong(TermsParams.TERMS_MINCOUNT, 1);
 
-      // init maximum frequency, default to max int
-      long freqmax = -1;
-      s = params.get(TermsParams.TERMS_MAXCOUNT);
-      if (s != null)  freqmax = Long.parseLong(s);
+      long freqmax = params.getLong(TermsParams.TERMS_MAXCOUNT, UNLIMITED_MAX_COUNT);
       if (freqmax < 0) {
         freqmax = Long.MAX_VALUE;
       }
 
-      // init limit, default to max int
-      long limit = 10;
-      s = params.get(TermsParams.TERMS_LIMIT);
-      if (s != null)  limit = Long.parseLong(s);
+      long limit = params.getLong(TermsParams.TERMS_LIMIT, 10);
       if (limit < 0) {
         limit = Long.MAX_VALUE;
       }
 
-      // loop though each field we want terms from
-      for (String key : fieldmap.keySet()) {
+      // loop through each field we want terms from
+      for (Map.Entry<String, HashMap<String, TermsResponse.Term>> entry : fieldmap.entrySet()) {
         NamedList<Object> fieldterms = new SimpleOrderedMap<>();
         TermsResponse.Term[] data = null;
         if (sort) {
-          data = getCountSorted(fieldmap.get(key));
+          data = getCountSorted(entry.getValue());
         } else {
-          data = getLexSorted(fieldmap.get(key));
+          data = getLexSorted(entry.getValue());
         }
 
         boolean includeTotalTermFreq = params.getBool(TermsParams.TERMS_TTF, false);
-        // loop though each term until we hit limit
+        // loop through each term until we hit limit
         int cnt = 0;
         for (TermsResponse.Term tc : data) {
           if (tc.getFrequency() >= freqmin && tc.getFrequency() <= freqmax) {
-            if (includeTotalTermFreq) {
-              NamedList<Number> termStats = new SimpleOrderedMap<>();
-              termStats.add("df", tc.getFrequency());
-              termStats.add("ttf", tc.getTotalTermFreq());
-              fieldterms.add(tc.getTerm(), termStats);
-            } else {
-              fieldterms.add(tc.getTerm(), num(tc.getFrequency()));
-            }
+            addTermToNamedList(fieldterms, tc.getTerm(), tc.getFrequency(), tc.getTotalTermFreq(), includeTotalTermFreq);
             cnt++;
           }
 
@@ -583,7 +577,7 @@ public class TermsComponent extends SearchComponent {
           }
         }
 
-        response.add(key, fieldterms);
+        response.add(entry.getKey(), fieldterms);
       }
 
       return response;
@@ -608,19 +602,7 @@ public class TermsComponent extends SearchComponent {
     public TermsResponse.Term[] getCountSorted(HashMap<String, TermsResponse.Term> data) {
       TermsResponse.Term[] arr = data.values().toArray(new TermsResponse.Term[data.size()]);
 
-      Arrays.sort(arr, (o1, o2) -> {
-        long freq1 = o1.getFrequency();
-        long freq2 = o2.getFrequency();
-
-        if (freq2 < freq1) {
-          return -1;
-        } else if (freq1 < freq2) {
-          return 1;
-        } else {
-          return o1.getTerm().compareTo(o2.getTerm());
-        }
-      });
-
+      Arrays.sort(arr, new TermCountComparator());
       return arr;
     }
   }
@@ -640,40 +622,30 @@ public class TermsComponent extends SearchComponent {
     for (String field : fields) {
       SchemaField sf = indexSearcher.getSchema().getField(field);
       FieldType fieldType = sf.getType();
-
+      NamedList<Object> termsMap = new SimpleOrderedMap<>();
+      
       if (fieldType.isPointField()) {
-        NamedList<Object> termsMap = new SimpleOrderedMap<>();
         for (String term : splitTerms) {
           Query q = fieldType.getFieldQuery(null, sf, term);
           int count = indexSearcher.getDocSet(q).size();
           termsMap.add(term, count);
         }
-        result.add(field, termsMap);
-        continue;
-      }
+      } else {
 
-      // Since splitTerms is already sorted, this array will also be sorted. NOTE: this may not be true, it depends on readableToIndexed.
-      Term[] terms = new Term[splitTerms.length];
-      for (int i = 0; i < splitTerms.length; i++) {
-        terms[i] = new Term(field, fieldType.readableToIndexed(splitTerms[i]));
-      }
-
-      TermContext[] termContexts = new TermContext[terms.length];
-      collectTermContext(topReaderContext, termContexts, terms);
-
-      NamedList<Object> termsMap = new SimpleOrderedMap<>();
-      for (int i = 0; i < terms.length; i++) {
-        if (termContexts[i] != null) {
-          String outTerm = fieldType.indexedToReadable(terms[i].bytes().utf8ToString());
-          int docFreq = termContexts[i].docFreq();
-          if (!includeTotalTermFreq) {
-            termsMap.add(outTerm, docFreq);
-          } else {
-            long totalTermFreq = termContexts[i].totalTermFreq();
-            NamedList<Long> termStats = new SimpleOrderedMap<>();
-            termStats.add("df", (long) docFreq);
-            termStats.add("ttf", totalTermFreq);
-            termsMap.add(outTerm, termStats);
+        // Since splitTerms is already sorted, this array will also be sorted. NOTE: this may not be true, it depends on readableToIndexed.
+        Term[] terms = new Term[splitTerms.length];
+        for (int i = 0; i < splitTerms.length; i++) {
+          terms[i] = new Term(field, fieldType.readableToIndexed(splitTerms[i]));
+        }
+  
+        TermStates[] termStates = new TermStates[terms.length];
+        collectTermStates(topReaderContext, termStates, terms);
+  
+        for (int i = 0; i < terms.length; i++) {
+          if (termStates[i] != null) {
+            String outTerm = fieldType.indexedToReadable(terms[i].bytes().utf8ToString());
+            int docFreq = termStates[i].docFreq();
+            addTermToNamedList(termsMap, outTerm, docFreq, termStates[i].totalTermFreq(), includeTotalTermFreq);
           }
         }
       }
@@ -682,8 +654,8 @@ public class TermsComponent extends SearchComponent {
     }
   }
 
-  private static void collectTermContext(IndexReaderContext topReaderContext, TermContext[] contextArray,
-      Term[] queryTerms) throws IOException {
+  private static void collectTermStates(IndexReaderContext topReaderContext, TermStates[] contextArray,
+                                        Term[] queryTerms) throws IOException {
     TermsEnum termsEnum = null;
     for (LeafReaderContext context : topReaderContext.leaves()) {
       for (int i = 0; i < queryTerms.length; i++) {
@@ -698,18 +670,55 @@ public class TermsComponent extends SearchComponent {
 
         if (termsEnum == TermsEnum.EMPTY) continue;
 
-        TermContext termContext = contextArray[i];
+        TermStates termStates = contextArray[i];
         if (termsEnum.seekExact(term.bytes())) {
-          if (termContext == null) {
-            termContext = new TermContext(topReaderContext);
-            contextArray[i] = termContext;
+          if (termStates == null) {
+            termStates = new TermStates(topReaderContext);
+            contextArray[i] = termStates;
           }
-          termContext.accumulateStatistics(termsEnum.docFreq(), termsEnum.totalTermFreq());
+          termStates.accumulateStatistics(termsEnum.docFreq(), termsEnum.totalTermFreq());
         }
       }
     }
   }
 
+  /**
+   * Helper method to add particular term to terms response
+   */
+  private static void addTermToNamedList(NamedList<Object> result, String term, long docFreq,
+                                         long totalTermFreq, boolean includeTotalTermFreq) {
+
+    if (includeTotalTermFreq) {
+        NamedList<Number> termStats = new SimpleOrderedMap<>();
+        termStats.add("df", docFreq);
+        termStats.add("ttf", totalTermFreq);
+        result.add(term, termStats);
+      } else {
+        result.add(term, TermsHelper.num(docFreq));
+      }
+  }
+
+  /**
+   * Comparator for {@link org.apache.solr.client.solrj.response.TermsResponse.Term} sorting
+   * This sorts term by frequency in descending order
+   */
+  public static class TermCountComparator implements Comparator<TermsResponse.Term> {
+
+    @Override
+    public int compare(TermsResponse.Term o1, TermsResponse.Term o2) {
+      long freq1 = o1.getFrequency();
+      long freq2 = o2.getFrequency();
+
+      if (freq2 < freq1) {
+        return -1;
+      } else if (freq1 < freq2) {
+        return 1;
+      } else {
+        return o1.getTerm().compareTo(o2.getTerm());
+      }
+    }
+  }
+
   private static void collectStats(SolrIndexSearcher searcher, NamedList<Number> stats) {
     int numDocs = searcher.getTopReaderContext().reader().numDocs();
     stats.add("numDocs", Long.valueOf(numDocs));
