@@ -84,6 +84,41 @@ public class JsonLoader extends ContentStreamLoader {
     new SingleThreadedJsonLoader(req, rsp, processor).load(req, rsp, stream, processor);
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
+  public static SolrInputDocument buildDoc(Map<String, Object> m) {
+    SolrInputDocument result = new SolrInputDocument();
+    for (Map.Entry<String, Object> e : m.entrySet()) {
+      if (mapEntryIsChildDoc(e.getValue())) { // parse child documents
+        if (e.getValue() instanceof List) {
+          List value = (List) e.getValue();
+          for (Object o : value) {
+            if (o instanceof Map) {
+              // retain the value as a list, even if the list contains a single value.
+              if(!result.containsKey(e.getKey())) {
+                result.setField(e.getKey(), new ArrayList<>(1));
+              }
+              result.addField(e.getKey(), buildDoc((Map) o));
+            }
+          }
+        } else if (e.getValue() instanceof Map) {
+          result.addField(e.getKey(), buildDoc((Map) e.getValue()));
+        }
+      } else {
+        result.setField(e.getKey(), e.getValue());
+      }
+    }
+    return result;
+  }
+
+  private static boolean mapEntryIsChildDoc(Object val) {
+    if(val instanceof List) {
+      @SuppressWarnings({"rawtypes"})
+      List listVal = (List) val;
+      if (listVal.size() == 0) return false;
+      return  listVal.get(0) instanceof Map;
+    }
+    return val instanceof Map;
+  }
 
   static class SingleThreadedJsonLoader extends ContentStreamLoader {
 
@@ -93,16 +128,11 @@ public class JsonLoader extends ContentStreamLoader {
     protected JSONParser parser;
     protected final int commitWithin;
     protected final boolean overwrite;
-    protected final boolean anonChildDoc;
 
-    /**
-     * {@link CommonParams#ANONYMOUS_CHILD_DOCS} Defaults to true.
-     */
-    public SingleThreadedJsonLoader(SolrQueryRequest req, SolrQueryResponse rsp, UpdateRequestProcessor processor) {
+    SingleThreadedJsonLoader(SolrQueryRequest req, SolrQueryResponse rsp, UpdateRequestProcessor processor) {
       this.processor = processor;
       this.req = req;
       this.rsp = rsp;
-      this.anonChildDoc = req.getParams().getBool(CommonParams.ANONYMOUS_CHILD_DOCS, true);
 
       commitWithin = req.getParams().getInt(UpdateParams.COMMIT_WITHIN, -1);
       overwrite = req.getParams().getBool(UpdateParams.OVERWRITE, true);
@@ -119,7 +149,7 @@ public class JsonLoader extends ContentStreamLoader {
         reader = stream.getReader();
         if (log.isTraceEnabled()) {
           String body = IOUtils.toString(reader);
-          log.trace("body", body);
+          log.trace("body: {}", body);
           reader = new StringReader(body);
         }
 
@@ -188,8 +218,10 @@ public class JsonLoader extends ContentStreamLoader {
           case JSONParser.BIGNUMBER:
           case JSONParser.BOOLEAN:
           case JSONParser.NULL:
-            log.info("Can't have a value here. Unexpected "
-                + JSONParser.getEventString(ev) + " at [" + parser.getPosition() + "]");
+            if (log.isInfoEnabled()) {
+              log.info("Can't have a value here. Unexpected {} at [{}]"
+                  , JSONParser.getEventString(ev), parser.getPosition());
+            }
 
           case JSONParser.OBJECT_START:
           case JSONParser.OBJECT_END:
@@ -197,7 +229,7 @@ public class JsonLoader extends ContentStreamLoader {
             break;
 
           default:
-            log.info("Noggit UNKNOWN_EVENT_ID: " + ev);
+            log.info("Noggit UNKNOWN_EVENT_ID: {}", ev);
             break;
         }
         // read the next event
@@ -222,7 +254,7 @@ public class JsonLoader extends ContentStreamLoader {
 
       JsonRecordReader jsonRecordReader = JsonRecordReader.getInst(split, Arrays.asList(fields));
       jsonRecordReader.streamRecords(parser, new JsonRecordReader.Handler() {
-        ArrayList docs = null;
+        ArrayList<Map<String, Object>> docs = null;
 
         @Override
         public void handle(Map<String, Object> record, String path) {
@@ -230,7 +262,7 @@ public class JsonLoader extends ContentStreamLoader {
 
           if (echo) {
             if (docs == null) {
-              docs = new ArrayList();
+              docs = new ArrayList<>();
               rsp.add("docs", docs);
             }
             changeChildDoc(copy);
@@ -250,39 +282,7 @@ public class JsonLoader extends ContentStreamLoader {
       });
     }
 
-    private SolrInputDocument buildDoc(Map<String, Object> m) {
-      SolrInputDocument result = new SolrInputDocument();
-      for (Map.Entry<String, Object> e : m.entrySet()) {
-        if (mapEntryIsChildDoc(e.getValue())) { // parse child documents
-          if (e.getValue() instanceof List) {
-            List value = (List) e.getValue();
-            for (Object o : value) {
-              if (o instanceof Map) {
-                if (anonChildDoc) {
-                  result.addChildDocument(buildDoc((Map) o));
-                } else {
-                  // retain the value as a list, even if the list contains a single value.
-                  if(!result.containsKey(e.getKey())) {
-                    result.setField(e.getKey(), new ArrayList<>(1));
-                  }
-                  result.addField(e.getKey(), buildDoc((Map) o));
-                }
-              }
-            }
-          } else if (e.getValue() instanceof Map) {
-            if (anonChildDoc) {
-              result.addChildDocument(buildDoc((Map) e.getValue()));
-            } else {
-              result.addField(e.getKey(), buildDoc((Map) e.getValue()));
-            }
-          }
-        } else {
-          result.setField(e.getKey(), e.getValue());
-        }
-      }
-      return result;
-    }
-
+    @SuppressWarnings({"unchecked", "rawtypes"})
     private Map<String, Object> getDocMap(Map<String, Object> record, JSONParser parser, String srcField, boolean mapUniqueKeyOnly) {
       Map result = record;
       if (srcField != null && parser instanceof RecordingJSONParser) {
@@ -429,6 +429,7 @@ public class JsonLoader extends ContentStreamLoader {
 
     void parseCommitOptions(CommitUpdateCommand cmd) throws IOException {
       assertNextEvent(JSONParser.OBJECT_START);
+      @SuppressWarnings({"unchecked"})
       final Map<String, Object> map = (Map) ObjectBuilder.getVal(parser);
 
       // SolrParams currently expects string values...
@@ -549,7 +550,7 @@ public class JsonLoader extends ContentStreamLoader {
         }
         String fieldName = parser.getString();
 
-        if (anonChildDoc && fieldName.equals(JsonLoader.CHILD_DOC_KEY)) {
+        if (fieldName.equals(JsonLoader.CHILD_DOC_KEY)) { // somewhat legacy
           ev = parser.nextEvent();
           assertEvent(ev, JSONParser.ARRAY_START);
           while ((ev = parser.nextEvent()) != JSONParser.ARRAY_END) {
@@ -558,94 +559,18 @@ public class JsonLoader extends ContentStreamLoader {
             sdoc.addChildDocument(parseDoc(ev));
           }
         } else {
-          SolrInputField sif = new SolrInputField(fieldName);
-          parseFieldValue(sif);
-          // pulling out the pieces may seem weird, but it's because
+          ev = parser.nextEvent();
+          Object val = parseFieldValue(ev, fieldName);
           // SolrInputDocument.addField will do the right thing
           // if the doc already has another value for this field
           // (ie: repeating fieldname keys)
-          sdoc.addField(sif.getName(), sif.getValue());
+          sdoc.addField(fieldName, val);
         }
 
       }
     }
 
-    private void parseFieldValue(SolrInputField sif) throws IOException {
-      int ev = parser.nextEvent();
-      if (ev == JSONParser.OBJECT_START) {
-        parseExtendedFieldValue(ev, sif);
-      } else {
-        Object val = parseNormalFieldValue(ev, sif);
-        sif.setValue(val);
-      }
-    }
-
-    /**
-     * A method to either extract an index time boost (deprecated), a map for atomic update, or a child document.
-     * firstly, a solr document SolrInputDocument constructed. It is then determined whether the document is indeed a childDocument(if it has a unique field).
-     * If so, it is added.
-     * Otherwise the document is looped over as a map, and is then parsed as an Atomic Update if that is the case.
-     * @param ev json parser event
-     * @param sif input field to add value to.
-     * @throws IOException in case of parsing exception.
-     */
-    private void parseExtendedFieldValue(int ev, SolrInputField sif) throws IOException {
-      assert ev == JSONParser.OBJECT_START;
-
-      SolrInputDocument extendedSolrDocument = parseDoc(ev);
-
-      if (isChildDoc(extendedSolrDocument)) {
-        sif.addValue(extendedSolrDocument);
-        return;
-      }
-
-      Object normalFieldValue = null;
-      Map<String, Object> extendedInfo = null;
-
-      for (SolrInputField entry: extendedSolrDocument) {
-        Object val = entry.getValue();
-        String label = entry.getName();
-        if ("boost".equals(label)) {
-          Object boostVal = val;
-          if (!(boostVal instanceof Double)) {
-            throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, "Boost should have number. "
-                + "Unexpected value: " + boostVal.toString() + "field=" + label);
-          }
-
-          String message = "Ignoring field boost: " + boostVal.toString() + " as index-time boosts are not supported anymore";
-          if (WARNED_ABOUT_INDEX_TIME_BOOSTS.compareAndSet(false, true)) {
-            log.warn(message);
-          } else {
-            log.debug(message);
-          }
-        } else if ("value".equals(label)) {
-          normalFieldValue = val;
-        } else {
-          // If we encounter other unknown map keys, then use a map
-          if (extendedInfo == null) {
-            extendedInfo = new HashMap<>(2);
-          }
-          // for now, the only extended info will be field values
-          // we could either store this as an Object or a SolrInputField
-          extendedInfo.put(label, val);
-        }
-        if (extendedInfo != null) {
-          if (normalFieldValue != null) {
-            extendedInfo.put("value", normalFieldValue);
-          }
-          sif.setValue(extendedInfo);
-        } else {
-          sif.setValue(normalFieldValue);
-        }
-      }
-    }
-
-
-    private Object parseNormalFieldValue(int ev, SolrInputField sif) throws IOException {
-      return ev == JSONParser.ARRAY_START ? parseArrayFieldValue(ev, sif): parseSingleFieldValue(ev, sif);
-    }
-
-    private Object parseSingleFieldValue(int ev, SolrInputField sif) throws IOException {
+    private Object parseFieldValue(int ev, String fieldName) throws IOException {
       switch (ev) {
         case JSONParser.STRING:
           return parser.getString();
@@ -661,29 +586,47 @@ public class JsonLoader extends ContentStreamLoader {
           parser.getNull();
           return null;
         case JSONParser.ARRAY_START:
-          return parseArrayFieldValue(ev, sif);
+          return parseArrayFieldValue(ev, fieldName);
         case JSONParser.OBJECT_START:
-          parseExtendedFieldValue(ev, sif);
-          return sif.getValue();
+          return parseObjectFieldValue(ev, fieldName);
         default:
           throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, "Error parsing JSON field value. "
-              + "Unexpected " + JSONParser.getEventString(ev) + " at [" + parser.getPosition() + "], field=" + sif.getName());
+              + "Unexpected " + JSONParser.getEventString(ev) + " at [" + parser.getPosition() + "], field=" + fieldName);
       }
     }
 
-
-    private List<Object> parseArrayFieldValue(int ev, SolrInputField sif) throws IOException {
+    @SuppressWarnings({"unchecked"})
+    private List<Object> parseArrayFieldValue(int ev, String fieldName) throws IOException {
       assert ev == JSONParser.ARRAY_START;
 
+      @SuppressWarnings({"rawtypes"})
       ArrayList lst = new ArrayList(2);
       for (; ; ) {
         ev = parser.nextEvent();
         if (ev == JSONParser.ARRAY_END) {
           return lst;
         }
-        Object val = parseSingleFieldValue(ev, sif);
-        lst.add(val);
-        sif.setValue(null);
+        lst.add(parseFieldValue(ev, fieldName));
+      }
+    }
+
+    /**
+     * Parses this object as either a map for atomic update, or a child document.
+     */
+    private Object parseObjectFieldValue(int ev, String fieldName) throws IOException {
+      assert ev == JSONParser.OBJECT_START;
+
+      SolrInputDocument extendedSolrDocument = parseDoc(ev);
+      // is this a partial update or a child doc?
+      if (isChildDoc(extendedSolrDocument)) {
+        return extendedSolrDocument;
+      } else {
+        //return extendedSolrDocument.toMap(new HashMap<>(extendedSolrDocument.size()));  not quite right
+        Map<String, Object> map = new HashMap<>(extendedSolrDocument.size());
+        for (SolrInputField inputField : extendedSolrDocument) {
+          map.put(inputField.getName(), inputField.getValue());
+        }
+        return map;
       }
     }
 
@@ -691,16 +634,9 @@ public class JsonLoader extends ContentStreamLoader {
       return extendedFieldValue.containsKey(req.getSchema().getUniqueKeyField().getName());
     }
 
-    private boolean mapEntryIsChildDoc(Object val) {
-      if(val instanceof List) {
-        List listVal = (List) val;
-        if (listVal.size() == 0) return false;
-        return  listVal.get(0) instanceof Map;
-      }
-      return val instanceof Map;
-    }
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private static Object changeChildDoc(Object o) {
     if (o instanceof List) {
       return ((List) o)
