@@ -64,9 +64,9 @@ import org.slf4j.MDC;
  * This class maintains a repository of named {@link MetricRegistry} instances, and provides several
  * helper methods for managing various aspects of metrics reporting:
  * <ul>
- *   <li>registry creation, clearing and removal,</li>
- *   <li>creation of most common metric implementations,</li>
- *   <li>management of {@link SolrMetricReporter}-s specific to a named registry.</li>
+ * <li>registry creation, clearing and removal,</li>
+ * <li>creation of most common metric implementations,</li>
+ * <li>management of {@link SolrMetricReporter}-s specific to a named registry.</li>
  * </ul>
  * {@link MetricRegistry} instances are automatically created when first referenced by name. Similarly,
  * instances of {@link Metric} implementations, such as {@link Meter}, {@link Counter}, {@link Timer} and
@@ -84,15 +84,21 @@ public class SolrMetricManager {
 
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
-  /** Common prefix for all registry names that Solr uses. */
+  /**
+   * Common prefix for all registry names that Solr uses.
+   */
   public static final String REGISTRY_NAME_PREFIX = "solr.";
 
-  /** Registry name for Jetty-specific metrics. This name is also subject to overrides controlled by
-   * system properties. This registry is shared between instances of {@link SolrMetricManager}. */
+  /**
+   * Registry name for Jetty-specific metrics. This name is also subject to overrides controlled by
+   * system properties. This registry is shared between instances of {@link SolrMetricManager}.
+   */
   public static final String JETTY_REGISTRY = REGISTRY_NAME_PREFIX + SolrInfoBean.Group.jetty.toString();
 
-  /** Registry name for JVM-specific metrics. This name is also subject to overrides controlled by
-   * system properties. This registry is shared between instances of {@link SolrMetricManager}. */
+  /**
+   * Registry name for JVM-specific metrics. This name is also subject to overrides controlled by
+   * system properties. This registry is shared between instances of {@link SolrMetricManager}.
+   */
   public static final String JVM_REGISTRY = REGISTRY_NAME_PREFIX + SolrInfoBean.Group.jvm.toString();
 
   private final ConcurrentMap<String, MetricRegistry> registries = new ConcurrentHashMap<>();
@@ -104,12 +110,14 @@ public class SolrMetricManager {
 
   public static final int DEFAULT_CLOUD_REPORTER_PERIOD = 60;
 
-  private MetricRegistry.MetricSupplier<Counter> counterSupplier;
-  private MetricRegistry.MetricSupplier<Meter> meterSupplier;
-  private MetricRegistry.MetricSupplier<Timer> timerSupplier;
-  private MetricRegistry.MetricSupplier<Histogram> histogramSupplier;
+  private final MetricsConfig metricsConfig;
+  private final MetricRegistry.MetricSupplier<Counter> counterSupplier;
+  private final MetricRegistry.MetricSupplier<Meter> meterSupplier;
+  private final MetricRegistry.MetricSupplier<Timer> timerSupplier;
+  private final MetricRegistry.MetricSupplier<Histogram> histogramSupplier;
 
   public SolrMetricManager() {
+    metricsConfig = new MetricsConfig.MetricsConfigBuilder().build();
     counterSupplier = MetricSuppliers.counterSupplier(null, null);
     meterSupplier = MetricSuppliers.meterSupplier(null, null);
     timerSupplier = MetricSuppliers.timerSupplier(null, null);
@@ -117,6 +125,7 @@ public class SolrMetricManager {
   }
 
   public SolrMetricManager(SolrResourceLoader loader, MetricsConfig metricsConfig) {
+    this.metricsConfig = metricsConfig;
     counterSupplier = MetricSuppliers.counterSupplier(loader, metricsConfig.getCounterSupplier());
     meterSupplier = MetricSuppliers.meterSupplier(loader, metricsConfig.getMeterSupplier());
     timerSupplier = MetricSuppliers.timerSupplier(loader, metricsConfig.getTimerSupplier());
@@ -140,6 +149,35 @@ public class SolrMetricManager {
     return histogramSupplier;
   }
 
+  /**
+   * Return an object used for representing a null (missing) numeric value.
+   */
+  public Object nullNumber() {
+    return metricsConfig.getNullNumber();
+  }
+
+  /**
+   * Return an object used for representing a "Not A Number" (NaN) value.
+   */
+  public Object notANumber() {
+    return metricsConfig.getNotANumber();
+  }
+
+  /**
+   * Return an object used for representing a null (missing) string value.
+   */
+  public Object nullString() {
+    return metricsConfig.getNullString();
+  }
+
+  /**
+   * Return an object used for representing a null (missing) object value.
+   */
+  public Object nullObject() {
+    return metricsConfig.getNullObject();
+  }
+
+
   /**
    * An implementation of {@link MetricFilter} that selects metrics
    * with names that start with one of prefixes.
@@ -151,8 +189,9 @@ public class SolrMetricManager {
 
     /**
      * Create a filter that uses the provided prefixes.
+     *
      * @param prefixes prefixes to use, must not be null. If empty then any
-     *               name will match, if not empty then match on any prefix will
+     *                 name will match, if not empty then match on any prefix will
      *                 succeed (logical OR).
      */
     public PrefixFilter(String... prefixes) {
@@ -190,6 +229,7 @@ public class SolrMetricManager {
 
     /**
      * Return the set of names that matched this filter.
+     *
      * @return matching names
      */
     public Set<String> getMatched() {
@@ -222,8 +262,9 @@ public class SolrMetricManager {
 
     /**
      * Create a filter that uses the provided prefix.
+     *
      * @param patterns regex patterns to use, must not be null. If empty then any
-     *               name will match, if not empty then match on any pattern will
+     *                 name will match, if not empty then match on any pattern will
      *                 succeed (logical OR).
      */
     public RegexFilter(String... patterns) throws PatternSyntaxException {
@@ -262,6 +303,7 @@ public class SolrMetricManager {
 
     /**
      * Return the set of names that matched this filter.
+     *
      * @return matching names
      */
     public Set<String> getMatched() {
@@ -283,6 +325,10 @@ public class SolrMetricManager {
     }
   }
 
+  /**
+   * An implementation of {@link MetricFilter} that selects metrics
+   * that match any filter in a list of filters.
+   */
   public static class OrFilter implements MetricFilter {
     List<MetricFilter> filters = new ArrayList<>();
 
@@ -313,6 +359,10 @@ public class SolrMetricManager {
     }
   }
 
+  /**
+   * An implementation of {@link MetricFilter} that selects metrics
+   * that match all filters in a list of filters.
+   */
   public static class AndFilter implements MetricFilter {
     List<MetricFilter> filters = new ArrayList<>();
 
@@ -354,18 +404,20 @@ public class SolrMetricManager {
   }
 
   /**
-   * Check whether a registry with a given (overridable) name already exists.
+   * Check whether a registry with a given name already exists.
+   *
    * @param name registry name
    * @return true if this name points to a registry that already exists, false otherwise
    */
   public boolean hasRegistry(String name) {
     Set<String> names = registryNames();
-    name = overridableRegistryName(name);
+    name = enforcePrefix(name);
     return names.contains(name);
   }
 
   /**
    * Return set of existing registry names that match a regex pattern
+   *
    * @param patterns regex patterns. NOTE: users need to make sure that patterns that
    *                 don't start with a wildcard use the full registry name starting with
    *                 {@link #REGISTRY_NAME_PREFIX}
@@ -379,7 +431,7 @@ public class SolrMetricManager {
     for (String pattern : patterns) {
       compiled.add(Pattern.compile(pattern));
     }
-    return registryNames((Pattern[])compiled.toArray(new Pattern[compiled.size()]));
+    return registryNames(compiled.toArray(new Pattern[compiled.size()]));
   }
 
   public Set<String> registryNames(Pattern... patterns) {
@@ -399,27 +451,24 @@ public class SolrMetricManager {
 
   /**
    * Check for predefined shared registry names. This compares the input name
-   * with normalized and possibly overriden names of predefined shared registries -
+   * with normalized names of predefined shared registries -
    * {@link #JVM_REGISTRY} and {@link #JETTY_REGISTRY}.
-   * @param registry already normalized and possibly overriden name
+   *
+   * @param registry already normalized name
    * @return true if the name matches one of shared registries
    */
   private static boolean isSharedRegistry(String registry) {
-    if (overridableRegistryName(JETTY_REGISTRY).equals(registry) ||
-        overridableRegistryName(JVM_REGISTRY).equals(registry)) {
-      return true;
-    } else {
-      return false;
-    }
+    return JETTY_REGISTRY.equals(registry) || JVM_REGISTRY.equals(registry);
   }
 
   /**
    * Get (or create if not present) a named registry
+   *
    * @param registry name of the registry
    * @return existing or newly created registry
    */
   public MetricRegistry registry(String registry) {
-    registry = overridableRegistryName(registry);
+    registry = enforcePrefix(registry);
     if (isSharedRegistry(registry)) {
       return SharedMetricRegistries.getOrCreate(registry);
     } else {
@@ -449,13 +498,14 @@ public class SolrMetricManager {
 
   /**
    * Remove a named registry.
+   *
    * @param registry name of the registry to remove
    */
   public void removeRegistry(String registry) {
     // close any reporters for this registry first
     closeReporters(registry, null);
-    // make sure we use a name with prefix, with overrides
-    registry = overridableRegistryName(registry);
+    // make sure we use a name with prefix
+    registry = enforcePrefix(registry);
     if (isSharedRegistry(registry)) {
       SharedMetricRegistries.remove(registry);
     } else {
@@ -472,14 +522,15 @@ public class SolrMetricManager {
    * Swap registries. This is useful eg. during
    * {@link org.apache.solr.core.SolrCore} rename or swap operations. NOTE:
    * this operation is not supported for shared registries.
+   *
    * @param registry1 source registry
    * @param registry2 target registry. Note: when used after core rename the target registry doesn't
    *                  exist, so the swap operation will only rename the existing registry without creating
    *                  an empty one under the previous name.
    */
   public void swapRegistries(String registry1, String registry2) {
-    registry1 = overridableRegistryName(registry1);
-    registry2 = overridableRegistryName(registry2);
+    registry1 = enforcePrefix(registry1);
+    registry2 = enforcePrefix(registry2);
     if (isSharedRegistry(registry1) || isSharedRegistry(registry2)) {
       throw new UnsupportedOperationException("Cannot swap shared registry: " + registry1 + ", " + registry2);
     }
@@ -503,25 +554,46 @@ public class SolrMetricManager {
     }
   }
 
+  /**
+   * Potential conflict resolution strategies when attempting to register a new metric that already exists
+   */
+  public enum ResolutionStrategy {
+    /**
+     * The existing metric will be kept and the new metric will be ignored
+     */
+    IGNORE,
+    /**
+     * The existing metric will be removed and replaced with the new metric
+     */
+    REPLACE,
+    /**
+     * An exception will be thrown. This is the default implementation behavior.
+     */
+    ERROR
+  }
+
   /**
    * Register all metrics in the provided {@link MetricSet}, optionally skipping those that
    * already exist.
-   * @param registry registry name
-   * @param metrics metric set to register
-   * @param force if true then already existing metrics with the same name will be replaced.
-   *                     When false and a metric with the same name already exists an exception
-   *                     will be thrown.
+   *
+   * @param registry   registry name
+   * @param metrics    metric set to register
+   * @param strategy   the conflict resolution strategy to use if the named metric already exists.
    * @param metricPath (optional) additional top-most metric name path elements
    * @throws Exception if a metric with this name already exists.
    */
-  public void registerAll(String registry, MetricSet metrics, boolean force, String... metricPath) throws Exception {
+  public void registerAll(String registry, MetricSet metrics, ResolutionStrategy strategy, String... metricPath) throws Exception {
     MetricRegistry metricRegistry = registry(registry);
     synchronized (metricRegistry) {
       Map<String, Metric> existingMetrics = metricRegistry.getMetrics();
       for (Map.Entry<String, Metric> entry : metrics.getMetrics().entrySet()) {
         String fullName = mkName(entry.getKey(), metricPath);
-        if (force && existingMetrics.containsKey(fullName)) {
-          metricRegistry.remove(fullName);
+        if (existingMetrics.containsKey(fullName)) {
+          if (strategy == ResolutionStrategy.REPLACE) {
+            metricRegistry.remove(fullName);
+          } else if (strategy == ResolutionStrategy.IGNORE) {
+            continue;
+          } // strategy == ERROR will fail when we try to register later
         }
         metricRegistry.register(fullName, entry.getValue());
       }
@@ -530,6 +602,7 @@ public class SolrMetricManager {
 
   /**
    * Remove all metrics from a specified registry.
+   *
    * @param registry registry name
    */
   public void clearRegistry(String registry) {
@@ -538,12 +611,13 @@ public class SolrMetricManager {
 
   /**
    * Remove some metrics from a named registry
-   * @param registry registry name
+   *
+   * @param registry   registry name
    * @param metricPath (optional) top-most metric name path elements. If empty then
-   *        this is equivalent to calling {@link #clearRegistry(String)},
-   *        otherwise non-empty elements will be joined using dotted notation
-   *        to form a fully-qualified prefix. Metrics with names that start
-   *        with the prefix will be removed.
+   *                   this is equivalent to calling {@link #clearRegistry(String)},
+   *                   otherwise non-empty elements will be joined using dotted notation
+   *                   to form a fully-qualified prefix. Metrics with names that start
+   *                   with the prefix will be removed.
    * @return set of metrics names that have been removed.
    */
   public Set<String> clearMetrics(String registry, String... metricPath) {
@@ -560,7 +634,8 @@ public class SolrMetricManager {
 
   /**
    * Retrieve matching metrics and their names.
-   * @param registry registry name.
+   *
+   * @param registry     registry name.
    * @param metricFilter filter (null is equivalent to {@link MetricFilter#ALL}).
    * @return map of matching names and metrics
    */
@@ -575,7 +650,8 @@ public class SolrMetricManager {
 
   /**
    * Create or get an existing named {@link Meter}
-   * @param registry registry name
+   *
+   * @param registry   registry name
    * @param metricName metric name, either final name or a fully-qualified name
    *                   using dotted notation
    * @param metricPath (optional) additional top-most metric name path elements
@@ -591,7 +667,8 @@ public class SolrMetricManager {
 
   /**
    * Create or get an existing named {@link Timer}
-   * @param registry registry name
+   *
+   * @param registry   registry name
    * @param metricName metric name, either final name or a fully-qualified name
    *                   using dotted notation
    * @param metricPath (optional) additional top-most metric name path elements
@@ -607,7 +684,8 @@ public class SolrMetricManager {
 
   /**
    * Create or get an existing named {@link Counter}
-   * @param registry registry name
+   *
+   * @param registry   registry name
    * @param metricName metric name, either final name or a fully-qualified name
    *                   using dotted notation
    * @param metricPath (optional) additional top-most metric name path elements
@@ -623,7 +701,8 @@ public class SolrMetricManager {
 
   /**
    * Create or get an existing named {@link Histogram}
-   * @param registry registry name
+   *
+   * @param registry   registry name
    * @param metricName metric name, either final name or a fully-qualified name
    *                   using dotted notation
    * @param metricPath (optional) additional top-most metric name path elements
@@ -639,11 +718,12 @@ public class SolrMetricManager {
 
   /**
    * Register an instance of {@link Metric}.
-   * @param registry registry name
-   * @param metric metric instance
-   * @param force if true then an already existing metric with the same name will be replaced.
-   *                     When false and a metric with the same name already exists an exception
-   *                     will be thrown.
+   *
+   * @param registry   registry name
+   * @param metric     metric instance
+   * @param force      if true then an already existing metric with the same name will be replaced.
+   *                   When false and a metric with the same name already exists an exception
+   *                   will be thrown.
    * @param metricName metric name, either final name or a fully-qualified name
    *                   using dotted notation
    * @param metricPath (optional) additional top-most metric name path elements
@@ -654,8 +734,8 @@ public class SolrMetricManager {
     if (info != null) {
       info.registerMetricName(fullName);
     }
-    synchronized (metricRegistry) {
-      if (force && metricRegistry.getMetrics().containsKey(fullName)) {
+    synchronized (metricRegistry) { // prevent race; register() throws if metric is already present
+      if (force) { // must remove any existing one if present
         metricRegistry.remove(fullName);
       }
       metricRegistry.register(fullName, metric);
@@ -691,31 +771,40 @@ public class SolrMetricManager {
       return gauge;
     }
   }
-
+  @SuppressWarnings({"unchecked", "rawtypes"})
   public void registerGauge(SolrInfoBean info, String registry, Gauge<?> gauge, String tag, boolean force, String metricName, String... metricPath) {
+    if (!metricsConfig.isEnabled()) {
+      gauge = MetricSuppliers.getNoOpGauge(gauge);
+    }
     registerMetric(info, registry, new GaugeWrapper(gauge, tag), force, metricName, metricPath);
   }
 
-  public int unregisterGauges(String registryName, String tag) {
-    if (tag == null) {
+  public int unregisterGauges(String registryName, String tagSegment) {
+    if (tagSegment == null) {
       return 0;
     }
     MetricRegistry registry = registry(registryName);
+    if (registry == null) return 0;
     AtomicInteger removed = new AtomicInteger();
     registry.removeMatching((name, metric) -> {
-      if (metric instanceof GaugeWrapper &&
-          tag.equals(((GaugeWrapper)metric).getTag())) {
-        removed.incrementAndGet();
-        return true;
-      } else {
-        return false;
+      if (metric instanceof GaugeWrapper) {
+        @SuppressWarnings({"rawtypes"})
+        GaugeWrapper wrapper = (GaugeWrapper) metric;
+        boolean toRemove = wrapper.getTag().contains(tagSegment);
+        if (toRemove) {
+          removed.incrementAndGet();
+        }
+        return toRemove;
       }
+      return false;
+
     });
     return removed.get();
   }
 
   /**
    * This method creates a hierarchical name with arbitrary levels of hierarchy
+   *
    * @param name the final segment of the name, must not be null or empty.
    * @param path optional path segments, starting from the top level. Empty or null
    *             segments will be skipped.
@@ -723,10 +812,16 @@ public class SolrMetricManager {
    * segments prepended to the name.
    */
   public static String mkName(String name, String... path) {
+    return makeName(path == null || path.length == 0 ? Collections.emptyList() : Arrays.asList(path),
+        name);
+
+  }
+
+  public static String makeName(List<String> path, String name) {
     if (name == null || name.isEmpty()) {
       throw new IllegalArgumentException("name must not be empty");
     }
-    if (path == null || path.length == 0) {
+    if (path == null || path.size() == 0) {
       return name;
     } else {
       StringBuilder sb = new StringBuilder();
@@ -747,28 +842,9 @@ public class SolrMetricManager {
     }
   }
 
-  /**
-   * Allows named registries to be renamed using System properties.
-   * This would be mostly be useful if you want to combine the metrics from a few registries for a single
-   * reporter.
-   * <p>For example, in order to collect metrics from related cores in a single registry you could specify
-   * the following system properties:</p>
-   * <pre>
-   *   ... -Dsolr.core.collection1=solr.core.allCollections -Dsolr.core.collection2=solr.core.allCollections
-   * </pre>
-   * <b>NOTE:</b> Once a registry is renamed in a way that its metrics are combined with another repository
-   * it is no longer possible to retrieve the original metrics until this renaming is removed and the Solr
-   * {@link org.apache.solr.core.SolrInfoBean.Group} of components that reported to that name is restarted.
-   * @param registry The name of the registry
-   * @return A potentially overridden (via System properties) registry name
-   */
-  public static String overridableRegistryName(String registry) {
-    String fqRegistry = enforcePrefix(registry);
-    return enforcePrefix(System.getProperty(fqRegistry,fqRegistry));
-  }
-
   /**
    * Enforces the leading {@link #REGISTRY_NAME_PREFIX} in a name.
+   *
    * @param name input name, possibly without the prefix
    * @return original name if it contained the prefix, or the
    * input name with the prefix prepended.
@@ -783,6 +859,7 @@ public class SolrMetricManager {
 
   /**
    * Helper method to construct a properly prefixed registry name based on the group.
+   *
    * @param group reporting group
    * @param names optional child elements of the registry name. If exactly one element is provided
    *              and it already contains the required prefix and group name then this value will be used,
@@ -805,7 +882,7 @@ public class SolrMetricManager {
     } else {
       fullName = MetricRegistry.name(group.toString(), names);
     }
-    return overridableRegistryName(fullName);
+    return enforcePrefix(fullName);
   }
 
   // reporter management
@@ -816,13 +893,14 @@ public class SolrMetricManager {
    * they will always be loaded for any group. These two attributes may also contain multiple comma- or
    * whitespace-separated values, in which case the reporter will be loaded for any matching value from
    * the list. If both attributes are present then only "group" attribute will be processed.
-   * @param pluginInfos plugin configurations
-   * @param loader resource loader
+   *
+   * @param pluginInfos   plugin configurations
+   * @param loader        resource loader
    * @param coreContainer core container
-   * @param solrCore optional solr core
-   * @param tag optional tag for the reporters, to distinguish reporters logically created for different parent
-   *            component instances.
-   * @param group selected group, not null
+   * @param solrCore      optional solr core
+   * @param tag           optional tag for the reporters, to distinguish reporters logically created for different parent
+   *                      component instances.
+   * @param group         selected group, not null
    * @param registryNames optional child registry name elements
    */
   public void loadReporters(PluginInfo[] pluginInfos, SolrResourceLoader loader, CoreContainer coreContainer, SolrCore solrCore, String tag, SolrInfoBean.Group group, String... registryNames) {
@@ -838,7 +916,7 @@ public class SolrMetricManager {
           String[] targets = target.split("[\\s,]+");
           boolean found = false;
           for (String t : targets) {
-            t = overridableRegistryName(t);
+            t = enforcePrefix(t);
             if (registryName.equals(t)) {
               found = true;
               break;
@@ -867,7 +945,7 @@ public class SolrMetricManager {
       try {
         loadReporter(registryName, loader, coreContainer, solrCore, info, tag);
       } catch (Exception e) {
-        log.warn("Error loading metrics reporter, plugin info: " + info, e);
+        log.warn("Error loading metrics reporter, plugin info: {}", info, e);
       }
     }
   }
@@ -900,43 +978,42 @@ public class SolrMetricManager {
 
   /**
    * Create and register an instance of {@link SolrMetricReporter}.
-   * @param registry reporter is associated with this registry
-   * @param loader loader to use when creating an instance of the reporter
+   *
+   * @param registry      reporter is associated with this registry
+   * @param loader        loader to use when creating an instance of the reporter
    * @param coreContainer core container
-   * @param solrCore optional solr core
-   * @param pluginInfo plugin configuration. Plugin "name" and "class" attributes are required.
-   * @param tag optional tag for the reporter, to distinguish reporters logically created for different parent
-   *            component instances.
+   * @param solrCore      optional solr core
+   * @param pluginInfo    plugin configuration. Plugin "name" and "class" attributes are required.
+   * @param tag           optional tag for the reporter, to distinguish reporters logically created for different parent
+   *                      component instances.
    * @throws Exception if any argument is missing or invalid
    */
+  @SuppressWarnings({"rawtypes"})
   public void loadReporter(String registry, SolrResourceLoader loader, CoreContainer coreContainer, SolrCore solrCore, PluginInfo pluginInfo, String tag) throws Exception {
     if (registry == null || pluginInfo == null || pluginInfo.name == null || pluginInfo.className == null) {
       throw new IllegalArgumentException("loadReporter called with missing arguments: " +
           "registry=" + registry + ", loader=" + loader + ", pluginInfo=" + pluginInfo);
     }
-    // make sure we use a name with prefix, with overrides
-    registry = overridableRegistryName(registry);
+    // make sure we use a name with prefix
+    registry = enforcePrefix(registry);
     SolrMetricReporter reporter = loader.newInstance(
         pluginInfo.className,
         SolrMetricReporter.class,
         new String[0],
-        new Class[]{SolrMetricManager.class, String.class},
+    new Class[]{SolrMetricManager.class, String.class},
         new Object[]{this, registry}
     );
     // prepare MDC for plugins that want to use its properties
-    MDCLoggingContext.setNode(coreContainer);
-    if (solrCore != null) {
-      MDCLoggingContext.setCore(solrCore);
-    }
+    MDCLoggingContext.setCoreDescriptor(coreContainer, solrCore == null ? null : solrCore.getCoreDescriptor());
     if (tag != null) {
       // add instance tag to MDC
       MDC.put("tag", "t:" + tag);
     }
     try {
       if (reporter instanceof SolrCoreReporter) {
-        ((SolrCoreReporter)reporter).init(pluginInfo, solrCore);
+        ((SolrCoreReporter) reporter).init(pluginInfo, solrCore);
       } else if (reporter instanceof SolrCoreContainerReporter) {
-        ((SolrCoreContainerReporter)reporter).init(pluginInfo, coreContainer);
+        ((SolrCoreContainerReporter) reporter).init(pluginInfo, coreContainer);
       } else {
         reporter.init(pluginInfo);
       }
@@ -968,7 +1045,7 @@ public class SolrMetricManager {
       }
       SolrMetricReporter oldReporter = perRegistry.get(name);
       if (oldReporter != null) { // close it
-        log.info("Replacing existing reporter '" + name + "' in registry '" + registry + "': " + oldReporter.toString());
+        log.info("Replacing existing reporter '{}' in registry'{}': {}", name, registry, oldReporter);
         oldReporter.close();
       }
       perRegistry.put(name, reporter);
@@ -980,22 +1057,23 @@ public class SolrMetricManager {
 
   /**
    * Close and unregister a named {@link SolrMetricReporter} for a registry.
+   *
    * @param registry registry name
-   * @param name reporter name
-   * @param tag optional tag for the reporter, to distinguish reporters logically created for different parent
-   *            component instances.
+   * @param name     reporter name
+   * @param tag      optional tag for the reporter, to distinguish reporters logically created for different parent
+   *                 component instances.
    * @return true if a named reporter existed and was closed.
    */
   public boolean closeReporter(String registry, String name, String tag) {
-    // make sure we use a name with prefix, with overrides
-    registry = overridableRegistryName(registry);
+    // make sure we use a name with prefix
+    registry = enforcePrefix(registry);
     try {
       if (!reportersLock.tryLock(10, TimeUnit.SECONDS)) {
-        log.warn("Could not obtain lock to modify reporters registry: " + registry);
+        log.warn("Could not obtain lock to modify reporters registry: {}", registry);
         return false;
       }
     } catch (InterruptedException e) {
-      log.warn("Interrupted while trying to obtain lock to modify reporters registry: " + registry);
+      log.warn("Interrupted while trying to obtain lock to modify reporters registry: {}", registry);
       return false;
     }
     try {
@@ -1013,7 +1091,7 @@ public class SolrMetricManager {
       try {
         reporter.close();
       } catch (Exception e) {
-        log.warn("Error closing metric reporter, registry=" + registry + ", name=" + name, e);
+        log.warn("Error closing metric reporter, registry={}, name={}", registry, name, e);
       }
       return true;
     } finally {
@@ -1023,6 +1101,7 @@ public class SolrMetricManager {
 
   /**
    * Close and unregister all {@link SolrMetricReporter}-s for a registry.
+   *
    * @param registry registry name
    * @return names of closed reporters
    */
@@ -1032,25 +1111,26 @@ public class SolrMetricManager {
 
   /**
    * Close and unregister all {@link SolrMetricReporter}-s for a registry.
+   *
    * @param registry registry name
-   * @param tag optional tag for the reporter, to distinguish reporters logically created for different parent
-   *            component instances.
+   * @param tag      optional tag for the reporter, to distinguish reporters logically created for different parent
+   *                 component instances.
    * @return names of closed reporters
    */
   public Set<String> closeReporters(String registry, String tag) {
-    // make sure we use a name with prefix, with overrides
-    registry = overridableRegistryName(registry);
+    // make sure we use a name with prefix
+    registry = enforcePrefix(registry);
     try {
       if (!reportersLock.tryLock(10, TimeUnit.SECONDS)) {
-        log.warn("Could not obtain lock to modify reporters registry: " + registry);
+        log.warn("Could not obtain lock to modify reporters registry: {}", registry);
         return Collections.emptySet();
       }
     } catch (InterruptedException e) {
-      log.warn("Interrupted while trying to obtain lock to modify reporters registry: " + registry);
+      log.warn("Interrupted while trying to obtain lock to modify reporters registry: {}", registry);
       return Collections.emptySet();
     }
-    log.info("Closing metric reporters for registry=" + registry + ", tag=" + tag);
     try {
+      log.info("Closing metric reporters for registry={} tag={}", registry, tag);
       Map<String, SolrMetricReporter> perRegistry = reporters.get(registry);
       if (perRegistry != null) {
         Set<String> names = new HashSet<>(perRegistry.keySet());
@@ -1063,7 +1143,7 @@ public class SolrMetricManager {
           try {
             reporter.close();
           } catch (IOException ioe) {
-            log.warn("Exception closing reporter " + reporter, ioe);
+            log.warn("Exception closing reporter {}", reporter, ioe);
           }
           removed.add(name);
         });
@@ -1076,24 +1156,28 @@ public class SolrMetricManager {
       }
     } finally {
       reportersLock.unlock();
+      if (log.isDebugEnabled()) {
+        log.debug("Finished closing registry={}, tag={}", registry, tag);
+      }
     }
   }
 
   /**
    * Get a map of reporters for a registry. Keys are reporter names, values are reporter instances.
+   *
    * @param registry registry name
    * @return map of reporters and their names, may be empty but never null
    */
   public Map<String, SolrMetricReporter> getReporters(String registry) {
-    // make sure we use a name with prefix, with overrides
-    registry = overridableRegistryName(registry);
+    // make sure we use a name with prefix
+    registry = enforcePrefix(registry);
     try {
       if (!reportersLock.tryLock(10, TimeUnit.SECONDS)) {
-        log.warn("Could not obtain lock to modify reporters registry: " + registry);
+        log.warn("Could not obtain lock to modify reporters registry: {}", registry);
         return Collections.emptyMap();
       }
     } catch (InterruptedException e) {
-      log.warn("Interrupted while trying to obtain lock to modify reporters registry: " + registry);
+      log.warn("Interrupted while trying to obtain lock to modify reporters registry: {}", registry);
       return Collections.emptyMap();
     }
     try {
@@ -1110,8 +1194,8 @@ public class SolrMetricManager {
   }
 
   private List<PluginInfo> prepareCloudPlugins(PluginInfo[] pluginInfos, String group,
-                                                      Map<String, String> defaultAttributes,
-                                                      Map<String, Object> defaultInitArgs) {
+                                               Map<String, String> defaultAttributes,
+                                               Map<String, Object> defaultInitArgs) {
     List<PluginInfo> result = new ArrayList<>();
     if (pluginInfos == null) {
       pluginInfos = new PluginInfo[0];
@@ -1129,6 +1213,7 @@ public class SolrMetricManager {
     return result;
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private PluginInfo preparePlugin(PluginInfo info, Map<String, String> defaultAttributes,
                                    Map<String, Object> defaultInitArgs) {
     if (info == null) {
@@ -1175,7 +1260,7 @@ public class SolrMetricManager {
       try {
         loadReporter(registryName, core, info, core.getMetricTag());
       } catch (Exception e) {
-        log.warn("Could not load shard reporter, pluginInfo=" + info, e);
+        log.warn("Could not load shard reporter, pluginInfo={}", info, e);
       }
     }
   }
@@ -1197,7 +1282,7 @@ public class SolrMetricManager {
       try {
         loadReporter(registryName, cc, info);
       } catch (Exception e) {
-        log.warn("Could not load cluster reporter, pluginInfo=" + info, e);
+        log.warn("Could not load cluster reporter, pluginInfo={}", info, e);
       }
     }
   }
