@@ -28,7 +28,6 @@ import javax.management.Query;
 import javax.management.QueryExp;
 import java.io.Closeable;
 import java.lang.invoke.MethodHandles;
-import java.lang.management.ManagementFactory;
 import java.util.HashMap;
 import java.util.Locale;
 import java.util.Map;
@@ -37,7 +36,7 @@ import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
 
 import com.codahale.metrics.Counter;
-import com.codahale.metrics.DefaultObjectNameFactory;
+import com.codahale.metrics.jmx.DefaultObjectNameFactory;
 import com.codahale.metrics.Gauge;
 import com.codahale.metrics.Histogram;
 import com.codahale.metrics.Meter;
@@ -46,7 +45,7 @@ import com.codahale.metrics.Metric;
 import com.codahale.metrics.MetricFilter;
 import com.codahale.metrics.MetricRegistry;
 import com.codahale.metrics.MetricRegistryListener;
-import com.codahale.metrics.ObjectNameFactory;
+import com.codahale.metrics.jmx.ObjectNameFactory;
 import com.codahale.metrics.Reporter;
 import com.codahale.metrics.Timer;
 import org.apache.solr.metrics.MetricsMap;
@@ -55,7 +54,7 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
- * This is a modified copy of Dropwizard's {@link com.codahale.metrics.JmxReporter} and classes that it internally uses,
+ * This is a modified copy of Dropwizard's {@link com.codahale.metrics.jmx.JmxReporter} and classes that it internally uses,
  * with a few important differences:
  * <ul>
  * <li>this class knows that it can directly use {@link MetricsMap} as a dynamic MBean.</li>
@@ -73,6 +72,9 @@ public class JmxMetricsReporter implements Reporter, Closeable {
     return new Builder(registry);
   }
 
+  /**
+   * Builder for the {@link JmxMetricsReporter} class.
+   */
   public static class Builder {
     private final MetricRegistry registry;
     private MBeanServer mBeanServer;
@@ -154,9 +156,6 @@ public class JmxMetricsReporter implements Reporter, Closeable {
     }
 
     public JmxMetricsReporter build() {
-      if (mBeanServer == null) {
-        mBeanServer = ManagementFactory.getPlatformMBeanServer();
-      }
       if (tag == null) {
         tag = Integer.toHexString(this.hashCode());
       }
@@ -523,9 +522,13 @@ public class JmxMetricsReporter implements Reporter, Closeable {
       if (mBeanServer.isRegistered(objectName)) {
         if (log.isDebugEnabled()) {
           Set<ObjectInstance> objects = mBeanServer.queryMBeans(objectName, null);
-          log.debug("## removing existing " + objects.size() + " bean(s) for " + objectName.getCanonicalName() + ", current tag=" + tag + ":");
+          if (log.isDebugEnabled()) {
+            log.debug("## removing existing {} bean(s) for {}, current tag={}:", objects.size(), objectName.getCanonicalName(), tag);
+          }
           for (ObjectInstance inst : objects) {
-            log.debug("## - tag=" + mBeanServer.getAttribute(inst.getObjectName(), INSTANCE_TAG));
+            if (log.isDebugEnabled()) {
+              log.debug("## - tag={}", mBeanServer.getAttribute(inst.getObjectName(), INSTANCE_TAG));
+            }
           }
         }
         mBeanServer.unregisterMBean(objectName);
@@ -539,7 +542,9 @@ public class JmxMetricsReporter implements Reporter, Closeable {
       } else {
         registered.put(objectName, objectName);
       }
-      log.debug("## registered " + objectInstance.getObjectName().getCanonicalName() + ", tag=" + tag);
+      if (log.isDebugEnabled()) {
+        log.debug("## registered {}, tag={}", objectInstance.getObjectName().getCanonicalName(), tag);
+      }
     }
 
     private void unregisterMBean(ObjectName originalObjectName) throws InstanceNotFoundException, MBeanRegistrationException {
@@ -549,7 +554,9 @@ public class JmxMetricsReporter implements Reporter, Closeable {
       }
       Set<ObjectInstance> objects = mBeanServer.queryMBeans(objectName, exp);
       for (ObjectInstance o : objects) {
-        log.debug("## Unregistered " + o.getObjectName().getCanonicalName() + ", tag=" + tag);
+        if (log.isDebugEnabled()) {
+          log.debug("## Unregistered {}, tag={}", o.getObjectName().getCanonicalName(), tag);
+        }
         mBeanServer.unregisterMBean(o.getObjectName());
       }
     }
@@ -560,8 +567,8 @@ public class JmxMetricsReporter implements Reporter, Closeable {
         if (filter.matches(name, gauge)) {
           final ObjectName objectName = createName("gauges", name);
           if (gauge instanceof SolrMetricManager.GaugeWrapper &&
-              ((SolrMetricManager.GaugeWrapper)gauge).getGauge() instanceof MetricsMap) {
-            MetricsMap mm = (MetricsMap)((SolrMetricManager.GaugeWrapper)gauge).getGauge();
+              ((SolrMetricManager.GaugeWrapper<?>)gauge).getGauge() instanceof MetricsMap) {
+            MetricsMap mm = (MetricsMap)((SolrMetricManager.GaugeWrapper<?>)gauge).getGauge();
             mm.setAttribute(new Attribute(INSTANCE_TAG, tag));
             // don't wrap it in a JmxGauge, it already supports all necessary JMX attributes
             registerMBean(mm, objectName);
@@ -739,9 +746,9 @@ public class JmxMetricsReporter implements Reporter, Closeable {
       } else if (v instanceof Timer) {
         listener.onTimerAdded(k, (Timer)v);
       } else if (v instanceof Gauge) {
-        listener.onGaugeAdded(k, (Gauge)v);
+        listener.onGaugeAdded(k, (Gauge<?>)v);
       } else {
-        log.warn("Unknown metric type " + v.getClass().getName() + " for metric '" + k + "', ignoring");
+        log.warn("Unknown metric type {} for metric '{}', ignoring", v.getClass().getName(), k);
       }
     });
   }
