@@ -23,6 +23,7 @@ import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.List;
 import java.util.Set;
+import java.util.stream.Collectors;
 
 import org.apache.lucene.document.DoublePoint;
 import org.apache.lucene.document.FloatPoint;
@@ -37,11 +38,10 @@ import org.apache.lucene.index.PostingsEnum;
 import org.apache.lucene.index.PrefixCodedTerms;
 import org.apache.lucene.index.ReaderUtil;
 import org.apache.lucene.index.Term;
-import org.apache.lucene.index.TermContext;
 import org.apache.lucene.index.TermState;
+import org.apache.lucene.index.TermStates;
 import org.apache.lucene.index.Terms;
 import org.apache.lucene.index.TermsEnum;
-import org.apache.lucene.search.BulkScorer;
 import org.apache.lucene.search.ConstantScoreQuery;
 import org.apache.lucene.search.ConstantScoreScorer;
 import org.apache.lucene.search.ConstantScoreWeight;
@@ -51,16 +51,19 @@ import org.apache.lucene.search.IndexSearcher;
 import org.apache.lucene.search.MatchAllDocsQuery;
 import org.apache.lucene.search.MatchNoDocsQuery;
 import org.apache.lucene.search.Query;
+import org.apache.lucene.search.QueryVisitor;
+import org.apache.lucene.search.ScoreMode;
 import org.apache.lucene.search.Scorer;
 import org.apache.lucene.search.Weight;
+import org.apache.lucene.util.Accountable;
 import org.apache.lucene.util.ArrayUtil;
-import org.apache.lucene.util.BitDocIdSet;
 import org.apache.lucene.util.BytesRef;
 import org.apache.lucene.util.BytesRefBuilder;
 import org.apache.lucene.util.BytesRefIterator;
 import org.apache.lucene.util.DocIdSetBuilder;
 import org.apache.lucene.util.FixedBitSet;
 import org.apache.lucene.util.FutureArrays;
+import org.apache.lucene.util.RamUsageEstimator;
 import org.apache.solr.common.params.SolrParams;
 import org.apache.solr.request.SolrQueryRequest;
 import org.apache.solr.schema.FieldType;
@@ -72,8 +75,7 @@ import org.apache.solr.schema.SchemaField;
  *  This allows graph traversals to skip traversing high frequency nodes which is often desirable from a performance standpoint.
  *
  *   Syntax: {!graphTerms f=field maxDocFreq=10000}term1,term2,term3
- **/
-
+ */
 public class GraphTermsQParserPlugin extends QParserPlugin {
   public static final String NAME = "graphTerms";
 
@@ -225,42 +227,29 @@ public class GraphTermsQParserPlugin extends QParserPlugin {
 
     @Override
     public String toString(String defaultField) {
-      StringBuilder builder = new StringBuilder();
-      boolean first = true;
-      for (Term term : this.queryTerms) {
-        if (!first) {
-          builder.append(',');
-        }
-        first = false;
-        builder.append(term.toString());
-      }
-
-      return builder.toString();
-    }
-
-    private class WeightOrDocIdSet {
-      final Weight weight;
-      final DocIdSet set;
-
-      WeightOrDocIdSet(DocIdSet bitset) {
-        this.set = bitset;
-        this.weight = null;
-      }
+      return Arrays.stream(this.queryTerms).map(Term::toString).collect(Collectors.joining(","));
     }
 
     @Override
-    public Weight createWeight(IndexSearcher searcher, boolean needsScores, float boost) throws IOException {
+    public void visit(QueryVisitor visitor) {
+      visitor.visitLeaf(this);
+    }
 
-      List<TermContext> finalContexts = new ArrayList();
-      List<Term> finalTerms = new ArrayList();
-      List<LeafReaderContext> contexts = searcher.getTopReaderContext().leaves();
-      TermContext[] termContexts = new TermContext[this.queryTerms.length];
-      collectTermContext(searcher.getIndexReader(), contexts, termContexts, this.queryTerms);
-      for(int i=0; i<termContexts.length; i++) {
-        TermContext termContext = termContexts[i];
-        if(termContext != null && termContext.docFreq() <= this.maxDocFreq) {
-          finalContexts.add(termContext);
-          finalTerms.add(queryTerms[i]);
+    @Override
+    public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost) throws IOException {
+
+      List<TermStates> finalContexts = new ArrayList<>();
+      List<Term> finalTerms = new ArrayList<>();
+      {
+        List<LeafReaderContext> contexts = searcher.getTopReaderContext().leaves();
+        TermStates[] termStates = new TermStates[this.queryTerms.length];
+        collectTermStates(searcher.getIndexReader(), contexts, termStates, this.queryTerms);
+        for(int i=0; i<termStates.length; i++) {
+          TermStates ts = termStates[i];
+          if(ts != null && ts.docFreq() <= this.maxDocFreq) {
+            finalContexts.add(ts);
+            finalTerms.add(queryTerms[i]);
+          }
         }
       }
 
@@ -274,61 +263,29 @@ public class GraphTermsQParserPlugin extends QParserPlugin {
           // order to protect highlighters
         }
 
-        private WeightOrDocIdSet rewrite(LeafReaderContext context) throws IOException {
+        @Override
+        public Scorer scorer(LeafReaderContext context) throws IOException {
           final LeafReader reader = context.reader();
           Terms terms = reader.terms(field);
-          if(terms == null) {
-            return new WeightOrDocIdSet(new BitDocIdSet(new FixedBitSet(reader.maxDoc()), 0));
+          if (terms == null) {
+            return null;
           }
           TermsEnum  termsEnum = terms.iterator();
           PostingsEnum docs = null;
           DocIdSetBuilder builder = new DocIdSetBuilder(reader.maxDoc(), terms);
           for (int i=0; i<finalContexts.size(); i++) {
-            TermContext termContext = finalContexts.get(i);
-            TermState termState = termContext.get(context.ord);
+            TermStates ts = finalContexts.get(i);
+            TermState termState = ts.get(context);
             if(termState != null) {
               Term term = finalTerms.get(i);
-              termsEnum.seekExact(term.bytes(), termContext.get(context.ord));
+              termsEnum.seekExact(term.bytes(), ts.get(context));
               docs = termsEnum.postings(docs, PostingsEnum.NONE);
               builder.add(docs);
             }
           }
-          return new WeightOrDocIdSet(builder.build());
-        }
-
-        private Scorer scorer(DocIdSet set) throws IOException {
-          if (set == null) {
-            return null;
-          }
-          final DocIdSetIterator disi = set.iterator();
-          if (disi == null) {
-            return null;
-          }
-          return new ConstantScoreScorer(this, score(), disi);
-        }
-
-        @Override
-        public BulkScorer bulkScorer(LeafReaderContext context) throws IOException {
-          final WeightOrDocIdSet weightOrBitSet = rewrite(context);
-          if (weightOrBitSet.weight != null) {
-            return weightOrBitSet.weight.bulkScorer(context);
-          } else {
-            final Scorer scorer = scorer(weightOrBitSet.set);
-            if (scorer == null) {
-              return null;
-            }
-            return new DefaultBulkScorer(scorer);
-          }
-        }
-
-        @Override
-        public Scorer scorer(LeafReaderContext context) throws IOException {
-          final WeightOrDocIdSet weightOrBitSet = rewrite(context);
-          if (weightOrBitSet.weight != null) {
-            return weightOrBitSet.weight.scorer(context);
-          } else {
-            return scorer(weightOrBitSet.set);
-          }
+          DocIdSet docIdSet = builder.build();
+          DocIdSetIterator disi = docIdSet.iterator();
+          return disi == null ? null : new ConstantScoreScorer(this, score(), scoreMode, disi);
         }
 
         @Override
@@ -339,10 +296,10 @@ public class GraphTermsQParserPlugin extends QParserPlugin {
       };
     }
 
-    private void collectTermContext(IndexReader reader,
-                                    List<LeafReaderContext> leaves,
-                                    TermContext[] contextArray,
-                                    Term[] queryTerms) throws IOException {
+    private void collectTermStates(IndexReader reader,
+                                   List<LeafReaderContext> leaves,
+                                   TermStates[] contextArray,
+                                   Term[] queryTerms) throws IOException {
       TermsEnum termsEnum = null;
       for (LeafReaderContext context : leaves) {
 
@@ -358,15 +315,15 @@ public class GraphTermsQParserPlugin extends QParserPlugin {
 
         for (int i = 0; i < queryTerms.length; i++) {
           Term term = queryTerms[i];
-          TermContext termContext = contextArray[i];
+          TermStates termStates = contextArray[i];
 
           if (termsEnum.seekExact(term.bytes())) {
-            if (termContext == null) {
-              contextArray[i] = new TermContext(reader.getContext(),
+            if (termStates == null) {
+              contextArray[i] = new TermStates(reader.getContext(),
                   termsEnum.termState(), context.ord, termsEnum.docFreq(),
                   termsEnum.totalTermFreq());
             } else {
-              termContext.register(termsEnum.termState(), context.ord,
+              termStates.register(termsEnum.termState(), context.ord,
                   termsEnum.docFreq(), termsEnum.totalTermFreq());
             }
           }
@@ -379,7 +336,9 @@ public class GraphTermsQParserPlugin extends QParserPlugin {
 
 
 // modified version of PointInSetQuery
-abstract class PointSetQuery extends Query implements DocSetProducer {
+abstract class PointSetQuery extends Query implements DocSetProducer, Accountable {
+  protected static final long BASE_RAM_BYTES = RamUsageEstimator.shallowSizeOfInstance(PointSetQuery.class);
+
   // A little bit overkill for us, since all of our "terms" are always in the same field:
   final PrefixCodedTerms sortedPackedPoints;
   final int sortedPackedPointsHashCode;
@@ -387,6 +346,7 @@ abstract class PointSetQuery extends Query implements DocSetProducer {
   final int bytesPerDim;
   final int numDims;
   int maxDocFreq = Integer.MAX_VALUE;
+  final long ramBytesUsed; // cache
 
   /**
    * Iterator of encoded point values.
@@ -544,6 +504,8 @@ abstract class PointSetQuery extends Query implements DocSetProducer {
     }
     sortedPackedPoints = builder.finish();
     sortedPackedPointsHashCode = sortedPackedPoints.hashCode();
+    ramBytesUsed = BASE_RAM_BYTES +
+        RamUsageEstimator.sizeOfObject(sortedPackedPoints);
   }
 
   private FixedBitSet getLiveDocs(IndexSearcher searcher) throws IOException {
@@ -566,6 +528,11 @@ abstract class PointSetQuery extends Query implements DocSetProducer {
     return getDocSet(searcher);
   }
 
+  @Override
+  public long ramBytesUsed() {
+    return ramBytesUsed;
+  }
+
   public DocSet getDocSet(IndexSearcher searcher) throws IOException {
     IndexReaderContext top = ReaderUtil.getTopLevelContext(searcher.getTopReaderContext());
     List<LeafReaderContext> segs = top.leaves();
@@ -603,7 +570,7 @@ abstract class PointSetQuery extends Query implements DocSetProducer {
 
 
   @Override
-  public final Weight createWeight(IndexSearcher searcher, boolean needsScores, float boost) throws IOException {
+  public final Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost) throws IOException {
     return new ConstantScoreWeight(this, boost) {
       Filter filter;
 
@@ -623,7 +590,7 @@ abstract class PointSetQuery extends Query implements DocSetProducer {
         if (readerSetIterator == null) {
           return null;
         }
-        return new ConstantScoreScorer(this, score(), readerSetIterator);
+        return new ConstantScoreScorer(this, score(), scoreMode, readerSetIterator);
       }
 
       @Override
@@ -780,5 +747,10 @@ abstract class PointSetQuery extends Query implements DocSetProducer {
     return sb.toString();
   }
 
+  @Override
+  public void visit(QueryVisitor visitor) {
+    visitor.visitLeaf(this);
+  }
+
   protected abstract String toString(byte[] value);
 }
