@@ -26,16 +26,16 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.commons.lang.ArrayUtils;
+import org.apache.commons.collections4.CollectionUtils;
+import org.apache.commons.lang3.ArrayUtils;
 import org.apache.lucene.index.ExitableDirectoryReader;
 import org.apache.lucene.index.IndexableField;
 import org.apache.lucene.queries.function.FunctionQuery;
 import org.apache.lucene.queries.function.ValueSource;
 import org.apache.lucene.queries.function.valuesource.QueryValueSource;
-import org.apache.lucene.search.BooleanClause.Occur;
-import org.apache.lucene.search.BooleanQuery;
 import org.apache.lucene.search.CachingCollector;
 import org.apache.lucene.search.Collector;
+import org.apache.lucene.search.MatchNoDocsQuery;
 import org.apache.lucene.search.MultiCollector;
 import org.apache.lucene.search.Query;
 import org.apache.lucene.search.ScoreDoc;
@@ -46,6 +46,7 @@ import org.apache.lucene.search.TopDocsCollector;
 import org.apache.lucene.search.TopFieldCollector;
 import org.apache.lucene.search.TopScoreDocCollector;
 import org.apache.lucene.search.TotalHitCountCollector;
+import org.apache.lucene.search.TotalHits;
 import org.apache.lucene.search.grouping.AllGroupHeadsCollector;
 import org.apache.lucene.search.grouping.AllGroupsCollector;
 import org.apache.lucene.search.grouping.FirstPassGroupingCollector;
@@ -58,6 +59,7 @@ import org.apache.lucene.search.grouping.ValueSourceGroupSelector;
 import org.apache.lucene.util.BytesRef;
 import org.apache.lucene.util.mutable.MutableValue;
 import org.apache.solr.common.SolrException;
+import org.apache.solr.common.SolrException.ErrorCode;
 import org.apache.solr.common.util.NamedList;
 import org.apache.solr.common.util.SimpleOrderedMap;
 import org.apache.solr.request.SolrQueryRequest;
@@ -81,6 +83,7 @@ public class Grouping {
   private final SolrIndexSearcher searcher;
   private final QueryResult qr;
   private final QueryCommand cmd;
+  @SuppressWarnings({"rawtypes"})
   private final List<Command> commands = new ArrayList<>();
   private final boolean main;
   private final boolean cacheSecondPassSearch;
@@ -102,7 +105,7 @@ public class Grouping {
   private Query query;
   private DocSet filter;
   private Filter luceneFilter;
-  private NamedList grouped = new SimpleOrderedMap();
+  private NamedList<Object> grouped = new SimpleOrderedMap<>();
   private Set<Integer> idSet = new LinkedHashSet<>();  // used for tracking unique docs when we need a doclist
   private int maxMatches;  // max number of matches from any grouping command
   private float maxScore = Float.NaN;  // max score seen in any doclist
@@ -133,7 +136,7 @@ public class Grouping {
     this.main = main;
   }
 
-  public void add(Grouping.Command groupingCommand) {
+  public void add(@SuppressWarnings({"rawtypes"})Grouping.Command groupingCommand) {
     commands.add(groupingCommand);
   }
 
@@ -179,6 +182,7 @@ public class Grouping {
   public void addFunctionCommand(String groupByStr, SolrQueryRequest request) throws SyntaxError {
     QParser parser = QParser.getParser(groupByStr, FunctionQParserPlugin.NAME, request);
     Query q = parser.getQuery();
+    @SuppressWarnings({"rawtypes"})
     final Grouping.Command gc;
     if (q instanceof FunctionQuery) {
       ValueSource valueSource = ((FunctionQuery) q).getValueSource();
@@ -222,6 +226,11 @@ public class Grouping {
   public void addQueryCommand(String groupByStr, SolrQueryRequest request) throws SyntaxError {
     QParser parser = QParser.getParser(groupByStr, request);
     Query gq = parser.getQuery();
+
+    if (gq == null) {
+      // normalize a null query to a query that matches nothing
+      gq = new MatchNoDocsQuery();
+    }
     Grouping.CommandQuery gc = new CommandQuery();
     gc.query = gq;
     gc.withinGroupSort = withinGroupSort;
@@ -243,7 +252,6 @@ public class Grouping {
       gc.docsPerGroup = gc.numGroups;  // doesn't make sense to limit to one
       gc.groupOffset = gc.offset;
     }
-
     commands.add(gc);
   }
 
@@ -287,6 +295,7 @@ public class Grouping {
     return this;
   }
 
+  @SuppressWarnings({"rawtypes"})
   public List<Command> getCommands() {
     return commands;
   }
@@ -316,13 +325,13 @@ public class Grouping {
     getDocList = (cmd.getFlags() & SolrIndexSearcher.GET_DOCLIST) != 0;
     query = QueryUtils.makeQueryable(cmd.getQuery());
 
-    for (Command cmd : commands) {
+    for (@SuppressWarnings({"rawtypes"})Command cmd : commands) {
       cmd.prepare();
     }
 
     AllGroupHeadsCollector<?> allGroupHeadsCollector = null;
     List<Collector> collectors = new ArrayList<>(commands.size());
-    for (Command cmd : commands) {
+    for (@SuppressWarnings({"rawtypes"})Command cmd : commands) {
       Collector collector = cmd.createFirstPassCollector();
       if (collector != null) {
         collectors.add(collector);
@@ -369,7 +378,7 @@ public class Grouping {
     }
 
     collectors.clear();
-    for (Command cmd : commands) {
+    for (@SuppressWarnings({"rawtypes"})Command cmd : commands) {
       Collector collector = cmd.createSecondPassCollector();
       if (collector != null)
         collectors.add(collector);
@@ -400,7 +409,7 @@ public class Grouping {
       }
     }
 
-    for (Command cmd : commands) {
+    for (@SuppressWarnings({"rawtypes"})Command cmd : commands) {
       cmd.finish();
     }
 
@@ -413,7 +422,7 @@ public class Grouping {
       for (int val : idSet) {
         ids[idx++] = val;
       }
-      qr.setDocList(new DocSlice(0, sz, ids, null, maxMatches, maxScore));
+      qr.setDocList(new DocSlice(0, sz, ids, null, maxMatches, maxScore, TotalHits.Relation.EQUAL_TO));
     }
   }
 
@@ -437,16 +446,9 @@ public class Grouping {
       collector = timeLimitingCollector;
     }
     try {
-      Query q = query;
-      if (luceneFilter != null) {
-        q = new BooleanQuery.Builder()
-            .add(q, Occur.MUST)
-            .add(luceneFilter, Occur.FILTER)
-            .build();
-      }
-      searcher.search(q, collector);
+      searcher.search(QueryUtils.combineQueryAndFilter(query, luceneFilter), collector);
     } catch (TimeLimitingCollector.TimeExceededException | ExitableDirectoryReader.ExitingReaderException x) {
-      log.warn( "Query: " + query + "; " + x.getMessage() );
+      log.warn("Query: {}; ", query, x);
       qr.setPartialResults(true);
     }
   }
@@ -590,6 +592,15 @@ public class Grouping {
       return null;
     }
 
+    protected void populateScoresIfNecessary() throws IOException {
+      if (needScores) {
+        for (GroupDocs<?> groups : result.groups) {
+          TopFieldCollector.populateScores(groups.scoreDocs, searcher, query);
+        }
+      }
+    }
+
+    @SuppressWarnings({"unchecked", "rawtypes"})
     protected NamedList commonResponse() {
       NamedList groupResult = new SimpleOrderedMap();
       grouped.add(key, groupResult);  // grouped={ key={
@@ -604,8 +615,9 @@ public class Grouping {
       return groupResult;
     }
 
-    protected DocList getDocList(GroupDocs groups) {
-      int max = Math.toIntExact(groups.totalHits);
+    protected DocList getDocList(@SuppressWarnings({"rawtypes"})GroupDocs groups) {
+      assert groups.totalHits.relation == TotalHits.Relation.EQUAL_TO;
+      int max = Math.toIntExact(groups.totalHits.value);
       int off = groupOffset;
       int len = docsPerGroup;
       if (format == Format.simple) {
@@ -627,7 +639,7 @@ public class Grouping {
 
       float score = groups.maxScore;
       maxScore = maxAvoidNaN(score, maxScore);
-      DocSlice docs = new DocSlice(off, Math.max(0, ids.length - off), ids, scores, groups.totalHits, score);
+      DocSlice docs = new DocSlice(off, Math.max(0, ids.length - off), ids, scores, groups.totalHits.value, score, TotalHits.Relation.EQUAL_TO);
 
       if (getDocList) {
         DocIterator iter = docs.iterator();
@@ -637,12 +649,15 @@ public class Grouping {
       return docs;
     }
 
-    protected void addDocList(NamedList rsp, GroupDocs groups) {
+    @SuppressWarnings({"unchecked"})
+    protected void addDocList(@SuppressWarnings({"rawtypes"})NamedList rsp
+            , @SuppressWarnings({"rawtypes"})GroupDocs groups) {
       rsp.add("doclist", getDocList(groups));
     }
 
     // Flatten the groups and get up offset + rows documents
     protected DocList createSimpleResponse() {
+      @SuppressWarnings({"rawtypes"})
       GroupDocs[] groups = result != null ? result.groups : new GroupDocs[0];
 
       List<Integer> ids = new ArrayList<>();
@@ -652,7 +667,7 @@ public class Grouping {
       float maxScore = Float.NaN;
 
       outer:
-      for (GroupDocs group : groups) {
+      for (@SuppressWarnings({"rawtypes"})GroupDocs group : groups) {
         maxScore = maxAvoidNaN(maxScore, group.maxScore);
 
         for (ScoreDoc scoreDoc : group.scoreDocs) {
@@ -669,7 +684,7 @@ public class Grouping {
       int len = docsGathered > offset ? docsGathered - offset : 0;
       int[] docs = ArrayUtils.toPrimitive(ids.toArray(new Integer[ids.size()]));
       float[] docScores = ArrayUtils.toPrimitive(scores.toArray(new Float[scores.size()]));
-      DocSlice docSlice = new DocSlice(offset, len, docs, docScores, getMatches(), maxScore);
+      DocSlice docSlice = new DocSlice(offset, len, docs, docScores, getMatches(), maxScore, TotalHits.Relation.EQUAL_TO);
 
       if (getDocList) {
         for (int i = offset; i < docs.length; i++) {
@@ -731,7 +746,7 @@ public class Grouping {
         return totalCount == TotalCount.grouped ? allGroupsCollector : null;
       }
 
-      topGroups = format == Format.grouped ? firstPass.getTopGroups(offset, false) : firstPass.getTopGroups(0, false);
+      topGroups = format == Format.grouped ? firstPass.getTopGroups(offset) : firstPass.getTopGroups(0);
       if (topGroups == null) {
         if (totalCount == TotalCount.grouped) {
           allGroupsCollector = new AllGroupsCollector<>(new TermGroupSelector(groupBy));
@@ -747,7 +762,7 @@ public class Grouping {
       groupedDocsToCollect = Math.max(groupedDocsToCollect, 1);
       Sort withinGroupSort = this.withinGroupSort != null ? this.withinGroupSort : Sort.RELEVANCE;
       secondPass = new TopGroupsCollector<>(new TermGroupSelector(groupBy),
-          topGroups, groupSort, withinGroupSort, groupedDocsToCollect, needScores, needScores, false
+          topGroups, groupSort, withinGroupSort, groupedDocsToCollect, needScores
       );
 
       if (totalCount == TotalCount.grouped) {
@@ -765,13 +780,18 @@ public class Grouping {
     }
 
     @Override
+    @SuppressWarnings({"unchecked"})
     protected void finish() throws IOException {
-      result = secondPass != null ? secondPass.getTopGroups(0) : null;
+      if (secondPass != null) {
+        result = secondPass.getTopGroups(0);
+        populateScoresIfNecessary();
+      }
       if (main) {
         mainResult = createSimpleResponse();
         return;
       }
 
+      @SuppressWarnings({"rawtypes"})
       NamedList groupResult = commonResponse();
 
       if (format == Format.simple) {
@@ -779,6 +799,7 @@ public class Grouping {
         return;
       }
 
+      @SuppressWarnings({"rawtypes"})
       List groupList = new ArrayList();
       groupResult.add("groups", groupList);        // grouped={ key={ groups=[
 
@@ -790,6 +811,7 @@ public class Grouping {
       if (numGroups == 0) return;
 
       for (GroupDocs<BytesRef> group : result.groups) {
+        @SuppressWarnings({"rawtypes"})
         NamedList nl = new SimpleOrderedMap();
         groupList.add(nl);                         // grouped={ key={ groups=[ {
 
@@ -800,9 +822,17 @@ public class Grouping {
         if (group.groupValue != null) {
           SchemaField schemaField = searcher.getSchema().getField(groupBy);
           FieldType fieldType = schemaField.getType();
-          String readableValue = fieldType.indexedToReadable(group.groupValue.utf8ToString());
-          IndexableField field = schemaField.createField(readableValue);
-          nl.add("groupValue", fieldType.toObject(field));
+          // use createFields so that fields having doc values are also supported
+          // TODO: currently, this path is called only for string field, so
+          // should we just use fieldType.toObject(schemaField, group.groupValue) here?
+          List<IndexableField> fields = schemaField.createFields(group.groupValue.utf8ToString());
+          if (CollectionUtils.isNotEmpty(fields)) {
+            nl.add("groupValue", fieldType.toObject(fields.get(0)));
+          } else {
+            throw new SolrException(ErrorCode.INVALID_STATE,
+                "Couldn't create schema field for grouping, group value: " + group.groupValue.utf8ToString()
+                + ", field: " + schemaField);
+          }
         } else {
           nl.add("groupValue", null);
         }
@@ -830,10 +860,12 @@ public class Grouping {
    * A group command for grouping on a query.
    */
   //NOTE: doesn't need to be generic. Maybe Command interface --> First / Second pass abstract impl.
+  @SuppressWarnings({"rawtypes"})
   public class CommandQuery extends Command {
 
     public Query query;
     TopDocsCollector topCollector;
+    MaxScoreCollector maxScoreCollector;
     FilterCollector collector;
 
     @Override
@@ -844,25 +876,38 @@ public class Grouping {
     @Override
     protected Collector createFirstPassCollector() throws IOException {
       DocSet groupFilt = searcher.getDocSet(query);
-      topCollector = newCollector(withinGroupSort, needScores);
-      collector = new FilterCollector(groupFilt, topCollector);
-      return collector;
-    }
-
-    TopDocsCollector newCollector(Sort sort, boolean needScores) throws IOException {
       int groupDocsToCollect = getMax(groupOffset, docsPerGroup, maxDoc);
-      if (sort == null || sort.equals(Sort.RELEVANCE)) {
-        return TopScoreDocCollector.create(groupDocsToCollect);
+      Collector subCollector;
+      if (withinGroupSort == null || withinGroupSort.equals(Sort.RELEVANCE)) {
+        subCollector = topCollector = TopScoreDocCollector.create(groupDocsToCollect, Integer.MAX_VALUE);
       } else {
-        return TopFieldCollector.create(searcher.weightSort(sort), groupDocsToCollect, false, needScores, needScores, true);
+        topCollector = TopFieldCollector.create(searcher.weightSort(withinGroupSort), groupDocsToCollect, Integer.MAX_VALUE);
+        if (needScores) {
+          maxScoreCollector = new MaxScoreCollector();
+          subCollector = MultiCollector.wrap(topCollector, maxScoreCollector);
+        } else {
+          subCollector = topCollector;
+        }
       }
+      collector = new FilterCollector(groupFilt, subCollector);
+      return collector;
     }
 
     @Override
     protected void finish() throws IOException {
-      TopDocsCollector topDocsCollector = (TopDocsCollector) collector.getDelegate();
-      TopDocs topDocs = topDocsCollector.topDocs();
-      GroupDocs<String> groupDocs = new GroupDocs<>(Float.NaN, topDocs.getMaxScore(), topDocs.totalHits, topDocs.scoreDocs, query.toString(), null);
+      TopDocs topDocs = topCollector.topDocs();
+      float maxScore;
+      if (withinGroupSort == null || withinGroupSort.equals(Sort.RELEVANCE)) {
+        maxScore = topDocs.scoreDocs.length == 0 ? Float.NaN : topDocs.scoreDocs[0].score;
+      } else if (needScores) {
+        // use top-level query to populate the scores
+        TopFieldCollector.populateScores(topDocs.scoreDocs, searcher, Grouping.this.query);
+        maxScore = maxScoreCollector.getMaxScore();
+      } else {
+        maxScore = Float.NaN;
+      }
+      
+      GroupDocs<String> groupDocs = new GroupDocs<>(Float.NaN, maxScore, topDocs.totalHits, topDocs.scoreDocs, query.toString(), null);
       if (main) {
         mainResult = getDocList(groupDocs);
       } else {
@@ -883,8 +928,10 @@ public class Grouping {
   public class CommandFunc extends Command<MutableValue> {
 
     public ValueSource groupBy;
+    @SuppressWarnings({"rawtypes"})
     Map context;
 
+    @SuppressWarnings({"unchecked"})
     private ValueSourceGroupSelector newSelector() {
       return new ValueSourceGroupSelector(groupBy, context);
     }
@@ -897,6 +944,7 @@ public class Grouping {
     Collection<SearchGroup<MutableValue>> topGroups;
 
     @Override
+    @SuppressWarnings({"unchecked"})
     protected void prepare() throws IOException {
       context = ValueSource.newContext(searcher);
       groupBy.createWeight(context, searcher);
@@ -923,7 +971,7 @@ public class Grouping {
         return totalCount == TotalCount.grouped ? allGroupsCollector : null;
       }
 
-      topGroups = format == Format.grouped ? firstPass.getTopGroups(offset, false) : firstPass.getTopGroups(0, false);
+      topGroups = format == Format.grouped ? firstPass.getTopGroups(offset) : firstPass.getTopGroups(0);
       if (topGroups == null) {
         if (totalCount == TotalCount.grouped) {
           allGroupsCollector = new AllGroupsCollector<>(newSelector());
@@ -939,7 +987,7 @@ public class Grouping {
       groupdDocsToCollect = Math.max(groupdDocsToCollect, 1);
       Sort withinGroupSort = this.withinGroupSort != null ? this.withinGroupSort : Sort.RELEVANCE;
       secondPass = new TopGroupsCollector<>(newSelector(),
-          topGroups, groupSort, withinGroupSort, groupdDocsToCollect, needScores, needScores, false
+          topGroups, groupSort, withinGroupSort, groupdDocsToCollect, needScores
       );
 
       if (totalCount == TotalCount.grouped) {
@@ -957,13 +1005,18 @@ public class Grouping {
     }
 
     @Override
+    @SuppressWarnings({"unchecked"})
     protected void finish() throws IOException {
-      result = secondPass != null ? secondPass.getTopGroups(0) : null;
+      if (secondPass != null) {
+        result = secondPass.getTopGroups(0);
+        populateScoresIfNecessary();
+      }
       if (main) {
         mainResult = createSimpleResponse();
         return;
       }
 
+      @SuppressWarnings({"rawtypes"})
       NamedList groupResult = commonResponse();
 
       if (format == Format.simple) {
@@ -971,6 +1024,7 @@ public class Grouping {
         return;
       }
 
+      @SuppressWarnings({"rawtypes"})
       List groupList = new ArrayList();
       groupResult.add("groups", groupList);        // grouped={ key={ groups=[
 
@@ -982,6 +1036,7 @@ public class Grouping {
       if (numGroups == 0) return;
 
       for (GroupDocs<MutableValue> group : result.groups) {
+        @SuppressWarnings({"rawtypes"})
         NamedList nl = new SimpleOrderedMap();
         groupList.add(nl);                         // grouped={ key={ groups=[ {
         nl.add("groupValue", group.groupValue.toObject());
