@@ -33,6 +33,7 @@ import org.apache.lucene.search.Query;
 import org.apache.solr.common.SolrException;
 import org.apache.solr.common.util.SimpleOrderedMap;
 import org.apache.solr.handler.component.ResponseBuilder;
+import org.apache.solr.request.SolrQueryRequest;
 import org.apache.solr.request.SolrRequestInfo;
 import org.apache.solr.schema.SchemaField;
 import org.apache.solr.search.BitDocSet;
@@ -52,7 +53,7 @@ public abstract class FacetProcessor<FacetRequestT extends FacetRequest>  {
   DocSet filter;  // additional filters specified by "filter"  // TODO: do these need to be on the context to support recomputing during multi-select?
   LinkedHashMap<String,SlotAcc> accMap;
   SlotAcc[] accs;
-  CountSlotAcc countAcc;
+  SlotAcc.CountSlotAcc countAcc;
 
   FacetProcessor(FacetContext fcontext, FacetRequestT freq) {
     this.fcontext = fcontext;
@@ -60,7 +61,7 @@ public abstract class FacetProcessor<FacetRequestT extends FacetRequest>  {
     fcontext.processor = this;
   }
 
-  public Object getResponse() {
+  public org.apache.solr.common.MapWriter getResponse() {
     return response;
   }
 
@@ -78,17 +79,10 @@ public abstract class FacetProcessor<FacetRequestT extends FacetRequest>  {
     // TODO: prevent parsing filters each time!
     for (Object rawFilter : filters) {
       if (rawFilter instanceof String) {
-        QParser parser = null;
-        try {
-          parser = QParser.getParser((String)rawFilter, fcontext.req);
-          parser.setIsFilter(true);
-          Query symbolicFilter = parser.getQuery();
-          qlist.add(symbolicFilter);
-        } catch (SyntaxError syntaxError) {
-          throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, syntaxError);
-        }
+        qlist.add(parserFilter((String) rawFilter, fcontext.req));
       } else if (rawFilter instanceof Map) {
 
+        @SuppressWarnings({"unchecked"})
         Map<String,Object> m = (Map<String, Object>) rawFilter;
         String type;
         Object args;
@@ -113,17 +107,11 @@ public abstract class FacetProcessor<FacetRequestT extends FacetRequest>  {
 
         String[] qstrings = fcontext.req.getParams().getParams(tag);
 
+        // idea is to support multivalued parameter ie, 0 or more values
+        // so, when value not specified, it is ignored rather than throwing exception
         if (qstrings != null) {
           for (String qstring : qstrings) {
-            QParser parser = null;
-            try {
-              parser = QParser.getParser((String) qstring, fcontext.req);
-              parser.setIsFilter(true);
-              Query symbolicFilter = parser.getQuery();
-              qlist.add(symbolicFilter);
-            } catch (SyntaxError syntaxError) {
-              throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, syntaxError);
-            }
+            qlist.add(parserFilter(qstring, fcontext.req));
           }
         }
 
@@ -135,6 +123,22 @@ public abstract class FacetProcessor<FacetRequestT extends FacetRequest>  {
     return qlist;
   }
 
+  private static Query parserFilter(String rawFilter, SolrQueryRequest req) {
+    QParser parser = null;
+    try {
+      parser = QParser.getParser(rawFilter, req);
+      parser.setIsFilter(true);
+      Query symbolicFilter = parser.getQuery();
+      if (symbolicFilter == null) {
+        throw new SolrException(SolrException.ErrorCode.BAD_REQUEST,
+            "QParser yields null, perhaps unresolved parameter reference in: "+rawFilter);
+      }
+      return symbolicFilter;
+    } catch (SyntaxError syntaxError) {
+      throw new SolrException(SolrException.ErrorCode.BAD_REQUEST, syntaxError);
+    }
+  }
+
   private void handleDomainChanges() throws IOException {
     if (freq.domain == null) return;
 
@@ -178,6 +182,7 @@ public abstract class FacetProcessor<FacetRequestT extends FacetRequest>  {
       return;
     }
 
+    @SuppressWarnings({"rawtypes"})
     Map tagMap = (Map) fcontext.req.getContext().get("tags");
     if (tagMap == null) {
       // no filters were tagged
@@ -304,8 +309,7 @@ public abstract class FacetProcessor<FacetRequestT extends FacetRequest>  {
 
     // allow a custom count acc to be used
     if (countAcc == null) {
-      countAcc = new CountSlotArrAcc(fcontext, slotCount);
-      countAcc.key = "count";
+      countAcc = new SlotAcc.CountSlotArrAcc(fcontext, slotCount);
     }
 
     for (Map.Entry<String,AggValueSource> entry : freq.getFacetStats().entrySet()) {
@@ -435,6 +439,7 @@ public abstract class FacetProcessor<FacetRequestT extends FacetRequest>  {
     }
   }
 
+  @SuppressWarnings({"unchecked"})
   void processSubs(SimpleOrderedMap<Object> response, Query filter, DocSet domain, boolean skip, Map<String,Object> facetInfo) throws IOException {
 
     boolean emptyDomain = domain == null || domain.size() == 0;
