@@ -57,6 +57,7 @@ import org.slf4j.LoggerFactory;
  *  This would also allow to not log document data for requests with commit=true
  *  in them (since we know that if the request succeeds, all docs will be committed)
  *
+ * @deprecated since 8.6; see <a href="https://cwiki.apache.org/confluence/display/SOLR/Deprecations">Deprecations</a>
  */
 public class HdfsTransactionLog extends TransactionLog {
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
@@ -82,9 +83,6 @@ public class HdfsTransactionLog extends TransactionLog {
     this.fs = fs;
 
     try {
-      if (debug) {
-        //log.debug("New TransactionLog file=" + tlogFile + ", exists=" + tlogFile.exists() + ", size=" + tlogFile.length() + ", openExisting=" + openExisting);
-      }
       this.tlogFile = tlogFile;
       
       if (fs.exists(tlogFile) && openExisting) {
@@ -121,7 +119,7 @@ public class HdfsTransactionLog extends TransactionLog {
         }
       } else {
         if (start > 0) {
-          log.error("New transaction log already exists:" + tlogFile + " size=" + tlogOutStream.size());
+          log.error("New transaction log already exists:{} size={}", tlogFile, tlogOutStream.size());
         }
 
         addGlobalStrings(globalStrings);
@@ -167,6 +165,7 @@ public class HdfsTransactionLog extends TransactionLog {
     return true;
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private void readHeader(FastInputStream fis) throws IOException {
     // read existing header
     boolean closeFis = false;
@@ -270,7 +269,7 @@ public class HdfsTransactionLog extends TransactionLog {
     synchronized (this) {
       if (fos == null) return;
       if (debug) {
-        log.debug("Closing output for " + tlogFile);
+        log.debug("Closing output for {}", tlogFile);
       }
       fos.flushBuffer();
       finalLogSize = fos.size();
@@ -405,7 +404,7 @@ public class HdfsTransactionLog extends TransactionLog {
 
       synchronized (HdfsTransactionLog.this) {
         if (trace) {
-          log.trace("Reading log record.  pos="+pos+" currentSize="+getLogSize());
+          log.trace("Reading log record.  pos={} currentSize={}", pos, getLogSize());
         }
 
         if (pos >= getLogSize()) {
@@ -491,6 +490,7 @@ public class HdfsTransactionLog extends TransactionLog {
 
         long lastVersion = Long.MIN_VALUE;
         while ( (o = super.next()) != null) {
+          @SuppressWarnings({"rawtypes"})
           List entry = (List) o;
           long version = (Long) entry.get(UpdateLog.VERSION_IDX);
           version = Math.abs(version);
