@@ -18,15 +18,12 @@ package org.apache.solr.update;
 
 import java.io.IOException;
 import java.lang.invoke.MethodHandles;
-import java.net.ConnectException;
 import java.net.SocketException;
+import java.net.SocketTimeoutException;
 import java.util.ArrayList;
 import java.util.Comparator;
-import java.util.HashSet;
 import java.util.List;
 import java.util.Optional;
-import java.util.Set;
-import java.util.function.Supplier;
 import java.util.stream.Collectors;
 
 import com.codahale.metrics.Counter;
@@ -41,14 +38,12 @@ import org.apache.solr.common.SolrException;
 import org.apache.solr.common.SolrInputDocument;
 import org.apache.solr.common.params.ModifiableSolrParams;
 import org.apache.solr.common.util.IOUtils;
-import org.apache.solr.common.util.StrUtils;
 import org.apache.solr.core.SolrCore;
 import org.apache.solr.core.SolrInfoBean;
 import org.apache.solr.handler.component.HttpShardHandlerFactory;
 import org.apache.solr.handler.component.ShardHandler;
 import org.apache.solr.handler.component.ShardRequest;
 import org.apache.solr.handler.component.ShardResponse;
-import org.apache.solr.logging.MDCLoggingContext;
 import org.apache.solr.metrics.SolrMetricManager;
 import org.apache.solr.metrics.SolrMetricProducer;
 import org.apache.solr.request.LocalSolrQueryRequest;
@@ -166,10 +161,11 @@ public class PeerSync implements SolrMetricProducer {
       syncErrors.inc();
       return PeerSyncResult.failure();
     }
-    MDCLoggingContext.setCore(core);
     Timer.Context timerContext = null;
     try {
-      log.info(msg() + "START replicas=" + replicas + " nUpdates=" + nUpdates);
+      if (log.isInfoEnabled()) {
+        log.info("{} START replicas={} nUpdates={}", msg(), replicas, nUpdates);
+      }
 
       // check if we already in sync to begin with 
       if(doFingerprint && alreadyInSync()) {
@@ -202,11 +198,14 @@ public class PeerSync implements SolrMetricProducer {
       } else {
         // we have no versions and hence no frame of reference to tell if we can use a peers
         // updates to bring us into sync
-        log.info(msg() + "DONE.  We have no versions.  sync failed.");
+        if (log.isInfoEnabled()) {
+          log.info("{} DONE. We have no versions. sync failed.", msg());
+        }
         for (;;)  {
           ShardResponse srsp = shardHandler.takeCompletedOrError();
           if (srsp == null) break;
           if (srsp.getException() == null)  {
+            @SuppressWarnings({"unchecked"})
             List<Long> otherVersions = (List<Long>)srsp.getSolrResponse().getResponse().get("versions");
             if (otherVersions != null && !otherVersions.isEmpty())  {
               syncErrors.inc();
@@ -225,7 +224,9 @@ public class PeerSync implements SolrMetricProducer {
         if (srsp == null) break;
         boolean success = handleResponse(srsp);
         if (!success) {
-          log.info(msg() + "DONE. sync failed");
+          if (log.isInfoEnabled()) {
+            log.info("{} DONE. sync failed", msg());
+          }
           shardHandler.cancelAll();
           syncErrors.inc();
           return PeerSyncResult.failure();
@@ -241,7 +242,9 @@ public class PeerSync implements SolrMetricProducer {
         }
       }
 
-      log.info(msg() + "DONE. sync " + (success ? "succeeded" : "failed"));
+      if (log.isInfoEnabled()) {
+        log.info("{} DONE. sync {}", msg(), (success ? "succeeded" : "failed"));
+      }
       if (!success) {
         syncErrors.inc();
       }
@@ -250,7 +253,6 @@ public class PeerSync implements SolrMetricProducer {
       if (timerContext != null) {
         timerContext.close();
       }
-      MDCLoggingContext.clear();
     }
   }
 
@@ -261,7 +263,18 @@ public class PeerSync implements SolrMetricProducer {
     for (String replica : replicas) {
       requestFingerprint(replica);
     }
-    
+
+    // We only compute fingerprint during leader election. Therefore after heavy indexing,
+    // the call to compute fingerprint takes awhile and slows the leader election.
+    // So we do it in parallel with fetching the fingerprint from the other replicas
+    IndexFingerprint ourFingerprint;
+    try {
+      ourFingerprint = IndexFingerprint.getFingerprint(core, Long.MAX_VALUE);
+    } catch (IOException e) {
+      log.warn("Could not confirm if we are already in sync. Continue with PeerSync");
+      return false;
+    }
+
     for (;;) {
       ShardResponse srsp = shardHandler.takeCompletedOrError();
       if (srsp == null) break;
@@ -275,19 +288,14 @@ public class PeerSync implements SolrMetricProducer {
         log.warn("Replica did not return a fingerprint - possibly an older Solr version or exception");
         continue;
       }
-      
-      try {
-        IndexFingerprint otherFingerprint = IndexFingerprint.fromObject(replicaFingerprint);
-        IndexFingerprint ourFingerprint = IndexFingerprint.getFingerprint(core, Long.MAX_VALUE);
-        if(IndexFingerprint.compare(otherFingerprint, ourFingerprint) == 0) {
-          log.info("We are already in sync. No need to do a PeerSync ");
-          return true;
-        }
-      } catch(IOException e) {
-        log.warn("Could not confirm if we are already in sync. Continue with PeerSync");
+
+      IndexFingerprint otherFingerprint = IndexFingerprint.fromObject(replicaFingerprint);
+      if(IndexFingerprint.compare(otherFingerprint, ourFingerprint) == 0) {
+        log.info("We are already in sync. No need to do a PeerSync ");
+        return true;
       }
     }
-    
+
     return false;
   }
   
@@ -339,22 +347,22 @@ public class PeerSync implements SolrMetricProducer {
         Throwable solrException = ((SolrServerException) srsp.getException())
             .getRootCause();
         boolean connectTimeoutExceptionInChain = connectTimeoutExceptionInChain(srsp.getException());
-        if (connectTimeoutExceptionInChain || solrException instanceof ConnectException || solrException instanceof ConnectTimeoutException
+        if (connectTimeoutExceptionInChain || solrException instanceof ConnectTimeoutException || solrException instanceof SocketTimeoutException
             || solrException instanceof NoHttpResponseException || solrException instanceof SocketException) {
-          log.warn(msg() + " couldn't connect to " + srsp.getShardAddress() + ", counting as success", srsp.getException());
 
+          log.warn("{} couldn't connect to {}, counting as success ", msg(), srsp.getShardAddress(), srsp.getException());
           return true;
         }
       }
       
       if (cantReachIsSuccess && sreq.purpose == 1 && srsp.getException() instanceof SolrException && ((SolrException) srsp.getException()).code() == 503) {
-        log.warn(msg() + " got a 503 from " + srsp.getShardAddress() + ", counting as success", srsp.getException());
+        log.warn("{} got a 503 from {}, counting as success ", msg(), srsp.getShardAddress(), srsp.getException());
         return true;
       }
       
       if (cantReachIsSuccess && sreq.purpose == 1 && srsp.getException() instanceof SolrException && ((SolrException) srsp.getException()).code() == 404) {
-        log.warn(msg() + " got a 404 from " + srsp.getShardAddress() + ", counting as success. " +
-            "Perhaps /get is not registered?", srsp.getException());
+        log.warn("{} got a 404 from {}, counting as success. {} Perhaps /get is not registered?"
+            , msg(), srsp.getShardAddress(), srsp.getException());
         return true;
       }
       
@@ -363,9 +371,9 @@ public class PeerSync implements SolrMetricProducer {
       
       // TODO: at least log???
       // srsp.getException().printStackTrace(System.out);
-     
-      log.warn(msg() + " exception talking to " + srsp.getShardAddress() + ", failed", srsp.getException());
-      
+
+      log.warn("{} exception talking to {}, failed", msg(), srsp.getShardAddress(), srsp.getException());
+
       return false;
     }
 
@@ -393,40 +401,18 @@ public class PeerSync implements SolrMetricProducer {
     }
   }
 
-  private boolean canHandleVersionRanges(String replica) {
-    SyncShardRequest sreq = new SyncShardRequest();
-    requests.add(sreq);
-
-    // determine if leader can handle version ranges
-    sreq.shards = new String[] {replica};
-    sreq.actualShards = sreq.shards;
-    sreq.params = new ModifiableSolrParams();
-    sreq.params.set("qt", "/get");
-    sreq.params.set(DISTRIB, false);
-    sreq.params.set("checkCanHandleVersionRanges", false);
-
-    ShardHandler sh = shardHandlerFactory.getShardHandler(client);
-    sh.submit(sreq, replica, sreq.params);
-
-    ShardResponse srsp = sh.takeCompletedIncludingErrors();
-    Boolean canHandleVersionRanges = srsp.getSolrResponse().getResponse().getBooleanArg("canHandleVersionRanges");
-
-    if (canHandleVersionRanges == null || canHandleVersionRanges.booleanValue() == false) {
-      return false;
-    }
-
-    return true;
-  }
-
   private boolean handleVersions(ShardResponse srsp) {
     // we retrieved the last N updates from the replica
+    @SuppressWarnings({"unchecked"})
     List<Long> otherVersions = (List<Long>)srsp.getSolrResponse().getResponse().get("versions");
     // TODO: how to handle short lists?
 
     SyncShardRequest sreq = (SyncShardRequest) srsp.getShardRequest();
     Object fingerprint = srsp.getSolrResponse().getResponse().get("fingerprint");
 
-    log.info(msg() + " Received " + otherVersions.size() + " versions from " + sreq.shards[0] + " fingerprint:" + fingerprint );
+    if (log.isInfoEnabled()) {
+      log.info("{} Received {} versions from {} fingerprint:{}", msg(), otherVersions.size(), sreq.shards[0], fingerprint);
+    }
     if (fingerprint != null) {
       sreq.fingerprint = IndexFingerprint.fromObject(fingerprint);
     }
@@ -437,8 +423,7 @@ public class PeerSync implements SolrMetricProducer {
     }
     
     MissedUpdatesRequest updatesRequest = missedUpdatesFinder.find(
-        otherVersions, sreq.shards[0],
-        () -> core.getSolrConfig().useRangeVersionsForPeerSync && canHandleVersionRanges(sreq.shards[0]));
+        otherVersions, sreq.shards[0]);
 
     if (updatesRequest == MissedUpdatesRequest.ALREADY_IN_SYNC) {
       return true;
@@ -469,7 +454,7 @@ public class PeerSync implements SolrMetricProducer {
       }
       return cmp == 0;  // currently, we only check for equality...
     } catch(IOException e){
-      log.error(msg() + "Error getting index fingerprint", e);
+      log.error("{} Error getting index fingerprint", msg(), e);
       return false;
     }
   }
@@ -477,7 +462,9 @@ public class PeerSync implements SolrMetricProducer {
   private boolean requestUpdates(ShardResponse srsp, String versionsAndRanges, long totalUpdates) {
     String replica = srsp.getShardRequest().shards[0];
 
-    log.info(msg() + "Requesting updates from " + replica + "n=" + totalUpdates + " versions=" + versionsAndRanges);
+    if (log.isInfoEnabled()) {
+      log.info("{} Requesting updates from {} n={} versions={}", msg(), replica, totalUpdates, versionsAndRanges);
+    }
 
     // reuse our original request object
     ShardRequest sreq = srsp.getShardRequest();
@@ -500,11 +487,12 @@ public class PeerSync implements SolrMetricProducer {
 
   private boolean handleUpdates(ShardResponse srsp) {
     // we retrieved the last N updates from the replica
+    @SuppressWarnings({"unchecked"})
     List<Object> updates = (List<Object>)srsp.getSolrResponse().getResponse().get("updates");
 
     SyncShardRequest sreq = (SyncShardRequest) srsp.getShardRequest();
     if (updates.size() < sreq.totalRequestedUpdates) {
-      log.error(msg() + " Requested " + sreq.totalRequestedUpdates + " updates from " + sreq.shards[0] + " but retrieved " + updates.size());
+      log.error("{} Requested {} updates from {} but retrieved {}", msg(), sreq.totalRequestedUpdates, sreq.shards[0], updates.size());
       return false;
     }
     
@@ -564,7 +552,9 @@ public class PeerSync implements SolrMetricProducer {
       if (!(o1 instanceof List)) return 1;
       if (!(o2 instanceof List)) return -1;
 
+      @SuppressWarnings({"rawtypes"})
       List lst1 = (List) o1;
+      @SuppressWarnings({"rawtypes"})
       List lst2 = (List) o2;
 
       long l1 = Math.abs((Long) lst1.get(1));
@@ -600,10 +590,11 @@ public class PeerSync implements SolrMetricProducer {
         for (Object obj : updates) {
           // should currently be a List<Oper,Ver,Doc/Id>
           o = obj;
+          @SuppressWarnings({"unchecked"})
           List<Object> entry = (List<Object>)o;
 
           if (debug) {
-            log.debug(logPrefix + "raw update record " + o);
+            log.debug("{} raw update record {}", logPrefix, o);
           }
 
           int oper = (Integer)entry.get(0) & UpdateLog.OPERATION_MASK;
@@ -622,7 +613,7 @@ public class PeerSync implements SolrMetricProducer {
               cmd.setVersion(version);
               cmd.setFlags(UpdateCommand.PEER_SYNC | UpdateCommand.IGNORE_AUTOCOMMIT);
               if (debug) {
-                log.debug(logPrefix + "add " + cmd + " id " + sdoc.getField(ID));
+                log.debug("{} add {} id {}", logPrefix, cmd, sdoc.getField(ID));
               }
               proc.processAdd(cmd);
               break;
@@ -635,7 +626,9 @@ public class PeerSync implements SolrMetricProducer {
               cmd.setVersion(version);
               cmd.setFlags(UpdateCommand.PEER_SYNC | UpdateCommand.IGNORE_AUTOCOMMIT);
               if (debug) {
-                log.debug(logPrefix + "delete " + cmd + " " + new BytesRef(idBytes).utf8ToString());
+                if (log.isDebugEnabled()) {
+                  log.debug("{} delete {} {}", logPrefix, cmd, new BytesRef(idBytes).utf8ToString());
+                }
               }
               proc.processDelete(cmd);
               break;
@@ -649,7 +642,7 @@ public class PeerSync implements SolrMetricProducer {
               cmd.setVersion(version);
               cmd.setFlags(UpdateCommand.PEER_SYNC | UpdateCommand.IGNORE_AUTOCOMMIT);
               if (debug) {
-                log.debug(logPrefix + "deleteByQuery " + cmd);
+                log.debug("{} deleteByQuery {}", logPrefix, cmd);
               }
               proc.processDelete(cmd);
               break;
@@ -659,7 +652,7 @@ public class PeerSync implements SolrMetricProducer {
               AddUpdateCommand cmd = UpdateLog.convertTlogEntryToAddUpdateCommand(req, entry, oper, version);
               cmd.setFlags(UpdateCommand.PEER_SYNC | UpdateCommand.IGNORE_AUTOCOMMIT);
               if (debug) {
-                log.debug(logPrefix + "inplace update " + cmd + " prevVersion=" + cmd.prevVersion + ", doc=" + cmd.solrDoc);
+                log.debug("{} inplace update {} prevVersion={} doc={}", logPrefix, cmd, cmd.prevVersion, cmd.solrDoc);
               }
               proc.processAdd(cmd);
               break;
@@ -674,16 +667,16 @@ public class PeerSync implements SolrMetricProducer {
       } catch (IOException e) {
         // TODO: should this be handled separately as a problem with us?
         // I guess it probably already will by causing replication to be kicked off.
-        log.error(logPrefix + "Error applying updates from " + updateFrom + " ,update=" + o, e);
+        log.error("{} Error applying updates from {}, update={}", logPrefix, updateFrom, o, e);
         throw e;
       } catch (Exception e) {
-        log.error(logPrefix + "Error applying updates from " + updateFrom + " ,update=" + o, e);
+        log.error("{} Error applying updates from {}, update={} ", logPrefix, updateFrom,  o, e);
         throw e;
       } finally {
         try {
           proc.finish();
         } catch (Exception e) {
-          log.error(logPrefix + "Error applying updates from " + updateFrom + " ,finish()", e);
+          log.error("{} Error applying updates from {}, finish()", logPrefix, updateFrom, e);
           throw e;
         } finally {
           IOUtils.closeQuietly(proc);
@@ -692,33 +685,14 @@ public class PeerSync implements SolrMetricProducer {
     }
   }
 
-  /**
-   * Helper class for doing comparison ourUpdates and other replicas's updates to find the updates that we missed
-   */
-  public static class MissedUpdatesFinder {
-    private List<Long> ourUpdates;
-    private Set<Long> ourUpdateSet;
-    private Set<Long> requestedUpdateSet;
+  static abstract class MissedUpdatesFinderBase {
+    long ourLowThreshold;  // 20th percentile
+    List<Long> ourUpdates;
 
-    private long ourLowThreshold;  // 20th percentile
-    private long ourHighThreshold; // 80th percentile
-    private long ourHighest;  // currently just used for logging/debugging purposes
-    private String logPrefix;
-    private long nUpdates;
-
-    MissedUpdatesFinder(List<Long> ourUpdates, String logPrefix, long nUpdates,
-                        long ourLowThreshold, long ourHighThreshold) {
+    MissedUpdatesFinderBase(List<Long> ourUpdates, long ourLowThreshold) {
       assert sorted(ourUpdates);
-
-      this.logPrefix = logPrefix;
       this.ourUpdates = ourUpdates;
       this.ourLowThreshold = ourLowThreshold;
-      this.ourHighThreshold = ourHighThreshold;
-      this.ourHighest = ourUpdates.get(0);
-      this.nUpdates = nUpdates;
-
-      this.ourUpdateSet = new HashSet<>(ourUpdates);
-      this.requestedUpdateSet = new HashSet<>();
     }
 
     private boolean sorted(List<Long> list) {
@@ -730,61 +704,7 @@ public class PeerSync implements SolrMetricProducer {
       return true;
     }
 
-    public MissedUpdatesRequest find(List<Long> otherVersions, Object updateFrom, Supplier<Boolean> canHandleVersionRanges) {
-      otherVersions.sort(absComparator);
-      if (debug) {
-        log.debug("{} sorted versions from {} = {}", logPrefix, otherVersions, updateFrom);
-      }
-
-      long otherHigh = percentile(otherVersions, .2f);
-      long otherLow = percentile(otherVersions, .8f);
-      long otherHighest = otherVersions.get(0);
-
-      if (ourHighThreshold < otherLow) {
-        // Small overlap between version windows and ours is older
-        // This means that we might miss updates if we attempted to use this method.
-        // Since there exists just one replica that is so much newer, we must
-        // fail the sync.
-        log.info("{} Our versions are too old. ourHighThreshold={} otherLowThreshold={} ourHighest={} otherHighest={}",
-            logPrefix, ourHighThreshold, otherLow, ourHighest, otherHighest);
-        return MissedUpdatesRequest.UNABLE_TO_SYNC;
-      }
-
-      if (ourLowThreshold > otherHigh) {
-        // Small overlap between windows and ours is newer.
-        // Using this list to sync would result in requesting/replaying results we don't need
-        // and possibly bringing deleted docs back to life.
-        log.info("{} Our versions are newer. ourHighThreshold={} otherLowThreshold={} ourHighest={} otherHighest={}",
-            logPrefix, ourHighThreshold, otherLow, ourHighest, otherHighest);
-
-        // Because our versions are newer, IndexFingerprint with the remote would not match us.
-        // We return true on our side, but the remote peersync with us should fail.
-        return MissedUpdatesRequest.ALREADY_IN_SYNC;
-      }
-
-      boolean completeList = otherVersions.size() < nUpdates;
-
-      MissedUpdatesRequest updatesRequest;
-      if (canHandleVersionRanges.get()) {
-        updatesRequest = handleVersionsWithRanges(otherVersions, completeList);
-      } else {
-        updatesRequest = handleIndividualVersions(otherVersions, completeList);
-      }
-
-      if (updatesRequest.totalRequestedUpdates > nUpdates) {
-        log.info("{} PeerSync will fail because number of missed updates is more than:{}", logPrefix, nUpdates);
-        return MissedUpdatesRequest.UNABLE_TO_SYNC;
-      }
-
-      if (updatesRequest == MissedUpdatesRequest.EMPTY) {
-        log.info("{} No additional versions requested. ourHighThreshold={} otherLowThreshold={} ourHighest={} otherHighest={}",
-            logPrefix, ourHighThreshold, otherLow, ourHighest, otherHighest);
-      }
-
-      return updatesRequest;
-    }
-
-    private MissedUpdatesRequest handleVersionsWithRanges(List<Long> otherVersions, boolean completeList) {
+    MissedUpdatesRequest handleVersionsWithRanges(List<Long> otherVersions, boolean completeList) {
       // we may endup asking for updates for too many versions, causing 2MB post payload limit. Construct a range of
       // versions to request instead of asking individual versions
       List<String> rangesToRequest = new ArrayList<>();
@@ -815,7 +735,7 @@ public class PeerSync implements SolrMetricProducer {
           ourUpdatesIndex--;
         } else {
           long rangeStart = otherVersions.get(otherUpdatesIndex);
-          while ((otherUpdatesIndex < otherVersions.size())
+          while (otherUpdatesIndex >= 0
               && (Math.abs(otherVersions.get(otherUpdatesIndex)) < Math.abs(ourUpdates.get(ourUpdatesIndex)))) {
             otherUpdatesIndex--;
             totalRequestedVersions++;
@@ -828,27 +748,74 @@ public class PeerSync implements SolrMetricProducer {
       String rangesToRequestStr = rangesToRequest.stream().collect(Collectors.joining(","));
       return MissedUpdatesRequest.of(rangesToRequestStr, totalRequestedVersions);
     }
+  }
 
-    private MissedUpdatesRequest handleIndividualVersions(List<Long> otherVersions, boolean completeList) {
-      List<Long> toRequest = new ArrayList<>();
-      for (Long otherVersion : otherVersions) {
-        // stop when the entries get old enough that reorders may lead us to see updates we don't need
-        if (!completeList && Math.abs(otherVersion) < ourLowThreshold) break;
+  /**
+   * Helper class for doing comparison ourUpdates and other replicas's updates to find the updates that we missed
+   */
+  public static class MissedUpdatesFinder extends MissedUpdatesFinderBase {
+    private long ourHighThreshold; // 80th percentile
+    private long ourHighest;  // currently just used for logging/debugging purposes
+    private String logPrefix;
+    private long nUpdates;
 
-        if (ourUpdateSet.contains(otherVersion) || requestedUpdateSet.contains(otherVersion)) {
-          // we either have this update, or already requested it
-          // TODO: what if the shard we previously requested this from returns failure (because it goes
-          // down)
-          continue;
-        }
+    MissedUpdatesFinder(List<Long> ourUpdates, String logPrefix, long nUpdates,
+                        long ourLowThreshold, long ourHighThreshold) {
+      super(ourUpdates, ourLowThreshold);
 
-        toRequest.add(otherVersion);
-        requestedUpdateSet.add(otherVersion);
-      }
-
-      return MissedUpdatesRequest.of(StrUtils.join(toRequest, ','), toRequest.size());
+      this.logPrefix = logPrefix;
+      this.ourHighThreshold = ourHighThreshold;
+      this.ourHighest = ourUpdates.get(0);
+      this.nUpdates = nUpdates;
     }
 
+    public MissedUpdatesRequest find(List<Long> otherVersions, Object updateFrom) {
+      otherVersions.sort(absComparator);
+      if (debug) {
+        log.debug("{} sorted versions from {} = {}", logPrefix, otherVersions, updateFrom);
+      }
+
+      long otherHigh = percentile(otherVersions, .2f);
+      long otherLow = percentile(otherVersions, .8f);
+      long otherHighest = otherVersions.get(0);
+
+      if (ourHighThreshold < otherLow) {
+        // Small overlap between version windows and ours is older
+        // This means that we might miss updates if we attempted to use this method.
+        // Since there exists just one replica that is so much newer, we must
+        // fail the sync.
+        log.info("{} Our versions are too old. ourHighThreshold={} otherLowThreshold={} ourHighest={} otherHighest={}",
+            logPrefix, ourHighThreshold, otherLow, ourHighest, otherHighest);
+        return MissedUpdatesRequest.UNABLE_TO_SYNC;
+      }
+
+      if (ourLowThreshold > otherHigh && ourHighest >= otherHighest) {
+        // Small overlap between windows and ours is newer.
+        // Using this list to sync would result in requesting/replaying results we don't need
+        // and possibly bringing deleted docs back to life.
+        log.info("{} Our versions are newer. ourHighThreshold={} otherLowThreshold={} ourHighest={} otherHighest={}",
+            logPrefix, ourHighThreshold, otherLow, ourHighest, otherHighest);
+
+        // Because our versions are newer, IndexFingerprint with the remote would not match us.
+        // We return true on our side, but the remote peersync with us should fail.
+        return MissedUpdatesRequest.ALREADY_IN_SYNC;
+      }
+
+      boolean completeList = otherVersions.size() < nUpdates;
+
+      MissedUpdatesRequest updatesRequest = handleVersionsWithRanges(otherVersions, completeList);
+      if (updatesRequest.totalRequestedUpdates > nUpdates) {
+        log.info("{} PeerSync will fail because number of missed updates is more than:{}", logPrefix, nUpdates);
+        return MissedUpdatesRequest.UNABLE_TO_SYNC;
+      }
+
+      if (updatesRequest == MissedUpdatesRequest.EMPTY) {
+        log.info("{} No additional versions requested. ourHighThreshold={} otherLowThreshold={} ourHighest={} otherHighest={}",
+            logPrefix, ourHighThreshold, otherLow, ourHighest, otherHighest);
+      }
+
+      return updatesRequest;
+    }
   }
 
   /**
