@@ -45,6 +45,8 @@ import org.apache.lucene.search.ConstantScoreWeight;
 import org.apache.lucene.search.DocIdSetIterator;
 import org.apache.lucene.search.IndexSearcher;
 import org.apache.lucene.search.Query;
+import org.apache.lucene.search.QueryVisitor;
+import org.apache.lucene.search.ScoreMode;
 import org.apache.lucene.search.Scorer;
 import org.apache.lucene.search.Weight;
 import org.apache.lucene.store.Directory;
@@ -173,7 +175,7 @@ public class SolrIndexSplitter {
           log.error("Original error closing IndexWriter:", e);
           throw new SolrException(SolrException.ErrorCode.SERVER_ERROR, "Error reopening IndexWriter after failed close", e1);
         }
-        throw new SolrException(SolrException.ErrorCode.SERVER_ERROR, "Error closing current IndexWriter, aborting offline split...", e);
+        throw new SolrException(SolrException.ErrorCode.SERVER_ERROR, "Error closing current IndexWriter, aborting 'link' split...", e);
       }
     }
     boolean success = false;
@@ -194,8 +196,9 @@ public class SolrIndexSplitter {
         t = timings.sub("parentApplyBufferedUpdates");
         ulog.applyBufferedUpdates();
         t.stop();
-        log.info("Splitting in 'offline' mode " + (success? "finished" : "FAILED") +
-            ": re-opened parent IndexWriter.");
+        if (log.isInfoEnabled()) {
+          log.info("Splitting in 'link' mode {}: re-opened parent IndexWriter.", (success ? "finished" : "FAILED"));
+        }
       }
     }
     results.add(CommonParams.TIMING, timings.asNamedList());
@@ -209,7 +212,9 @@ public class SolrIndexSplitter {
     SolrIndexConfig parentConfig = searcher.getCore().getSolrConfig().indexConfig;
     String timestamp = new SimpleDateFormat(SnapShooter.DATE_FMT, Locale.ROOT).format(new Date());
 
-    log.info("SolrIndexSplitter: partitions=" + numPieces + " segments=" + leaves.size());
+    if (log.isInfoEnabled()) {
+      log.info("SolrIndexSplitter: partitions={} segments={}", numPieces, leaves.size());
+    }
     RTimerTree t;
 
     // this tracks round-robin assignment of docs to partitions
@@ -305,7 +310,10 @@ public class SolrIndexSplitter {
           t = timings.sub("addIndexes");
           t.resume();
           for (int segmentNumber = 0; segmentNumber<leaves.size(); segmentNumber++) {
-            log.info("SolrIndexSplitter: partition #" + partitionNumber + " partitionCount=" + numPieces + (ranges != null ? " range=" + ranges.get(partitionNumber) : "") + " segment #"+segmentNumber + " segmentCount=" + leaves.size());
+            if (log.isInfoEnabled()) {
+              log.info("SolrIndexSplitter: partition # {} partitionCount={} {} segment #={} segmentCount={}", partitionNumber, numPieces
+                  , (ranges != null ? " range=" + ranges.get(partitionNumber) : ""), segmentNumber, leaves.size()); // nowarn
+            }
             CodecReader subReader = SlowCodecReaderWrapper.wrap(leaves.get(segmentNumber).reader());
             iw.addIndexes(new LiveDocsReader(subReader, segmentDocSets.get(segmentNumber)[partitionNumber]));
           }
@@ -356,7 +364,7 @@ public class SolrIndexSplitter {
           subCore.getUpdateHandler().newIndexWriter(false);
           openNewSearcher(subCore);
         } catch (Exception e) {
-          log.error("Failed to switch sub-core " + indexDirPath + " to " + hardLinkPath + ", split will fail.", e);
+          log.error("Failed to switch sub-core {} to {}, split will fail", indexDirPath, hardLinkPath, e);
           switchOk = false;
           break;
         }
@@ -393,7 +401,7 @@ public class SolrIndexSplitter {
           try {
             openNewSearcher(subCore);
           } catch (Exception e) {
-            log.warn("Error rolling back failed split of " + hardLinkPath, e);
+            log.warn("Error rolling back failed split of {}", hardLinkPath, e);
           }
         }
         t.stop();
@@ -422,6 +430,7 @@ public class SolrIndexSplitter {
   }
 
   private void openNewSearcher(SolrCore core) throws Exception {
+    @SuppressWarnings({"rawtypes"})
     Future[] waitSearcher = new Future[1];
     core.getSearcher(true, false, waitSearcher, true);
     if (waitSearcher[0] != null) {
@@ -450,7 +459,7 @@ public class SolrIndexSplitter {
     }
 
     @Override
-    public Weight createWeight(IndexSearcher searcher, boolean needsScores, float boost) throws IOException {
+    public Weight createWeight(IndexSearcher searcher, ScoreMode scoreMode, float boost) throws IOException {
       return new ConstantScoreWeight(this, boost) {
 
         @Override
@@ -459,8 +468,11 @@ public class SolrIndexSplitter {
           t.resume();
           FixedBitSet set = findDocsToDelete(context);
           t.pause();
-          log.info("### partition=" + partition + ", leaf=" + context + ", maxDoc=" + context.reader().maxDoc() +
-          ", numDels=" + context.reader().numDeletedDocs() + ", setLen=" + set.length() + ", setCard=" + set.cardinality());
+          if (log.isInfoEnabled()) {
+            log.info("### partition={}, leaf={}, maxDoc={}, numDels={}, setLen={}, setCard={}"
+            , partition, context, context.reader().maxDoc()
+            ,context.reader().numDeletedDocs(), set.length(), set.cardinality());
+          }
           Bits liveDocs = context.reader().getLiveDocs();
           if (liveDocs != null) {
             // check that we don't delete already deleted docs
@@ -468,10 +480,10 @@ public class SolrIndexSplitter {
             dels.flip(0, dels.length());
             dels.and(set);
             if (dels.cardinality() > 0) {
-              log.error("### INVALID DELS " + dels.cardinality());
+              log.error("### INVALID DELS {}", dels.cardinality());
             }
           }
-          return new ConstantScoreScorer(this, score(), new BitSetIterator(set, set.length()));
+          return new ConstantScoreScorer(this, score(), scoreMode, new BitSetIterator(set, set.length()));
         }
 
         @Override
@@ -528,6 +540,11 @@ public class SolrIndexSplitter {
     public int hashCode() {
       return partition;
     }
+
+    @Override
+    public void visit(QueryVisitor visitor) {
+      visitor.visitLeaf(this);
+    }
   }
 
   static FixedBitSet[] split(LeafReaderContext readerContext, int numPieces, SchemaField field, DocRouter.Range[] rangesArr,
