@@ -188,6 +188,7 @@ public class CloneFieldUpdateProcessorFactory
   /** @see #dest */
   private Pattern pattern = null;
 
+  @SuppressWarnings("WeakerAccess")
   protected final FieldNameSelector getSourceSelector() {
     if (null != srcSelector) return srcSelector;
 
@@ -197,7 +198,7 @@ public class CloneFieldUpdateProcessorFactory
 
   @SuppressWarnings("unchecked")
   @Override
-  public void init(NamedList args) {
+  public void init(@SuppressWarnings({"rawtypes"})NamedList args) {
 
     // high level (loose) check for which type of config we have.
     // 
@@ -226,7 +227,7 @@ public class CloneFieldUpdateProcessorFactory
    * "source" and "dest" init params do <em>not</em> exist.
    */
   @SuppressWarnings("unchecked")
-  private void initSimpleRegexReplacement(NamedList args) {
+  private void initSimpleRegexReplacement(@SuppressWarnings({"rawtypes"})NamedList args) {
     // The syntactic sugar for the case where there is only one regex pattern for source and the same pattern
     // is used for the destination pattern...
     //
@@ -282,7 +283,7 @@ public class CloneFieldUpdateProcessorFactory
    * "source" and "dest" init params <em>do</em> exist.
    */
   @SuppressWarnings("unchecked")
-  private void initSourceSelectorSyntax(NamedList args) {
+  private void initSourceSelectorSyntax(@SuppressWarnings({"rawtypes"})NamedList args) {
     // Full and complete syntax where source and dest are mandatory.
     //
     // source may be a single string or a selector.
@@ -306,6 +307,7 @@ public class CloneFieldUpdateProcessorFactory
     if (1 == sources.size()) {
       if (sources.get(0) instanceof NamedList) {
         // nested set of selector options
+        @SuppressWarnings({"rawtypes"})
         NamedList selectorConfig = (NamedList) args.remove(SOURCE_PARAM);
 
         srcInclusions = parseSelectorParams(selectorConfig);
@@ -321,6 +323,7 @@ public class CloneFieldUpdateProcessorFactory
             throw new SolrException(SERVER_ERROR, "Init param '" + SOURCE_PARAM +
                                     "' child 'exclude' must be <lst/>");
           }
+          @SuppressWarnings({"rawtypes"})
           NamedList exc = (NamedList) excObj;
           srcExclusions.add(parseSelectorParams(exc));
           if (0 < exc.size()) {
@@ -352,6 +355,7 @@ public class CloneFieldUpdateProcessorFactory
     }
     
     if (d instanceof NamedList) {
+      @SuppressWarnings({"rawtypes"})
       NamedList destList = (NamedList) d;
 
       Object patt = destList.remove(PATTERN_PARAM);
@@ -433,8 +437,10 @@ public class CloneFieldUpdateProcessorFactory
             if (matcher.find()) {
               resolvedDest = matcher.replaceAll(dest);
             } else {
-              log.debug("CloneFieldUpdateProcessor.srcSelector.shouldMutate(\"{}\") returned true, " +
-                  "but replacement pattern did not match, field skipped.", fname);
+              if (log.isDebugEnabled()) {
+                log.debug("CloneFieldUpdateProcessor.srcSelector.shouldMutate('{}') returned true, but replacement pattern did not match, field skipped."
+                    , fname);
+              }
               continue;
             }
           }
@@ -457,8 +463,8 @@ public class CloneFieldUpdateProcessorFactory
           destMap.put(resolvedDest, destField);
         }
 
-        for (String dest : destMap.keySet()) {
-          doc.put(dest, destMap.get(dest));
+        for (Map.Entry<String, SolrInputField> entry : destMap.entrySet()) {
+          doc.put(entry.getKey(), entry.getValue());
         }
         super.processAdd(cmd);
       }
@@ -466,7 +472,7 @@ public class CloneFieldUpdateProcessorFactory
   }
 
   /** macro */
-  private static SelectorParams parseSelectorParams(NamedList args) {
+  private static SelectorParams parseSelectorParams(@SuppressWarnings({"rawtypes"})NamedList args) {
     return FieldMutatingUpdateProcessorFactory.parseSelectorParams(args);
   }
 
