@@ -69,27 +69,6 @@ public class DocBasedVersionConstraintsProcessor extends UpdateRequestProcessor
   private final boolean useFieldCache;
 
   private long oldSolrVersion;  // current _version_ of the doc in the index/update log
-  
-  /**
-   * @deprecated Use {@link #DocBasedVersionConstraintsProcessor(List, boolean, List, boolean, boolean, NamedList, SolrQueryRequest, UpdateRequestProcessor)}
-   */
-  @Deprecated
-  public DocBasedVersionConstraintsProcessor(List<String> versionFields,
-      boolean ignoreOldUpdates,
-      List<String> deleteVersionParamNames,
-      boolean supportMissingVersionOnOldDocs,
-      boolean useFieldCache,
-      SolrQueryRequest req,
-      UpdateRequestProcessor next ) {
-    this(versionFields,
-        ignoreOldUpdates,
-        deleteVersionParamNames,
-        supportMissingVersionOnOldDocs,
-        useFieldCache,
-        null,
-        req,
-        next);
-  }
 
   public DocBasedVersionConstraintsProcessor(List<String> versionFields,
                                              boolean ignoreOldUpdates,
@@ -239,7 +218,7 @@ public class DocBasedVersionConstraintsProcessor extends UpdateRequestProcessor
 
   private DocFoundAndOldUserAndSolrVersions getOldUserVersionsFromStored(BytesRef indexedDocId) throws IOException {
     // stored fields only...
-    SolrInputDocument oldDoc = RealTimeGetComponent.getInputDocument(core, indexedDocId);
+    SolrInputDocument oldDoc = RealTimeGetComponent.getInputDocument(core, indexedDocId, RealTimeGetComponent.Resolution.DOC);
     if (null == oldDoc) {
       return DocFoundAndOldUserAndSolrVersions.NOT_FOUND;
     } else {
@@ -346,7 +325,9 @@ public class DocBasedVersionConstraintsProcessor extends UpdateRequestProcessor
    * @param userVersionFieldName Field name of the user versions being compared
    * @return True if acceptable, false if not.
    */
-  protected boolean newUpdateComparePasses(Comparable newUserVersion, Comparable oldUserVersion, String userVersionFieldName) {
+  @SuppressWarnings({"unchecked"})
+  protected boolean newUpdateComparePasses(@SuppressWarnings({"rawtypes"})Comparable newUserVersion,
+                                           @SuppressWarnings({"rawtypes"})Comparable oldUserVersion, String userVersionFieldName) {
     return oldUserVersion.compareTo(newUserVersion) < 0;
   }
 
@@ -372,10 +353,12 @@ public class DocBasedVersionConstraintsProcessor extends UpdateRequestProcessor
     return values;
   }
 
+  @SuppressWarnings({"unchecked"})
   private static FunctionValues getFunctionValues(LeafReaderContext segmentContext,
                                           SchemaField field,
                                           SolrIndexSearcher searcher) throws IOException {
     ValueSource vs = field.getType().getValueSource(field, null);
+    @SuppressWarnings({"rawtypes"})
     Map context = ValueSource.newContext(searcher);
     vs.createWeight(context, searcher);
     return vs.getValues(context, segmentContext);
@@ -389,7 +372,8 @@ public class DocBasedVersionConstraintsProcessor extends UpdateRequestProcessor
       return true;
     }
     // if phase==TOLEADER, we can't just assume we are the leader... let the normal logic check.
-    return !distribProc.isLeader(cmd);
+    distribProc.setupRequest(cmd);
+    return !distribProc.isLeader();
   }
 
   @Override
@@ -428,7 +412,7 @@ public class DocBasedVersionConstraintsProcessor extends UpdateRequestProcessor
   private static void logOverlyFailedRetries(int i, UpdateCommand cmd) {
     // Log a warning every 256 retries.... even a few retries should normally be very unusual.
     if ((i&0xff) == 0xff) {
-      log.warn("Unusual number of optimistic concurrency retries: retries=" + i + " cmd=" + cmd);
+      log.warn("Unusual number of optimistic concurrency retries: retries={} cmd={}", i, cmd);
     }
   }
 
