@@ -16,6 +16,7 @@
  */
 package org.apache.solr.util;
 
+import java.io.IOException;
 import java.lang.invoke.MethodHandles;
 import java.lang.reflect.Method;
 import java.util.Collections;
@@ -31,7 +32,7 @@ import java.util.concurrent.atomic.AtomicInteger;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import org.apache.solr.cloud.ZkController;
+import org.apache.lucene.index.IndexWriter;
 import org.apache.solr.common.NonExistentCoreException;
 import org.apache.solr.common.SolrException;
 import org.apache.solr.common.SolrException.ErrorCode;
@@ -53,7 +54,7 @@ import org.slf4j.LoggerFactory;
  * @lucene.internal
  */
 public class TestInjection {
-  
+
   public static class TestShutdownFailError extends OutOfMemoryError {
 
     public TestShutdownFailError(String msg) {
@@ -73,9 +74,11 @@ public class TestInjection {
    * If non-null, then this class should be used for accessing random entropy
    * @see #random
    */
+  @SuppressWarnings({"rawtypes"})
   private static final Class LUCENE_TEST_CASE;
   
   static {
+    @SuppressWarnings({"rawtypes"})
     Class nonFinalTemp = null;
     try {
       ClassLoader classLoader = MethodHandles.lookup().lookupClass().getClassLoader();
@@ -96,6 +99,7 @@ public class TestInjection {
       return null;
     } else {
       try {
+        @SuppressWarnings({"unchecked"})
         Method randomMethod = LUCENE_TEST_CASE.getMethod("random");
         return (Random) randomMethod.invoke(null);
       } catch (Exception e) {
@@ -105,11 +109,13 @@ public class TestInjection {
   }
   
   public volatile static String nonGracefullClose = null;
-
+  
   public volatile static String failReplicaRequests = null;
   
   public volatile static String failUpdateRequests = null;
 
+  public volatile static String leaderTragedy = null;
+
   public volatile static String nonExistentCoreExceptionAfterUnload = null;
 
   public volatile static String updateLogReplayRandomPause = null;
@@ -128,7 +134,11 @@ public class TestInjection {
 
   public volatile static CountDownLatch splitLatch = null;
 
-  public volatile static String waitForReplicasInSync = "true:60";
+  public volatile static CountDownLatch directUpdateLatch = null;
+
+  public volatile static CountDownLatch reindexLatch = null;
+
+  public volatile static String reindexFailure = null;
 
   public volatile static String failIndexFingerprintRequests = null;
 
@@ -138,12 +148,25 @@ public class TestInjection {
 
   private volatile static AtomicInteger countPrepRecoveryOpPauseForever = new AtomicInteger(0);
 
-  public volatile static Integer delayBeforeSlaveCommitRefresh=null;
+  public volatile static Integer delayBeforeFollowerCommitRefresh=null;
 
   public volatile static Integer delayInExecutePlanAction=null;
 
+  public volatile static Integer delayBeforeCreatingNewDocSet = null;
+
+  public volatile static AtomicInteger countDocSetDelays = new AtomicInteger(0);
+
   public volatile static boolean failInExecutePlanAction = false;
 
+  /**
+   * Defaults to <code>false</code>, If set to <code>true</code>, 
+   * then {@link #injectSkipIndexWriterCommitOnClose} will return <code>true</code>
+   *
+   * @see #injectSkipIndexWriterCommitOnClose
+   * @see org.apache.solr.update.DirectUpdateHandler2#closeWriter
+   */
+  public volatile static boolean skipIndexWriterCommitOnClose = false;
+
   public volatile static boolean uifOutOfMemoryError = false;
 
   private volatile static CountDownLatch notifyPauseForeverDone = new CountDownLatch(1);
@@ -157,6 +180,7 @@ public class TestInjection {
     nonGracefullClose = null;
     failReplicaRequests = null;
     failUpdateRequests = null;
+    leaderTragedy = null;
     nonExistentCoreExceptionAfterUnload = null;
     updateLogReplayRandomPause = null;
     updateRandomPause = null;
@@ -164,14 +188,19 @@ public class TestInjection {
     splitFailureBeforeReplicaCreation = null;
     splitFailureAfterReplicaCreation = null;
     splitLatch = null;
+    directUpdateLatch = null;
+    reindexLatch = null;
+    reindexFailure = null;
     prepRecoveryOpPauseForever = null;
     countPrepRecoveryOpPauseForever = new AtomicInteger(0);
-    waitForReplicasInSync = "true:60";
     failIndexFingerprintRequests = null;
     wrongIndexFingerprint = null;
-    delayBeforeSlaveCommitRefresh = null;
+    delayBeforeFollowerCommitRefresh = null;
     delayInExecutePlanAction = null;
+    delayBeforeCreatingNewDocSet = null;
+    countDocSetDelays.set(0);
     failInExecutePlanAction = false;
+    skipIndexWriterCommitOnClose = false;
     uifOutOfMemoryError = false;
     notifyPauseForeverDone();
     newSearcherHooks.clear();
@@ -274,6 +303,21 @@ public class TestInjection {
     return true;
   }
 
+  /**
+   * Returns the value of {@link #skipIndexWriterCommitOnClose}.
+   *
+   * @param indexWriter used only for logging
+   * @see #skipIndexWriterCommitOnClose
+   * @see org.apache.solr.update.DirectUpdateHandler2#closeWriter
+   */
+  public static boolean injectSkipIndexWriterCommitOnClose(Object indexWriter) {
+    if (skipIndexWriterCommitOnClose) {
+      log.info("Inject failure: skipIndexWriterCommitOnClose={}: {}",
+               skipIndexWriterCommitOnClose, indexWriter);
+    }
+    return skipIndexWriterCommitOnClose;
+  }
+  
   public static boolean injectFailReplicaRequests() {
     if (failReplicaRequests != null) {
       Random rand = random();
@@ -305,6 +349,39 @@ public class TestInjection {
 
     return true;
   }
+
+  public static boolean injectLeaderTragedy(SolrCore core) {
+    if (leaderTragedy != null) {
+      Random rand = random();
+      if (null == rand) return true;
+
+      Pair<Boolean, Integer> pair = parseValue(leaderTragedy);
+      boolean enabled = pair.first();
+      int chanceIn100 = pair.second();
+
+      if (! core.getCoreDescriptor().getCloudDescriptor().isLeader()) {
+        return true;
+      }
+
+      if (enabled && rand.nextInt(100) >= (100 - chanceIn100)) {
+        RefCounted<IndexWriter> writer = null;
+        try {
+          writer = core.getSolrCoreState().getIndexWriter(null);
+          writer.get().onTragicEvent(new Exception("injected tragedy"), "injection");
+        } catch (IOException e) {
+          // Problem getting the writer, but that will likely bubble up later
+          return true;
+        } finally {
+          if (writer != null) {
+            writer.decref();
+          }
+        }
+
+        throw new SolrException(ErrorCode.SERVER_ERROR, "Random tragedy fail");
+      }
+    }
+    return true;
+  }
   
   public static boolean injectNonExistentCoreExceptionAfterUnload(String cname) {
     if (nonExistentCoreExceptionAfterUnload != null) {
@@ -407,7 +484,7 @@ public class TestInjection {
       boolean enabled = pair.first();
       int chanceIn100 = pair.second();
       if (enabled && rand.nextInt(100) >= (100 - chanceIn100)) {
-        log.info("Injecting failure: " + label);
+        log.info("Injecting failure: {}", label);
         throw new SolrException(ErrorCode.SERVER_ERROR, "Error: " + label);
       }
     }
@@ -434,14 +511,47 @@ public class TestInjection {
     return true;
   }
 
-  public static boolean waitForInSyncWithLeader(SolrCore core, ZkController zkController, String collection, String shardId) {
-    // NOTE: this method should do *NOTHING* unless LUCENE_TEST_CASE is non-null
-    
-    if (waitForReplicasInSync == null) return true;
-    
-    return true; // No-Op: see SOLR-12313
+  public static boolean injectDirectUpdateLatch() {
+    if (directUpdateLatch != null) {
+      try {
+        log.info("Waiting in DirectUpdateHandler2 for up to 60s");
+        return directUpdateLatch.await(60, TimeUnit.SECONDS);
+      } catch (InterruptedException e) {
+        Thread.currentThread().interrupt();
+      }
+    }
+    return true;
   }
-  
+
+  public static boolean injectReindexFailure() {
+    if (reindexFailure != null)  {
+      Random rand = random();
+      if (null == rand) return true;
+
+      Pair<Boolean,Integer> pair = parseValue(reindexFailure);
+      boolean enabled = pair.first();
+      int chanceIn100 = pair.second();
+      if (enabled && rand.nextInt(100) >= (100 - chanceIn100)) {
+        log.info("Test injection failure");
+        throw new SolrException(ErrorCode.SERVER_ERROR, "Test injection failure");
+      }
+    }
+    return true;
+  }
+
+
+  public static boolean injectReindexLatch() {
+    if (reindexLatch != null) {
+      try {
+        log.info("Waiting in ReindexCollectionCmd for up to 60s");
+        return reindexLatch.await(60, TimeUnit.SECONDS);
+      } catch (InterruptedException e) {
+        Thread.currentThread().interrupt();
+      }
+    }
+    return true;
+  }
+
   private static Pair<Boolean,Integer> parseValue(final String raw) {
     if (raw == null) return new Pair<>(false, 0);
     Matcher m = ENABLED_PERCENT.matcher(raw);
@@ -456,11 +566,11 @@ public class TestInjection {
     return new Pair<>(Boolean.parseBoolean(val), Integer.parseInt(percent));
   }
 
-  public static boolean injectDelayBeforeSlaveCommitRefresh() {
-    if (delayBeforeSlaveCommitRefresh!=null) {
+  public static boolean injectDelayBeforeFollowerCommitRefresh() {
+    if (delayBeforeFollowerCommitRefresh!=null) {
       try {
-        log.info("Pausing IndexFetcher for {}ms", delayBeforeSlaveCommitRefresh);
-        Thread.sleep(delayBeforeSlaveCommitRefresh);
+        log.info("Pausing IndexFetcher for {}ms", delayBeforeFollowerCommitRefresh);
+        Thread.sleep(delayBeforeFollowerCommitRefresh);
       } catch (InterruptedException e) {
         Thread.currentThread().interrupt();
       }
@@ -475,6 +585,22 @@ public class TestInjection {
     return true;
   }
 
+  public static boolean injectDocSetDelay(Object query) {
+    if (delayBeforeCreatingNewDocSet != null) {
+      countDocSetDelays.incrementAndGet();
+      try {
+        log.info("Pausing DocSet for {}ms: {}", delayBeforeCreatingNewDocSet, query);
+        if (log.isDebugEnabled()) {
+          log.debug("", new Exception("Stack Trace"));
+        }
+        Thread.sleep(delayBeforeCreatingNewDocSet);
+      } catch (InterruptedException e) {
+        Thread.currentThread().interrupt();
+      }
+    }
+    return true;
+  }
+
   static Set<Hook> newSearcherHooks = ConcurrentHashMap.newKeySet();
   
   public interface Hook {
