@@ -31,7 +31,7 @@ import java.util.concurrent.Callable;
 import java.util.concurrent.ExecutorCompletionService;
 import java.util.concurrent.Future;
 
-import org.apache.commons.lang.StringUtils;
+import org.apache.commons.lang3.StringUtils;
 import org.apache.lucene.util.LuceneTestCase.Slow;
 import org.apache.solr.SolrTestCaseJ4.SuppressSSL;
 import org.apache.solr.client.solrj.SolrClient;
@@ -117,6 +117,7 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
   }
   
   @Test
+  @SuppressWarnings({"unchecked"})
   public void test() throws Exception {
     
     assertEquals(clients.size(), jettys.size());
@@ -209,7 +210,10 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
     query("q","*:*", "sort",i1+" desc", "fl","*,score");
     query("q","*:*", "sort","n_tl1 asc", "fl","*,score"); 
     query("q","*:*", "sort","n_tl1 desc");
+    
     handle.put("maxScore", SKIPVAL);
+    testMinExactCount();
+    
     query("q","{!func}"+i1);// does not expect maxScore. So if it comes ,ignore it. JavaBinCodec.writeSolrDocumentList()
     //is agnostic of request params.
     handle.remove("maxScore");
@@ -596,7 +600,7 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
           if (shardReq.params.getBool(StatsParams.STATS, false)) {
             numStatsShardRequests++;
             for (ShardResponse shardRsp : shardReq.sreq.responses) {
-              NamedList<Object> shardStats = 
+              NamedList<Object> shardStats =
                 ((NamedList<NamedList<NamedList<Object>>>)
                  shardRsp.getSolrResponse().getResponse().get("stats")).get("stats_fields").get(i1);
 
@@ -626,7 +630,7 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
       //
       assertEquals("wrong min", -987.0D, (Double)s.getMin(), 0.0001D );
       assertEquals("wrong mean", 377.153846D, (Double)s.getMean(), 0.0001D );
-      assertEquals("wrong stddev", 1271.76215D, (Double)s.getStddev(), 0.0001D );
+      assertEquals("wrong stddev", 1271.76215D, s.getStddev(), 0.0001D );
       //
       assertNull("expected null for count", s.getCount());
       assertNull("expected null for calcDistinct", s.getCountDistinct());
@@ -680,7 +684,7 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
 
         // ignore the FieldStatsInfo convinience class, and look directly at the NamedList
         // so we don't need any sort of crazy reflection
-        NamedList<Object> svals = 
+        NamedList<Object> svals =
           ((NamedList<NamedList<NamedList<Object>>>)
            rsp.getResponse().get("stats")).get("stats_fields").get(i1);
 
@@ -803,7 +807,7 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
       // NOTE: min is expected to be null even though requested because of no values
       assertEquals("wrong min", null, s.getMin()); 
       assertTrue("mean should be NaN", ((Double)s.getMean()).isNaN());
-      assertEquals("wrong stddev", 0.0D, (Double)s.getStddev(), 0.0D );
+      assertEquals("wrong stddev", 0.0D, s.getStddev(), 0.0D );
 
       // things that we didn't ask for, so they better be null
       assertNull("expected null for count", s.getCount());
@@ -921,13 +925,11 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
 
     //SOLR 3161 ensure shards.qt=/update fails (anything but search handler really)
     // Also see TestRemoteStreaming#testQtUpdateFails()
-    try {
-      ignoreException("isShard is only acceptable");
-      // query("q","*:*","shards.qt","/update","stream.body","<delete><query>*:*</query></delete>");
-      // fail();
-    } catch (SolrException e) {
-      //expected
-    }
+
+    //SolrException e = expectThrows(SolrException.class, () -> {
+    //  ignoreException("isShard is only acceptable");
+    //  query("q","*:*","shards.qt","/update","stream.body","<delete><query>*:*</query></delete>");
+    //});
     unIgnoreException("isShard is only acceptable");
 
     // test debugging
@@ -1084,11 +1086,38 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
     assertEquals(new EnumFieldValue(11, "Critical"),
                  rsp.getFieldStatsInfo().get(fieldName).getMax());
 
-    handle.put("severity", UNORDERED); // this is stupid, but stats.facet doesn't garuntee order
+    handle.put("severity", UNORDERED); // this is stupid, but stats.facet doesn't guarantee order
     query("q", "*:*", "stats", "true", "stats.field", fieldName, 
           "stats.facet", fieldName);
   }
 
+  private void testMinExactCount() throws Exception {
+    assertIsExactHitCount("q","{!cache=false}dog OR men OR cow OR country OR dumpty", CommonParams.MIN_EXACT_COUNT, "200", CommonParams.ROWS, "2", CommonParams.SORT, "score desc, id asc");
+    assertIsExactHitCount("q","{!cache=false}dog OR men OR cow OR country OR dumpty", CommonParams.MIN_EXACT_COUNT, "-1", CommonParams.ROWS, "2", CommonParams.SORT, "score desc, id asc");
+    assertIsExactHitCount("q","{!cache=false}dog OR men OR cow OR country OR dumpty", CommonParams.MIN_EXACT_COUNT, "1", CommonParams.ROWS, "200", CommonParams.SORT, "score desc, id asc");
+    assertIsExactHitCount("q","{!cache=false}dog OR men OR cow OR country OR dumpty", "facet", "true", "facet.field", s1, CommonParams.MIN_EXACT_COUNT,"1", CommonParams.ROWS, "200", CommonParams.SORT, "score desc, id asc");
+    assertIsExactHitCount("q","{!cache=false}id:1", CommonParams.MIN_EXACT_COUNT,"1", CommonParams.ROWS, "1");
+    assertApproximatedHitCount("q","{!cache=false}dog OR men OR cow OR country OR dumpty", CommonParams.MIN_EXACT_COUNT,"2", CommonParams.ROWS, "2", CommonParams.SORT, "score desc, id asc");
+  }
+  
+  private void assertIsExactHitCount(Object... requestParams) throws Exception {
+    QueryResponse response = query(requestParams);
+    assertNotNull("Expecting exact hit count in response: " + response.getResults().toString(),
+        response.getResults().getNumFoundExact());
+    assertTrue("Expecting exact hit count in response: " + response.getResults().toString(),
+        response.getResults().getNumFoundExact());
+  }
+  
+  private void assertApproximatedHitCount(Object...requestParams) throws Exception {
+    handle.put("numFound", SKIPVAL);
+    QueryResponse response = query(requestParams);
+    assertNotNull("Expecting numFoundExact in response: " + response.getResults().toString(),
+        response.getResults().getNumFoundExact());
+    assertFalse("Expecting aproximated results in response: " + response.getResults().toString(),
+        response.getResults().getNumFoundExact());
+    handle.remove("numFound", SKIPVAL);
+  }
+
   /** comparing results with facet.method=uif */
   private void queryAndCompareUIF(Object ... params) throws Exception {
     final QueryResponse expect = query(params);
@@ -1126,6 +1155,7 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
     }
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   protected void queryPartialResults(final List<String> upShards,
                                      final List<SolrClient> upClients,
                                      Object... q) throws Exception {
@@ -1252,41 +1282,34 @@ public class TestDistributedSearch extends BaseDistributedSearchTestCase {
   private void validateCommonQueryParameters() throws Exception {
     ignoreException("parameter cannot be negative");
 
-    try {
+    SolrException e1 = expectThrows(SolrException.class, () -> {
       SolrQuery query = new SolrQuery();
       query.setParam("start", "non_numeric_value").setQuery("*");
       QueryResponse resp = query(query);
-      fail("Expected the last query to fail, but got response: " + resp);
-    } catch (SolrException e) {
-      assertEquals(ErrorCode.BAD_REQUEST.code, e.code());
-    }
+    });
+    assertEquals(ErrorCode.BAD_REQUEST.code, e1.code());
 
-    try {
+    SolrException e2 = expectThrows(SolrException.class, () -> {
       SolrQuery query = new SolrQuery();
       query.setStart(-1).setQuery("*");
       QueryResponse resp = query(query);
-      fail("Expected the last query to fail, but got response: " + resp);
-    } catch (SolrException e) {
-      assertEquals(ErrorCode.BAD_REQUEST.code, e.code());
-    }
+    });
+    assertEquals(ErrorCode.BAD_REQUEST.code, e2.code());
 
-    try {
+    SolrException e3 = expectThrows(SolrException.class, () -> {
       SolrQuery query = new SolrQuery();
       query.setRows(-1).setStart(0).setQuery("*");
       QueryResponse resp = query(query);
-      fail("Expected the last query to fail, but got response: " + resp);
-    } catch (SolrException e) {
-      assertEquals(ErrorCode.BAD_REQUEST.code, e.code());
-    }
+    });
+    assertEquals(ErrorCode.BAD_REQUEST.code, e3.code());
 
-    try {
+    SolrException e4 = expectThrows(SolrException.class, () -> {
       SolrQuery query = new SolrQuery();
       query.setParam("rows", "non_numeric_value").setQuery("*");
       QueryResponse resp = query(query);
-      fail("Expected the last query to fail, but got response: " + resp);
-    } catch (SolrException e) {
-      assertEquals(ErrorCode.BAD_REQUEST.code, e.code());
-    }
+    });
+    assertEquals(ErrorCode.BAD_REQUEST.code, e4.code());
+
     resetExceptionIgnores();
   }
 }
