@@ -21,7 +21,6 @@ import java.nio.file.Files;
 import java.nio.file.Path;
 import java.nio.file.Paths;
 import java.util.EnumSet;
-import java.util.List;
 import java.util.Set;
 import java.util.concurrent.Semaphore;
 import java.util.concurrent.TimeUnit;
@@ -36,8 +35,8 @@ import org.apache.solr.client.solrj.request.CoreStatus;
 import org.apache.solr.cloud.overseer.OverseerAction;
 import org.apache.solr.common.SolrException;
 import org.apache.solr.common.SolrInputDocument;
-import org.apache.solr.common.cloud.CollectionStateWatcher;
 import org.apache.solr.common.cloud.DocCollection;
+import org.apache.solr.common.cloud.DocCollectionWatcher;
 import org.apache.solr.common.cloud.Replica;
 import org.apache.solr.common.cloud.Slice;
 import org.apache.solr.common.cloud.ZkNodeProps;
@@ -47,7 +46,6 @@ import org.apache.solr.common.util.TimeSource;
 import org.apache.solr.common.util.Utils;
 import org.apache.solr.core.ZkContainer;
 import org.apache.solr.util.TimeOut;
-import org.apache.zookeeper.KeeperException;
 import org.junit.After;
 import org.junit.Before;
 import org.junit.BeforeClass;
@@ -90,7 +88,7 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
   }
 
   @Test
-  @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
+  // commented out on: 01-Apr-2019   @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
   public void deleteLiveReplicaTest() throws Exception {
 
     final String collectionName = "delLiveColl";
@@ -102,7 +100,11 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
 
     DocCollection state = getCollectionState(collectionName);
     Slice shard = getRandomShard(state);
-    Replica replica = getRandomReplica(shard, (r) -> r.getState() == Replica.State.ACTIVE);
+    
+    // don't choose the leader to shutdown, it just complicates things unneccessarily
+    Replica replica = getRandomReplica(shard, (r) ->
+                                       ( r.getState() == Replica.State.ACTIVE &&
+                                         ! r.equals(shard.getLeader())));
 
     CoreStatus coreStatus = getCoreStatus(replica);
     Path dataDir = Paths.get(coreStatus.getDataDirectory());
@@ -117,15 +119,27 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
 
     JettySolrRunner replicaJetty = cluster.getReplicaJetty(replica);
     ZkStateReaderAccessor accessor = new ZkStateReaderAccessor(replicaJetty.getCoreContainer().getZkController().getZkStateReader());
-    Set<CollectionStateWatcher> watchers = accessor.getStateWatchers(collectionName);
+
+    final long preDeleteWatcherCount = countUnloadCoreOnDeletedWatchers
+      (accessor.getStateWatchers(collectionName));
+    
     CollectionAdminRequest.deleteReplica(collectionName, shard.getName(), replica.getName())
         .process(cluster.getSolrClient());
     waitForState("Expected replica " + replica.getName() + " to have been removed", collectionName, (n, c) -> {
       Slice testShard = c.getSlice(shard.getName());
       return testShard.getReplica(replica.getName()) == null;
     });
-    // the core no longer watch collection state since it was removed
-    assertEquals(watchers.size() - 1, accessor.getStateWatchers(collectionName).size());
+    
+    // the core should no longer have a watch collection state since it was removed
+    // the core should no longer have a watch collection state since it was removed
+    TimeOut timeOut = new TimeOut(60, TimeUnit.SECONDS, TimeSource.NANO_TIME);
+    timeOut.waitFor("Waiting for core's watcher to be removed", () -> {
+        final long postDeleteWatcherCount = countUnloadCoreOnDeletedWatchers
+          (accessor.getStateWatchers(collectionName));
+        log.info("preDeleteWatcherCount={} vs postDeleteWatcherCount={}",
+                 preDeleteWatcherCount, postDeleteWatcherCount);
+        return (preDeleteWatcherCount - 1L == postDeleteWatcherCount);
+      });
 
     assertFalse("Data directory for " + replica.getName() + " should have been removed", Files.exists(dataDir));
 
@@ -167,23 +181,19 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
     CollectionAdminRequest.deleteReplicasFromShard(collectionName, "shard1", 2).process(cluster.getSolrClient());
     waitForState("Expected a single shard with a single replica", collectionName, clusterShape(1, 1));
 
-    try {
-      CollectionAdminRequest.deleteReplicasFromShard(collectionName, "shard1", 1).process(cluster.getSolrClient());
-      fail("Expected Exception, Can't delete the last replica by count");
-    } catch (SolrException e) {
-      // expected
-      assertEquals(SolrException.ErrorCode.BAD_REQUEST.code, e.code());
-      assertTrue(e.getMessage().contains("There is only one replica available"));
-    }
+    SolrException e = expectThrows(SolrException.class,
+        "Can't delete the last replica by count",
+        () -> CollectionAdminRequest.deleteReplicasFromShard(collectionName, "shard1", 1).process(cluster.getSolrClient())
+    );
+    assertEquals(SolrException.ErrorCode.BAD_REQUEST.code, e.code());
+    assertTrue(e.getMessage().contains("There is only one replica available"));
     DocCollection docCollection = getCollectionState(collectionName);
     // We know that since leaders are preserved, PULL replicas should not be left alone in the shard
     assertEquals(0, docCollection.getSlice("shard1").getReplicas(EnumSet.of(Replica.Type.PULL)).size());
-
-
   }
 
   @Test
-  @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
+  // commented out on: 17-Feb-2019   @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
   public void deleteReplicaByCountForAllShards() throws Exception {
 
     final String collectionName = "deleteByCountNew";
@@ -223,19 +233,28 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
     cluster.getSolrClient().add(collectionName, new SolrInputDocument("id", "2"));
     cluster.getSolrClient().commit(collectionName);
 
+    cluster.waitForActiveCollection(collectionName, 1, 3);
+
     Slice shard = getCollectionState(collectionName).getSlice("shard1");
-    Replica replica = getRandomReplica(shard);
+
+    // don't choose the leader to shutdown, it just complicates things unneccessarily
+    Replica replica = getRandomReplica(shard, (r) ->
+                                       ( r.getState() == Replica.State.ACTIVE &&
+                                         ! r.equals(shard.getLeader())));
+    
     JettySolrRunner replicaJetty = cluster.getReplicaJetty(replica);
     ZkStateReaderAccessor accessor = new ZkStateReaderAccessor(replicaJetty.getCoreContainer().getZkController().getZkStateReader());
-    Set<CollectionStateWatcher> watchers = accessor.getStateWatchers(collectionName);
+
+    final long preDeleteWatcherCount = countUnloadCoreOnDeletedWatchers
+      (accessor.getStateWatchers(collectionName));
 
     ZkNodeProps m = new ZkNodeProps(
         Overseer.QUEUE_OPERATION, OverseerAction.DELETECORE.toLower(),
         ZkStateReader.CORE_NAME_PROP, replica.getCoreName(),
         ZkStateReader.NODE_NAME_PROP, replica.getNodeName(),
+        ZkStateReader.BASE_URL_PROP, replicaJetty.getCoreContainer().getZkController().getZkStateReader().getBaseUrlForNodeName(replica.getNodeName()),
         ZkStateReader.COLLECTION_PROP, collectionName,
-        ZkStateReader.CORE_NODE_NAME_PROP, replica.getName(),
-        ZkStateReader.BASE_URL_PROP, replica.getBaseUrl());
+        ZkStateReader.CORE_NODE_NAME_PROP, replica.getName());
 
     cluster.getOpenOverseer().getStateUpdateQueue().offer(Utils.toJSON(m));
 
@@ -246,18 +265,23 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
     timeOut.waitFor("Waiting for replica get unloaded", () ->
         replicaJetty.getCoreContainer().getCoreDescriptor(replica.getCoreName()) == null
     );
-    // the core no longer watch collection state since it was removed
+    
+    // the core should no longer have a watch collection state since it was removed
     timeOut = new TimeOut(60, TimeUnit.SECONDS, TimeSource.NANO_TIME);
-    timeOut.waitFor("Waiting for watcher get removed", () ->
-        watchers.size() - 1 == accessor.getStateWatchers(collectionName).size()
-    );
-
+    timeOut.waitFor("Waiting for core's watcher to be removed", () -> {
+        final long postDeleteWatcherCount = countUnloadCoreOnDeletedWatchers
+          (accessor.getStateWatchers(collectionName));
+        log.info("preDeleteWatcherCount={} vs postDeleteWatcherCount={}",
+                 preDeleteWatcherCount, postDeleteWatcherCount);
+        return (preDeleteWatcherCount - 1L == postDeleteWatcherCount);
+      });
+    
     CollectionAdminRequest.deleteCollection(collectionName).process(cluster.getSolrClient());
   }
 
   @Test
   @Slow
-  @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
+  // commented out on: 17-Feb-2019   @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
   public void raceConditionOnDeleteAndRegisterReplica() throws Exception {
     raceConditionOnDeleteAndRegisterReplica("false");
     CollectionAdminRequest.setClusterProperty(ZkStateReader.LEGACY_CLOUD, null).process(cluster.getSolrClient());
@@ -265,13 +289,13 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
   
   @Test
   @Slow
-  @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
+  // commented out on: 17-Feb-2019   @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
   public void raceConditionOnDeleteAndRegisterReplicaLegacy() throws Exception {
     raceConditionOnDeleteAndRegisterReplica("true");
     CollectionAdminRequest.setClusterProperty(ZkStateReader.LEGACY_CLOUD, null).process(cluster.getSolrClient());
   }
 
-  @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
+  // commented out on: 17-Feb-2019   @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
   public void raceConditionOnDeleteAndRegisterReplica(String legacyCloud) throws Exception {
     
     CollectionAdminRequest.setClusterProperty(ZkStateReader.LEGACY_CLOUD, legacyCloud).process(cluster.getSolrClient());
@@ -310,9 +334,9 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
               Overseer.QUEUE_OPERATION, OverseerAction.DELETECORE.toLower(),
               ZkStateReader.CORE_NAME_PROP, replica1.getCoreName(),
               ZkStateReader.NODE_NAME_PROP, replica1.getNodeName(),
+              ZkStateReader.BASE_URL_PROP, replica1Jetty.getCoreContainer().getZkController().getZkStateReader().getBaseUrlForNodeName(replica1.getNodeName()),
               ZkStateReader.COLLECTION_PROP, collectionName,
-              ZkStateReader.CORE_NODE_NAME_PROP, replica1.getName(),
-              ZkStateReader.BASE_URL_PROP, replica1.getBaseUrl());
+              ZkStateReader.CORE_NODE_NAME_PROP, replica1.getName());
           cluster.getOpenOverseer().getStateUpdateQueue().offer(Utils.toJSON(m));
 
           boolean replicaDeleted = false;
@@ -386,6 +410,7 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
     });
 
     leaderJetty.start();
+    cluster.waitForActiveCollection(collectionName, 1, 2);
 
     CollectionAdminRequest.deleteCollection(collectionName).process(cluster.getSolrClient());
   }
@@ -442,22 +467,25 @@ public class DeleteReplicaTest extends SolrCloudTestCase {
     try {
       cluster.getSolrClient().waitForState(collectionName, 20, TimeUnit.SECONDS, (liveNodes, collectionState) -> collectionState.getReplicas().size() == 1);
     } catch (TimeoutException e) {
-      log.info("Timeout wait for state {}", getCollectionState(collectionName));
+      if (log.isInfoEnabled()) {
+        log.info("Timeout wait for state {}", getCollectionState(collectionName));
+      }
       throw e;
     }
+  }
 
-    TimeOut timeOut = new TimeOut(20, TimeUnit.SECONDS, TimeSource.NANO_TIME);
-    timeOut.waitFor("Time out waiting for LIR state get removed", () -> {
-      String lirPath = ZkController.getLeaderInitiatedRecoveryZnodePath(collectionName, "shard1");
-      try {
-        List<String> children = zkClient().getChildren(lirPath, null, true);
-        return children.size() == 0;
-      } catch (KeeperException.NoNodeException e) {
-        return true;
-      } catch (Exception e) {
-        throw new AssertionError(e);
-      }
-    });
+  /** 
+   * Helper method for counting the number of instances of <code>UnloadCoreOnDeletedWatcher</code>
+   * that exist on a given node.
+   *
+   * This is useful for verifying that deleting a replica correctly removed it's watchers.
+   *
+   * (Note: tests should not assert specific values, since multiple replicas may exist on the same 
+   * node. Instead tests should only assert that the number of watchers has decreased by 1 per known 
+   * replica removed)
+   */
+  private static final long countUnloadCoreOnDeletedWatchers(final Set<DocCollectionWatcher> watchers) {
+    return watchers.stream().filter(w -> w instanceof ZkController.UnloadCoreOnDeletedWatcher).count();
   }
 }
 
