@@ -50,8 +50,9 @@ import org.apache.solr.client.solrj.response.QueryResponse;
 import org.apache.solr.client.solrj.response.schema.SchemaResponse;
 import org.apache.solr.common.SolrDocumentList;
 import org.apache.solr.common.SolrInputDocument;
-import org.junit.After;
+import org.junit.AfterClass;
 import org.junit.Before;
+import org.junit.BeforeClass;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.RuleChain;
@@ -76,8 +77,8 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
   volatile static List<FieldProps> fieldsToTestGroupSortFirst = null;
   volatile static List<FieldProps> fieldsToTestGroupSortLast = null;
 
-  @Before
-  public void createCluster() throws Exception {
+  @BeforeClass
+  public static void createCluster() throws Exception {
     System.setProperty("managed.schema.mutable", "true");
     configureCluster(2)
         .addConfig("conf1", TEST_PATH().resolve("configsets").resolve("cloud-managed").resolve("conf"))
@@ -98,7 +99,8 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
             new FieldProps("floatField", "float", 1),
             new FieldProps("dateField", "date", 1),
             new FieldProps("stringField", "string", 1),
-            new FieldProps("boolField", "boolean", 1)
+            new FieldProps("boolField", "boolean", 1),
+            new FieldProps("sortableText", "sortabletext", 1)
         ));
 
     fieldsToTestMulti =
@@ -109,10 +111,11 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
             new FieldProps("floatFieldMulti", "float", 5),
             new FieldProps("dateFieldMulti", "date", 5),
             new FieldProps("stringFieldMulti", "string", 5),
-            new FieldProps("boolFieldMulti", "boolean", 2)
+            new FieldProps("boolFieldMulti", "boolean", 2),
+            new FieldProps("sortableFieldMulti", "sortabletext", 5)
         ));
 
-    // Fields to test for grouping and sorting with sortMinssingFirst/Last.
+    // Fields to test for grouping and sorting with sortMissingFirst/Last.
     fieldsToTestGroupSortFirst =
         Collections.unmodifiableList(Arrays.asList(
             new FieldProps("intGSF", "int"),
@@ -121,7 +124,8 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
             new FieldProps("floatGSF", "float"),
             new FieldProps("dateGSF", "date"),
             new FieldProps("stringGSF", "string"),
-            new FieldProps("boolGSF", "boolean")
+            new FieldProps("boolGSF", "boolean"),
+            new FieldProps("sortableGSF", "sortabletext")
         ));
 
     fieldsToTestGroupSortLast =
@@ -132,7 +136,8 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
             new FieldProps("floatGSL", "float"),
             new FieldProps("dateGSL", "date"),
             new FieldProps("stringGSL", "string"),
-            new FieldProps("boolGSL", "boolean")
+            new FieldProps("boolGSL", "boolean"),
+            new FieldProps("sortableGSL", "sortabletext")
         ));
 
     List<Update> updateList = new ArrayList<>(fieldsToTestSingle.size() +
@@ -164,16 +169,28 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
   }
 
 
-  @After
-  public void after() throws Exception {
+  @AfterClass
+  public static void shutdown() throws Exception {
     shutdownCluster();
-    
-    fieldsToTestSingle = null;
-    fieldsToTestMulti = null;
-    fieldsToTestGroupSortFirst = null;
-    fieldsToTestGroupSortLast = null;
   }
 
+  @Before
+  public void clean() throws IOException, SolrServerException {
+    CloudSolrClient client = cluster.getSolrClient();
+    client.deleteByQuery("*:*");
+    client.commit();
+    resetFields(fieldsToTestSingle);
+    resetFields(fieldsToTestMulti);
+    resetFields(fieldsToTestGroupSortFirst);
+    resetFields(fieldsToTestGroupSortLast);
+
+  }
+
+  void resetFields(List<FieldProps> fieldProps) {
+    for (FieldProps prop : fieldProps) {
+      prop.resetBase();
+    }
+  }
   @Test
   public void testDistribFaceting() throws IOException, SolrServerException {
     // For this test, I want to insure that there are shards that do _not_ have a doc with any of the DV_only 
@@ -210,18 +227,16 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
     final QueryResponse rsp = client.query(COLLECTION, solrQuery);
 
     for (FieldProps props : fieldsToTestSingle) {
-      testFacet(props, rsp);
+      doTestFacet(props, rsp);
     }
 
     for (FieldProps props : fieldsToTestMulti) {
-      testFacet(props, rsp);
+      doTestFacet(props, rsp);
     }
-
   }
 
   // We should be able to sort thing with missing first/last and that are _NOT_ present at all on one server.
   @Test
-  @AwaitsFix(bugUrl = "https://issues.apache.org/jira/browse/SOLR-12028")
   public void testGroupingSorting() throws IOException, SolrServerException {
     CloudSolrClient client = cluster.getSolrClient();
 
@@ -239,7 +254,7 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
     new UpdateRequest()
         .add(docs)
         .commit(client, COLLECTION);
-    
+
     checkSortOrder(client, fieldsToTestGroupSortFirst, "asc", new String[]{"4", "2", "1", "3"}, new String[]{"4", "1", "2", "3"});
     checkSortOrder(client, fieldsToTestGroupSortFirst, "desc", new String[]{"3", "1", "2", "4"}, new String[]{"2", "3", "1", "4"});
 
@@ -267,7 +282,6 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
   }
 
   @Test
-  // 12-Jun-2018 @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028")
   public void testGroupingDocAbsent() throws IOException, SolrServerException {
     List<SolrInputDocument> docs = new ArrayList<>(4);
     docs.add(makeGSDoc(2, fieldsToTestGroupSortFirst, null));
@@ -299,11 +313,9 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
       GroupCommand fieldCommand = commands.get(0);
       int expected = 4;
       if (prop.getName().startsWith("bool")) expected = 3; //true, false and null
-      
+
       List<Group> fieldCommandGroups = fieldCommand.getValues();
-      if (!prop.getName().startsWith("intGSF")) { // TODO: can be 3 or 4
-        assertEquals("Did not find the expected number of groups for field " + prop.getName(), expected, fieldCommandGroups.size());
-      }
+      assertEquals("Did not find the expected number of groups for field " + prop.getName(), expected, fieldCommandGroups.size());
     }
   }
 
@@ -311,18 +323,29 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
   // Verify that we actually form groups that are "expected". Most of the processing takes some care to 
   // make sure all the values for each field are unique. We need to have docs that have values that are _not_
   // unique.
-  // 12-Jun-2018 @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 04-May-2018
-  // commented 15-Sep-2018 @LuceneTestCase.BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 2-Aug-2018
-  public void testGroupingDVOnly() throws IOException, SolrServerException {
+  public void testGroupingDVOnlySortFirst() throws IOException, SolrServerException {
+    doGroupingDvOnly(fieldsToTestGroupSortFirst, "boolGSF");
+  }
+
+  @Test
+  public void testGroupingDVOnlySortLast() throws IOException, SolrServerException {
+    doGroupingDvOnly(fieldsToTestGroupSortLast, "boolGSL");
+  }
+
+  private void doGroupingDvOnly(List<FieldProps> fieldProps, String boolName) throws IOException, SolrServerException {
     List<SolrInputDocument> docs = new ArrayList<>(50);
     for (int idx = 0; idx < 49; ++idx) {
       SolrInputDocument doc = new SolrInputDocument();
       doc.addField("id", idx);
-      boolean doInc = ((idx % 7) == 0);
-      for (FieldProps prop : fieldsToTestGroupSortFirst) {
-        doc.addField(prop.getName(), prop.getValue(doInc));
+
+      // Every 7th doc we bump a counter by some random amount
+      for (FieldProps prop : fieldProps) {
+        doc.addField(prop.getName(), prop.getValue((idx % 7) == 0));
       }
       docs.add(doc);
+
+      // Every fifth time through we add a doc with no values in any of the "fields of interest", so there should be
+      // 10 docs with nulls
       if ((idx % 5) == 0) {
         doc = new SolrInputDocument();
         doc.addField("id", idx + 10_000);
@@ -337,20 +360,18 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
         .commit(client, COLLECTION);
 
     // OK, we should have one group with 10 entries for null, a group with 1 entry and 7 groups with 7
-    for (FieldProps prop : fieldsToTestGroupSortFirst) {
-      // Special handling until SOLR-9802 is fixed
+    for (FieldProps prop : fieldProps) {
+
+      // Solr 9802
       if (prop.getName().startsWith("date")) continue;
-      // SOLR-9802 to here
-      
-      // TODO: gsf fails this
-      if (prop.getName().endsWith("GSF") ) continue;
 
       final SolrQuery solrQuery = new SolrQuery(
           "q", "*:*",
           "rows", "100",
           "group", "true",
           "group.field", prop.getName(),
-          "group.limit", "100");
+          "group.limit", "100",
+          "group.sort", "id asc");
 
       final QueryResponse rsp = client.query(COLLECTION, solrQuery);
 
@@ -376,13 +397,13 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
             case 25:
             case 24:
               ++boolCount;
-              assertEquals("We should have more counts for boolean fields!", "boolGSF", prop.getName());
+              assertEquals("We should have more counts for boolean fields!", boolName, prop.getName());
               break;
             
             default:
-              if (!prop.getName().equals("intGSF")) { // TODO: this can be 6 or 8 as well
-                fail("Unexpected number of elements in the group for " + prop.getName() + ": " + grp.getResult().size() + " rsp: " + rsp);
-              }
+              fail("Unexpected number of elements in the group for '" + prop.getName() + "' size: '" + grp.getResult().size()
+                  + "' GroupValue: '" + grp.getGroupValue()
+                  + "' rsp: " + rsp);
           }
         }
       }
@@ -442,7 +463,7 @@ public class DocValuesNotIndexedTest extends SolrCloudTestCase {
   }
 
 
-  private void testFacet(FieldProps props, QueryResponse rsp) {
+  private void doTestFacet(FieldProps props, QueryResponse rsp) {
     String name = props.getName();
     final List<FacetField.Count> counts = rsp.getFacetField(name).getValues();
     long expectedCount = props.getExpectedCount();
@@ -470,20 +491,22 @@ class FieldProps {
     this.expectedCount = expectedCount;
     resetBase();
   }
+
+  // There's a vague chance that counts will roll over, so let's insure we have some room to grow in a positive direction
   void resetBase() {
     if (name.startsWith("int")) {
-      base = Math.abs(random().nextInt());
+      base = Math.abs(random().nextInt()) / 2;
     } else if (name.startsWith("long")) {
-      base = Math.abs(random().nextLong());
+      base = Math.abs(random().nextLong()) / 2;
     } else if (name.startsWith("float")) {
-      base = Math.abs(random().nextFloat());
+      base = Math.abs(random().nextFloat()) / 2;
     } else if (name.startsWith("double")) {
-      base = Math.abs(random().nextDouble());
+      base = Math.abs(random().nextDouble()) / 2;
     } else if (name.startsWith("date")) {
-      base = Math.abs(random().nextLong());
+      base = Math.abs(random().nextLong()) / 2;
     } else if (name.startsWith("bool")) {
       base = true; // Must start with a known value since bools only have a two values....
-    } else if (name.startsWith("string")) {
+    } else if (name.startsWith("string") || name.startsWith("sortable")) {
       base = "base_string_" + random().nextInt(1_000_000) + "_";
     } else {
       throw new RuntimeException("Should have found a prefix for the field before now!");
@@ -509,7 +532,7 @@ class FieldProps {
 
   public String getValue(boolean incrementCounter) {
     if (incrementCounter) {
-      counter += random().nextInt(10) + 10_000;
+      counter += random().nextInt(10_000) + 1; // Must add something because nextInt could return zero
     }
     if (name.startsWith("int")) {
       return Integer.toString((int) base + counter);
@@ -531,7 +554,7 @@ class FieldProps {
       base = !((boolean) base);
       return ret;
     }
-    if (name.startsWith("string")) {
+    if (name.startsWith("string") || name.startsWith("sortable")) {
       return String.format(Locale.ROOT, "%s_%08d", (String) base, counter);
     }
     throw new RuntimeException("Should have found a prefix for the field before now!");
