@@ -22,7 +22,6 @@ import static org.apache.solr.common.cloud.Replica.State.RECOVERING;
 import java.io.File;
 import java.io.IOException;
 import java.lang.invoke.MethodHandles;
-import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Collection;
 import java.util.Collections;
@@ -31,7 +30,6 @@ import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Optional;
-import java.util.Properties;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import org.apache.lucene.util.LuceneTestCase.Slow;
@@ -51,7 +49,6 @@ import org.apache.solr.common.cloud.ClusterState;
 import org.apache.solr.common.cloud.DocCollection;
 import org.apache.solr.common.cloud.Replica;
 import org.apache.solr.common.cloud.Slice;
-import org.apache.solr.common.cloud.SolrZkClient;
 import org.apache.solr.common.cloud.ZkCoreNodeProps;
 import org.apache.solr.common.cloud.ZkStateReader;
 import org.apache.solr.common.util.NamedList;
@@ -59,7 +56,6 @@ import org.apache.solr.common.util.TimeSource;
 import org.apache.solr.core.CoreContainer;
 import org.apache.solr.core.SolrCore;
 import org.apache.solr.update.UpdateLog;
-import org.apache.solr.util.MockCoreContainer.MockCoreDescriptor;
 import org.apache.solr.util.RTimer;
 import org.apache.solr.util.TestInjection;
 import org.apache.solr.util.TimeOut;
@@ -88,8 +84,6 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
   // give plenty of time for replicas to recover when running in slow Jenkins test envs
   protected static final int maxWaitSecsToSeeAllActive = 90;
 
-  private final boolean onlyLeaderIndexes = random().nextBoolean();
-
   @BeforeClass
   public static void setupSysProps() {
     System.setProperty("socketTimeout", "10000");
@@ -105,11 +99,6 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
     fixShardCount(3);
   }
 
-  @Override
-  protected boolean useTlogReplicas() {
-    return false; // TODO: tlog replicas makes commits take way to long due to what is likely a bug and it's TestInjection use
-  }
-
   /**
    * We need to turn off directUpdatesToLeadersOnly due to SOLR-9512
    */
@@ -140,8 +129,6 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
   public void test() throws Exception {
     waitForThingsToLevelOut(30000);
 
-    testLeaderInitiatedRecoveryCRUD();
-
     testDoRecoveryOnRestart();
 
     // test a 1x2 collection
@@ -165,62 +152,6 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
     log.info("HttpPartitionTest succeeded ... shutting down now!");
   }
 
-  /**
-   * Tests handling of different format of lir nodes
-   */
-  //TODO remove in SOLR-11812
-  protected void testLeaderInitiatedRecoveryCRUD() throws Exception {
-    String testCollectionName = "c8n_crud_1x2";
-    String shardId = "shard1";
-    createCollectionRetry(testCollectionName, "conf1", 1, 2, 1);
-    cloudClient.setDefaultCollection(testCollectionName);
-
-    Replica leader = cloudClient.getZkStateReader().getLeaderRetry(testCollectionName, shardId);
-    JettySolrRunner leaderJetty = getJettyOnPort(getReplicaPort(leader));
-
-    CoreContainer cores = leaderJetty.getCoreContainer();
-    ZkController zkController = cores.getZkController();
-    assertNotNull("ZkController is null", zkController);
-
-    Replica notLeader =
-        ensureAllReplicasAreActive(testCollectionName, shardId, 1, 2, maxWaitSecsToSeeAllActive).get(0);
-
-    ZkCoreNodeProps replicaCoreNodeProps = new ZkCoreNodeProps(notLeader);
-    String replicaUrl = replicaCoreNodeProps.getCoreUrl();
-
-    MockCoreDescriptor cd = new MockCoreDescriptor() {
-      public CloudDescriptor getCloudDescriptor() {
-        return new CloudDescriptor(leader.getStr(ZkStateReader.CORE_NAME_PROP), new Properties(), this) {
-          @Override
-          public String getCoreNodeName() {
-            return leader.getName();
-          }
-          @Override
-          public boolean isLeader() {
-            return true;
-          }
-        };
-      }
-    };
-
-    zkController.updateLeaderInitiatedRecoveryState(testCollectionName, shardId, notLeader.getName(), DOWN, cd, true);
-    Map<String,Object> lirStateMap = zkController.getLeaderInitiatedRecoveryStateObject(testCollectionName, shardId, notLeader.getName());
-    assertNotNull(lirStateMap);
-    assertSame(DOWN, Replica.State.getState((String) lirStateMap.get(ZkStateReader.STATE_PROP)));
-
-    // test old non-json format handling
-    SolrZkClient zkClient = zkController.getZkClient();
-    String znodePath = zkController.getLeaderInitiatedRecoveryZnodePath(testCollectionName, shardId, notLeader.getName());
-    zkClient.setData(znodePath, "down".getBytes(StandardCharsets.UTF_8), true);
-    lirStateMap = zkController.getLeaderInitiatedRecoveryStateObject(testCollectionName, shardId, notLeader.getName());
-    assertNotNull(lirStateMap);
-    assertSame(DOWN, Replica.State.getState((String) lirStateMap.get(ZkStateReader.STATE_PROP)));
-    zkClient.delete(znodePath, -1, false);
-
-    // try to clean up
-    attemptCollectionDelete(cloudClient, testCollectionName);
-  }
-
   private void testDoRecoveryOnRestart() throws Exception {
     String testCollectionName = "collDoRecoveryOnRestart";
     try {
@@ -329,10 +260,10 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
       maxVersionBefore = ulog.getCurrentMaxVersion();
     }
     assertNotNull("max version bucket seed not set for core " + coreName, maxVersionBefore);
-    log.info("Looked up max version bucket seed "+maxVersionBefore+" for core "+coreName);
+    log.info("Looked up max version bucket seed {} for core {}", maxVersionBefore, coreName);
 
     // now up the stakes and do more docs
-    int numDocs = TEST_NIGHTLY ? 1000 : 100;
+    int numDocs = TEST_NIGHTLY ? 1000 : 105;
     boolean hasPartition = false;
     for (int d = 0; d < numDocs; d++) {
       // create / restore partition every 100 docs
@@ -365,15 +296,15 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
     try (SolrCore core = coreContainer.getCore(coreName)) {
       assertNotNull("Core '" + coreName + "' not found for replica: " + notLeader.getName(), core);
       Long currentMaxVersion = core.getUpdateHandler().getUpdateLog().getCurrentMaxVersion();
-      log.info("After recovery, looked up NEW max version bucket seed " + currentMaxVersion +
-          " for core " + coreName + ", was: " + maxVersionBefore);
+      log.info("After recovery, looked up NEW max version bucket seed {} for core {}, was: {}"
+          , currentMaxVersion, coreName, maxVersionBefore);
       assertTrue("max version bucket seed not updated after recovery!", currentMaxVersion > maxVersionBefore);
     }
 
     // verify all docs received
     assertDocsExistInAllReplicas(notLeaders, testCollectionName, 1, numDocs + 3);
 
-    log.info("testRf2 succeeded ... deleting the "+testCollectionName+" collection");
+    log.info("testRf2 succeeded ... deleting the {} collection", testCollectionName);
 
     // try to clean up
     attemptCollectionDelete(cloudClient, testCollectionName);
@@ -443,13 +374,14 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
     
     assertDocsExistInAllReplicas(notLeaders, testCollectionName, 1, 4);
 
-    log.info("testRf3 succeeded ... deleting the "+testCollectionName+" collection");
+    log.info("testRf3 succeeded ... deleting the {} collection", testCollectionName);
 
     // try to clean up
     attemptCollectionDelete(cloudClient, testCollectionName);
   }
 
   // test inspired by SOLR-6511
+  @SuppressWarnings({"try"})
   protected void testLeaderZkSessionLoss() throws Exception {
 
     String testCollectionName = "c8n_1x2_leader_session_loss";
@@ -502,7 +434,9 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
 
     // TODO: This test logic seems to be timing dependent and fails on Jenkins
     // need to come up with a better approach
-    log.info("Sending doc 2 to old leader "+leader.getName());
+    if (log.isInfoEnabled()) {
+      log.info("Sending doc 2 to old leader {}", leader.getName());
+    }
     try ( HttpSolrClient leaderSolr = getHttpSolrClient(leader, testCollectionName)) {
     
       leaderSolr.add(doc);
@@ -527,7 +461,7 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
     waitToSeeReplicasActive(testCollectionName, "shard1", replicasToCheck, 30);
     assertDocsExistInAllReplicas(participatingReplicas, testCollectionName, 1, 2);
 
-    log.info("testLeaderZkSessionLoss succeeded ... deleting the "+testCollectionName+" collection");
+    log.info("testLeaderZkSessionLoss succeeded ... deleting the {} collection", testCollectionName);
 
     // try to clean up
     attemptCollectionDelete(cloudClient, testCollectionName);
@@ -551,8 +485,12 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
     List<Replica> replicas = new ArrayList<Replica>();
     replicas.addAll(activeReplicas.values());
     return replicas;
-  }  
+  }
 
+  /**
+   * Assert docs exists in {@code notLeaders} replicas, docs must also exist in the shard1 leader as well.
+   * This method uses RTG for validation therefore it must work for asserting both TLOG and NRT replicas.
+   */
   protected void assertDocsExistInAllReplicas(List<Replica> notLeaders,
       String testCollectionName, int firstDocId, int lastDocId)
       throws Exception {
@@ -630,12 +568,14 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
   }
 
   protected void assertDocNotExists(HttpSolrClient solr, String coll, String docId) throws Exception {
+    @SuppressWarnings({"rawtypes"})
     NamedList rsp = realTimeGetDocId(solr, docId);
     String match = JSONTestUtil.matchObj("/id", rsp.get("doc"), Integer.valueOf(docId));
     assertTrue("Doc with id=" + docId + " is found in " + solr.getBaseURL()
         + " due to: " + match + "; rsp="+rsp, match != null);
   }
 
+  @SuppressWarnings({"rawtypes"})
   private NamedList realTimeGetDocId(HttpSolrClient solr, String docId) throws SolrServerException, IOException {
     QueryRequest qr = new QueryRequest(params("qt", "/get", "id", docId, "distrib", "false"));
     return solr.request(qr);
@@ -674,7 +614,9 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
 
         final Replica.State state = replica.getState();
         if (state != Replica.State.ACTIVE) {
-          log.info("Replica " + replica.getName() + " is currently " + state);
+          if (log.isInfoEnabled()) {
+            log.info("Replica {} is currently {}", replica.getName(), state);
+          }
           allReplicasUp = false;
         }
       }
@@ -691,7 +633,9 @@ public class HttpPartitionTest extends AbstractFullDistribZkTestBase {
       fail("Didn't see replicas "+ replicasToCheck +
           " come up within " + maxWaitMs + " ms! ClusterState: " + printClusterStateInfo(testCollectionName));
 
-    log.info("Took {} ms to see replicas [{}] become active.", timer.getTime(), replicasToCheck);
+    if (log.isInfoEnabled()) {
+      log.info("Took {} ms to see replicas [{}] become active.", timer.getTime(), replicasToCheck);
+    }
   }
 
 }
