@@ -26,7 +26,6 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
-import org.apache.commons.lang.StringUtils;
 import org.apache.lucene.util.TestUtil;
 import org.apache.solr.SolrTestCaseJ4.SuppressSSL;
 import org.apache.solr.client.solrj.SolrServerException;
@@ -362,7 +361,7 @@ public class TestCloudPivotFacet extends AbstractFullDistribZkTestBase {
       // no stats for this pivot, nothing to check
 
       // TODO: use a trace param to know if/how-many to expect ?
-      log.info("No stats to check for => " + message);
+      log.info("No stats to check for => {}", message);
       return;
     }
     
@@ -380,7 +379,7 @@ public class TestCloudPivotFacet extends AbstractFullDistribZkTestBase {
         // StatsComponent results being "null" (and not even included in the 
         // getFieldStatsInfo() Map due to specila SolrJ logic) 
 
-        log.info("Requested stats missing in verification query, pivot stats: " + pivotStats);
+        log.info("Requested stats missing in verification query, pivot stats: {}", pivotStats);
         assertEquals("Special Count", 0L, pivotStats.getCount().longValue());
         assertEquals("Special Missing", 
                      constraint.getCount(), pivotStats.getMissing().longValue());
@@ -496,7 +495,7 @@ public class TestCloudPivotFacet extends AbstractFullDistribZkTestBase {
       // makes it a robust test (especially for multi-valued fields)
       fields[i] = fieldNames[TestUtil.nextInt(random(),0,fieldNames.length-1)];
     }
-    return StringUtils.join(fields, ",");
+    return String.join(",", fields);
   }
 
   /**
