@@ -16,7 +16,6 @@
  */
 package org.apache.solr.cloud;
 
-import junit.framework.Assert;
 import org.apache.hadoop.util.Time;
 import org.apache.lucene.util.LuceneTestCase;
 import org.apache.solr.SolrTestCaseJ4;
@@ -26,10 +25,13 @@ import org.apache.solr.client.solrj.impl.LBHttpSolrClient;
 import org.apache.solr.client.solrj.SolrClient;
 import org.apache.solr.client.solrj.SolrRequest;
 import org.apache.solr.client.solrj.embedded.JettySolrRunner;
+import org.apache.solr.client.solrj.request.AbstractUpdateRequest.ACTION;
 import org.apache.solr.client.solrj.request.CollectionAdminRequest;
 import org.apache.solr.client.solrj.request.DelegationTokenRequest;
+import org.apache.solr.client.solrj.request.UpdateRequest;
 import org.apache.solr.client.solrj.response.DelegationTokenResponse;
 import org.apache.solr.common.SolrException.ErrorCode;
+import org.apache.solr.common.SolrInputDocument;
 import org.apache.solr.common.cloud.SolrZkClient;
 import org.apache.solr.common.params.SolrParams;
 import org.apache.solr.common.params.ModifiableSolrParams;
@@ -83,12 +85,16 @@ public class TestSolrCloudWithDelegationTokens extends SolrTestCaseJ4 {
   public static void shutdown() throws Exception {
     if (miniCluster != null) {
       miniCluster.shutdown();
+      miniCluster = null;
+    }
+    if (null != solrClientPrimary) {
+      solrClientPrimary.close();
+      solrClientPrimary = null;
+    }
+    if (null != solrClientSecondary) {
+      solrClientSecondary.close();
+      solrClientSecondary = null;
     }
-    miniCluster = null;
-    solrClientPrimary.close();
-    solrClientPrimary = null;
-    solrClientSecondary.close();
-    solrClientSecondary = null;
     System.clearProperty("authenticationPlugin");
     System.clearProperty(KerberosPlugin.DELEGATION_TOKEN_ENABLED);
     System.clearProperty("solr.kerberos.cookie.domain");
@@ -160,9 +166,10 @@ public class TestSolrCloudWithDelegationTokens extends SolrTestCaseJ4 {
       }
       Thread.sleep(1000);
     }
-    assertEquals("Did not receieve excepted status code", expectedStatusCode, lastStatusCode);
+    assertEquals("Did not receive expected status code", expectedStatusCode, lastStatusCode);
   }
 
+  @SuppressWarnings({"rawtypes"})
   private SolrRequest getAdminRequest(final SolrParams params) {
     return new CollectionAdminRequest.List() {
       @Override
@@ -173,7 +180,19 @@ public class TestSolrCloudWithDelegationTokens extends SolrTestCaseJ4 {
       }
     };
   }
+  @SuppressWarnings({"rawtypes"})
+  private SolrRequest getUpdateRequest(boolean commit) {
+    UpdateRequest request = new UpdateRequest();
+    if (commit) {
+      request.setAction(ACTION.COMMIT, false, false);
+    }
+    SolrInputDocument doc = new SolrInputDocument();
+    doc.addField("id", "dummy_id");
+    request.add(doc);
+    return request;
+  }
 
+  @SuppressWarnings({"unchecked"})
   private int getStatusCode(String token, final String user, final String op, HttpSolrClient client)
   throws Exception {
     SolrClient delegationTokenClient;
@@ -194,6 +213,7 @@ public class TestSolrCloudWithDelegationTokens extends SolrTestCaseJ4 {
       ModifiableSolrParams p = new ModifiableSolrParams();
       if (user != null) p.set(USER_PARAM, user);
       if (op != null) p.set("op", op);
+      @SuppressWarnings({"rawtypes"})
       SolrRequest req = getAdminRequest(p);
       if (user != null || op != null) {
         Set<String> queryParams = new HashSet<>();
@@ -212,7 +232,8 @@ public class TestSolrCloudWithDelegationTokens extends SolrTestCaseJ4 {
     }
   }
 
-  private void doSolrRequest(HttpSolrClient client, SolrRequest request,
+  private void doSolrRequest(HttpSolrClient client,
+                             @SuppressWarnings({"rawtypes"})SolrRequest request,
       int expectedStatusCode) throws Exception {
     try {
       client.request(request);
@@ -222,6 +243,17 @@ public class TestSolrCloudWithDelegationTokens extends SolrTestCaseJ4 {
     }
   }
 
+  private void doSolrRequest(HttpSolrClient client,
+                             @SuppressWarnings({"rawtypes"})SolrRequest request, String collectionName,
+      int expectedStatusCode) throws Exception {
+    try {
+      client.request(request, collectionName);
+      assertEquals(HttpStatus.SC_OK, expectedStatusCode);
+    } catch (HttpSolrClient.RemoteSolrException ex) {
+      assertEquals(expectedStatusCode, ex.code());
+    }
+  }
+
   private void verifyTokenValid(String token) throws Exception {
      // pass with token
     doSolrRequest(token, HttpStatus.SC_OK, solrClientPrimary);
@@ -382,36 +414,75 @@ public class TestSolrCloudWithDelegationTokens extends SolrTestCaseJ4 {
     String token = getDelegationToken(null, "bar", solrClientPrimary);
     assertNotNull(token);
 
+    @SuppressWarnings({"rawtypes"})
     SolrRequest request = getAdminRequest(new ModifiableSolrParams());
 
     // test without token
-    HttpSolrClient ss =
-        new HttpSolrClient.Builder(solrClientPrimary.getBaseURL().toString())
-            .withResponseParser(solrClientPrimary.getParser())
-            .build();
+    final HttpSolrClient ssWoToken =
+      new HttpSolrClient.Builder(solrClientPrimary.getBaseURL().toString())
+          .withResponseParser(solrClientPrimary.getParser())
+          .build();
     try {
-      doSolrRequest(ss, request, ErrorCode.UNAUTHORIZED.code);
+      doSolrRequest(ssWoToken, request, ErrorCode.UNAUTHORIZED.code);
     } finally {
-      ss.close();
+      ssWoToken.close();
     }
 
-    ss = new HttpSolrClient.Builder(solrClientPrimary.getBaseURL().toString())
+    final HttpSolrClient ssWToken = new HttpSolrClient.Builder(solrClientPrimary.getBaseURL().toString())
         .withKerberosDelegationToken(token)
         .withResponseParser(solrClientPrimary.getParser())
         .build();
     try {
       // test with token via property
-      doSolrRequest(ss, request, HttpStatus.SC_OK);
+      doSolrRequest(ssWToken, request, HttpStatus.SC_OK);
 
       // test with param -- should throw an exception
       ModifiableSolrParams tokenParam = new ModifiableSolrParams();
       tokenParam.set("delegation", "invalidToken");
-      try {
-        doSolrRequest(ss, getAdminRequest(tokenParam), ErrorCode.FORBIDDEN.code);
-        Assert.fail("Expected exception");
-      } catch (IllegalArgumentException ex) {}
+      expectThrows(IllegalArgumentException.class,
+          () -> doSolrRequest(ssWToken, getAdminRequest(tokenParam), ErrorCode.FORBIDDEN.code));
     } finally {
-      ss.close();
+      ssWToken.close();
     }
   }
+
+  /**
+   * Test HttpSolrServer's delegation token support for Update Requests
+   */
+  @Test
+  public void testDelegationTokenSolrClientWithUpdateRequests() throws Exception {
+    String collectionName = "testDelegationTokensWithUpdate";
+
+    // Get token
+    String token = getDelegationToken(null, "bar", solrClientPrimary);
+    assertNotNull(token);
+
+    // Tests with update request.
+    // Before SOLR-13921, the request without commit will fail with a NullpointerException in DelegationTokenHttpSolrClient.createMethod
+    // due to a missing null check in the createMethod. (When requesting a commit, the setAction method will call setParams on the
+    // request so there is no NPE in the createMethod.)
+    final HttpSolrClient scUpdateWToken = new HttpSolrClient.Builder(solrClientPrimary.getBaseURL().toString())
+        .withKerberosDelegationToken(token)
+        .withResponseParser(solrClientPrimary.getParser())
+        .build();
+
+    // Create collection
+    CollectionAdminRequest.Create create = CollectionAdminRequest.createCollection(collectionName, 1, 1);
+    create.process(scUpdateWToken);
+
+    try {
+      // test update request with token via property and commit=true
+      @SuppressWarnings({"rawtypes"})
+      SolrRequest request = getUpdateRequest(true);
+      doSolrRequest(scUpdateWToken, request, collectionName, HttpStatus.SC_OK);
+
+      // test update request with token via property and commit=false
+      request = getUpdateRequest(false);
+      doSolrRequest(scUpdateWToken, request, collectionName, HttpStatus.SC_OK);
+
+    } finally {
+      scUpdateWToken.close();
+    }
+  }
+
 }
