@@ -16,9 +16,9 @@
  */
 package org.apache.solr.cloud.api.collections;
 
-import static org.apache.solr.common.cloud.ZkStateReader.CORE_NAME_PROP;
-import static org.apache.solr.common.cloud.ZkStateReader.REPLICATION_FACTOR;
-
+import javax.management.MBeanServer;
+import javax.management.MBeanServerFactory;
+import javax.management.ObjectName;
 import java.io.IOException;
 import java.lang.invoke.MethodHandles;
 import java.lang.management.ManagementFactory;
@@ -38,11 +38,7 @@ import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
 
-import javax.management.MBeanServer;
-import javax.management.MBeanServerFactory;
-import javax.management.ObjectName;
-
-import org.apache.commons.io.IOUtils;
+import com.google.common.collect.ImmutableList;
 import org.apache.lucene.util.LuceneTestCase.Slow;
 import org.apache.lucene.util.TestUtil;
 import org.apache.solr.client.solrj.SolrQuery;
@@ -70,7 +66,6 @@ import org.apache.solr.common.params.CollectionParams.CollectionAction;
 import org.apache.solr.common.params.CoreAdminParams;
 import org.apache.solr.common.params.ModifiableSolrParams;
 import org.apache.solr.common.util.NamedList;
-import org.apache.solr.common.util.SimpleOrderedMap;
 import org.apache.solr.common.util.TimeSource;
 import org.apache.solr.core.CoreContainer;
 import org.apache.solr.core.SolrCore;
@@ -83,7 +78,8 @@ import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
-import com.google.common.collect.ImmutableList;
+import static org.apache.solr.common.cloud.ZkStateReader.CORE_NAME_PROP;
+import static org.apache.solr.common.cloud.ZkStateReader.REPLICATION_FACTOR;
 
 /**
  * Tests the Cloud Collections API.
@@ -92,6 +88,10 @@ import com.google.common.collect.ImmutableList;
 public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
+  protected String getConfigSet() {
+    return "cloud-minimal";
+  }
+
   @Before
   public void setupCluster() throws Exception {
     // we don't want this test to have zk timeouts
@@ -99,24 +99,28 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     System.setProperty("createCollectionWaitTimeTillActive", "5");
     TestInjection.randomDelayInCoreCreation = "true:5";
     System.setProperty("validateAfterInactivity", "200");
-    
-    String solrXml = IOUtils.toString(CollectionsAPIDistributedZkTest.class.getResourceAsStream("/solr/solr-jmxreporter.xml"), "UTF-8");
+    System.setProperty("solr.allowPaths", "*");
+
     configureCluster(4)
-        .addConfig("conf", configset("cloud-minimal"))
-        .addConfig("conf2", configset("cloud-minimal-jmx"))
-        .withSolrXml(solrXml)
+        .addConfig("conf", configset(getConfigSet()))
+        .addConfig("conf2", configset(getConfigSet()))
+        .withSolrXml(TEST_PATH().resolve("solr.xml"))
         .configure();
   }
   
   @After
   public void tearDownCluster() throws Exception {
-    shutdownCluster();
-    System.clearProperty("createCollectionWaitTimeTillActive");
+    try {
+      shutdownCluster();
+    } finally {
+      System.clearProperty("createCollectionWaitTimeTillActive");
+      System.clearProperty("solr.allowPaths");
+      super.tearDown();
+    }
   }
 
   @Test
   public void testCreationAndDeletion() throws Exception {
-
     String collectionName = "created_and_deleted";
 
     CollectionAdminRequest.createCollection(collectionName, "conf", 1, 1).process(cluster.getSolrClient());
@@ -128,13 +132,10 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
         .contains(collectionName));
 
     assertFalse(cluster.getZkClient().exists(ZkStateReader.COLLECTIONS_ZKNODE + "/" + collectionName, true));
-
-
   }
 
   @Test
   public void deleteCollectionRemovesStaleZkCollectionsNode() throws Exception {
-    
     String collectionName = "out_of_sync_collection";
 
     // manually create a collections zknode
@@ -147,12 +148,10 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
                   .contains(collectionName));
     
     assertFalse(cluster.getZkClient().exists(ZkStateReader.COLLECTIONS_ZKNODE + "/" + collectionName, true));
-
   }
 
   @Test
   public void deletePartiallyCreatedCollection() throws Exception {
-
     final String collectionName = "halfdeletedcollection";
 
     assertEquals(0, CollectionAdminRequest.createCollection(collectionName, "conf", 2, 1)
@@ -174,12 +173,10 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
         .process(cluster.getSolrClient());
 
     assertTrue(CollectionAdminRequest.listCollections(cluster.getSolrClient()).contains(collectionName));
-
   }
 
   @Test
   public void deleteCollectionOnlyInZk() throws Exception {
-
     final String collectionName = "onlyinzk";
 
     // create the collections node, but nothing else
@@ -193,12 +190,10 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     CollectionAdminRequest.createCollection(collectionName, "conf", 2, 1)
         .process(cluster.getSolrClient());
     assertTrue(CollectionAdminRequest.listCollections(cluster.getSolrClient()).contains(collectionName));
-
   }
 
   @Test
-  public void testBadActionNames() throws Exception {
-
+  public void testBadActionNames() {
     // try a bad action
     ModifiableSolrParams params = new ModifiableSolrParams();
     params.set("action", "BADACTION");
@@ -211,16 +206,15 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     expectThrows(Exception.class, () -> {
       cluster.getSolrClient().request(request);
     });
-
   }
 
   @Test
   public void testMissingRequiredParameters() {
-
     ModifiableSolrParams params = new ModifiableSolrParams();
     params.set("action", CollectionAction.CREATE.toString());
     params.set("numShards", 2);
     // missing required collection parameter
+    @SuppressWarnings({"rawtypes"})
     final SolrRequest request = new QueryRequest(params);
     request.setPath("/admin/collections");
 
@@ -231,18 +225,16 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
 
   @Test
   public void testTooManyReplicas() {
-
+    @SuppressWarnings({"rawtypes"})
     CollectionAdminRequest req = CollectionAdminRequest.createCollection("collection", "conf", 2, 10);
 
     expectThrows(Exception.class, () -> {
       cluster.getSolrClient().request(req);
     });
-
   }
 
   @Test
   public void testMissingNumShards() {
-
     // No numShards should fail
     ModifiableSolrParams params = new ModifiableSolrParams();
     params.set("action", CollectionAction.CREATE.toString());
@@ -250,18 +242,17 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     params.set(REPLICATION_FACTOR, 10);
     params.set("collection.configName", "conf");
 
+    @SuppressWarnings({"rawtypes"})
     final SolrRequest request = new QueryRequest(params);
     request.setPath("/admin/collections");
 
     expectThrows(Exception.class, () -> {
       cluster.getSolrClient().request(request);
     });
-
   }
 
   @Test
   public void testZeroNumShards() {
-
     ModifiableSolrParams params = new ModifiableSolrParams();
     params.set("action", CollectionAction.CREATE.toString());
     params.set("name", "acollection");
@@ -269,12 +260,12 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     params.set("numShards", 0);
     params.set("collection.configName", "conf");
 
+    @SuppressWarnings({"rawtypes"})
     final SolrRequest request = new QueryRequest(params);
     request.setPath("/admin/collections");
     expectThrows(Exception.class, () -> {
       cluster.getSolrClient().request(request);
     });
-
   }
 
   @Test
@@ -298,24 +289,15 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     String nn1 = cluster.getJettySolrRunner(0).getNodeName();
     String nn2 = cluster.getJettySolrRunner(1).getNodeName();
 
-    CollectionAdminResponse resp = CollectionAdminRequest.createCollection("halfcollection", "conf", 2, 1)
-        .setCreateNodeSet(nn1 + "," + nn2)
-        .process(cluster.getSolrClient());
-    
-    SimpleOrderedMap success = (SimpleOrderedMap) resp.getResponse().get("success");
-    SimpleOrderedMap failure = (SimpleOrderedMap) resp.getResponse().get("failure");
-
-    assertNotNull(resp.toString(), success);
-    assertNotNull(resp.toString(), failure);
-    
-    String val1 = success.getVal(0).toString();
-    String val2 = failure.getVal(0).toString();
-    assertTrue(val1.contains("SolrException") || val2.contains("SolrException"));
+    expectThrows(HttpSolrClient.RemoteSolrException.class, () -> {
+      CollectionAdminResponse resp = CollectionAdminRequest.createCollection("halfcollection", "conf", 2, 1)
+          .setCreateNodeSet(nn1 + "," + nn2)
+          .process(cluster.getSolrClient());
+    });
   }
 
   @Test
   public void testNoConfigSetExist() throws Exception {
-
     expectThrows(Exception.class, () -> {
       CollectionAdminRequest.createCollection("noconfig", "conf123", 1, 1)
           .process(cluster.getSolrClient());
@@ -329,14 +311,12 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
 
   @Test
   public void testCoresAreDistributedAcrossNodes() throws Exception {
-
     CollectionAdminRequest.createCollection("nodes_used_collection", "conf", 2, 2)
         .process(cluster.getSolrClient());
 
     Set<String> liveNodes = cluster.getSolrClient().getZkStateReader().getClusterState().getLiveNodes();
 
-    List<String> createNodeList = new ArrayList<>();
-    createNodeList.addAll(liveNodes);
+    List<String> createNodeList = new ArrayList<>(liveNodes);
 
     DocCollection collection = getCollectionState("nodes_used_collection");
     for (Slice slice : collection.getSlices()) {
@@ -346,13 +326,12 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     }
 
     assertEquals(createNodeList.toString(), 0, createNodeList.size());
-
   }
 
   @Test
   public void testDeleteNonExistentCollection() throws Exception {
 
-    SolrException e = expectThrows(SolrException.class, () -> {
+    expectThrows(SolrException.class, () -> {
       CollectionAdminRequest.deleteCollection("unknown_collection").process(cluster.getSolrClient());
     });
 
@@ -374,24 +353,19 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
   }
 
   @Test
-  public void testMaxNodesPerShard() throws Exception {
-
-    // test maxShardsPerNode
+  public void testMaxNodesPerShard() {
     int numLiveNodes = cluster.getJettySolrRunners().size();
     int numShards = (numLiveNodes/2) + 1;
     int replicationFactor = 2;
-    int maxShardsPerNode = 1;
 
-    SolrException e = expectThrows(SolrException.class, () -> {
+    expectThrows(SolrException.class, () -> {
       CollectionAdminRequest.createCollection("oversharded", "conf", numShards, replicationFactor)
           .process(cluster.getSolrClient());
     });
-
   }
 
   @Test
   public void testCreateNodeSet() throws Exception {
-
     JettySolrRunner jetty1 = cluster.getRandomJetty(random());
     JettySolrRunner jetty2 = cluster.getRandomJetty(random());
 
@@ -406,13 +380,14 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
       String replicaUrl = replica.getCoreUrl();
       boolean matchingJetty = false;
       for (String jettyUrl : baseUrls) {
-        if (replicaUrl.startsWith(jettyUrl))
+        if (replicaUrl.startsWith(jettyUrl)) {
           matchingJetty = true;
+        }
       }
-      if (matchingJetty == false)
+      if (matchingJetty == false) {
         fail("Expected replica to be on " + baseUrls + " but was on " + replicaUrl);
+      }
     }
-
   }
 
   @Test
@@ -507,13 +482,11 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
       for (int z = 1; z < createRequests[j].getNumShards(); z++) {
         zkStateReader.getLeaderRetry(collectionName, "shard" + z, 10000);
       }      // make sure we again have leaders for each shard
-      
     }
   }
 
   @Test
   public void testCollectionReload() throws Exception {
-
     final String collectionName = "reloaded_collection";
     CollectionAdminRequest.createCollection(collectionName, "conf", 2, 2).process(cluster.getSolrClient());
 
@@ -533,21 +506,17 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     CoreContainer cores = jetty.getCoreContainer();
     Collection<SolrCore> theCores = cores.getCores();
     for (SolrCore core : theCores) {
-
       // look for core props file
-      Path instancedir = (Path) core.getResourceLoader().getInstancePath();
+      Path instancedir = core.getInstancePath();
       assertTrue("Could not find expected core.properties file", Files.exists(instancedir.resolve("core.properties")));
 
       Path expected = Paths.get(jetty.getSolrHome()).toAbsolutePath().resolve(core.getName());
 
       assertTrue("Expected: " + expected + "\nFrom core stats: " + instancedir, Files.isSameFile(expected, instancedir));
-
     }
   }
 
   private boolean waitForReloads(String collectionName, Map<String,Long> urlToTimeBefore) throws SolrServerException, IOException {
-
-
     TimeOut timeout = new TimeOut(45, TimeUnit.SECONDS, TimeSource.NANO_TIME);
 
     boolean allTimesAreCorrect = false;
@@ -596,29 +565,27 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     }
   }
   
-  private void checkNoTwoShardsUseTheSameIndexDir() throws Exception {
+  private void checkNoTwoShardsUseTheSameIndexDir() {
     Map<String, Set<String>> indexDirToShardNamesMap = new HashMap<>();
     
     List<MBeanServer> servers = new LinkedList<>();
     servers.add(ManagementFactory.getPlatformMBeanServer());
     servers.addAll(MBeanServerFactory.findMBeanServer(null));
     for (final MBeanServer server : servers) {
-      Set<ObjectName> mbeans = new HashSet<>();
-      mbeans.addAll(server.queryNames(null, null));
+      Set<ObjectName> mbeans = new HashSet<>(server.queryNames(null, null));
       for (final ObjectName mbean : mbeans) {
-
         try {
           Map<String, String> props = mbean.getKeyPropertyList();
           String category = props.get("category");
           String name = props.get("name");
-          if ((category != null && category.toString().equals(Category.CORE.toString())) &&
+          if ((category != null && category.equals(Category.CORE.toString())) &&
               (name != null && name.equals("indexDir"))) {
             String indexDir = server.getAttribute(mbean, "Value").toString();
             String key = props.get("dom2") + "." + props.get("dom3") + "." + props.get("dom4");
             if (!indexDirToShardNamesMap.containsKey(indexDir)) {
-              indexDirToShardNamesMap.put(indexDir.toString(), new HashSet<>());
+              indexDirToShardNamesMap.put(indexDir, new HashSet<>());
             }
-            indexDirToShardNamesMap.get(indexDir.toString()).add(key);
+            indexDirToShardNamesMap.get(indexDir).add(key);
           }
         } catch (Exception e) {
           // ignore, just continue - probably a "Value" attribute
@@ -638,7 +605,6 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
             + entry.getKey());
       }
     }
-
   }
 
   @Test
@@ -660,8 +626,7 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     Replica newReplica = grabNewReplica(response, getCollectionState(collectionName));
 
     assertEquals("Replica should be created on the right node",
-        cluster.getSolrClient().getZkStateReader().getBaseUrlForNodeName(nodeList.get(0)),
-        newReplica.getStr(ZkStateReader.BASE_URL_PROP));
+        cluster.getSolrClient().getZkStateReader().getBaseUrlForNodeName(nodeList.get(0)), newReplica.getBaseUrl());
 
     Path instancePath = createTempDir();
     response = CollectionAdminRequest.addReplicaToShard(collectionName, "shard1")
@@ -670,7 +635,7 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     newReplica = grabNewReplica(response, getCollectionState(collectionName));
     assertNotNull(newReplica);
 
-    try (HttpSolrClient coreclient = getHttpSolrClient(newReplica.getStr(ZkStateReader.BASE_URL_PROP))) {
+    try (HttpSolrClient coreclient = getHttpSolrClient(newReplica.getBaseUrl())) {
       CoreAdminResponse status = CoreAdminRequest.getStatus(newReplica.getStr("core"), coreclient);
       NamedList<Object> coreStatus = status.getCoreStatus(newReplica.getStr("core"));
       String instanceDirStr = (String) coreStatus.get("instanceDir");
@@ -700,7 +665,6 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
 
     newReplica = grabNewReplica(response, getCollectionState(collectionName));
     assertEquals("'core' should be 'propertyDotName' ", "propertyDotName", newReplica.getStr("core"));
-
   }
 
   private Replica grabNewReplica(CollectionAdminResponse response, DocCollection docCollection) {
@@ -713,5 +677,4 @@ public class CollectionsAPIDistributedZkTest extends SolrCloudTestCase {
     }
     throw new AssertionError("Can not find " + replicaName + " from " + docCollection);
   }
-
 }
