@@ -17,19 +17,6 @@
 
 package org.apache.solr.cloud.autoscaling;
 
-import java.io.IOException;
-import java.lang.invoke.MethodHandles;
-import java.util.ArrayList;
-import java.util.Collection;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Map;
-import java.util.Set;
-import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.concurrent.atomic.AtomicReference;
-
 import org.apache.lucene.util.LuceneTestCase;
 import org.apache.solr.client.solrj.SolrRequest;
 import org.apache.solr.client.solrj.cloud.NodeStateProvider;
@@ -55,13 +42,18 @@ import org.apache.solr.common.util.Pair;
 import org.apache.solr.common.util.Utils;
 import org.apache.solr.core.SolrResourceLoader;
 import org.apache.solr.util.LogLevel;
-import org.junit.After;
-import org.junit.Before;
-import org.junit.BeforeClass;
-import org.junit.Test;
+import org.junit.*;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import java.io.IOException;
+import java.lang.invoke.MethodHandles;
+import java.util.*;
+import java.util.concurrent.CountDownLatch;
+import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicBoolean;
+import java.util.concurrent.atomic.AtomicReference;
+
 import static java.nio.charset.StandardCharsets.UTF_8;
 import static org.apache.solr.common.params.CollectionParams.CollectionAction.MOVEREPLICA;
 
@@ -75,12 +67,14 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
   private static final AtomicBoolean fired = new AtomicBoolean(false);
   private static final int NODE_COUNT = 1;
   private static CountDownLatch triggerFiredLatch = new CountDownLatch(1);
+  @SuppressWarnings({"rawtypes"})
   private static final AtomicReference<Map> actionContextPropsRef = new AtomicReference<>();
   private static final AtomicReference<TriggerEvent> eventRef = new AtomicReference<>();
   private static SolrCloudManager cloudManager;
 
   @BeforeClass
   public static void setupCluster() throws Exception {
+    System.setProperty("metricsEnabled", "true");
     configureCluster(NODE_COUNT)
         .addConfig("conf", configset("cloud-minimal"))
         .configure();
@@ -121,6 +115,7 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
         "      {'nodeRole':'overseer', 'replica':0}" +
         "    ]" +
         "}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setClusterPolicyCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
@@ -161,11 +156,22 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     SolrCloudManager cloudManager = cluster.getJettySolrRunner(0).getCoreContainer().getZkController().getSolrCloudManager();
     for (String node: cloudManager.getClusterStateProvider().getLiveNodes()) {
       Map<String, Object> values = cloudManager.getNodeStateProvider().getNodeValues(node, ImplicitSnitch.tags);
-      log.debug("* Node values: " + node + "\n" + Utils.toJSONString(values));
+      if (log.isDebugEnabled()) {
+        log.debug("* Node values: {}\n{}", node, Utils.toJSONString(values));
+      }
+    }
+    if (log.isDebugEnabled()) {
+      log.debug("* Live nodes: {}", cloudManager.getClusterStateProvider().getLiveNodes());
     }
-    log.debug("* Live nodes: " + cloudManager.getClusterStateProvider().getLiveNodes());
     ClusterState state = cloudManager.getClusterStateProvider().getClusterState();
-    state.forEachCollection(coll -> log.debug("* Collection " + coll.getName() + " state: " + coll));
+    if (log.isDebugEnabled()) {
+      state.forEachCollection(coll -> log.debug("* Collection {} state: {}", coll.getName(), coll));
+    }
+  }
+
+  @AfterClass
+  public static void cleanUpAfterClass() throws Exception {
+    cloudManager = null;
   }
 
   @Test
@@ -186,6 +192,7 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
         "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction'}," +
         "{'name':'test','class':'" + ComputePlanActionTest.AssertingTriggerAction.class.getName() + "'}]" +
         "}}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
@@ -222,11 +229,14 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
 
     assertTrue("Trigger was not fired even after 10 seconds", triggerFiredLatch.await(10, TimeUnit.SECONDS));
     assertTrue(fired.get());
+    @SuppressWarnings({"rawtypes"})
     Map context = actionContextPropsRef.get();
     assertNotNull(context);
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<SolrRequest> operations = (List<SolrRequest>) context.get("operations");
     assertNotNull("The operations computed by ComputePlanAction should not be null , "+ getNodeStateProviderState() + eventRef.get(), operations);
     assertEquals("ComputePlanAction should have computed exactly 1 operation", 1, operations.size());
+    @SuppressWarnings({"rawtypes"})
     SolrRequest solrRequest = operations.get(0);
     SolrParams params = solrRequest.getParams();
     assertEquals("Expected MOVEREPLICA action after adding node", MOVEREPLICA, CollectionParams.CollectionAction.get(params.get("action")));
@@ -271,6 +281,7 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
         "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction'}," +
         "{'name':'test','class':'" + ComputePlanActionTest.AssertingTriggerAction.class.getName() + "'}]" +
         "}}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
@@ -312,14 +323,18 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     assertEquals(TriggerEventType.NODELOST, triggerEvent.getEventType());
     // TODO assertEquals(stoppedNodeName, triggerEvent.getProperty(TriggerEvent.NODE_NAME));
 
+    @SuppressWarnings({"rawtypes"})
     Map context = actionContextPropsRef.get();
     assertNotNull(context);
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<SolrRequest> operations = (List<SolrRequest>) context.get("operations");
     assertNotNull("The operations computed by ComputePlanAction should not be null "+ getNodeStateProviderState() + actionContextPropsRef.get(), operations);
-    operations.forEach(solrRequest -> log.info(solrRequest.getParams().toString()));
+    if (log.isInfoEnabled()) {
+      operations.forEach(solrRequest -> log.info(solrRequest.getParams().toString()));
+    }
     assertEquals("ComputePlanAction should have computed exactly 2 operation", 2, operations.size());
 
-    for (SolrRequest solrRequest : operations) {
+    for (@SuppressWarnings({"rawtypes"})SolrRequest solrRequest : operations) {
       SolrParams params = solrRequest.getParams();
       assertEquals("Expected MOVEREPLICA action after adding node", MOVEREPLICA, CollectionParams.CollectionAction.get(params.get("action")));
       String moved = params.get("replica");
@@ -328,7 +343,7 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
   }
 
   @Test
-  @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 14-Oct-2018
+  // commented out on: 17-Feb-2019   @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 14-Oct-2018
   public void testNodeAdded() throws Exception {
     CloudSolrClient solrClient = cluster.getSolrClient();
     String setTriggerCommand = "{" +
@@ -340,6 +355,7 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
         "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction'}," +
         "{'name':'test','class':'" + ComputePlanActionTest.AssertingTriggerAction.class.getName() + "'}]" +
         "}}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
@@ -379,11 +395,14 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     JettySolrRunner runner = cluster.startJettySolrRunner();
     assertTrue("Trigger was not fired even after 5 seconds", triggerFiredLatch.await(5, TimeUnit.SECONDS));
     assertTrue(fired.get());
+    @SuppressWarnings({"rawtypes"})
     Map context = actionContextPropsRef.get();
     assertNotNull(context);
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<SolrRequest> operations = (List<SolrRequest>) context.get("operations");
     assertNotNull("The operations computed by ComputePlanAction should not be null" + getNodeStateProviderState() + context, operations);
     assertEquals("ComputePlanAction should have computed exactly 1 operation", 1, operations.size());
+    @SuppressWarnings({"rawtypes"})
     SolrRequest request = operations.get(0);
     SolrParams params = request.getParams();
     assertEquals("Expected MOVEREPLICA action after adding node", MOVEREPLICA, CollectionParams.CollectionAction.get(params.get("action")));
@@ -412,6 +431,7 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     @Override
     public void process(TriggerEvent event, ActionContext context) {
       if (expectedNode != null) {
+        @SuppressWarnings({"rawtypes"})
         Collection nodes = (Collection) event.getProperty(TriggerEvent.NODE_NAMES);
         if (nodes == null || !nodes.contains(expectedNode)) return;//this is not the event we are looking for
       }
@@ -430,8 +450,37 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
 
   @Test
   //2018-06-18 (commented) @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 09-Apr-2018
-  public void testSelectedCollections() throws Exception {
-    log.info("Found number of jetties: {}", cluster.getJettySolrRunners().size());
+  public void testSelectedCollectionsByName() throws Exception {
+    String collectionsFilter = "'testSelected1,testSelected2'";
+    testCollectionsPredicate(collectionsFilter, Collections.emptyMap());
+  }
+
+  @Test
+  //2018-06-18 (commented) @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 09-Apr-2018
+  public void testSelectedCollectionsByPolicy() throws Exception {
+    CloudSolrClient solrClient = cluster.getSolrClient();
+    String setSearchPolicyCommand = "{" +
+            " 'set-policy': {" +
+            "   'search': [" +
+            "      {'replica':'<5', 'shard': '#EACH', 'node': '#ANY'}," +
+            "    ]" +
+            "}}";
+    @SuppressWarnings({"rawtypes"})
+    SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setSearchPolicyCommand);
+    NamedList<Object> response = solrClient.request(req);
+    assertEquals(response.get("result").toString(), "success");
+
+    String collectionsFilter = "{'policy': 'search'}";
+    Map<String, String> createCollectionParameters = new HashMap<>();
+    createCollectionParameters.put("testSelected1", "search");
+    createCollectionParameters.put("testSelected2", "search");
+    testCollectionsPredicate(collectionsFilter, createCollectionParameters);
+  }
+
+  private void testCollectionsPredicate(String collectionsFilter, Map<String, String> createCollectionParameters) throws Exception {
+    if (log.isInfoEnabled()) {
+      log.info("Found number of jetties: {}", cluster.getJettySolrRunners().size());
+    }
     // start 3 more nodes
     cluster.startJettySolrRunner();
     cluster.startJettySolrRunner();
@@ -441,28 +490,38 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
 
     CloudSolrClient solrClient = cluster.getSolrClient();
     String setTriggerCommand = "{" +
-        "'set-trigger' : {" +
-        "'name' : 'node_lost_trigger'," +
-        "'event' : 'nodeLost'," +
-        "'waitFor' : '1s'," +
-        "'enabled' : true," +
-        "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction', 'collections' : 'testSelected1,testSelected2'}," +
-        "{'name':'test','class':'" + ComputePlanActionTest.AssertingTriggerAction.class.getName() + "'}]" +
-        "}}";
+            "'set-trigger' : {" +
+            "'name' : 'node_lost_trigger'," +
+            "'event' : 'nodeLost'," +
+            "'waitFor' : '1s'," +
+            "'enabled' : true," +
+            "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction', 'collections' : " + collectionsFilter + "}," +
+            "{'name':'test','class':'" + ComputePlanActionTest.AssertingTriggerAction.class.getName() + "'}]" +
+            "}}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
 
     CollectionAdminRequest.Create create = CollectionAdminRequest.createCollection("testSelected1",
         "conf", 2, 2);
+    if (createCollectionParameters.get("testSelected1") != null)  {
+      create.setPolicy(createCollectionParameters.get("testSelected1"));
+    }
     create.process(solrClient);
 
     create = CollectionAdminRequest.createCollection("testSelected2",
         "conf", 2, 2);
+    if (createCollectionParameters.get("testSelected2") != null)  {
+      create.setPolicy(createCollectionParameters.get("testSelected2"));
+    }
     create.process(solrClient);
 
     create = CollectionAdminRequest.createCollection("testSelected3",
         "conf", 2, 2);
+    if (createCollectionParameters.get("testSelected3") != null)  {
+      create.setPolicy(createCollectionParameters.get("testSelected3"));
+    }
     create.process(solrClient);
     
     cluster.waitForActiveCollection("testSelected1", 2, 4);
@@ -500,11 +559,14 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     }
     assertTrue("Trigger was not fired even after 5 seconds", triggerFiredLatch.await(5, TimeUnit.SECONDS));
     assertTrue(fired.get());
+    @SuppressWarnings({"rawtypes"})
     Map context = actionContextPropsRef.get();
     assertNotNull(context);
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<SolrRequest> operations = (List<SolrRequest>) context.get("operations");
     assertNotNull("The operations computed by ComputePlanAction should not be null. " + getNodeStateProviderState() + context, operations);
     assertEquals("ComputePlanAction should have computed exactly 2 operations", 2, operations.size());
+    @SuppressWarnings({"rawtypes"})
     SolrRequest request = operations.get(0);
     SolrParams params = request.getParams();
     assertEquals("Expected MOVEREPLICA action after adding node", MOVEREPLICA, CollectionParams.CollectionAction.get(params.get("action")));
@@ -521,7 +583,16 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     int numShards = 1;
     int numCollections = 5;
 
-    nodeAddedTriggerWithAddReplicaPreferredOp(collectionNamePrefix, numShards, numCollections);
+    nodeAddedTriggerWithAddReplicaPreferredOp(collectionNamePrefix, numShards, numCollections, null);
+  }
+
+  @Test
+  public void testNodeAddedTriggerWithAddReplicaPreferredOpReplicaType_1Shard() throws Exception {
+    String collectionNamePrefix = "testNodeAddedTriggerWithAddReplicaPreferredOpReplicaType_1Shard";
+    int numShards = 1;
+    int numCollections = 5;
+
+    nodeAddedTriggerWithAddReplicaPreferredOpReplicaType(collectionNamePrefix, numShards, numCollections, null);
   }
 
   @Test
@@ -531,11 +602,19 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     int numShards = 2;
     int numCollections = 5;
 
-    nodeAddedTriggerWithAddReplicaPreferredOp(collectionNamePrefix, numShards, numCollections);
+    nodeAddedTriggerWithAddReplicaPreferredOp(collectionNamePrefix, numShards, numCollections, null);
   }
 
-  private void nodeAddedTriggerWithAddReplicaPreferredOp(String collectionNamePrefix, int numShards, int numCollections) throws Exception {
-    CloudSolrClient solrClient = cluster.getSolrClient();
+  @Test
+  public void testNodeAddedTriggerWithAddReplicaPreferredOp_2Shard_OpLimit() throws Exception {
+    String collectionNamePrefix = "testNodeAddedTriggerWithAddReplicaPreferredOp_2Shard";
+    int numShards = 2;
+    int numCollections = 5;
+
+    nodeAddedTriggerWithAddReplicaPreferredOp(collectionNamePrefix, numShards, numCollections, 1);
+  }
+
+  private void nodeAddedTriggerWithAddReplicaPreferredOp(String collectionNamePrefix, int numShards, int numCollections, Integer maxOps) throws Exception {
     String setTriggerCommand = "{" +
         "'set-trigger' : {" +
         "'name' : 'node_added_trigger'," +
@@ -546,9 +625,6 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
         "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction'}," +
         "{'name':'test','class':'" + AssertingTriggerAction.class.getName() + "'}]" +
         "}}";
-    SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
-    NamedList<Object> response = solrClient.request(req);
-    assertEquals(response.get("result").toString(), "success");
 
     String setClusterPolicyCommand = "{" +
         " 'set-cluster-policy': [" +
@@ -557,13 +633,63 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
         "      {'nodeRole':'overseer', 'replica':0}" +
         "    ]" +
         "}";
+
+    nodeAddedTriggerWithAddReplicaPreferredOp(collectionNamePrefix, numShards, numCollections, setTriggerCommand, setClusterPolicyCommand, maxOps);
+  }
+
+  private void nodeAddedTriggerWithAddReplicaPreferredOpReplicaType(String collectionNamePrefix, int numShards, int numCollections, Integer maxOps) throws Exception {
+    String setTriggerCommand = "{" +
+        "'set-trigger' : {" +
+        "'name' : 'node_added_trigger'," +
+        "'event' : 'nodeAdded'," +
+        "'waitFor' : '1s'," +
+        "'enabled' : true," +
+        "'" + AutoScalingParams.PREFERRED_OP + "':'addreplica'," +
+        "'" + AutoScalingParams.REPLICA_TYPE + "':'" + Replica.Type.PULL + "'," +
+        "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction'}," +
+        "{'name':'test','class':'" + AssertingTriggerAction.class.getName() + "'}]" +
+        "}}";
+
+    String setClusterPolicyCommand = "{" +
+        " 'set-cluster-policy': [" +
+        "      {'cores':'<" + (1 + numCollections * numShards) + "', 'node':'#ANY'}," +
+        "      {'replica':'<2', 'shard': '#EACH', 'node': '#ANY'}," +
+        "      {'nodeRole':'overseer', 'replica':0}" +
+        "    ]" +
+        "}";
+
+    nodeAddedTriggerWithAddReplicaPreferredOp(collectionNamePrefix, numShards, numCollections, setTriggerCommand, setClusterPolicyCommand, maxOps, 0, 1, 0);
+  }
+
+  private void nodeAddedTriggerWithAddReplicaPreferredOp(String collectionNamePrefix, int numShards, int numCollections, String setTriggerCommand, String setClusterPolicyCommand, Integer maxOps) throws Exception {
+    nodeAddedTriggerWithAddReplicaPreferredOp(collectionNamePrefix, numShards, numCollections, setTriggerCommand, setClusterPolicyCommand, maxOps, 1, null, null);
+  }
+  private void nodeAddedTriggerWithAddReplicaPreferredOp(String collectionNamePrefix, int numShards, int numCollections, String setTriggerCommand, String setClusterPolicyCommand,
+                                                         Integer maxOps,
+                                                         Integer nNrtReplicas, Integer nTlogReplicas, Integer nPullReplicas) throws Exception {
+    CloudSolrClient solrClient = cluster.getSolrClient();
+    @SuppressWarnings({"rawtypes"})
+    SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
+    NamedList<Object> response = solrClient.request(req);
+    assertEquals(response.get("result").toString(), "success");
+
     req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setClusterPolicyCommand);
     response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
 
+    if (maxOps != null) {
+      String setMaxOpsCommand = "{" +
+          " 'set-properties': {" +
+          "   'maxComputeOperations': " + maxOps +
+          "  }" +
+          "}";
+      req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setMaxOpsCommand);
+      response = solrClient.request(req);
+      assertEquals(response.get("result").toString(), "success");
+    }
 
     CollectionAdminRequest.Create create = CollectionAdminRequest.createCollection(collectionNamePrefix + "_0",
-        "conf", numShards, 1).setMaxShardsPerNode(2);
+        "conf", numShards, nNrtReplicas, nTlogReplicas, nPullReplicas).setMaxShardsPerNode(2);
     create.process(solrClient);
 
     waitForState("Timed out waiting for replicas of new collection to be active",
@@ -574,10 +700,18 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     cluster.waitForAllNodes(30);
     assertTrue(triggerFiredLatch.await(30, TimeUnit.SECONDS));
     assertTrue(fired.get());
+    @SuppressWarnings({"rawtypes"})
     Map actionContext = actionContextPropsRef.get();
+    @SuppressWarnings({"rawtypes"})
     List operations = (List) actionContext.get("operations");
     assertNotNull(operations);
-    assertEquals(numShards, operations.size());
+    int numExpectedOps;
+    if (maxOps != null && maxOps > 0) {
+      numExpectedOps = maxOps;
+    } else {
+      numExpectedOps = numShards;
+    }
+    assertEquals(numExpectedOps, operations.size());
     Set<String> affectedShards = new HashSet<>(2);
     for (Object operation : operations) {
       assertTrue(operation instanceof CollectionAdminRequest.AddReplica);
@@ -586,7 +720,7 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
       assertEquals(collectionNamePrefix + "_0", addReplica.getCollection());
       affectedShards.add(addReplica.getShard());
     }
-    assertEquals(numShards, affectedShards.size());
+    assertEquals(numExpectedOps, affectedShards.size());
 
     for (int i = 1; i < numCollections; i++) {
       create = CollectionAdminRequest.createCollection(collectionNamePrefix + "_" + i,
@@ -606,7 +740,12 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     actionContext = actionContextPropsRef.get();
     operations = (List) actionContext.get("operations");
     assertNotNull(operations);
-    assertEquals(numCollections * numShards, operations.size());
+    if (maxOps != null && maxOps > 0) {
+      numExpectedOps = maxOps;
+    } else {
+      numExpectedOps = numCollections * numShards;
+    }
+    assertEquals(numExpectedOps, operations.size());
     Set<String> affectedCollections = new HashSet<>(numCollections);
     affectedShards = new HashSet<>(numShards);
     Set<Pair<String, String>> affectedCollShards = new HashSet<>(numCollections * numShards);
@@ -618,13 +757,19 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
       affectedShards.add(addReplica.getShard());
       affectedCollShards.add(new Pair<>(addReplica.getCollection(), addReplica.getShard()));
     }
-    assertEquals(numCollections, affectedCollections.size());
-    assertEquals(numShards, affectedShards.size());
-    assertEquals(numCollections * numShards, affectedCollShards.size());
+    if (maxOps != null && maxOps > 0) {
+      assertEquals(numExpectedOps, affectedCollections.size());
+      assertEquals(numExpectedOps, affectedShards.size());
+      assertEquals(numExpectedOps, affectedCollShards.size());
+    } else {
+      assertEquals(numCollections, affectedCollections.size());
+      assertEquals(numShards, affectedShards.size());
+      assertEquals(numCollections * numShards, affectedCollShards.size());
+    }
   }
 
   @Test
-  @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 14-Oct-2018
+  // commented out on: 17-Feb-2019   @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 14-Oct-2018
   public void testNodeLostTriggerWithDeleteNodePreferredOp() throws Exception {
     String collectionNamePrefix = "testNodeLostTriggerWithDeleteNodePreferredOp";
     int numCollections = 1 + random().nextInt(3), numShards = 1 + random().nextInt(3);
@@ -641,6 +786,7 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
         "{'name':'execute_plan','class':'solr.ExecutePlanAction'}" +
         "{'name':'test','class':'" + AssertingTriggerAction.class.getName() + "'}]" +
         "}}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
@@ -671,7 +817,9 @@ public class ComputePlanActionTest extends SolrCloudTestCase {
     cluster.stopJettySolrRunner(newNode);
     assertTrue(triggerFiredLatch.await(30, TimeUnit.SECONDS));
     assertTrue(fired.get());
+    @SuppressWarnings({"rawtypes"})
     Map actionContext = actionContextPropsRef.get();
+    @SuppressWarnings({"rawtypes"})
     List operations = (List) actionContext.get("operations");
     assertNotNull(operations);
     assertEquals(1, operations.size());
