@@ -33,8 +33,8 @@ import org.apache.solr.client.solrj.SolrResponse;
 import org.apache.solr.client.solrj.cloud.SolrCloudManager;
 import org.apache.solr.client.solrj.cloud.autoscaling.TriggerEventType;
 import org.apache.solr.client.solrj.request.CollectionAdminRequest;
-import org.apache.solr.cloud.CloudTestUtils;
 import org.apache.solr.cloud.CloudTestUtils.AutoScalingRequest;
+import org.apache.solr.cloud.CloudUtil;
 import org.apache.solr.cloud.autoscaling.ActionContext;
 import org.apache.solr.cloud.autoscaling.ComputePlanAction;
 import org.apache.solr.cloud.autoscaling.ScheduledTriggers;
@@ -69,6 +69,7 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
   private static final AtomicBoolean fired = new AtomicBoolean(false);
   private static final int NODE_COUNT = 1;
   private static CountDownLatch triggerFiredLatch = new CountDownLatch(1);
+  @SuppressWarnings({"rawtypes"})
   private static final AtomicReference<Map> actionContextPropsRef = new AtomicReference<>();
   private static final AtomicReference<TriggerEvent> eventRef = new AtomicReference<>();
 
@@ -86,6 +87,7 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
         "      {'nodeRole':'overseer', 'replica':0}" +
         "    ]" +
         "}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setClusterPolicyCommand);
     SolrResponse rsp = cluster.request(req);
     NamedList<Object> response = rsp.getResponse();
@@ -106,12 +108,21 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
 
   @After
   public void printState() throws Exception {
-    log.info("-------------_ FINAL STATE --------------");
-    log.info("* Node values: " + Utils.toJSONString(cluster.getSimNodeStateProvider().simGetAllNodeValues()));
-    log.info("* Live nodes: " + cluster.getClusterStateProvider().getLiveNodes());
+    if (null == cluster) {
+      // test didn't init, nothing to do
+      return;
+    }
+
+    if (log.isInfoEnabled()) {
+      log.info("-------------_ FINAL STATE --------------");
+      log.info("* Node values: {}", Utils.toJSONString(cluster.getSimNodeStateProvider().simGetAllNodeValues())); // logOk
+      log.info("* Live nodes: {}", cluster.getClusterStateProvider().getLiveNodes()); // logOk
+    }
     ClusterState state = cluster.getClusterStateProvider().getClusterState();
-    for (String coll: cluster.getSimClusterStateProvider().simListCollections()) {
-      log.info("* Collection " + coll + " state: " + state.getCollection(coll));
+    if (log.isInfoEnabled()) {
+      for (String coll : cluster.getSimClusterStateProvider().simListCollections()) {
+        log.info("* Collection {} state: {}", coll, state.getCollection(coll)); // logOk
+      }
     }
     shutdownCluster();
   }
@@ -135,6 +146,7 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
         "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction'}," +
         "{'name':'test','class':'" + TestSimComputePlanAction.AssertingTriggerAction.class.getName() + "'}]" +
         "}}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
@@ -145,8 +157,8 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
         "conf",1, 2);
     create.process(solrClient);
 
-    CloudTestUtils.waitForState(cluster, "Timed out waiting for replicas of new collection to be active",
-        "testNodeLost", CloudTestUtils.clusterShape(1, 2, false, true));
+    CloudUtil.waitForState(cluster, "Timed out waiting for replicas of new collection to be active",
+        "testNodeLost", CloudUtil.clusterShape(1, 2, false, true));
 
     ClusterState clusterState = cluster.getClusterStateProvider().getClusterState();
     log.debug("-- cluster state: {}", clusterState);
@@ -166,11 +178,14 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
 
     assertTrue("Trigger was not fired even after 10 seconds", triggerFiredLatch.await(10, TimeUnit.SECONDS));
     assertTrue(fired.get());
+    @SuppressWarnings({"rawtypes"})
     Map context = actionContextPropsRef.get();
     assertNotNull(context);
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<SolrRequest> operations = (List<SolrRequest>) context.get("operations");
     assertNotNull("The operations computed by ComputePlanAction should not be null , " + eventRef.get(), operations);
     assertEquals("ComputePlanAction should have computed exactly 1 operation", 1, operations.size());
+    @SuppressWarnings({"rawtypes"})
     SolrRequest solrRequest = operations.get(0);
     SolrParams params = solrRequest.getParams();
     assertEquals("Expected MOVEREPLICA action after adding node", MOVEREPLICA, CollectionParams.CollectionAction.get(params.get("action")));
@@ -200,6 +215,7 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
         "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction'}," +
         "{'name':'test','class':'" + AssertingTriggerAction.class.getName() + "'}]" +
         "}}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
@@ -211,8 +227,8 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
     create.setMaxShardsPerNode(2);
     create.process(solrClient);
 
-    CloudTestUtils.waitForState(cluster, "Timed out waiting for replicas of new collection to be active",
-        "testNodeWithMultipleReplicasLost", CloudTestUtils.clusterShape(2, 3, false, true));
+    CloudUtil.waitForState(cluster, "Timed out waiting for replicas of new collection to be active",
+        "testNodeWithMultipleReplicasLost", CloudUtil.clusterShape(2, 3, false, true));
 
     ClusterState clusterState = cluster.getClusterStateProvider().getClusterState();
     log.debug("-- cluster state: {}", clusterState);
@@ -240,20 +256,24 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
     assertEquals(TriggerEventType.NODELOST, triggerEvent.getEventType());
     // TODO assertEquals(stoppedNodeName, triggerEvent.getProperty(TriggerEvent.NODE_NAME));
 
+    @SuppressWarnings({"rawtypes"})
     Map context = actionContextPropsRef.get();
     assertNotNull(context);
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<SolrRequest> operations = (List<SolrRequest>) context.get("operations");
     assertNotNull("The operations computed by ComputePlanAction should not be null " + actionContextPropsRef.get() + "\nevent: " + eventRef.get(), operations);
-    operations.forEach(solrRequest -> log.info(solrRequest.getParams().toString()));
-    
+    if (log.isInfoEnabled()) {
+      operations.forEach(solrRequest -> log.info("{}", solrRequest.getParams()));
+    }
+
     // TODO: this can be 3!
     // assertEquals("ComputePlanAction should have computed exactly 2 operation", 2, operations.size());
 
-    for (SolrRequest solrRequest : operations) {
+    for (@SuppressWarnings({"rawtypes"})SolrRequest solrRequest : operations) {
       SolrParams params = solrRequest.getParams();
       assertEquals("Expected MOVEREPLICA action after adding node", MOVEREPLICA, CollectionParams.CollectionAction.get(params.get("action")));
       String moved = params.get("replica");
-      
+
       // TODO: this can fail!
       // assertTrue(replicasToBeMoved.stream().anyMatch(replica -> replica.getName().equals(moved)));
     }
@@ -274,6 +294,7 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
         "'actions' : [{'name':'compute_plan', 'class' : 'solr.ComputePlanAction'}," +
         "{'name':'test','class':'" + TestSimComputePlanAction.AssertingTriggerAction.class.getName() + "'}]" +
         "}}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setTriggerCommand);
     NamedList<Object> response = solrClient.request(req);
     assertEquals(response.get("result").toString(), "success");
@@ -296,7 +317,7 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
         "conf",1, 4).setMaxShardsPerNode(-1);
     create.process(solrClient);
 
-    CloudTestUtils.waitForState(cluster, "Timed out waiting for replicas of new collection to be active",
+    CloudUtil.waitForState(cluster, "Timed out waiting for replicas of new collection to be active",
         "testNodeAdded", (liveNodes, collectionState) -> collectionState.getReplicas().stream().allMatch(replica -> replica.isActive(liveNodes)));
 
     // reset to the original policy which has only 1 replica per shard per node
@@ -317,16 +338,23 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
     String newNode = cluster.simAddNode();
     assertTrue("Trigger was not fired even after 5 seconds", triggerFiredLatch.await(5, TimeUnit.SECONDS));
     assertTrue(fired.get());
+    @SuppressWarnings({"rawtypes"})
     Map context = actionContextPropsRef.get();
     assertNotNull(context);
-    log.info("Node values: " + Utils.toJSONString(cluster.getSimNodeStateProvider().simGetAllNodeValues()));
-    log.info("Live nodes: " + cluster.getClusterStateProvider().getLiveNodes() + ", collection state: " + cluster.getClusterStateProvider().getClusterState().getCollection("testNodeAdded"));
+    if (log.isInfoEnabled()) {
+      log.info("Node values: {}", Utils.toJSONString(cluster.getSimNodeStateProvider().simGetAllNodeValues()));
+      log.info("Live nodes: {}, collection state: {}"
+          , cluster.getClusterStateProvider().getLiveNodes(), cluster.getClusterStateProvider().getClusterState().getCollection("testNodeAdded")); // logOk
+    }
+
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<SolrRequest> operations = (List<SolrRequest>) context.get("operations");
     assertNotNull("The operations computed by ComputePlanAction should not be null" + context, operations);
 
     // TODO: can be 2!
     // assertEquals("ComputePlanAction should have computed exactly 1 operation, but was: " + operations, 1, operations.size());
-    
+
+    @SuppressWarnings({"rawtypes"})
     SolrRequest request = operations.get(0);
     SolrParams params = request.getParams();
     assertEquals("Expected MOVEREPLICA action after adding node", MOVEREPLICA, CollectionParams.CollectionAction.get(params.get("action")));
@@ -355,6 +383,7 @@ public class TestSimComputePlanAction extends SimSolrCloudTestCase {
     @Override
     public void process(TriggerEvent event, ActionContext context) {
       if (expectedNode != null) {
+        @SuppressWarnings({"rawtypes"})
         Collection nodes = (Collection) event.getProperty(TriggerEvent.NODE_NAMES);
         if (nodes == null || !nodes.contains(expectedNode)) return;//this is not the event we are looking for
       }
