@@ -76,9 +76,13 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     request.addFieldType("pint");
     request.setFieldValue("5");
     
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> nl = handler.handleAnalysisRequest(request, h.getCore().getLatestSchema());
+    @SuppressWarnings({"rawtypes"})
     NamedList pintNL = (NamedList)nl.get("field_types").get("pint");
+    @SuppressWarnings({"rawtypes"})
     NamedList indexNL = (NamedList)pintNL.get("index");
+    @SuppressWarnings({"rawtypes"})
     ArrayList analyzerNL = (ArrayList)indexNL.get("org.apache.solr.schema.FieldType$DefaultAnalyzer$1");
     String text = (String)((NamedList)analyzerNL.get(0)).get("text"); 
     assertEquals("5", text);
@@ -156,15 +160,9 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     params.remove(CommonParams.Q);
     params.remove(AnalysisParams.QUERY);
     params.remove(AnalysisParams.FIELD_VALUE);
-    try {
-      request = handler.resolveAnalysisRequest(req);
-      fail("Analysis request must fail if all of q, analysis.query or analysis.value are absent");
-    } catch (SolrException e) {
-      if (e.code() != SolrException.ErrorCode.BAD_REQUEST.code)  {
-        fail("Unexpected exception");
-      }
-    } catch (Exception e) {
-      fail("Unexpected exception");
+    try (SolrQueryRequest solrQueryRequest = new LocalSolrQueryRequest(h.getCore(), params)) {
+      SolrException ex = expectThrows(SolrException.class, () -> handler.resolveAnalysisRequest(solrQueryRequest));
+      assertEquals(SolrException.ErrorCode.BAD_REQUEST.code, ex.code());
     }
 
     req.close();
@@ -175,6 +173,7 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
    * org.apache.solr.schema.IndexSchema)}
    */
   @Test
+  @SuppressWarnings({"unchecked"})
   public void testHandleAnalysisRequest() throws Exception {
 
     FieldAnalysisRequest request = new FieldAnalysisRequest();
@@ -186,17 +185,22 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     request.setQuery("fox brown");
     request.setShowMatch(true);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> result = handler.handleAnalysisRequest(request, h.getCore().getLatestSchema());
     assertTrue("result is null and it shouldn't be", result != null);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> fieldTypes = result.get("field_types");
     assertNotNull("field_types should never be null", fieldTypes);
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> textType = fieldTypes.get("text");
     assertNotNull("expecting result for field type 'text'", textType);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<List<NamedList>> indexPart = textType.get("index");
     assertNotNull("expecting an index token analysis for field type 'text'", indexPart);
 
+    @SuppressWarnings({"rawtypes"})
     List<NamedList> tokenList = indexPart.get("org.apache.lucene.analysis.standard.StandardTokenizer");
     assertNotNull("Expcting StandardTokenizer analysis breakdown", tokenList);
     assertEquals(tokenList.size(), 10);
@@ -246,6 +250,7 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     assertToken(tokenList.get(6), new TokenInfo("brown", null, "<ALPHANUM>", 39, 44, 9, new int[]{9,9,9,9}, null, true));
     assertToken(tokenList.get(7), new TokenInfo("dog", null, "<ALPHANUM>", 45, 49, 10, new int[]{10,10,10,10}, null, false));
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<List<NamedList>> queryPart = textType.get("query");
     assertNotNull("expecting a query token analysis for field type 'text'", queryPart);
 
@@ -270,6 +275,7 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     assertToken(tokenList.get(0), new TokenInfo("fox", null, "<ALPHANUM>", 0, 3, 1, new int[]{1,1,1,1}, null, false));
     assertToken(tokenList.get(1), new TokenInfo("brown", null, "<ALPHANUM>", 4, 9, 2, new int[]{2,2,2,2}, null, false));
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> nameTextType = fieldTypes.get("nametext");
     assertNotNull("expecting result for field type 'nametext'", nameTextType);
 
@@ -296,9 +302,11 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     assertToken(tokenList.get(0), new TokenInfo("fox", null, "word", 0, 3, 1, new int[]{1}, null, false));
     assertToken(tokenList.get(1), new TokenInfo("brown", null, "word", 4, 9, 2, new int[]{2}, null, false));
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> fieldNames = result.get("field_names");
     assertNotNull("field_nameds should never be null", fieldNames);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> whitetok = fieldNames.get("whitetok");
     assertNotNull("expecting result for field 'whitetok'", whitetok);
 
@@ -328,6 +336,7 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     assertToken(tokenList.get(0), new TokenInfo("fox", null, "word", 0, 3, 1, new int[]{1}, null, false));
     assertToken(tokenList.get(1), new TokenInfo("brown", null, "word", 4, 9, 2, new int[]{2}, null, false));
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> keywordtok = fieldNames.get("keywordtok");
     assertNotNull("expecting result for field 'keywordtok'", keywordtok);
 
@@ -357,20 +366,25 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     request.setFieldValue("<html><body>whátëvêr</body></html>");
     request.setShowMatch(false);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> result = handler.handleAnalysisRequest(request, h.getCore().getLatestSchema());
     assertTrue("result is null and it shouldn't be", result != null);
 
+    @SuppressWarnings({"unchecked", "rawtypes"})
     NamedList<NamedList> fieldTypes = result.get("field_types");
     assertNotNull("field_types should never be null", fieldTypes);
+    @SuppressWarnings({"unchecked", "rawtypes"})
     NamedList<NamedList> textType = fieldTypes.get("charfilthtmlmap");
     assertNotNull("expecting result for field type 'charfilthtmlmap'", textType);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList indexPart = textType.get("index");
     assertNotNull("expecting an index token analysis for field type 'charfilthtmlmap'", indexPart);
 
     assertEquals("\n\nwhátëvêr\n\n", indexPart.get("org.apache.lucene.analysis.charfilter.HTMLStripCharFilter"));
     assertEquals("\n\nwhatever\n\n", indexPart.get("org.apache.lucene.analysis.charfilter.MappingCharFilter"));
 
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<NamedList> tokenList = (List<NamedList>)indexPart.get(MockTokenizer.class.getName());
     assertNotNull("Expecting MockTokenizer analysis breakdown", tokenList);
     assertEquals(tokenList.size(), 1);
@@ -385,17 +399,22 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     request.setFieldValue("hi, 3456-12 a Test");
     request.setShowMatch(false);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> result = handler.handleAnalysisRequest(request, h.getCore().getLatestSchema());
     assertTrue("result is null and it shouldn't be", result != null);
 
+    @SuppressWarnings({"unchecked", "rawtypes"})
     NamedList<NamedList> fieldTypes = result.get("field_types");
     assertNotNull("field_types should never be null", fieldTypes);
+    @SuppressWarnings({"unchecked", "rawtypes"})
     NamedList<NamedList> textType = fieldTypes.get("skutype1");
     assertNotNull("expecting result for field type 'skutype1'", textType);
 
+    @SuppressWarnings({"unchecked", "rawtypes"})
     NamedList<List<NamedList>> indexPart = textType.get("index");
     assertNotNull("expecting an index token analysis for field type 'skutype1'", indexPart);
 
+    @SuppressWarnings({"rawtypes"})
     List<NamedList> tokenList = indexPart.get(MockTokenizer.class.getName());
     assertNotNull("Expcting MockTokenizer analysis breakdown", tokenList);
     assertEquals(4, tokenList.size());
@@ -429,14 +448,17 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     request.addFieldType("location_rpt");
     request.setFieldValue("MULTIPOINT ((10 40), (40 30), (20 20), (30 10))");
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> result = handler.handleAnalysisRequest(request, h.getCore().getLatestSchema());
+    @SuppressWarnings({"unchecked", "rawtypes"})
     NamedList<List<NamedList>> tokens = (NamedList<List<NamedList>>)
         ((NamedList)result.get("field_types").get("location_rpt")).get("index");
-    List<NamedList> tokenList = tokens.get("org.apache.lucene.spatial.prefix.BytesRefIteratorTokenStream");
+    @SuppressWarnings({"rawtypes"})
+    List<NamedList> tokenList = tokens.get("org.apache.lucene.spatial.prefix.PrefixTreeStrategy$ShapeTokenStream");
 
 
     List<String> vals = new ArrayList<>(tokenList.size());
-    for(NamedList v : tokenList) {
+    for(@SuppressWarnings({"rawtypes"})NamedList v : tokenList) {
       vals.add( (String)v.get("text") );
     }
     Collections.sort(vals);
@@ -468,8 +490,10 @@ public class FieldAnalysisRequestHandlerTest extends AnalysisRequestHandlerTestB
     );
     fieldType.setIndexAnalyzer(analyzer);
 
+    @SuppressWarnings({"rawtypes"})
     NamedList<NamedList> result = handler.analyzeValues(request, fieldType, "fieldNameUnused");
     // just test that we see "900" in the flags attribute here
+    @SuppressWarnings({"unchecked", "rawtypes"})
     List<NamedList> tokenInfoList = (List<NamedList>) result.findRecursive("index", CustomTokenFilter.class.getName());
     // '1' from CustomTokenFilter plus 900 from CustomFlagsAttributeImpl.
     assertEquals(901, tokenInfoList.get(0).get("org.apache.lucene.analysis.tokenattributes.FlagsAttribute#flags"));
