@@ -24,6 +24,7 @@ import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.TimeUnit;
 import java.util.stream.Collectors;
 
+import org.apache.lucene.util.LuceneTestCase;
 import org.apache.solr.client.solrj.SolrClient;
 import org.apache.solr.client.solrj.SolrRequest;
 import org.apache.solr.client.solrj.cloud.autoscaling.TriggerEventProcessorStage;
@@ -49,6 +50,7 @@ import org.apache.solr.common.params.CommonParams;
 import org.apache.solr.common.params.ModifiableSolrParams;
 import org.apache.solr.common.util.NamedList;
 import org.apache.solr.util.LogLevel;
+import org.junit.AfterClass;
 import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Test;
@@ -56,6 +58,7 @@ import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 @LogLevel("org.apache.solr.cloud.autoscaling=DEBUG;org.apache.solr.cloud.Overseer=DEBUG;org.apache.solr.cloud.overseer=DEBUG;org.apache.solr.client.solrj.cloud.autoscaling=DEBUG")
+@LuceneTestCase.BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // fixed on 9.0, broken? in 8.x
 public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
 
@@ -76,6 +79,7 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
 
   @BeforeClass
   public static void setupCluster() throws Exception {
+    System.setProperty("metricsEnabled", "true");
     configureCluster(2)
         .addConfig("conf", configset("cloud-minimal"))
         .configure();
@@ -97,6 +101,11 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
     cluster.waitForActiveCollection(COLL_NAME, 1, 3);
   }
 
+  @AfterClass
+  public static void releaseClient() throws Exception {
+    solrClient = null;
+  }
+
   public static class TesterListener extends TriggerListenerBase {
 
     @Override
@@ -124,6 +133,7 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
         "      {'replica':'<2', 'shard': '#EACH', 'node': '#ANY'}" +
         "    ]" +
         "}";
+    @SuppressWarnings({"rawtypes"})
     SolrRequest req = AutoScalingRequest.create(SolrRequest.METHOD.POST, setClusterPolicyCommand);
     solrClient.request(req);
 
@@ -270,7 +280,7 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
     JettySolrRunner jetty = cluster.startJettySolrRunner();
     cluster.waitForAllNodes(30);
     String nodeAddedName = jetty.getNodeName();
-    log.info("### Added node " + nodeAddedName);
+    log.info("### Added node {}", nodeAddedName);
     boolean await = actionFiredLatch.await(60, TimeUnit.SECONDS);
     assertTrue("action did not execute", await);
 
@@ -360,7 +370,7 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
       }
     }
     assertNotNull("no suitable node found", nodeToKill);
-    log.info("### Stopping node " + nodeToKill);
+    log.info("### Stopping node {}", nodeToKill);
     for (int i = 0; i < cluster.getJettySolrRunners().size(); i++) {
       if (cluster.getJettySolrRunner(i).getNodeName().equals(nodeToKill)) {
         JettySolrRunner j = cluster.stopJettySolrRunner(i);
@@ -368,7 +378,7 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
         break;
       }
     }
-    log.info("### Stopped node " + nodeToKill);
+    log.info("### Stopped node {}", nodeToKill);
     await = actionFiredLatch.await(60, TimeUnit.SECONDS);
     assertTrue("action did not execute", await);
 
@@ -399,17 +409,19 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
     QueryResponse rsp = client.query(query);
     SolrDocumentList docs = rsp.getResults();
     if (docs.size() != expected) {
-      log.info("History query: " + query);
-      log.info("Wrong response: " + rsp);
+      log.info("History query: {}", query);
+      log.info("Wrong response: {}", rsp);
       ModifiableSolrParams fullQuery = params(CommonParams.QT, CommonParams.AUTOSCALING_HISTORY_PATH);
-      log.info("Full response: " + client.query(fullQuery));
+      if (log.isInfoEnabled()) {
+        log.info("Full response: {}", client.query(fullQuery));
+      }
     }
     assertEquals("Wrong number of documents", expected, docs.size());
     return docs;
   }
 
   private static void waitForRecovery(String collection) throws Exception {
-    log.info("Waiting for recovery of " + collection);
+    log.info("Waiting for recovery of {}", collection);
     boolean recovered = false;
     boolean allActive = true;
     boolean hasLeaders = true;
@@ -417,7 +429,7 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
     for (int i = 0; i < 300; i++) {
       ClusterState state = solrClient.getZkStateReader().getClusterState();
       collState = getCollectionState(collection);
-      log.debug("###### " + collState);
+      log.debug("###### {}", collState);
       Collection<Replica> replicas = collState.getReplicas();
       allActive = true;
       hasLeaders = true;
@@ -425,11 +437,11 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
         for (Replica r : replicas) {
           if (state.getLiveNodes().contains(r.getNodeName())) {
             if (!r.isActive(state.getLiveNodes())) {
-              log.info("Not active: " + r);
+              log.info("Not active: {}", r);
               allActive = false;
             }
           } else {
-            log.info("Replica no longer on a live node, ignoring: " + r);
+            log.info("Replica no longer on a live node, ignoring: {}", r);
           }
         }
       } else {
@@ -444,7 +456,7 @@ public class AutoscalingHistoryHandlerTest extends SolrCloudTestCase {
         recovered = true;
         break;
       } else {
-        log.info("--- waiting, allActive=" + allActive + ", hasLeaders=" + hasLeaders);
+        log.info("--- waiting, allActive={}, hasLeaders={}", allActive, hasLeaders);
         Thread.sleep(1000);
       }
     }
