@@ -16,8 +16,18 @@
  */
 package org.apache.solr.handler.component;
 
+import java.io.File;
+import java.io.IOException;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashSet;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Map;
+import java.util.Map.Entry;
+import java.util.Set;
+
 import org.apache.commons.io.FileUtils;
-import org.apache.commons.lang.StringUtils;
 import org.apache.solr.SolrJettyTestBase;
 import org.apache.solr.client.solrj.SolrClient;
 import org.apache.solr.client.solrj.SolrQuery;
@@ -34,17 +44,6 @@ import org.junit.AfterClass;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
-import java.io.File;
-import java.io.IOException;
-import java.util.ArrayList;
-import java.util.Arrays;
-import java.util.HashSet;
-import java.util.Iterator;
-import java.util.List;
-import java.util.Map;
-import java.util.Map.Entry;
-import java.util.Set;
-
 public class DistributedDebugComponentTest extends SolrJettyTestBase {
   
   private static SolrClient collection1;
@@ -99,12 +98,18 @@ public class DistributedDebugComponentTest extends SolrJettyTestBase {
   
   @AfterClass
   public static void destroyThings() throws Exception {
-    collection1.close();
-    collection2.close();
-    collection1 = null;
-    collection2 = null;
-    jetty.stop();
-    jetty=null;
+    if (null != collection1) {
+      collection1.close();
+      collection1 = null;
+    }
+    if (null != collection2) {
+      collection2.close();
+      collection2 = null;
+    }
+    if (null != jetty) {
+      jetty.stop();
+      jetty=null;
+    }
     resetExceptionIgnores();
     systemClearPropertySolrDisableShardsWhitelist();
   }
@@ -118,6 +123,10 @@ public class DistributedDebugComponentTest extends SolrJettyTestBase {
     query.set("distrib", "true");
     query.setFields("id", "text");
     query.set("shards", shard1 + "," + shard2);
+    
+    if (random().nextBoolean()) {
+      query.add("omitHeader", Boolean.toString(random().nextBoolean()));
+    }
     QueryResponse response = collection1.query(query);
     NamedList<Object> track = (NamedList<Object>) response.getDebugMap().get("track");
     assertNotNull(track);
@@ -139,13 +148,6 @@ public class DistributedDebugComponentTest extends SolrJettyTestBase {
     assertElementsPresent((NamedList<String>)((NamedList<Object>)track.get("GET_FIELDS")).get(shard2), 
         "QTime", "ElapsedTime", "RequestPurpose", "NumFound", "Response");
     
-    query.add("omitHeader", "true");
-    response = collection1.query(query);
-    assertNull("QTime is not included in the response when omitHeader is set to true", 
-        ((NamedList<Object>)response.getDebugMap().get("track")).findRecursive("EXECUTE_QUERY", shard1, "QTime"));
-    assertNull("QTime is not included in the response when omitHeader is set to true", 
-        ((NamedList<Object>)response.getDebugMap().get("track")).findRecursive("GET_FIELDS", shard2, "QTime"));
-    
     query.setQuery("id:1");
     response = collection1.query(query);
     track = (NamedList<Object>) response.getDebugMap().get("track");
@@ -187,7 +189,7 @@ public class DistributedDebugComponentTest extends SolrJettyTestBase {
       } else if (random().nextBoolean()) {
         shards.remove(shard2);
       }
-      q.set("shards", StringUtils.join(shards, ","));
+      q.set("shards", String.join(",", shards));
 
 
       List<String> debug = new ArrayList<String>(10);
@@ -206,7 +208,7 @@ public class DistributedDebugComponentTest extends SolrJettyTestBase {
         debug.add("true");
         all = true;
       }
-      q.set("debug", (String[])debug.toArray(new String[debug.size()]));
+      q.set("debug", debug.toArray(new String[debug.size()]));
 
       QueryResponse r = client.query(q);
       try {
@@ -386,27 +388,24 @@ public class DistributedDebugComponentTest extends SolrJettyTestBase {
   }
   
   public void testTolerantSearch() throws SolrServerException, IOException {
-    String badShard = "[ff01::0083]:3334";
+    String badShard = DEAD_HOST_1;
     SolrQuery query = new SolrQuery();
     query.setQuery("*:*");
     query.set("debug",  "true");
     query.set("distrib", "true");
     query.setFields("id", "text");
     query.set("shards", shard1 + "," + shard2 + "," + badShard);
-    try {
-      ignoreException("Server refused connection");
-      // verify that the request would fail if shards.tolerant=false
-      collection1.query(query);
-      fail("Expecting exception");
-    } catch (SolrException e) {
-      //expected
-    }
+
+    // verify that the request would fail if shards.tolerant=false
+    ignoreException("Server refused connection");
+    expectThrows(SolrException.class, () -> collection1.query(query));
+
     query.set(ShardParams.SHARDS_TOLERANT, "true");
     QueryResponse response = collection1.query(query);
     assertTrue((Boolean)response.getResponseHeader().get(SolrQueryResponse.RESPONSE_HEADER_PARTIAL_RESULTS_KEY));
     @SuppressWarnings("unchecked")
-    NamedList<String> badShardTrack = (NamedList<String>) ((NamedList<NamedList<String>>)
-        ((NamedList<NamedList<NamedList<String>>>)response.getDebugMap().get("track")).get("EXECUTE_QUERY")).get(badShard);
+    NamedList<String> badShardTrack =
+            (((NamedList<NamedList<NamedList<String>>>)response.getDebugMap().get("track")).get("EXECUTE_QUERY")).get(badShard);
     assertEquals("Unexpected response size for shard", 1, badShardTrack.size());
     Entry<String, String> exception = badShardTrack.iterator().next();
     assertEquals("Expected key 'Exception' not found", "Exception", exception.getKey());
@@ -423,7 +422,7 @@ public class DistributedDebugComponentTest extends SolrJettyTestBase {
 
   @SuppressWarnings({"unchecked", "rawtypes"})
   private void assertSameKeys(NamedList object, NamedList object2) {
-    Iterator<Map.Entry<String,Object>> iteratorObj2 = ((NamedList)object2).iterator();
+    Iterator<Map.Entry<String,Object>> iteratorObj2 = (object2).iterator();
     for (Map.Entry<String,Object> entry:(NamedList<Object>)object) {
       assertTrue(iteratorObj2.hasNext());
       Map.Entry<String,Object> entry2 = iteratorObj2.next();
