@@ -28,24 +28,29 @@ import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
 
+import com.fasterxml.jackson.databind.ObjectMapper;
 import org.apache.lucene.util.TestUtil;
 import org.apache.solr.SolrTestCaseJ4;
 import org.apache.solr.common.SolrInputDocument;
+import org.apache.solr.common.params.StreamParams;
 import org.apache.solr.common.util.SuppressForbidden;
 import org.apache.solr.common.util.Utils;
+import org.apache.solr.index.LogDocMergePolicyFactory;
 import org.apache.solr.request.SolrQueryRequest;
 import org.apache.solr.schema.SchemaField;
-import org.codehaus.jackson.map.ObjectMapper;
 import org.junit.Before;
 import org.junit.BeforeClass;
 import org.junit.Test;
-import org.noggit.ObjectBuilder;
 
 public class TestExportWriter extends SolrTestCaseJ4 {
+
+  private ObjectMapper mapper = new ObjectMapper();
   
   @BeforeClass
   public static void beforeClass() throws Exception {
-    System.setProperty("export.test", "true");
+    // force LogDocMergePolicy so that we get a predictable doc order
+    // when testing index order results
+    systemSetPropertySolrTestsMergePolicyFactory(LogDocMergePolicyFactory.class.getName());
     initCore("solrconfig-sortingresponse.xml","schema-sortingresponse.xml");
   }
 
@@ -58,29 +63,6 @@ public class TestExportWriter extends SolrTestCaseJ4 {
 
   }
 
-  @Test
-  public void testSortingOnFieldWithNoValues() throws Exception {
-    assertU(delQ("*:*"));
-    assertU(commit());
-
-    assertU(adoc("id","1"));
-    assertU(commit());
-
-    // 10 fields
-    List<String> fieldNames = new ArrayList<>(Arrays.asList("floatdv", "intdv", "stringdv", "longdv", "doubledv",
-        "datedv", "booleandv", "field1_s_dv", "field2_i_p", "field3_l_p"));
-    for (String sortField : fieldNames) {
-      String resp = h.query(req("q", "*:*", "qt", "/export", "fl", "id," + sortField, "sort", sortField + " desc"));
-      assertJsonEquals(resp, "{\n" +
-          "  \"responseHeader\":{\"status\":0},\n" +
-          "  \"response\":{\n" +
-          "    \"numFound\":1,\n" +
-          "    \"docs\":[{\n" +
-          "        \"id\":\"1\"}]}}");
-    }
-
-  }
-
   @Test
   public void testEmptyValues() throws Exception {
     //Index 2 document with one document that doesn't have field2_i_p
@@ -107,6 +89,29 @@ public class TestExportWriter extends SolrTestCaseJ4 {
 
   }
 
+  @Test
+  public void testSortingOnFieldWithNoValues() throws Exception {
+    assertU(delQ("*:*"));
+    assertU(commit());
+
+    assertU(adoc("id","1"));
+    assertU(commit());
+
+    // 10 fields
+    List<String> fieldNames = new ArrayList<>(Arrays.asList("floatdv", "intdv", "stringdv", "longdv", "doubledv",
+        "datedv", "booleandv", "field1_s_dv", "field2_i_p", "field3_l_p"));
+    for (String sortField : fieldNames) {
+      String resp = h.query(req("q", "*:*", "qt", "/export", "fl", "id," + sortField, "sort", sortField + " desc"));
+      assertJsonEquals(resp, "{\n" +
+          "  \"responseHeader\":{\"status\":0},\n" +
+          "  \"response\":{\n" +
+          "    \"numFound\":1,\n" +
+          "    \"docs\":[{\n" +
+          "        \"id\":\"1\"}]}}");
+    }
+
+  }
+
   public static void createIndex() {
     assertU(adoc("id","1",
                  "floatdv","2.1",
@@ -129,7 +134,13 @@ public class TestExportWriter extends SolrTestCaseJ4 {
                  "datedv_m", "2017-06-16T01:00:00Z",
                  "datedv_m", "2017-06-16T02:00:00Z",
                  "datedv_m", "2017-06-16T03:00:00Z",
-                 "datedv_m", "2017-06-16T04:00:00Z"));
+                 "datedv_m", "2017-06-16T04:00:00Z",
+                 "sortabledv_m", "this is some text one_1",
+                 "sortabledv_m", "this is some text two_1",
+                 "sortabledv_m", "this is some text three_1",
+                 "sortabledv_m_udvas", "this is some text one_1",
+                 "sortabledv_m_udvas", "this is some text two_1",
+                 "sortabledv_m_udvas", "this is some text three_1"));
 
     assertU(adoc("id","7",
         "floatdv","2.1",
@@ -166,7 +177,9 @@ public class TestExportWriter extends SolrTestCaseJ4 {
         "int_is_t", "1",
         "int_is_t", "1",
         "int_is_t", "1",
-        "int_is_t", "1"));
+        "int_is_t", "1",
+        "sortabledv", "this is some text_1",
+        "sortabledv_udvas", "this is some text_1"));
     assertU(commit());
     assertU(adoc("id","8",
         "floatdv","2.1",
@@ -191,7 +204,16 @@ public class TestExportWriter extends SolrTestCaseJ4 {
         "int_is_p", "1",
         "int_is_p", "1",
         "int_is_p", "1",
-        "int_is_p", "1"));
+        "int_is_p", "1",
+        "sortabledv", "this is some text_2",
+        "sortabledv_udvas", "this is some text_2",
+        "sortabledv_m", "this is some text one_2",
+        "sortabledv_m", "this is some text two_2",
+        "sortabledv_m", "this is some text three_2",
+        "sortabledv_m_udvas", "this is some text one_2",
+        "sortabledv_m_udvas", "this is some text two_2",
+        "sortabledv_m_udvas", "this is some text three_2"
+    ));
     assertU(commit());
 
 
@@ -491,6 +513,33 @@ public class TestExportWriter extends SolrTestCaseJ4 {
 
     s =  h.query(req("q", "id:8", "qt", "/export", "fl", "stringdv", "sort", "intdv asc"));
     assertJsonEquals(s, "{\"responseHeader\": {\"status\": 0}, \"response\":{\"numFound\":1, \"docs\":[{\"stringdv\":\"chello \\\"world\\\"\"}]}}");
+
+    // Test sortable text fields:
+    s =  h.query(req("q", "id:(1 OR 3 OR 8)", "qt", "/export", "fl", "sortabledv_m_udvas,sortabledv_udvas", "sort", "sortabledv_udvas asc"));
+    assertJsonEquals(s, "{\n" +
+        "  \"responseHeader\":{\"status\":0},\n" +
+        "  \"response\":{\n" +
+        "    \"numFound\":3,\n" +
+        "    \"docs\":[{\n" +
+        "        \"sortabledv_m_udvas\":[\"this is some text one_1\"\n" +
+        "          ,\"this is some text three_1\"\n" +
+        "          ,\"this is some text two_1\"]}\n" +
+        "      ,{\n" +
+        "        \"sortabledv_udvas\":\"this is some text_1\"}\n" +
+        "      ,{\n" +
+        "        \"sortabledv_m_udvas\":[\"this is some text one_2\"\n" +
+        "          ,\"this is some text three_2\"\n" +
+        "          ,\"this is some text two_2\"],\n" +
+        "        \"sortabledv_udvas\":\"this is some text_2\"}]}}");
+
+    s =  h.query(req("q", "id:(1 OR 3 OR 8)", "qt", "/export", "fl", "sortabledv_m", "sort", "sortabledv_udvas asc"));
+    assertTrue("Should have 400 status when exporting sortabledv_m, it does not have useDocValuesAsStored='true'", s.contains("\"status\":400}"));
+    assertTrue("Should have a cause when exporting sortabledv_m, it does not have useDocValuesAsStored='true'", s.contains("Must have useDocValuesAsStored='true' to be used with export writer"));
+
+    s =  h.query(req("q", "id:(1 OR 3 OR 8)", "qt", "/export", "fl", "sortabledv", "sort", "sortabledv_udvas asc"));
+    assertTrue("Should have 400 status when exporting sortabledv, it does not have useDocValuesAsStored='true'", s.contains("\"status\":400}"));
+    assertTrue("Should have a cause when exporting sortabledv, it does not have useDocValuesAsStored='true'", s.contains("Must have useDocValuesAsStored='true' to be used with export writer"));
+
   }
 
   private void assertJsonEquals(String actual, String expected) {
@@ -660,6 +709,80 @@ public class TestExportWriter extends SolrTestCaseJ4 {
     validateSort(numDocs);
   }
 
+  private void createLargeIndex() throws Exception {
+    int BATCH_SIZE = 5000;
+    int NUM_BATCHES = 20;
+    SolrInputDocument[] docs = new SolrInputDocument[BATCH_SIZE];
+    for (int i = 0; i < NUM_BATCHES; i++) {
+      for (int j = 0; j < BATCH_SIZE; j++) {
+        docs[j] = new SolrInputDocument(
+            "id", String.valueOf(i * BATCH_SIZE + j),
+            "sortabledv", TestUtil.randomSimpleString(random(), 2, 3),
+            "sortabledv_udvas", String.valueOf((i + j) % 101),
+            "small_i_p", String.valueOf((i + j) % 37)
+            );
+      }
+      updateJ(jsonAdd(docs), null);
+    }
+    assertU(commit());
+  }
+
+  @Test
+  public void testExpr() throws Exception {
+    assertU(delQ("*:*"));
+    assertU(commit());
+    createLargeIndex();
+    SolrQueryRequest req = req("q", "*:*", "qt", "/export", "fl", "id", "sort", "id asc", "expr", "top(n=2,input(),sort=\"id desc\")");
+    assertJQ(req,
+        "response/numFound==100000",
+        "response/docs/[0]/id=='99999'",
+        "response/docs/[1]/id=='99998'"
+        );
+    req = req("q", "*:*", "qt", "/export", "fl", "id,sortabledv_udvas", "sort", "sortabledv_udvas asc", "expr", "unique(input(),over=\"sortabledv_udvas\")");
+    String rsp = h.query(req);
+    @SuppressWarnings({"unchecked"})
+    Map<String, Object> rspMap = mapper.readValue(rsp, HashMap.class);
+    @SuppressWarnings({"unchecked"})
+    List<Map<String, Object>> docs = (List<Map<String, Object>>) Utils.getObjectByPath(rspMap, false, "/response/docs");
+    assertNotNull("missing document results: " + rspMap, docs);
+    assertEquals("wrong number of unique docs", 101, docs.size());
+    for (int i = 0; i < 100; i++) {
+      boolean found = false;
+      String si = String.valueOf(i);
+      for (int j = 0; j < docs.size(); j++) {
+        if (docs.get(j).get("sortabledv_udvas").equals(si)) {
+          found = true;
+          break;
+        }
+      }
+      assertTrue("missing value " + i + " in results", found);
+    }
+    req = req("q", "*:*", "qt", "/export", "fl", "id,sortabledv_udvas,small_i_p", "sort", "sortabledv_udvas asc", "expr", "rollup(input(),over=\"sortabledv_udvas\", sum(small_i_p),avg(small_i_p),min(small_i_p),count(*))");
+    rsp = h.query(req);
+    rspMap = mapper.readValue(rsp, HashMap.class);
+    docs = (List<Map<String, Object>>) Utils.getObjectByPath(rspMap, false, "/response/docs");
+    assertNotNull("missing document results: " + rspMap, docs);
+    assertEquals("wrong number of unique docs", 101, docs.size());
+    for (Map<String, Object> doc : docs) {
+      assertNotNull("missing sum: " + doc, doc.get("sum(small_i_p)"));
+      assertEquals(18000.0, ((Number)doc.get("sum(small_i_p)")).doubleValue(), 1000.0);
+      assertNotNull("missing avg: " + doc, doc.get("avg(small_i_p)"));
+      assertEquals(18.0, ((Number)doc.get("avg(small_i_p)")).doubleValue(), 1.0);
+      assertNotNull("missing count: " + doc, doc.get("count(*)"));
+      assertEquals(1000.0, ((Number)doc.get("count(*)")).doubleValue(), 100.0);
+    }
+    // try invalid field types
+    req = req("q", "*:*", "qt", "/export", "fl", "id,sortabledv,small_i_p", "sort", "sortabledv asc", "expr", "unique(input(),over=\"sortabledv\")");
+    rsp = h.query(req);
+    rspMap = mapper.readValue(rsp, HashMap.class);
+    assertEquals("wrong response status", 400, ((Number)Utils.getObjectByPath(rspMap, false, "/responseHeader/status")).intValue());
+    docs = (List<Map<String, Object>>) Utils.getObjectByPath(rspMap, false, "/response/docs");
+    assertEquals("wrong number of docs", 1, docs.size());
+    Map<String, Object> doc = docs.get(0);
+    assertTrue("doc doesn't have exception", doc.containsKey(StreamParams.EXCEPTION));
+    assertTrue("wrong exception message", doc.get(StreamParams.EXCEPTION).toString().contains("Must have useDocValuesAsStored='true'"));
+  }
+
   private void validateSort(int numDocs) throws Exception {
     // 10 fields
     List<String> fieldNames = new ArrayList<>(Arrays.asList("floatdv", "intdv", "stringdv", "longdv", "doubledv",
@@ -681,13 +804,16 @@ public class TestExportWriter extends SolrTestCaseJ4 {
     String fieldsStr = String.join(",", fieldStrs); // fl :  field1, field2
 
     String resp = h.query(req("q", "*:*", "qt", "/export", "fl", "id," + fieldsStr, "sort", sortStr));
-    ObjectMapper mapper = new ObjectMapper();
+    @SuppressWarnings({"rawtypes"})
     HashMap respMap = mapper.readValue(resp, HashMap.class);
+    @SuppressWarnings({"rawtypes"})
     List docs = (ArrayList) ((HashMap) respMap.get("response")).get("docs");
 
     SolrQueryRequest selectReq = req("q", "*:*", "qt", "/select", "fl", "id," + fieldsStr, "sort", sortStr, "rows", Integer.toString(numDocs), "wt", "json");
     String response = h.query(selectReq);
-    Map rsp = (Map)ObjectBuilder.fromJSON(response);
+    @SuppressWarnings({"rawtypes"})
+    Map rsp = (Map)Utils.fromJSONString(response);
+    @SuppressWarnings({"rawtypes"})
     List doclist = (List)(((Map)rsp.get("response")).get("docs"));
 
     assert docs.size() == numDocs;
