@@ -22,6 +22,7 @@ import java.util.Iterator;
 import org.apache.lucene.util.TestUtil;
 import org.apache.solr.SolrTestCaseJ4;
 import org.apache.solr.common.SolrDocument;
+import org.apache.solr.common.SolrException;
 import org.apache.solr.common.SolrInputDocument;
 import org.apache.solr.request.SolrQueryRequest;
 import org.apache.solr.response.BasicResultContext;
@@ -29,6 +30,8 @@ import org.junit.After;
 import org.junit.BeforeClass;
 import org.junit.Test;
 
+import static org.hamcrest.core.StringContains.containsString;
+
 public class TestChildDocTransformer extends SolrTestCaseJ4 {
 
   private static String ID_FIELD = "id";
@@ -36,7 +39,7 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
 
   @BeforeClass
   public static void beforeClass() throws Exception {
-    initCore("solrconfig.xml","schema.xml"); // *not* the "nest" schema version
+    initCore("solrconfig.xml","schema-root.xml"); // *not* the "nest" schema
   }
 
   @After
@@ -92,16 +95,38 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
         "/response/result/doc[1]/doc[1]/str[@name='id']='3'" ,
         "/response/result/doc[1]/doc[2]/str[@name='id']='5'"};
 
+    assertQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
+        "fl", "*,[child]"), test1);
 
+    // shows parentFilter specified (not necessary any more) and also child
+    assertQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
+        "fl", "id, subject,[child childFilter=\"title:foo\"]"), test2);
 
     assertQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "*,[child parentFilter=\"subject:parentDocument\"]"), test1);
+        "fl", "id, subject,[child childFilter=\"title:bar\" limit=2]"), test3);
 
-    assertQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "id, subject,[child parentFilter=\"subject:parentDocument\" childFilter=\"title:foo\"]"), test2);
+    SolrException e = expectThrows(SolrException.class, () -> {
+      h.query(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
+          "fl", "id, subject,[child parentFilter=\"subject:bleh\" childFilter=\"title:bar\" limit=2]"));
+    });
+    assertEquals(SolrException.ErrorCode.BAD_REQUEST.code, e.code());
+    assertThat(e.getMessage(),
+        containsString("Parent filter 'QueryBitSetProducer(subject:bleh)' doesn't match any parent documents"));
 
-    assertQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "id, subject,[child parentFilter=\"subject:parentDocument\" childFilter=\"title:bar\" limit=2]"), test3);
+    e = expectThrows(SolrException.class, () -> {
+      h.query(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
+          "fl", "id, subject,[child parentFilter=e childFilter=\"title:bar\" limit=2]"));
+    });
+    assertEquals(SolrException.ErrorCode.BAD_REQUEST.code, e.code());
+    assertThat(e.getMessage(),
+        containsString("Parent filter 'QueryBitSetProducer(text:e)' doesn't match any parent documents"));
+
+    e = expectThrows(SolrException.class, () -> {
+      h.query(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
+          "fl", "id, subject,[child parentFilter=\"\"]"));
+    });
+    assertEquals(SolrException.ErrorCode.BAD_REQUEST.code, e.code());
+    assertThat(e.getMessage(), containsString("Invalid Parent filter '', resolves to null"));
   }
   
   private void testSubQueryXML() {
@@ -216,13 +241,13 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
     };
 
     assertJQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "*,[child parentFilter=\"subject:parentDocument\"]"), test1);
+        "fl", "*,[child]"), test1);
 
     assertJQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "id, subject,[child parentFilter=\"subject:parentDocument\" childFilter=\"title:foo\"]"), test2);
+        "fl", "id, subject,[child childFilter=\"title:foo\"]"), test2);
 
     assertJQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "id, subject,[child parentFilter=\"subject:parentDocument\" childFilter=\"title:bar\" limit=3]"), test3);
+        "fl", "id, subject,[child childFilter=\"title:bar\" limit=3]"), test3);
   }
 
   private void testChildDocNonStoredDVFields() throws Exception {
@@ -247,26 +272,26 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
     };
 
     assertJQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "*,[child parentFilter=\"subject:parentDocument\"]"), test1);
+        "fl", "*,[child]"), test1);
 
     assertJQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "intDvoDefault, subject,[child parentFilter=\"subject:parentDocument\" childFilter=\"title:foo\"]"), test2);
+        "fl", "intDvoDefault, subject,[child childFilter=\"title:foo\"]"), test2);
 
     assertJQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "intDvoDefault, subject,[child parentFilter=\"subject:parentDocument\" childFilter=\"title:bar\" limit=2]"), test3);
+        "fl", "intDvoDefault, subject,[child childFilter=\"title:bar\" limit=2]"), test3);
 
   }
 
   private void testChildReturnFields() throws Exception {
 
     assertJQ(req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "*,[child parentFilter=\"subject:parentDocument\" fl=\"intDvoDefault,child_fl:[value v='child_fl_test']\"]"),
+        "fl", "*,[child fl=\"intDvoDefault,child_fl:[value v='child_fl_test']\"]"),
         "/response/docs/[0]/intDefault==42",
         "/response/docs/[0]/_childDocuments_/[0]/intDvoDefault==42",
         "/response/docs/[0]/_childDocuments_/[0]/child_fl=='child_fl_test'");
 
     try(SolrQueryRequest req = req("q", "*:*", "fq", "subject:\"parentDocument\" ",
-        "fl", "intDefault,[child parentFilter=\"subject:parentDocument\" fl=\"intDvoDefault, [docid]\"]")) {
+        "fl", "intDefault,[child fl=\"intDvoDefault, [docid]\"]")) {
       BasicResultContext res = (BasicResultContext) h.queryAndResponse("/select", req).getResponse();
       Iterator<SolrDocument> docsStreamer = res.getProcessedDocuments();
       while (docsStreamer.hasNext()) {
@@ -356,12 +381,12 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
     String[] tests = new String[] {
         "/response/docs/[0]/id=='1'",
         "/response/docs/[0]/_childDocuments_/[0]/id=='2'",
-        "/response/docs/[0]/_childDocuments_/[0]/cat/[0]/=='childDocument'",
-        "/response/docs/[0]/_childDocuments_/[0]/title/[0]/=='" + titleVals[0] + "'",
+        "/response/docs/[0]/_childDocuments_/[0]/cat=='childDocument'",
+        "/response/docs/[0]/_childDocuments_/[0]/title=='" + titleVals[0] + "'",
         "/response/docs/[1]/id=='4'",
         "/response/docs/[1]/_childDocuments_/[0]/id=='5'",
-        "/response/docs/[1]/_childDocuments_/[0]/cat/[0]/=='childDocument'",
-        "/response/docs/[1]/_childDocuments_/[0]/title/[0]/=='" + titleVals[1] + "'"
+        "/response/docs/[1]/_childDocuments_/[0]/cat=='childDocument'",
+        "/response/docs/[1]/_childDocuments_/[0]/title=='" + titleVals[1] + "'"
     };
 
 
@@ -377,6 +402,14 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
                  "fl", "id, cat, title, [child childFilter='cat:childDocument' parentFilter=\"subject:parentDocument\"]"),
              tests);
 
+    // shows if parentFilter matches all docs, then there are effectively no child docs
+    assertJQ(req("q", "*:*",
+        "sort", "id asc",
+        "fq", "subject:\"parentDocument\" ",
+        "fl", "id,[child childFilter='cat:childDocument' parentFilter=\"*:*\"]"),
+        "/response==" +
+            "{'numFound':2,'start':0,'numFoundExact':true,'docs':[{'id':'1'},{'id':'4'}]}");
+
   }
   
   private void testSubQueryParentFilterJSON() throws Exception {
@@ -384,12 +417,12 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
     String[] tests = new String[] {
         "/response/docs/[0]/id=='1'",
         "/response/docs/[0]/children/docs/[0]/id=='2'",
-        "/response/docs/[0]/children/docs/[0]/cat/[0]/=='childDocument'",
-        "/response/docs/[0]/children/docs/[0]/title/[0]/=='" + titleVals[0] + "'",
+        "/response/docs/[0]/children/docs/[0]/cat=='childDocument'",
+        "/response/docs/[0]/children/docs/[0]/title=='" + titleVals[0] + "'",
         "/response/docs/[1]/id=='4'",
         "/response/docs/[1]/children/docs/[0]/id=='5'",
-        "/response/docs/[1]/children/docs/[0]/cat/[0]/=='childDocument'",
-        "/response/docs/[1]/children/docs/[0]/title/[0]/=='" + titleVals[1] + "'"
+        "/response/docs/[1]/children/docs/[0]/cat=='childDocument'",
+        "/response/docs/[1]/children/docs/[0]/title=='" + titleVals[1] + "'"
     };
 
 
@@ -417,23 +450,23 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
         "//*[@numFound='2']",
         "/response/result/doc[1]/str[@name='id']='1'" ,
         "/response/result/doc[1]/doc[1]/str[@name='id']='2'" ,
-        "/response/result/doc[1]/doc[1]/arr[@name='cat']/str[1]='childDocument'" ,
-        "/response/result/doc[1]/doc[1]/arr[@name='title']/str[1]='" + titleVals[0] + "'" ,
+        "/response/result/doc[1]/doc[1]/str[@name='cat']='childDocument'" ,
+        "/response/result/doc[1]/doc[1]/str[@name='title']='" + titleVals[0] + "'" ,
         "/response/result/doc[2]/str[@name='id']='4'" ,
         "/response/result/doc[2]/doc[1]/str[@name='id']='5'",
-        "/response/result/doc[2]/doc[1]/arr[@name='cat']/str[1]='childDocument'",
-        "/response/result/doc[2]/doc[1]/arr[@name='title']/str[1]='" + titleVals[1] + "'"};
+        "/response/result/doc[2]/doc[1]/str[@name='cat']='childDocument'",
+        "/response/result/doc[2]/doc[1]/str[@name='title']='" + titleVals[1] + "'"};
 
     assertQ(req("q", "*:*", 
                 "sort", "id asc",
                 "fq", "subject:\"parentDocument\" ",
-                "fl", "*,[child childFilter='cat:childDocument' parentFilter=\"subject:parentDocument\"]"), 
+                "fl", "*,[child childFilter='cat:childDocument']"), 
             tests);
 
     assertQ(req("q", "*:*", 
                 "sort", "id asc",
                 "fq", "subject:\"parentDocument\" ",
-                "fl", "id, cat, title, [child childFilter='cat:childDocument' parentFilter=\"subject:parentDocument\"]"),
+                "fl", "id, cat, title, [child childFilter='cat:childDocument']"),
             tests);
   }
 
@@ -443,12 +476,12 @@ public class TestChildDocTransformer extends SolrTestCaseJ4 {
         "//*[@numFound='2']",
         "/response/result/doc[1]/str[@name='id']='1'" ,
         "/response/result/doc[1]/result[@name='children'][@numFound=1]/doc[1]/str[@name='id']='2'" ,
-        "/response/result/doc[1]/result[@name='children'][@numFound=1]/doc[1]/arr[@name='cat']/str[1]='childDocument'" ,
-        "/response/result/doc[1]/result[@name='children'][@numFound=1]/doc[1]/arr[@name='title']/str[1]='" + titleVals[0] + "'" ,
+        "/response/result/doc[1]/result[@name='children'][@numFound=1]/doc[1]/str[@name='cat']='childDocument'" ,
+        "/response/result/doc[1]/result[@name='children'][@numFound=1]/doc[1]/str[@name='title']='" + titleVals[0] + "'" ,
         "/response/result/doc[2]/str[@name='id']='4'" ,
         "/response/result/doc[2]/result[@name='children'][@numFound=1]/doc[1]/str[@name='id']='5'",
-        "/response/result/doc[2]/result[@name='children'][@numFound=1]/doc[1]/arr[@name='cat']/str[1]='childDocument'",
-        "/response/result/doc[2]/result[@name='children'][@numFound=1]/doc[1]/arr[@name='title']/str[1]='" + titleVals[1] + "'"};
+        "/response/result/doc[2]/result[@name='children'][@numFound=1]/doc[1]/str[@name='cat']='childDocument'",
+        "/response/result/doc[2]/result[@name='children'][@numFound=1]/doc[1]/str[@name='title']='" + titleVals[1] + "'"};
 
     assertQ(req(
         "q", "*:*", "fq", "subject:\"parentDocument\" ",
