@@ -48,14 +48,14 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
 
   @BeforeClass
   public static void beforeClass() throws Exception {
-    initCore("solrconfig-update-processor-chains.xml", "schema-nest.xml"); // use "nest" schema
+    initCore("solrconfig-minimal.xml", "schema-nest.xml"); // use "nest" schema
 
     if(random().nextBoolean()) {
       idCounter.set(-100); // start docIDs at -100 for these random docs we don't care about (all less than 0)
       // create random segments
       final int numOfDocs = 10;
       for(int i = 0; i < numOfDocs; ++i) {
-        updateJ(generateDocHierarchy(i), params("update.chain", "nested"));
+        updateJ(generateDocHierarchy(i), null);
         if(random().nextBoolean()) {
           assertU(commit());
         }
@@ -75,6 +75,21 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
     assertU(commit());
   }
 
+  @Test
+  public void testNonTrivialChildFilter() throws Exception {
+    // just check we don't throw an exception. This used to throw before SOLR-15152
+    assertQ(
+            req(
+                    "q",
+                    "*:*",
+                    "sort",
+                    "id asc",
+                    "fl",
+                    "*, _nest_path_, [child childFilter='type_s:Regular OR type_s:Chocolate']",
+                    "fq",
+                    fqToExcludeNonTestedDocs));
+  }
+
   @Test
   public void testParentFilterJSON() throws Exception {
     indexSampleData(numberOfDocsPerNestedTest);
@@ -111,7 +126,7 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
   public void testParentFilterLimitJSON() throws Exception {
     indexSampleData(numberOfDocsPerNestedTest);
 
-    try(SolrQueryRequest req = req("q", "type_s:donut", "sort", "id asc", "fl", "id, type_s, toppings, _nest_path_, [child childFilter='_nest_path_:\"toppings/\"' limit=1]",
+    try(SolrQueryRequest req = req("q", "type_s:donut", "sort", "id asc", "fl", "id, type_s, toppings, _nest_path_, [child childFilter='{!field f=_nest_path_}/toppings' limit=1]",
         "fq", fqToExcludeNonTestedDocs)) {
       BasicResultContext res = (BasicResultContext) h.queryAndResponse("/select", req).getResponse();
       Iterator<SolrDocument> docsStreamer = res.getProcessedDocuments();
@@ -135,6 +150,53 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
     );
   }
 
+  @Test
+  public void testWithDeletedChildren() throws Exception {
+    // add a doc to create another segment
+    final String addNonTestedDoc =
+        "{\n" +
+          "\"add\": {\n" +
+            "\"doc\": {\n" +
+                "\"id\": " + -1000 + ", \n" +
+                "\"type_s\": \"cake\", \n" +
+            "}\n" +
+          "}\n" +
+        "}";
+
+    if (random().nextBoolean()) {
+      updateJ(addNonTestedDoc, null);
+      assertU(commit());
+    }
+
+    indexSampleData(numberOfDocsPerNestedTest);
+    // delete toppings path
+    assertU(delQ("_nest_path_:\\/toppings"));
+    assertU(commit());
+
+    try(SolrQueryRequest req = req("q", "type_s:donut", "sort", "id asc", "fl", "id, type_s, toppings, _nest_path_, [child childFilter='_nest_path_:\\\\/toppings' limit=1]",
+        "fq", fqToExcludeNonTestedDocs)) {
+      BasicResultContext res = (BasicResultContext) h.queryAndResponse("/select", req).getResponse();
+      Iterator<SolrDocument> docsStreamer = res.getProcessedDocuments();
+      while (docsStreamer.hasNext()) {
+        SolrDocument doc = docsStreamer.next();
+        cleanSolrDocumentFields(doc);
+        assertFalse("root doc should not have anonymous child docs", doc.hasChildDocuments());
+        assertNull("should not include deleted docs", doc.getFieldValues("toppings"));
+      }
+    }
+
+    assertJQ(req("q", "type_s:donut",
+        "sort", "id asc",
+        "fl", "*, [child limit=1]",
+        "fq", fqToExcludeNonTestedDocs),
+        "/response/docs/[0]/type_s==donut",
+        "/response/docs/[0]/lonely/test_s==testing",
+        "/response/docs/[0]/lonely/lonelyGrandChild/test2_s==secondTest",
+        // "!" (negate): don't find toppings.  The "limit" kept us from reaching these, which follow lonely.
+        "!/response/docs/[0]/toppings/[0]/type_s==Regular"
+    );
+  }
+
   @Test
   public void testChildFilterLimitJSON() throws Exception {
     indexSampleData(numberOfDocsPerNestedTest);
@@ -165,19 +227,19 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
     indexSampleData(2);
     String[] tests = {
         "/response/numFound==4",
-        "/response/docs/[0]/_nest_path_=='toppings#0'",
-        "/response/docs/[1]/_nest_path_=='toppings#0'",
-        "/response/docs/[2]/_nest_path_=='toppings#1'",
-        "/response/docs/[3]/_nest_path_=='toppings#1'",
+        "/response/docs/[0]/_nest_path_=='/toppings#0'",
+        "/response/docs/[1]/_nest_path_=='/toppings#0'",
+        "/response/docs/[2]/_nest_path_=='/toppings#1'",
+        "/response/docs/[3]/_nest_path_=='/toppings#1'",
     };
 
-    assertJQ(req("q", "_nest_path_:*toppings/",
+    assertJQ(req("q", "_nest_path_:*toppings",
         "sort", "_nest_path_ asc",
         "fl", "*, id_i, _nest_path_",
         "fq", fqToExcludeNonTestedDocs),
         tests);
 
-    assertJQ(req("q", "+_nest_path_:\"toppings/\"",
+    assertJQ(req("q", "+_nest_path_:\"/toppings\"",
         "sort", "_nest_path_ asc",
         "fl", "*, _nest_path_",
         "fq", fqToExcludeNonTestedDocs),
@@ -219,13 +281,40 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
       }
     }
 
-
-
+    // test full path
     assertJQ(req("q", "type_s:donut",
         "sort", "id asc",
         "fl", "*,[child childFilter='toppings/ingredients/name_s:cocoa']",
         "fq", fqToExcludeNonTestedDocs),
         tests);
+
+    // test partial path
+    assertJQ(req("q", "type_s:donut",
+        "sort", "id asc",
+        "fl", "*,[child childFilter='ingredients/name_s:cocoa']",
+        "fq", fqToExcludeNonTestedDocs),
+        tests);
+
+    // test absolute path
+    assertJQ(req("q", "type_s:donut",
+        "sort", "id asc",
+        "fl", "*,[child childFilter='/toppings/ingredients/name_s:cocoa']",
+        "fq", fqToExcludeNonTestedDocs),
+        tests);
+  }
+
+  @Test
+  public void testNestPathTransformerMatches() throws Exception {
+    indexSampleData(numberOfDocsPerNestedTest);
+
+    // test partial path
+    // should not match any child docs
+    assertQ(req("q", "type_s:donut",
+        "sort", "id asc",
+        "fl", "*,[child childFilter='redients/name_s:cocoa']",
+        "fq", fqToExcludeNonTestedDocs),
+        "//result/doc/str[@name='type_s'][.='donut']", "not(//result/doc/arr[@name='toppings'])"
+        );
   }
 
   @Test
@@ -287,7 +376,7 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
             "}\n" +
           "}\n" +
         "}";
-    updateJ(addDocWoChildren, params("update.chain", "nested"));
+    updateJ(addDocWoChildren, null);
     assertU(commit());
 
     assertJQ(req("q", "type_s:cake",
@@ -299,7 +388,7 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
 
   private void indexSampleData(int numDocs) throws Exception {
     for(int i = 0; i < numDocs; ++i) {
-      updateJ(generateDocHierarchy(i), params("update.chain", "nested"));
+      updateJ(generateDocHierarchy(i), null);
     }
     assertU(commit());
   }
@@ -308,6 +397,7 @@ public class TestChildDocTransformerHierarchy extends SolrTestCaseJ4 {
     return idCounter.incrementAndGet();
   }
 
+  @SuppressWarnings({"unchecked"})
   private static void cleanSolrDocumentFields(SolrDocument input) {
     for(String fieldName: fieldsToRemove) {
       input.removeFields(fieldName);
