@@ -16,41 +16,42 @@
  */
 package org.apache.solr.rest.schema;
 
-import org.apache.commons.io.FileUtils;
-
-import org.apache.lucene.search.similarities.DFISimilarity;
-import org.apache.lucene.search.similarities.PerFieldSimilarityWrapper;
-import org.apache.lucene.search.similarities.BM25Similarity;
-import org.apache.lucene.misc.SweetSpotSimilarity;
-import org.apache.lucene.search.similarities.Similarity;
-
-import org.apache.solr.common.SolrDocumentList;
-import org.apache.solr.core.SolrCore;
-import org.apache.solr.core.CoreContainer;
-import org.apache.solr.schema.SimilarityFactory;
-import org.apache.solr.search.similarities.SchemaSimilarityFactory;
-import org.apache.solr.util.RESTfulServerProvider;
-import org.apache.solr.util.RestTestBase;
-import org.apache.solr.util.RestTestHarness;
-
-import org.junit.After;
-import org.junit.Before;
-import org.noggit.JSONParser;
-import org.noggit.ObjectBuilder;
-import org.slf4j.Logger;
-import org.slf4j.LoggerFactory;
-
 import java.io.File;
-import java.io.StringReader;
 import java.lang.invoke.MethodHandles;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.HashMap;
 import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Set;
 import java.util.function.Consumer;
 
+import org.apache.commons.io.FileUtils;
+import org.apache.lucene.misc.SweetSpotSimilarity;
+import org.apache.lucene.search.similarities.BM25Similarity;
+import org.apache.lucene.search.similarities.DFISimilarity;
+import org.apache.lucene.search.similarities.PerFieldSimilarityWrapper;
+import org.apache.lucene.search.similarities.Similarity;
+import org.apache.solr.client.solrj.SolrQuery;
+import org.apache.solr.client.solrj.request.schema.SchemaRequest;
+import org.apache.solr.common.SolrDocumentList;
+import org.apache.solr.core.CoreContainer;
+import org.apache.solr.core.SolrCore;
+import org.apache.solr.schema.IndexSchema;
+import org.apache.solr.schema.SimilarityFactory;
+import org.apache.solr.search.similarities.SchemaSimilarityFactory;
+import org.apache.solr.util.RESTfulServerProvider;
+import org.apache.solr.util.RestTestBase;
+import org.apache.solr.util.RestTestHarness;
+import org.junit.After;
+import org.junit.Before;
+import org.junit.Test;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
+
+import static org.apache.solr.common.util.Utils.fromJSONString;
+
 
 public class TestBulkSchemaAPI extends RestTestBase {
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
@@ -108,15 +109,20 @@ public class TestBulkSchemaAPI extends RestTestBase {
         "    }";
 
     String response = restTestHarness.post("/schema", json(payload));
-    Map map = (Map) ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    @SuppressWarnings({"rawtypes"})
+    Map map = (Map) fromJSONString(response);
+    @SuppressWarnings({"rawtypes"})
     Map error = (Map)map.get("error");
     assertNotNull("No errors", error);
+    @SuppressWarnings({"rawtypes"})
     List details = (List)error.get("details");
     assertNotNull("No details", details);
     assertEquals("Wrong number of details", 2, details.size());
+    @SuppressWarnings({"rawtypes"})
     List firstErrorList = (List)((Map)details.get(0)).get("errorMessages");
     assertEquals(1, firstErrorList.size());
     assertTrue (((String)firstErrorList.get(0)).contains("Field 'a1': Field type 'string1' not found.\n"));
+    @SuppressWarnings({"rawtypes"})
     List secondErrorList = (List)((Map)details.get(1)).get("errorMessages");
     assertEquals(1, secondErrorList.size());
     assertTrue (((String)secondErrorList.get(0)).contains("is a required field"));
@@ -147,12 +153,16 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     String response = restTestHarness.post("/schema",
         json(addFieldTypeAnalyzerWithClass + ',' + charFilters + tokenizer + filters + suffix));
-    Map map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    @SuppressWarnings({"rawtypes"})
+    Map map = (Map) fromJSONString(response);
+    @SuppressWarnings({"rawtypes"})
     Map error = (Map)map.get("error");
     assertNotNull("No errors", error);
+    @SuppressWarnings({"rawtypes"})
     List details = (List)error.get("details");
     assertNotNull("No details", details);
     assertEquals("Wrong number of details", 1, details.size());
+    @SuppressWarnings({"rawtypes"})
     List errorList = (List)((Map)details.get(0)).get("errorMessages");
     assertEquals(1, errorList.size());
     assertTrue (((String)errorList.get(0)).contains
@@ -160,7 +170,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     response = restTestHarness.post("/schema",
         json(addFieldTypeAnalyzerWithClass + ',' + tokenizer + filters + suffix));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     error = (Map)map.get("error");
     assertNotNull("No errors", error);
     details = (List)error.get("details");
@@ -173,7 +183,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     response = restTestHarness.post("/schema",
         json(addFieldTypeAnalyzerWithClass + ',' + filters + suffix));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     error = (Map)map.get("error");
     assertNotNull("No errors", error);
     details = (List)error.get("details");
@@ -185,14 +195,15 @@ public class TestBulkSchemaAPI extends RestTestBase {
         ("An analyzer with a class property may not define any filters!"));
 
     response = restTestHarness.post("/schema", json(addFieldTypeAnalyzerWithClass + suffix));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(response, map.get("error"));
 
     map = getObj(restTestHarness, "myNewTextFieldWithAnalyzerClass", "fieldTypes");
     assertNotNull(map);
+    @SuppressWarnings({"rawtypes"})
     Map analyzer = (Map)map.get("analyzer");
     assertEquals("org.apache.lucene.analysis.core.WhitespaceAnalyzer", String.valueOf(analyzer.get("class")));
-    assertEquals("5.0.0", String.valueOf(analyzer.get("luceneMatchVersion")));
+    assertEquals("5.0.0", String.valueOf(analyzer.get(IndexSchema.LUCENE_MATCH_VERSION_PARAM)));
   }
 
   public void testAddFieldMatchingExistingDynamicField() throws Exception {
@@ -200,6 +211,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     String newFieldName = "attr_non_dynamic";
 
+    @SuppressWarnings({"rawtypes"})
     Map map = getObj(harness, newFieldName, "fields");
     assertNull("Field '" + newFieldName + "' already exists in the schema", map);
 
@@ -220,7 +232,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     String response = harness.post("/schema", json(payload));
 
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(response, map.get("error"));
 
     map = getObj(harness, newFieldName, "fields");
@@ -242,13 +254,15 @@ public class TestBulkSchemaAPI extends RestTestBase {
         "    }";
 
     String response = harness.post("/schema", json(payload));
-    Map map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    @SuppressWarnings({"rawtypes"})
+    Map map = (Map) fromJSONString(response);
     assertNotNull(response, map.get("error"));
 
     map = getObj(harness, newFieldName, "dynamicFields");
     assertNull(newFieldName + " illegal dynamic field should not have been added to schema", map);
   }
 
+  @SuppressWarnings({"rawtypes"})
   public void testAddIllegalFields() throws Exception {
     RestTestHarness harness = restTestHarness;
 
@@ -265,7 +279,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
         "}";
 
     String response = harness.post("/schema", json(payload));
-    Map map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    Map map = (Map) fromJSONString(response);
     assertNotNull(response, map.get("error"));
 
     map = getObj(harness, newFieldName, "fields");
@@ -287,10 +301,11 @@ public class TestBulkSchemaAPI extends RestTestBase {
         "}";
 
     response = harness.post("/schema", json(payload));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNotNull(response, map.get("error"));
   }
 
+  @SuppressWarnings({"rawtypes"})
   public void testAddFieldWithExistingCatchallDynamicField() throws Exception {
     RestTestHarness harness = restTestHarness;
 
@@ -319,7 +334,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     String response = harness.post("/schema", json(payload));
 
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(response, map.get("error"));
 
     map = getObj(harness, "*", "dynamicFields");
@@ -336,13 +351,14 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     response = harness.post("/schema", json(payload));
 
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(response, map.get("error"));
 
     map = getObj(harness, newFieldName, "fields");
     assertNotNull("Field '" + newFieldName + "' is not in the schema", map);
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   public void testMultipleCommands() throws Exception{
     RestTestHarness harness = restTestHarness;
 
@@ -519,7 +535,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
     
     String response = harness.post("/schema", json(payload));
 
-    Map map = (Map) ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    Map map = (Map) fromJSONString(response);
     assertNull(response, map.get("error"));
 
     m = getObj(harness, "a1", "fields");
@@ -618,7 +634,183 @@ public class TestBulkSchemaAPI extends RestTestBase {
     assertNotNull("'attr_*' dynamic field does not exist in the schema", m);
     assertEquals("string", m.get("type"));
   }
-  
+
+  public void testCopyFieldRules() throws Exception {
+    RestTestHarness harness = restTestHarness;
+
+    @SuppressWarnings({"rawtypes"})
+    Map m = getObj(harness, "name", "fields");
+    assertNotNull("'name' field does not exist in the schema", m);
+
+    m = getObj(harness, "bind", "fields");
+    assertNotNull("'bind' field does not exist in the schema", m);
+
+    @SuppressWarnings({"rawtypes"})
+    List l = getSourceCopyFields(harness, "bleh_s");
+    assertTrue("'bleh_s' copyField rule exists in the schema", l.isEmpty());
+
+    String payload = "{\n" +
+        "          'add-copy-field' : {\n" +
+        "                       'source' :'bleh_s',\n" +
+        "                       'dest':'name'\n" +
+        "                       }\n" +
+        "          }\n";
+    String response = harness.post("/schema", json(payload));
+
+    @SuppressWarnings({"rawtypes"})
+    Map map = (Map) fromJSONString(response);
+    assertNull(response, map.get("error"));
+
+    l = getSourceCopyFields(harness, "bleh_s");
+    assertFalse("'bleh_s' copyField rule doesn't exist", l.isEmpty());
+    assertEquals("bleh_s", ((Map)l.get(0)).get("source"));
+    assertEquals("name", ((Map)l.get(0)).get("dest"));
+
+    // delete copy field rule
+    payload = "{\n" +
+        "          'delete-copy-field' : {\n" +
+        "                       'source' :'bleh_s',\n" +
+        "                       'dest':'name'\n" +
+        "                       }\n" +
+        "          }\n";
+
+    response = harness.post("/schema", json(payload));
+    map = (Map) fromJSONString(response);
+    assertNull(response, map.get("error"));
+    l = getSourceCopyFields(harness, "bleh_s");
+    assertTrue("'bleh_s' copyField rule exists in the schema", l.isEmpty());
+
+    // copy and delete with multiple destination
+    payload = "{\n" +
+        "          'add-copy-field' : {\n" +
+        "                       'source' :'bleh_s',\n" +
+        "                       'dest':['name','bind']\n" +
+        "                       }\n" +
+        "          }\n";
+    response = harness.post("/schema", json(payload));
+    map = (Map) fromJSONString(response);
+    assertNull(response, map.get("error"));
+
+    l = getSourceCopyFields(harness, "bleh_s");
+    assertEquals(2, l.size());
+
+    payload = "{\n" +
+        "          'delete-copy-field' : {\n" +
+        "                       'source' :'bleh_s',\n" +
+        "                       'dest':['name','bind']\n" +
+        "                       }\n" +
+        "          }\n";
+
+    response = harness.post("/schema", json(payload));
+    map = (Map) fromJSONString(response);
+    assertNull(response, map.get("error"));
+    l = getSourceCopyFields(harness, "bleh_s");
+    assertTrue("'bleh_s' copyField rule exists in the schema", l.isEmpty());
+  }
+
+  @SuppressWarnings({"rawtypes"})
+  public void testCopyFieldWithReplace() throws Exception {
+    RestTestHarness harness = restTestHarness;
+    String newFieldName = "test_solr_14950";
+
+    // add-field-type
+    String addFieldTypeAnalyzer = "{\n" +
+        "'add-field-type' : {" +
+        "    'name' : 'myNewTextField',\n" +
+        "    'class':'solr.TextField',\n" +
+        "    'analyzer' : {\n" +
+        "        'charFilters' : [{\n" +
+        "                'class':'solr.PatternReplaceCharFilterFactory',\n" +
+        "                'replacement':'$1$1',\n" +
+        "                'pattern':'([a-zA-Z])\\\\\\\\1+'\n" +
+        "            }],\n" +
+        "        'tokenizer' : { 'class':'solr.WhitespaceTokenizerFactory' },\n" +
+        "        'filters' : [{ 'class':'solr.ASCIIFoldingFilterFactory' }]\n" +
+        "    }\n"+
+        "}}";
+
+    String response = restTestHarness.post("/schema", json(addFieldTypeAnalyzer));
+    Map map = (Map) fromJSONString(response);
+    assertNull(response, map.get("error"));
+    map = getObj(harness, "myNewTextField", "fieldTypes");
+    assertNotNull("'myNewTextField' field type does not exist in the schema", map);
+
+    // add-field
+    String payload = "{\n" +
+        "    'add-field' : {\n" +
+        "                 'name':'" + newFieldName + "',\n" +
+        "                 'type':'myNewTextField',\n" +
+        "                 'stored':true,\n" +
+        "                 'indexed':true\n" +
+        "                 }\n" +
+        "    }";
+
+    response = harness.post("/schema", json(payload));
+
+    map = (Map) fromJSONString(response);
+    assertNull(response, map.get("error"));
+
+    Map m = getObj(harness, newFieldName, "fields");
+    assertNotNull("'"+ newFieldName + "' field does not exist in the schema", m);
+
+    // add copy-field with explicit source and destination
+    List l = getSourceCopyFields(harness, "bleh_s");
+    assertTrue("'bleh_s' copyField rule exists in the schema", l.isEmpty());
+
+    payload = "{\n" +
+        "          'add-copy-field' : {\n" +
+        "                       'source' :'bleh_s',\n" +
+        "                       'dest':'"+ newFieldName + "'\n" +
+        "                       }\n" +
+        "          }\n";
+    response = harness.post("/schema", json(payload));
+
+    map = (Map) fromJSONString(response);
+    assertNull(response, map.get("error"));
+
+    l = getSourceCopyFields(harness, "bleh_s");
+    assertFalse("'bleh_s' copyField rule doesn't exist", l.isEmpty());
+    assertEquals("bleh_s", ((Map)l.get(0)).get("source"));
+    assertEquals(newFieldName, ((Map)l.get(0)).get("dest"));
+
+    // replace-field-type
+    String replaceFieldTypeAnalyzer = "{\n" +
+        "'replace-field-type' : {" +
+        "    'name' : 'myNewTextField',\n" +
+        "    'class':'solr.TextField',\n" +
+        "    'analyzer' : {\n" +
+        "        'tokenizer' : { 'class':'solr.WhitespaceTokenizerFactory' },\n" +
+        "        'filters' : [{ 'class':'solr.ASCIIFoldingFilterFactory' }]\n" +
+        "    }\n"+
+        "}}";
+
+    response = restTestHarness.post("/schema", json(replaceFieldTypeAnalyzer));
+    map = (Map) fromJSONString(response);
+    assertNull(response, map.get("error"));
+
+    map = getObj(restTestHarness, "myNewTextField", "fieldTypes");
+    assertNotNull(map);
+    Map analyzer = (Map)map.get("analyzer");
+    assertNull("'myNewTextField' shouldn't contain charFilters", analyzer.get("charFilters"));
+
+    l = getSourceCopyFields(harness, "bleh_s");
+    assertFalse("'bleh_s' copyField rule doesn't exist", l.isEmpty());
+    assertEquals("bleh_s", ((Map)l.get(0)).get("source"));
+    assertEquals(newFieldName, ((Map)l.get(0)).get("dest"));
+
+    // with replace-field
+    String replaceField = "{'replace-field' : {'name':'" + newFieldName + "', 'type':'string'}}";
+    response = harness.post("/schema", json(replaceField));
+    map = (Map) fromJSONString(response);
+    assertNull(map.get("error"));
+
+    l = getSourceCopyFields(harness, "bleh_s");
+    assertFalse("'bleh_s' copyField rule doesn't exist", l.isEmpty());
+    assertEquals("bleh_s", ((Map)l.get(0)).get("source"));
+    assertEquals(newFieldName, ((Map)l.get(0)).get("dest"));
+  }
+
+  @SuppressWarnings({"unchecked", "rawtypes"})
   public void testDeleteAndReplace() throws Exception {
     RestTestHarness harness = restTestHarness;
 
@@ -660,7 +852,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     String response = harness.post("/schema", json(cmds));
 
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(response, map.get("error"));
 
     map = getObj(harness, "NewFieldType", "fieldTypes");
@@ -714,14 +906,14 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     cmds = "{'delete-field-type' : {'name':'NewFieldType'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     Object errors = map.get("error");
     assertNotNull(errors);
     assertTrue(errors.toString().contains("Can't delete 'NewFieldType' because it's the field type of "));
 
     cmds = "{'delete-field' : {'name':'NewField1'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     errors = map.get("error");
     assertNotNull(errors);
     assertTrue(errors.toString().contains
@@ -729,7 +921,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     cmds = "{'delete-field' : {'name':'NewField2'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     errors = map.get("error");
     assertNotNull(errors);
     assertTrue(errors.toString().contains
@@ -737,7 +929,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     cmds = "{'replace-field' : {'name':'NewField1', 'type':'string'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(map.get("error"));
     // Make sure the copy field directives with source NewField1 are preserved
     list = getSourceCopyFields(harness, "NewField1");
@@ -751,7 +943,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     cmds = "{'delete-dynamic-field' : {'name':'NewDynamicField1*'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     errors = map.get("error");
     assertNotNull(errors);
     assertTrue(errors.toString().contains
@@ -759,7 +951,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     cmds = "{'replace-field' : {'name':'NewField2', 'type':'string'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     errors = map.get("error");
     assertNull(errors);
     // Make sure the copy field directives with destination NewField2 are preserved
@@ -776,7 +968,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     cmds = "{'replace-dynamic-field' : {'name':'NewDynamicField2*', 'type':'string'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     errors = map.get("error");
     assertNull(errors);
     // Make sure the copy field directives with source NewDynamicField2* are preserved
@@ -786,7 +978,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     cmds = "{'replace-dynamic-field' : {'name':'NewDynamicField1*', 'type':'string'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     errors = map.get("error");
     assertNull(errors);
     // Make sure the copy field directives with destinations matching NewDynamicField1* are preserved
@@ -796,7 +988,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     cmds = "{'replace-field-type': {'name':'NewFieldType', 'class':'solr.BinaryField'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(map.get("error"));
     // Make sure the copy field directives with sources and destinations of type NewFieldType are preserved
     list = getDestCopyFields(harness, "NewField3");
@@ -816,7 +1008,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
         "                        {'source':'NewField4',         'dest':['NewField1', 'NewField2', 'NewField3']}]\n" +
         "}\n";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(map.get("error"));
     list = getSourceCopyFields(harness, "NewField1");
     assertEquals(0, list.size());
@@ -831,7 +1023,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
     
     cmds = "{'delete-field': [{'name':'NewField1'},{'name':'NewField2'},{'name':'NewField3'},{'name':'NewField4'}]}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(map.get("error"));
 
     cmds = "{'delete-dynamic-field': [{'name':'NewDynamicField1*'}," +
@@ -839,14 +1031,15 @@ public class TestBulkSchemaAPI extends RestTestBase {
         "                             {'name':'NewDynamicField3*'}]\n" +
         "}\n";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(map.get("error"));
     
     cmds = "{'delete-field-type':{'name':'NewFieldType'}}";
     response = harness.post("/schema", json(cmds));
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(map.get("error"));
   }
+
   public void testSortableTextFieldWithAnalyzer() throws Exception {
     String fieldTypeName = "sort_text_type";
     String fieldName = "sort_text";
@@ -867,9 +1060,11 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     String response = restTestHarness.post("/schema", json(payload));
 
-    Map map = (Map) ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    @SuppressWarnings({"rawtypes"})
+    Map map = (Map) fromJSONString(response);
     assertNull(response, map.get("errors"));
 
+    @SuppressWarnings({"rawtypes"})
     Map fields = getObj(restTestHarness, fieldName, "fields");
     assertNotNull("field " + fieldName + " not created", fields);
 
@@ -901,7 +1096,25 @@ public class TestBulkSchemaAPI extends RestTestBase {
     }
     
   }
-  
+
+  @Test
+  public void testAddNewFieldAndQuery() throws Exception {
+    getSolrClient().add(Arrays.asList(
+        sdoc("id", "1", "term_s", "tux")));
+
+    getSolrClient().commit(true, true);
+    Map<String,Object> attrs = new HashMap<>();
+    attrs.put("name", "newstringtestfield");
+    attrs.put("type", "string");
+
+    new SchemaRequest.AddField(attrs).process(getSolrClient());
+
+    SolrQuery query = new SolrQuery("*:*");
+    query.addFacetField("newstringtestfield");
+    int size = getSolrClient().query(query).getResults().size();
+    assertEquals(1, size);
+  }
+
   public void testSimilarityParser() throws Exception {
     RestTestHarness harness = restTestHarness;
 
@@ -927,9 +1140,11 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     String response = harness.post("/schema", json(payload));
 
-    Map map = (Map) ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    @SuppressWarnings({"rawtypes"})
+    Map map = (Map) fromJSONString(response);
     assertNull(response, map.get("error"));
 
+    @SuppressWarnings({"rawtypes"})
     Map fields = getObj(harness, fieldName, "fields");
     assertNotNull("field " + fieldName + " not created", fields);
     
@@ -954,7 +1169,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
 
     response = harness.post("/schema", json(payload));
 
-    map = (Map)ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    map = (Map) fromJSONString(response);
     assertNull(response, map.get("error"));
     fields = getObj(harness, fieldName, "fields");
     assertNotNull("field " + fieldName + " not created", fields);
@@ -964,26 +1179,30 @@ public class TestBulkSchemaAPI extends RestTestBase {
         sim -> assertEquals("Unexpected discountedOverlaps", discountOverlaps, sim.getDiscountOverlaps()));
   }
 
+  @SuppressWarnings({"rawtypes"})
   public static Map getObj(RestTestHarness restHarness, String fld, String key) throws Exception {
     Map map = getRespMap(restHarness);
     List l = (List) ((Map)map.get("schema")).get(key);
     for (Object o : l) {
-      Map m = (Map) o;
+      @SuppressWarnings({"rawtypes"})Map m = (Map) o;
       if (fld.equals(m.get("name"))) 
         return m;
     }
     return null;
   }
 
+  @SuppressWarnings({"rawtypes"})
   public static Map getRespMap(RestTestHarness restHarness) throws Exception {
     return getAsMap("/schema", restHarness);
   }
 
+  @SuppressWarnings({"rawtypes"})
   public static Map getAsMap(String uri, RestTestHarness restHarness) throws Exception {
     String response = restHarness.query(uri);
-    return (Map) ObjectBuilder.getVal(new JSONParser(new StringReader(response)));
+    return (Map) fromJSONString(response);
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   public static List getSourceCopyFields(RestTestHarness harness, String src) throws Exception {
     Map map = getRespMap(harness);
     List l = (List) ((Map)map.get("schema")).get("copyFields");
@@ -995,6 +1214,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
     return result;
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   public static List getDestCopyFields(RestTestHarness harness, String dest) throws Exception {
     Map map = getRespMap(harness);
     List l = (List) ((Map)map.get("schema")).get("copyFields");
@@ -1012,6 +1232,7 @@ public class TestBulkSchemaAPI extends RestTestBase {
    * Executes each of the specified Similarity-accepting validators.
    */
   @SafeVarargs
+  @SuppressWarnings({"unchecked", "varargs"})
   private static <T extends Similarity> void assertFieldSimilarity(String fieldname, Class<T> expected, Consumer<T>... validators) {
     CoreContainer cc = jetty.getCoreContainer();
     try (SolrCore core = cc.getCore("collection1")) {
