@@ -20,7 +20,6 @@ import java.io.IOException;
 import java.io.PrintWriter;
 import java.io.StringWriter;
 import java.util.ArrayList;
-import java.util.Collection;
 import java.util.Collections;
 import java.util.Comparator;
 import java.util.HashMap;
@@ -32,18 +31,20 @@ import org.apache.lucene.index.IndexReaderContext;
 import org.apache.lucene.index.LeafReaderContext;
 import org.apache.lucene.index.NumericDocValues;
 import org.apache.lucene.index.ReaderUtil;
-import org.apache.lucene.search.DocIdSetIterator;
 import org.apache.lucene.search.FieldComparator;
 import org.apache.lucene.search.IndexSearcher;
 import org.apache.lucene.search.LeafCollector;
 import org.apache.lucene.search.LeafFieldComparator;
 import org.apache.lucene.search.Query;
+import org.apache.lucene.search.QueryVisitor;
+import org.apache.lucene.search.Scorable;
 import org.apache.lucene.search.ScoreDoc;
-import org.apache.lucene.search.Scorer;
+import org.apache.lucene.search.ScoreMode;
 import org.apache.lucene.search.Sort;
 import org.apache.lucene.search.SortField;
 import org.apache.lucene.search.TopDocs;
 import org.apache.lucene.search.TopDocsCollector;
+import org.apache.lucene.search.TotalHits;
 import org.apache.lucene.search.Weight;
 import org.apache.lucene.util.InPlaceMergeSorter;
 import org.apache.lucene.util.PriorityQueue;
@@ -107,8 +108,13 @@ public class RankQueryTestPlugin extends QParserPlugin {
       return false;
     }
 
-    public Weight createWeight(IndexSearcher indexSearcher, boolean needsScores, float boost) throws IOException{
-      return q.createWeight(indexSearcher, needsScores, boost);
+    public Weight createWeight(IndexSearcher indexSearcher, ScoreMode scoreMode, float boost) throws IOException{
+      return q.createWeight(indexSearcher, scoreMode, boost);
+    }
+
+    @Override
+    public void visit(QueryVisitor visitor) {
+
     }
 
     @Override
@@ -126,6 +132,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
       this.mergeStrategy = mergeStrategy;
     }
 
+    @SuppressWarnings({"rawtypes"})
     public TopDocsCollector getTopDocsCollector(int len, QueryCommand cmd, IndexSearcher searcher) {
       if(collector == 0)
         return new TestCollector(null);
@@ -159,6 +166,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
 
     }
 
+    @SuppressWarnings({"unchecked"})
     public void merge(ResponseBuilder rb, ShardRequest sreq) {
 
       // id to shard mapping, to eliminate any accidental dups
@@ -177,7 +185,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
       long numFound = 0;
       Float maxScore=null;
       boolean partialResults = false;
-      List<ShardDoc> shardDocs = new ArrayList();
+      List<ShardDoc> shardDocs = new ArrayList<>();
 
       for (ShardResponse srsp : sreq.responses) {
         SolrDocumentList docs = null;
@@ -419,7 +427,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
             }
 
             doc -= currentLeaf.docBase;  // adjust for what segment this is in
-            leafComparator.setScorer(new FakeScorer(doc, score));
+            leafComparator.setScorer(new ScoreAndDoc(doc, score));
             leafComparator.copy(0, doc);
             Object val = comparator.value(0);
             if (null != ft) val = ft.marshalSortValue(val);
@@ -433,13 +441,12 @@ public class RankQueryTestPlugin extends QParserPlugin {
       }
     }
 
-    private static class FakeScorer extends Scorer {
+    private static class ScoreAndDoc extends Scorable {
 
       final int docid;
       final float score;
 
-      FakeScorer(int docid, float score) {
-        super(null);
+      ScoreAndDoc(int docid, float score) {
         this.docid = docid;
         this.score = score;
       }
@@ -450,26 +457,12 @@ public class RankQueryTestPlugin extends QParserPlugin {
       }
 
       @Override
-      public float score() throws IOException {
+      public float score() {
         return score;
       }
-
-      @Override
-      public DocIdSetIterator iterator() {
-        throw new UnsupportedOperationException();
-      }
-
-      @Override
-      public Weight getWeight() {
-        throw new UnsupportedOperationException();
-      }
-
-      @Override
-      public Collection<ChildScorer> getChildren() {
-        throw new UnsupportedOperationException();
-      }
     }
 
+    @SuppressWarnings({"unchecked"})
     public void merge(ResponseBuilder rb, ShardRequest sreq) {
 
       // id to shard mapping, to eliminate any accidental dups
@@ -488,7 +481,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
       long numFound = 0;
       Float maxScore=null;
       boolean partialResults = false;
-      List<ShardDoc> shardDocs = new ArrayList();
+      List<ShardDoc> shardDocs = new ArrayList<>();
 
       for (ShardResponse srsp : sreq.responses) {
         SolrDocumentList docs = null;
@@ -545,8 +538,11 @@ public class RankQueryTestPlugin extends QParserPlugin {
         SortSpec ss = rb.getSortSpec();
         Sort sort = ss.getSort();
 
+        @SuppressWarnings({"rawtypes"})
         NamedList sortFieldValues = (NamedList)(srsp.getSolrResponse().getResponse().get("merge_values"));
+        @SuppressWarnings({"rawtypes"})
         NamedList unmarshalledSortFieldValues = unmarshalSortValues(ss, sortFieldValues, schema);
+        @SuppressWarnings({"rawtypes"})
         List lst = (List)unmarshalledSortFieldValues.getVal(0);
 
         for (int i=0; i<docs.size(); i++) {
@@ -624,6 +620,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
       }
     }
 
+    @SuppressWarnings({"unchecked", "rawtypes"})
     private NamedList unmarshalSortValues(SortSpec sortSpec,
                                           NamedList sortFieldValues,
                                           IndexSchema schema) {
@@ -667,11 +664,13 @@ public class RankQueryTestPlugin extends QParserPlugin {
   }
 
 
+  @SuppressWarnings({"rawtypes"})
   static class TestCollector extends TopDocsCollector {
 
-    private List<ScoreDoc> list = new ArrayList();
+    private List<ScoreDoc> list = new ArrayList<>();
 
-    public TestCollector(PriorityQueue pq) {
+    @SuppressWarnings({"unchecked"})
+    public TestCollector(@SuppressWarnings({"rawtypes"})PriorityQueue pq) {
       super(pq);
     }
 
@@ -682,7 +681,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
       return new LeafCollector() {
         
         @Override
-        public void setScorer(Scorer scorer) throws IOException {}
+        public void setScorer(Scorable scorer) throws IOException {}
         
         public void collect(int doc) throws IOException {
           long value;
@@ -700,6 +699,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
       return list.size();
     }
 
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public TopDocs topDocs() {
       Collections.sort(list, new Comparator() {
         public int compare(Object o1, Object o2) {
@@ -715,7 +715,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
         }
       });
       ScoreDoc[] scoreDocs = list.toArray(new ScoreDoc[list.size()]);
-      return new TopDocs(list.size(), scoreDocs, 0.0f);
+      return new TopDocs(new TotalHits(list.size(), TotalHits.Relation.EQUAL_TO), scoreDocs);
     }
 
     public TopDocs topDocs(int start, int len) {
@@ -727,16 +727,18 @@ public class RankQueryTestPlugin extends QParserPlugin {
     }
     
     @Override
-    public boolean needsScores() {
-      return true;
+    public ScoreMode scoreMode() {
+      return ScoreMode.COMPLETE;
     }
   }
 
+  @SuppressWarnings({"rawtypes"})
   static class TestCollector1 extends TopDocsCollector {
 
-    private List<ScoreDoc> list = new ArrayList();
+    private List<ScoreDoc> list = new ArrayList<>();
 
-    public TestCollector1(PriorityQueue pq) {
+    @SuppressWarnings({"unchecked"})
+    public TestCollector1(@SuppressWarnings({"rawtypes"})PriorityQueue pq) {
       super(pq);
     }
 
@@ -745,10 +747,10 @@ public class RankQueryTestPlugin extends QParserPlugin {
       final int base = context.docBase;
       return new LeafCollector() {
         
-        Scorer scorer;
+        Scorable scorer;
         
         @Override
-        public void setScorer(Scorer scorer) throws IOException {
+        public void setScorer(Scorable scorer) throws IOException {
           this.scorer = scorer;
         }
         
@@ -762,6 +764,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
       return list.size();
     }
 
+    @SuppressWarnings({"unchecked", "rawtypes"})
     public TopDocs topDocs() {
       Collections.sort(list, new Comparator() {
         public int compare(Object o1, Object o2) {
@@ -777,7 +780,7 @@ public class RankQueryTestPlugin extends QParserPlugin {
         }
       });
       ScoreDoc[] scoreDocs = list.toArray(new ScoreDoc[list.size()]);
-      return new TopDocs(list.size(), scoreDocs, 0.0f);
+      return new TopDocs(new TotalHits(list.size(), TotalHits.Relation.EQUAL_TO), scoreDocs);
     }
 
     public TopDocs topDocs(int start, int len) {
@@ -789,8 +792,8 @@ public class RankQueryTestPlugin extends QParserPlugin {
     }
     
     @Override
-    public boolean needsScores() {
-      return true;
+    public ScoreMode scoreMode() {
+      return ScoreMode.COMPLETE;
     }
   }
 
