@@ -28,6 +28,7 @@ import org.apache.solr.SolrTestCaseHS;
 import org.apache.solr.client.solrj.SolrClient;
 import org.apache.solr.common.params.ModifiableSolrParams;
 import org.apache.solr.common.util.SimpleOrderedMap;
+import org.apache.solr.common.util.Utils;
 import org.apache.solr.request.SolrQueryRequest;
 import org.junit.AfterClass;
 import org.junit.BeforeClass;
@@ -117,11 +118,13 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
     JSONParser parser = new JSONParser(json);
     ObjectBuilder ob = new ObjectBuilder(parser) {
       @Override
+      @SuppressWarnings({"rawtypes"})
       public Object newObject() throws IOException {
         return new SimpleOrderedMap();
       }
 
       @Override
+      @SuppressWarnings({"unchecked", "rawtypes"})
       public void addKeyVal(Object map, Object key, Object val) throws IOException {
         ((SimpleOrderedMap) map).add(key.toString(), val);
       }
@@ -134,8 +137,9 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
     SolrQueryRequest req = req();
     try {
       int nShards = responsesAndTests.length / 2;
-      Object jsonFacet = ObjectBuilder.fromJSON(facet);
-      FacetParser parser = new FacetTopParser(req);
+      Object jsonFacet = Utils.fromJSONString(facet);
+      @SuppressWarnings({"rawtypes"})
+      FacetParser parser = new FacetParser.FacetTopParser(req);
       FacetRequest facetRequest = parser.parse(jsonFacet);
 
       FacetMerger merger = null;
@@ -210,7 +214,7 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
         null,
         null
     );
-    
+
     // same test, but nested in a terms facet
     doTestRefine("{top:{type:terms, field:Afield, facet:{x : {type:terms, field:X, limit:2, refine:true} } } }",
         "{top: {buckets:[{val:'A', count:2, x:{buckets:[{val:x1, count:5},{val:x2, count:3}], more:true} } ] } }",
@@ -276,7 +280,7 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
                  "    between:{ x:{_l : [x1]} }" + 
                  "} } ");
     // a range face w/o any sub facets shouldn't require any refinement
-    doTestRefine("{top:{type:range, other:all, field:R, start:0, end:3, gap:2 } }" +
+    doTestRefine("{top:{type:range, other:all, field:R, start:0, end:3, gap:2 } }" ,
                  // phase #1
                  "{top: {buckets:[{val:0, count:2}, {val:2, count:2}]," +
                  "       before:{count:3},after:{count:47}," +
@@ -289,7 +293,39 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
                  // refinement...
                  null,
                  null);
-    
+
+    // same test, but nested in range facet with ranges
+    doTestRefine("{top:{type:range, field:R, ranges:[{from:0, to:1}], facet:{x : {type:terms, field:X, limit:2, refine:true} } } }",
+        "{top: {buckets:[{val:\"[0,1)\", count:2, x:{buckets:[{val:x1, count:5},{val:x2, count:3}],more:true} } ] } }",
+        "{top: {buckets:[{val:\"[0,1)\", count:1, x:{buckets:[{val:x2, count:4},{val:x3, count:2}],more:true} } ] } }",
+        null,
+        "=={top: {" +
+            "_s:[  [\"[0,1)\" , {x:{_l:[x1]}} ]  ]" +
+            "    }  " +
+            "}"
+    );
+
+    doTestRefine("{top:{type:range, field:R, ranges:[{from:\"*\", to:1}], facet:{x : {type:terms, field:X, limit:2, refine:true} } } }",
+        "{top: {buckets:[{val:\"[*,1)\", count:2, x:{buckets:[{val:x1, count:5},{val:x2, count:3}],more:true} } ] } }",
+        "{top: {buckets:[{val:\"[*,1)\", count:1, x:{buckets:[{val:x2, count:4},{val:x3, count:2}],more:true} } ] } }",
+        null,
+        "=={top: {" +
+            "_s:[  [\"[*,1)\" , {x:{_l:[x1]}} ]  ]" +
+            "    }  " +
+            "}"
+    );
+
+    // a range facet w/o any sub facets shouldn't require any refinement
+    // other and include ignored for ranges
+    doTestRefine("{top:{type:range, other:all, field:R, ranges:[{from:0, to:2},{from:2, to:3}] } }",
+            // phase #1
+            "{top: {buckets:[{val:\"[0,2)\", count:2}, {val:\"[2,3)\", count:2}]," +
+            "       } }",
+        "{top: {buckets:[{val:\"[0,2)\", count:2}, {val:\"[2,3)\", count:19}]," +
+            "       } }",
+        // refinement...
+        null,
+        null);
 
     // for testing partial _p, we need a partial facet within a partial facet
     doTestRefine("{top:{type:terms, field:Afield, refine:true, limit:1, facet:{x : {type:terms, field:X, limit:1, refine:true} } } }",
@@ -470,7 +506,7 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
   
     initServers();
     final Client client = servers.getClient(random().nextInt());
-    client.queryDefaults().set("shards", servers.getShards(), "debugQuery", Boolean.toString(random().nextBoolean()));
+    client.queryDefaults().set("shards", servers.getShards()).set("debugQuery", Boolean.toString(random().nextBoolean()));
 
     List<SolrClient> clients = client.getClientProvider().all();
     assertTrue(clients.size() >= 3); // we only use 2, but assert 3 to also test empty shard
@@ -560,7 +596,7 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
   public void testSortedFacetRefinementPushingNonRefinedBucketBackIntoTopN() throws Exception {
     initServers();
     final Client client = servers.getClient(random().nextInt());
-    client.queryDefaults().set("shards", servers.getShards(), "debugQuery", Boolean.toString(random().nextBoolean()));
+    client.queryDefaults().set("shards", servers.getShards()).set("debugQuery", Boolean.toString(random().nextBoolean()));
 
     List<SolrClient> clients = client.getClientProvider().all();
     assertTrue(clients.size() >= 3); // we only use 2, but assert 3 to also test empty shard
@@ -792,7 +828,7 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
   public void testProcessEmptyRefinement() throws Exception {
     initServers();
     final Client client = servers.getClient(random().nextInt());
-    client.queryDefaults().set("shards", servers.getShards(), "debugQuery", Boolean.toString(random().nextBoolean()));
+    client.queryDefaults().set("shards", servers.getShards()).set("debugQuery", Boolean.toString(random().nextBoolean()));
 
     List<SolrClient> clients = client.getClientProvider().all();
     assertTrue(clients.size() >= 3); // we only use 2, but assert at least 3 to also test empty shard
@@ -848,7 +884,7 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
   private int initSomeDocsWhere1ShardHasOnlyParentFacetField() throws Exception {
     initServers();
     final Client client = servers.getClient(random().nextInt());
-    client.queryDefaults().set("shards", servers.getShards(), "debugQuery", Boolean.toString(random().nextBoolean()));
+    client.queryDefaults().set("shards", servers.getShards()).set("debugQuery", Boolean.toString(random().nextBoolean()));
 
     final List<SolrClient> clients = client.getClientProvider().all();
     assertTrue(clients.size() >= 2);
@@ -1007,7 +1043,7 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
   public void doBasicRefinement(ModifiableSolrParams p) throws Exception {
     initServers();
     Client client = servers.getClient(random().nextInt());
-    client.queryDefaults().set("shards", servers.getShards(), "debugQuery", Boolean.toString(random().nextBoolean()));
+    client.queryDefaults().set("shards", servers.getShards()).set("debugQuery", Boolean.toString(random().nextBoolean()));
     
     List<SolrClient> clients = client.getClientProvider().all();
     assertTrue(clients.size() >= 3);
@@ -1049,7 +1085,7 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
      , "facets=={foo:555}"
      );
      ****/
-    for (String method : new String[]{"","dvhash","stream","uif","enum","stream","smart"}) {
+    for (String method : new String[]{"","dv", "dvhash","stream","uif","enum","stream","smart"}) {
       if (method.equals("")) {
         p.remove("terms");
       } else {
@@ -1179,10 +1215,13 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
                                     "{ type:func, func:'relatedness($fore,$back)', min_popularity:0.2 }")) {
         client.testJQ(params(p, "rows", "0", "q", "*:*", "fore", "${xy_s}:X", "back", "${num_d}:[0 TO 100]",
                              "json.facet", "{"
-                             + "   cat0:{ ${terms} type:terms, field: ${cat_s}, "
+                             + "   cat0:{ ${terms} type:terms, field: ${cat_s}, allBuckets:true, "
                              + "          sort:'count desc', limit:1, overrequest:0, refine:true, "
                              + "          facet:{ s:"+s+"} } }")
-                      , "facets=={ count:8, cat0:{ buckets:[ "
+                      , "facets=={ count:8, cat0:{ "
+                      // 's' key must not exist in the allBuckets bucket
+                      + "   allBuckets: { count:8 }"
+                      + "   buckets:[ "
                       + "   { val:A, count:4, "
                       + "     s : { relatedness: 0.00496, "
                       //+ "           foreground_count: 3, "
@@ -1198,11 +1237,14 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
       // same query with a high min_pop should result in a -Infinity relatedness score
       client.testJQ(params(p, "rows", "0", "q", "*:*", "fore", "${xy_s}:X", "back", "${num_d}:[0 TO 100]",
                            "json.facet", "{"
-                           + "   cat0:{ ${terms} type:terms, field: ${cat_s}, "
+                           + "   cat0:{ ${terms} type:terms, field: ${cat_s},  allBuckets:true,"
                            + "          sort:'count desc', limit:1, overrequest:0, refine:true, "
                            + "          facet:{ s:{ type:func, func:'relatedness($fore,$back)', "
                            + "                      min_popularity:0.6 } } } }")
-                    , "facets=={ count:8, cat0:{ buckets:[ "
+                    , "facets=={ count:8, cat0:{ "
+                    // 's' key must not exist in the allBuckets bucket
+                    + "   allBuckets: { count:8 }"
+                    + "   buckets:[ "
                     + "   { val:A, count:4, "
                     + "     s : { relatedness: '-Infinity', "
                     //+ "           foreground_count: 3, "
@@ -1215,6 +1257,22 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
                     "}"
                     );
 
+      // really special case: allBuckets when there are no regular buckets...
+      for (String refine : Arrays.asList("", "refine: true,", "refine:false,")) {
+        client.testJQ(params(p, "rows", "0", "q", "*:*", "fore", "${xy_s}:X", "back", "${num_d}:[0 TO 100]",
+                             "json.facet", "{"
+                             + "   cat0:{ ${terms} type:terms, field: bogus_field_s, allBuckets:true, "
+                             + refine
+                             + "          facet:{ s:{ type:func, func:'relatedness($fore,$back)' } } } }")
+                      , "facets=={ count:8, cat0:{ "
+                      // 's' key must not exist in the allBuckets bucket
+                      + "    allBuckets: { count:0 }"
+                      + "    buckets:[ ]"
+                      + "} }"
+                      );
+      }
+
+
       // SKG under nested facet where some terms only exist on one shard
       { 
         // sub-bucket order should change as sort direction changes
@@ -1329,17 +1387,24 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
       );
 
       // test filling in missing "allBuckets"
-      client.testJQ(params(p, "q", "*:*",
+      client.testJQ(params(p, "q", "*:*", 
           "json.facet", "{" +
-              "  cat :{${terms} type:terms, field:${cat_s}, limit:1, overrequest:0, refine:false, allBuckets:true, facet:{  xy:{${terms} type:terms, field:${xy_s}, limit:1, overrequest:0, allBuckets:true, refine:false}  }  }" +
+              "  cat0:{${terms} type:terms, field:${cat_s}, limit:1, overrequest:0, refine:false, allBuckets:true, facet:{  xy:{${terms} type:terms, field:${xy_s}, limit:1, overrequest:0, allBuckets:true, refine:false}  }  }" +
+              ", cat1:{${terms} type:terms, field:${cat_s}, limit:1, overrequest:0, refine:true, allBuckets:true, sort:'min asc', facet:{  min:'min(${num_d})' }  }" +
               ", cat2:{${terms} type:terms, field:${cat_s}, limit:1, overrequest:0, refine:true , allBuckets:true, facet:{  xy:{${terms} type:terms, field:${xy_s}, limit:1, overrequest:0, allBuckets:true, refine:true }  }  }" +
-              ", cat3:{${terms} type:terms, field:${cat_s}, limit:1, overrequest:0, refine:true , allBuckets:true, facet:{  xy:{${terms} type:terms, field:${xy_s}, limit:1, overrequest:0, allBuckets:true, refine:true , facet:{f:'sum(${num_d})'}   }  }  }" +
+              ", cat3:{${terms} type:terms, field:${cat_s}, limit:1, overrequest:0, refine:true , allBuckets:true, facet:{  xy:{${terms} type:terms, field:${xy_s}, limit:1, overrequest:0, allBuckets:true, refine:true , facet:{sum:'sum(${num_d})'}   }  }  }" +
+              ", cat4:{${terms} type:terms, field:${cat_s}, limit:1, overrequest:0, refine:true , allBuckets:true, facet:{  xy:{${terms} type:terms, field:${xy_s}, limit:1, overrequest:0, allBuckets:true, refine:true , sort:'sum asc', facet:{sum:'sum(${num_d})'}   }  }  }" +
+              // using overrefine only so we aren't fooled by 'local maximum' and ask all shards for 'B'
+              ", cat5:{${terms} type:terms, field:${cat_s}, limit:1, overrequest:0, refine:true, overrefine:2, allBuckets:true,  sort:'min desc' facet:{  min:'min(${num_d})', xy:{${terms} type:terms, field:${xy_s}, limit:1, overrequest:0, allBuckets:true, refine:true, facet:{sum:'sum(${num_d})'}   }  }  }" +
               "}"
           )
           , "facets=={ count:8" +
-              ", cat:{ allBuckets:{count:8}, buckets:[  {val:A, count:3, xy:{buckets:[{count:2, val:X}], allBuckets:{count:3}}}]  }" +
+              ",cat0:{ allBuckets:{count:8}, buckets:[  {val:A, count:3, xy:{buckets:[{count:2, val:X}], allBuckets:{count:3}}}]  }" +
+              ",cat1:{ allBuckets:{count:8, min:-19.0 }, buckets:[  {val:A, count:4, min:-19.0 }]  }" +
               ",cat2:{ allBuckets:{count:8}, buckets:[  {val:A, count:4, xy:{buckets:[{count:3, val:X}], allBuckets:{count:4}}}]  }" +
-              ",cat3:{ allBuckets:{count:8}, buckets:[  {val:A, count:4, xy:{buckets:[{count:3, val:X, f:23.0}], allBuckets:{count:4, f:4.0}}}]  }" +
+              ",cat3:{ allBuckets:{count:8}, buckets:[  {val:A, count:4, xy:{buckets:[{count:3, val:X, sum:23.0}], allBuckets:{count:4, sum:4.0}}}]  }" +
+              ",cat4:{ allBuckets:{count:8}, buckets:[  {val:A, count:4, xy:{buckets:[{count:1, val:Y, sum:-19.0}], allBuckets:{count:4, sum:4.0}}}]  }" +
+              ",cat5:{ allBuckets:{count:8, min:-19.0 }, buckets:[  {val:B, count:4, min:-11.0, xy:{buckets:[{count:2, val:X, sum:6.0}], allBuckets:{count:4, sum:-2.0}}}]  }" +
               "}"
       );
 
@@ -1407,22 +1472,47 @@ public class TestJsonFacetRefinement extends SolrTestCaseHS {
     } // end method loop
   }
 
-  // Unlike solrconfig.xml this test using solrconfig-tlog.xml should not fail with too-many-exceptions (see TestSolrQueryParser.testManyClauses)
-  @Test
-  public void testManyClauses() throws Exception {
-    String a = "1 a 2 b 3 c 10 d 11 12 "; // 10 terms
-    StringBuilder sb = new StringBuilder("id:(");
-    for (int i = 0; i < 1024; i++) { // historically, the max number of boolean clauses defaulted to 1024
-      sb.append('z').append(i).append(' ');
+  @AwaitsFix(bugUrl="https://issues.apache.org/jira/browse/SOLR-14595")
+  public void testIndexAscRefineConsistency() throws Exception {
+    initServers();
+    final Client client = servers.getClient(random().nextInt());
+    client.queryDefaults().set("shards", servers.getShards()).set("debugQuery", Boolean.toString(random().nextBoolean()));
+
+    List<SolrClient> clients = client.getClientProvider().all();
+    assertTrue(clients.size() >= 3);
+    final SolrClient c0 = clients.get(0);
+    final SolrClient c1 = clients.get(1);
+    final SolrClient c2 = clients.get(2);
+
+    client.deleteByQuery("*:*", null);
+    int id = 0;
+    
+    c0.add(sdoc("id", id++, "cat_s", "Z", "price_i", 10));
+    
+    c1.add(sdoc("id", id++, "cat_s", "Z", "price_i", -5000));
+    c1.add(sdoc("id", id++, "cat_s", "X", "price_i", 2,       "child_s", "A" ));
+    
+    c2.add(sdoc("id", id++, "cat_s", "X", "price_i", 2,       "child_s", "B" ));
+    c2.add(sdoc("id", id++, "cat_s", "X", "price_i", 2,       "child_s", "C" ));
+    
+    client.commit();
+
+    // TODO once SOLR-14595 is fixed, modify test to check full EnumSet, not just these two...
+    for (String m : Arrays.asList("smart", "enum")) {
+      client.testJQ(params("q", "*:*", "rows", "0", "json.facet", "{"
+                           + " cat : { type:terms, field:cat_s, limit:1, refine:true,"
+                           + "         overrequest:0, " // to trigger parent refinement given small data set
+                           + "         sort:'sum desc', "
+                           + "         facet: { sum : 'sum(price_i)', "
+                           + "                  child_"+m+" : { "
+                           + "                     type:terms, field:child_s, limit:1, refine:true,"
+                           + "                     sort:'index asc', method:" + m + " } "
+                           + "       }} }"
+                           )
+                    , "facets=={ count:5"
+                    + ", cat:{buckets:[ { val:X, count:3, sum:6.0, "
+                    + "                   child_"+m+":{buckets:[{val:A, count:1}]}}]}}"
+                    );
     }
-    sb.append(a);
-    sb.append(")");
-
-    String q = sb.toString();
-
-    ignoreException("Too many clauses");
-    assertJQ(req("q", q)
-        , "/response/numFound==");
   }
-
 }
