@@ -35,6 +35,7 @@ import org.apache.lucene.index.IndexReader;
 import org.apache.lucene.index.IndexWriter;
 import org.apache.lucene.index.IndexWriterConfig;
 import org.apache.lucene.index.MultiReader;
+import org.apache.lucene.index.NoMergePolicy;
 import org.apache.lucene.index.RandomIndexWriter;
 import org.apache.lucene.index.Term;
 import org.apache.solr.legacy.LegacyDoubleField;
@@ -50,15 +51,16 @@ import org.apache.lucene.search.Sort;
 import org.apache.lucene.search.SortField;
 import org.apache.lucene.search.TermQuery;
 import org.apache.lucene.search.TopDocs;
+import org.apache.lucene.search.similarities.BM25Similarity;
 import org.apache.lucene.store.Directory;
 import org.apache.solr.uninverting.UninvertingReader.Type;
-import org.apache.lucene.util.LuceneTestCase;
+import org.apache.solr.SolrTestCase;
 import org.apache.lucene.util.TestUtil;
 
 /*
  * Tests sorting (but with fieldcache instead of docvalues)
  */
-public class TestFieldCacheSort extends LuceneTestCase {
+public class TestFieldCacheSort extends SolrTestCase {
 
   public void testString() throws IOException {
     testString(SortField.Type.STRING);
@@ -87,7 +89,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", sortType));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(2, td.totalHits);
+    assertEquals(2, td.totalHits.value);
     // 'bar' comes before 'foo'
     assertEquals("bar", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("foo", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -126,7 +128,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", sortType));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null comes first
     assertNull(searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("bar", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -163,7 +165,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", sortType, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(2, td.totalHits);
+    assertEquals(2, td.totalHits.value);
     // 'foo' comes after 'bar' in reverse order
     assertEquals("foo", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("bar", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -203,7 +205,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sf);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null comes first
     assertNull(searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("bar", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -244,7 +246,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sf);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     assertEquals("foo", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("bar", searcher.doc(td.scoreDocs[1].doc).get("value"));
     // null comes last
@@ -286,7 +288,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sf);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     assertEquals("bar", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("foo", searcher.doc(td.scoreDocs[1].doc).get("value"));
     // null comes last
@@ -328,7 +330,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sf);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null comes first
     assertNull(searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("foo", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -355,7 +357,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(SortField.FIELD_DOC);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(2, td.totalHits);
+    assertEquals(2, td.totalHits.value);
     // docid 0, then docid 1
     assertEquals(0, td.scoreDocs[0].doc);
     assertEquals(1, td.scoreDocs[1].doc);
@@ -381,7 +383,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField(null, SortField.Type.DOC, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(2, td.totalHits);
+    assertEquals(2, td.totalHits.value);
     // docid 1, then docid 0
     assertEquals(1, td.scoreDocs[0].doc);
     assertEquals(0, td.scoreDocs[1].doc);
@@ -407,11 +409,11 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort();
 
     TopDocs actual = searcher.search(new TermQuery(new Term("value", "foo")), 10, sort);
-    assertEquals(2, actual.totalHits);
+    assertEquals(2, actual.totalHits.value);
 
     TopDocs expected = searcher.search(new TermQuery(new Term("value", "foo")), 10);
     // the two topdocs should be the same
-    assertEquals(expected.totalHits, actual.totalHits);
+    assertEquals(expected.totalHits.value, actual.totalHits.value);
     for (int i = 0; i < actual.scoreDocs.length; i++) {
       assertEquals(actual.scoreDocs[i].doc, expected.scoreDocs[i].doc);
     }
@@ -434,14 +436,16 @@ public class TestFieldCacheSort extends LuceneTestCase {
     writer.close();
     
     IndexSearcher searcher = newSearcher(ir);
+    // this test expects the freq to make doc 1 scores greater than doc 0
+    searcher.setSimilarity(new BM25Similarity());
     Sort sort = new Sort(new SortField(null, SortField.Type.SCORE, true));
 
     TopDocs actual = searcher.search(new TermQuery(new Term("value", "foo")), 10, sort);
-    assertEquals(2, actual.totalHits);
+    assertEquals(2, actual.totalHits.value);
 
     TopDocs expected = searcher.search(new TermQuery(new Term("value", "foo")), 10);
     // the two topdocs should be the reverse of each other
-    assertEquals(expected.totalHits, actual.totalHits);
+    assertEquals(expected.totalHits.value, actual.totalHits.value);
     assertEquals(actual.scoreDocs[0].doc, expected.scoreDocs[1].doc);
     assertEquals(actual.scoreDocs[1].doc, expected.scoreDocs[0].doc);
     TestUtil.checkReader(ir);
@@ -473,7 +477,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.INT));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // numeric order
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -505,7 +509,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.INT));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as a 0
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertNull(searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -539,7 +543,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sortField);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as a Integer.MAX_VALUE
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -573,7 +577,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.INT, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // reverse numeric order
     assertEquals("300000", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -604,7 +608,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.INT));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // numeric order
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -634,7 +638,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.INT));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as a 0
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertNull(searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -666,7 +670,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sortField);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as a Integer.MAX_VALUE
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -697,7 +701,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.INT, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // reverse numeric order
     assertEquals("300000", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -731,7 +735,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.LONG));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // numeric order
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -763,7 +767,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.LONG));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as 0
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertNull(searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -797,7 +801,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sortField);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as Long.MAX_VALUE
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -831,7 +835,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.LONG, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // reverse numeric order
     assertEquals("3000000000", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -862,7 +866,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.LONG));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // numeric order
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -892,7 +896,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.LONG));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as 0
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertNull(searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -924,7 +928,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sortField);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as Long.MAX_VALUE
     assertEquals("-1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -955,7 +959,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.LONG, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // reverse numeric order
     assertEquals("3000000000", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -989,7 +993,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.FLOAT));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // numeric order
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1021,7 +1025,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.FLOAT));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as 0
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertNull(searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1055,7 +1059,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sortField);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as Float.MAX_VALUE
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1089,7 +1093,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.FLOAT, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // reverse numeric order
     assertEquals("30.1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1120,7 +1124,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.FLOAT));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // numeric order
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1150,7 +1154,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.FLOAT));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as 0
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertNull(searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1182,7 +1186,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sortField);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // null is treated as Float.MAX_VALUE
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1213,7 +1217,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.FLOAT, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(3, td.totalHits);
+    assertEquals(3, td.totalHits.value);
     // reverse numeric order
     assertEquals("30.1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1251,7 +1255,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.DOUBLE));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(4, td.totalHits);
+    assertEquals(4, td.totalHits.value);
     // numeric order
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2333333333332", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1283,7 +1287,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.DOUBLE));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(2, td.totalHits);
+    assertEquals(2, td.totalHits.value);
     // numeric order
     double v0 = searcher.doc(td.scoreDocs[0].doc).getField("value").numericValue().doubleValue();
     double v1 = searcher.doc(td.scoreDocs[1].doc).getField("value").numericValue().doubleValue();
@@ -1323,7 +1327,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.DOUBLE));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(4, td.totalHits);
+    assertEquals(4, td.totalHits.value);
     // null treated as a 0
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertNull(searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1362,7 +1366,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sortField);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(4, td.totalHits);
+    assertEquals(4, td.totalHits.value);
     // null treated as Double.MAX_VALUE
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2333333333332", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1401,7 +1405,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.DOUBLE, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(4, td.totalHits);
+    assertEquals(4, td.totalHits.value);
     // numeric order
     assertEquals("30.1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2333333333333", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1436,7 +1440,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.DOUBLE));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(4, td.totalHits);
+    assertEquals(4, td.totalHits.value);
     // numeric order
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2333333333332", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1466,7 +1470,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.DOUBLE));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(2, td.totalHits);
+    assertEquals(2, td.totalHits.value);
     // numeric order
     double v0 = searcher.doc(td.scoreDocs[0].doc).getField("value").numericValue().doubleValue();
     double v1 = searcher.doc(td.scoreDocs[1].doc).getField("value").numericValue().doubleValue();
@@ -1503,7 +1507,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.DOUBLE));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(4, td.totalHits);
+    assertEquals(4, td.totalHits.value);
     // null treated as a 0
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertNull(searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1539,7 +1543,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(sortField);
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(4, td.totalHits);
+    assertEquals(4, td.totalHits.value);
     // null treated as Double.MAX_VALUE
     assertEquals("-1.3", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2333333333332", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1574,7 +1578,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.DOUBLE, true));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(4, td.totalHits);
+    assertEquals(4, td.totalHits.value);
     // numeric order
     assertEquals("30.1", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("4.2333333333333", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1587,7 +1591,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
   
   public void testEmptyStringVsNullStringSort() throws Exception {
     Directory dir = newDirectory();
-    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig(new MockAnalyzer(random())));
+    IndexWriter w = new IndexWriter(dir, newIndexWriterConfig(new MockAnalyzer(random())).setMergePolicy(NoMergePolicy.INSTANCE));
     Document doc = new Document();
     doc.add(newStringField("f", "", Field.Store.NO));
     doc.add(newStringField("t", "1", Field.Store.NO));
@@ -1602,7 +1606,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     w.close();
     IndexSearcher s = newSearcher(r);
     TopDocs hits = s.search(new TermQuery(new Term("t", "1")), 10, new Sort(new SortField("f", SortField.Type.STRING)));
-    assertEquals(2, hits.totalHits);
+    assertEquals(2, hits.totalHits.value);
     // null sorts first
     assertEquals(1, hits.scoreDocs[0].doc);
     assertEquals(0, hits.scoreDocs[1].doc);
@@ -1661,12 +1665,6 @@ public class TestFieldCacheSort extends LuceneTestCase {
                     Collections.singletonMap("id", Type.LEGACY_INTEGER));
     w.close();
     Query q = new TermQuery(new Term("body", "text"));
-    IndexSearcher s = newSearcher(r);
-    float maxScore = s.search(q , 10).getMaxScore();
-    assertEquals(maxScore, s.search(q, 3, Sort.INDEXORDER, random().nextBoolean(), true).getMaxScore(), 0.0);
-    assertEquals(maxScore, s.search(q, 3, Sort.RELEVANCE, random().nextBoolean(), true).getMaxScore(), 0.0);
-    assertEquals(maxScore, s.search(q, 3, new Sort(new SortField[] {new SortField("id", SortField.Type.INT, false)}), random().nextBoolean(), true).getMaxScore(), 0.0);
-    assertEquals(maxScore, s.search(q, 3, new Sort(new SortField[] {new SortField("id", SortField.Type.INT, true)}), random().nextBoolean(), true).getMaxScore(), 0.0);
     TestUtil.checkReader(r);
     r.close();
     d.close();
@@ -1678,28 +1676,28 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Query query = new TermQuery(new Term("contents", "foo"));
   
     Sort sort = new Sort();
-    TopDocs td = empty.search(query, 10, sort, true, true);
-    assertEquals(0, td.totalHits);
+    TopDocs td = empty.search(query, 10, sort, true);
+    assertEquals(0, td.totalHits.value);
 
     sort.setSort(SortField.FIELD_DOC);
-    td = empty.search(query, 10, sort, true, true);
-    assertEquals(0, td.totalHits);
+    td = empty.search(query, 10, sort, true);
+    assertEquals(0, td.totalHits.value);
 
     sort.setSort(new SortField("int", SortField.Type.INT), SortField.FIELD_DOC);
-    td = empty.search(query, 10, sort, true, true);
-    assertEquals(0, td.totalHits);
+    td = empty.search(query, 10, sort, true);
+    assertEquals(0, td.totalHits.value);
     
     sort.setSort(new SortField("string", SortField.Type.STRING, true), SortField.FIELD_DOC);
-    td = empty.search(query, 10, sort, true, true);
-    assertEquals(0, td.totalHits);
+    td = empty.search(query, 10, sort, true);
+    assertEquals(0, td.totalHits.value);
     
     sort.setSort(new SortField("string_val", SortField.Type.STRING_VAL, true), SortField.FIELD_DOC);
-    td = empty.search(query, 10, sort, true, true);
-    assertEquals(0, td.totalHits);
+    td = empty.search(query, 10, sort, true);
+    assertEquals(0, td.totalHits.value);
 
     sort.setSort(new SortField("float", SortField.Type.FLOAT), new SortField("string", SortField.Type.STRING));
-    td = empty.search(query, 10, sort, true, true);
-    assertEquals(0, td.totalHits);
+    td = empty.search(query, 10, sort, true);
+    assertEquals(0, td.totalHits.value);
   }
   
   /** Tests sorting a single document */
@@ -1717,7 +1715,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.STRING));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(1, td.totalHits);
+    assertEquals(1, td.totalHits.value);
     assertEquals("foo", searcher.doc(td.scoreDocs[0].doc).get("value"));
     TestUtil.checkReader(ir);
     ir.close();
@@ -1739,10 +1737,10 @@ public class TestFieldCacheSort extends LuceneTestCase {
     Sort sort = new Sort(new SortField("value", SortField.Type.STRING));
 
     TopDocs expected = searcher.search(new TermQuery(new Term("value", "foo")), 10);
-    assertEquals(1, expected.totalHits);
-    TopDocs actual = searcher.search(new TermQuery(new Term("value", "foo")), 10, sort, true, true);
+    assertEquals(1, expected.totalHits.value);
+    TopDocs actual = searcher.search(new TermQuery(new Term("value", "foo")), 10, sort, true);
     
-    assertEquals(expected.totalHits, actual.totalHits);
+    assertEquals(expected.totalHits.value, actual.totalHits.value);
     assertEquals(expected.scoreDocs[0].score, actual.scoreDocs[0].score, 0F);
     TestUtil.checkReader(ir);
     ir.close();
@@ -1774,7 +1772,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
                          new SortField("value", SortField.Type.STRING));
 
     TopDocs td = searcher.search(new MatchAllDocsQuery(), 10, sort);
-    assertEquals(2, td.totalHits);
+    assertEquals(2, td.totalHits.value);
     // 'bar' comes before 'foo'
     assertEquals("bar", searcher.doc(td.scoreDocs[0].doc).get("value"));
     assertEquals("foo", searcher.doc(td.scoreDocs[1].doc).get("value"));
@@ -1802,7 +1800,7 @@ public class TestFieldCacheSort extends LuceneTestCase {
     bq.add(new TermQuery(new Term("value", "foo")), Occur.SHOULD);
     bq.add(new MatchAllDocsQuery(), Occur.SHOULD);
     TopDocs td = searcher.search(bq.build(), 10, sort);
-    assertEquals(2, td.totalHits);
+    assertEquals(2, td.totalHits.value);
     if (Float.isNaN(td.scoreDocs[0].score) == false && Float.isNaN(td.scoreDocs[1].score) == false) {
       assertEquals(1, td.scoreDocs[0].doc);
       assertEquals(0, td.scoreDocs[1].doc);
