@@ -47,6 +47,7 @@ import org.apache.solr.cloud.AbstractFullDistribZkTestBase;
 import org.apache.solr.cloud.ZkShardTerms;
 import org.apache.solr.common.SolrDocument;
 import org.apache.solr.common.SolrDocumentList;
+import org.apache.solr.common.SolrException;
 import org.apache.solr.common.SolrInputDocument;
 import org.apache.solr.common.cloud.ClusterState;
 import org.apache.solr.common.cloud.DocCollection;
@@ -56,18 +57,21 @@ import org.apache.solr.common.cloud.ZkStateReader;
 import org.apache.solr.common.params.SolrParams;
 import org.apache.solr.common.util.ExecutorUtil;
 import org.apache.solr.common.util.NamedList;
+import org.apache.solr.common.util.SolrNamedThreadFactory;
 import org.apache.solr.common.util.TimeSource;
 import org.apache.solr.index.NoMergePolicyFactory;
 import org.apache.solr.update.processor.DistributedUpdateProcessor;
-import org.apache.solr.util.DefaultSolrThreadFactory;
 import org.apache.solr.util.RefCounted;
 import org.apache.solr.util.TimeOut;
 import org.apache.zookeeper.KeeperException;
+import org.hamcrest.MatcherAssert;
 import org.junit.BeforeClass;
 import org.junit.Test;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import static org.hamcrest.core.StringContains.containsString;
+
 /**
  * Tests the in-place updates (docValues updates) for a one shard, three replica cluster.
  */
@@ -125,7 +129,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
   @SuppressWarnings("unchecked")
   //28-June-2018 @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 21-May-2018
   // commented 4-Sep-2018 @LuceneTestCase.BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // 2-Aug-2018
-  @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
+  // commented out on: 17-Feb-2019   @BadApple(bugUrl="https://issues.apache.org/jira/browse/SOLR-12028") // annotated on: 24-Dec-2018
   public void test() throws Exception {
     waitForRecoveriesToFinish(true);
 
@@ -178,6 +182,8 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     resetDelays();
     reorderedDBQsResurrectionTest();
     resetDelays();
+    setNullForDVEnabledField();
+    resetDelays();
     
     // AwaitsFix this test fails easily
     // reorderedDBQsUsingUpdatedValueFromADroppedUpdate();
@@ -219,6 +225,45 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     assertEquals(2, NONLEADERS.size());
   }
 
+  private void setNullForDVEnabledField() throws Exception {
+    // to test set=null
+    // should this test be here? As set null would be an atomic update
+    clearIndex();
+    commit();
+
+    buildRandomIndex(0);
+    float inplace_updatable_float = 1;
+
+    // update doc, set
+    index("id", 0, "inplace_updatable_float", map("set", inplace_updatable_float));
+
+    LEADER.commit();
+    SolrDocument sdoc = LEADER.getById("0");  // RTG straight from the index
+    assertEquals(inplace_updatable_float, sdoc.get("inplace_updatable_float"));
+    assertEquals("title0", sdoc.get("title_s"));
+    long version0 = (long) sdoc.get("_version_");
+
+    for (SolrClient client : NONLEADERS) {
+      SolrDocument doc = client.getById(String.valueOf(0), params("distrib", "false"));
+      assertEquals(inplace_updatable_float, doc.get("inplace_updatable_float"));
+      assertEquals(version0, doc.get("_version_"));
+    }
+
+    index("id", 0, "inplace_updatable_float", map("set", null));
+    LEADER.commit();
+
+    sdoc = LEADER.getById("0");  // RTG straight from the index
+    assertNull(sdoc.get("inplace_updatable_float"));
+    assertEquals("title0", sdoc.get("title_s"));
+    long version1 = (long) sdoc.get("_version_");
+
+    for (SolrClient client : NONLEADERS) {
+      SolrDocument doc = client.getById(String.valueOf(0), params("distrib", "false"));
+      assertNull(doc.get("inplace_updatable_float"));
+      assertEquals(version1, doc.get("_version_"));
+    }
+  }
+
   final int NUM_RETRIES = 100, WAIT_TIME = 50;
 
   // The following should work: full update to doc 0, in-place update for doc 0, delete doc 0
@@ -249,13 +294,15 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
 
     // order the updates correctly for NONLEADER 1
     for (UpdateRequest update : updates) {
-      log.info("Issuing well ordered update: " + update.getDocuments());
+      if (log.isInfoEnabled()) {
+        log.info("Issuing well ordered update: {}", update.getDocuments());
+      }
       NONLEADERS.get(1).request(update);
     }
 
     // Reordering needs to happen using parallel threads
     ExecutorService threadpool = 
-        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new DefaultSolrThreadFactory(getTestName()));
+        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new SolrNamedThreadFactory(getTestName()));
 
     // re-order the updates for NONLEADER 0
     List<UpdateRequest> reorderedUpdates = new ArrayList<>(updates);
@@ -306,7 +353,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
 
     // Reordering needs to happen using parallel threads
     ExecutorService threadpool =
-        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new DefaultSolrThreadFactory(getTestName()));
+        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new SolrNamedThreadFactory(getTestName()));
 
     // re-order the updates by swapping the last two
     List<UpdateRequest> reorderedUpdates = new ArrayList<>(updates);
@@ -342,7 +389,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     // number of docs we're testing (0 <= id), index may contain additional random docs (id < 0)
     int numDocs = atLeast(100);
     if (onlyLeaderIndexes) numDocs = TestUtil.nextInt(random(), 10, 50);
-    log.info("Trying num docs = " + numDocs);
+    log.info("Trying num docs = {}", numDocs);
     final List<Integer> ids = new ArrayList<Integer>(numDocs);
     for (int id = 0; id < numDocs; id++) {
       ids.add(id);
@@ -351,7 +398,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     buildRandomIndex(101.0F, ids);
     
     List<Integer> luceneDocids = new ArrayList<>(numDocs);
-    List<Float> valuesList = new ArrayList<Float>(numDocs);
+    List<Number> valuesList = new ArrayList<>(numDocs);
     SolrParams params = params("q", "id:[0 TO *]", "fl", "*,[docid]", "rows", String.valueOf(numDocs), "sort", "id_i asc");
     SolrDocumentList results = LEADER.query(params).getResults();
     assertEquals(numDocs, results.size());
@@ -359,10 +406,10 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
       luceneDocids.add((Integer) doc.get("[docid]"));
       valuesList.add((Float) doc.get("inplace_updatable_float"));
     }
-    log.info("Initial results: "+results);
+    log.info("Initial results: {}", results);
     
     // before we do any atomic operations, sanity check our results against all clients
-    assertDocIdsAndValuesAgainstAllClients("sanitycheck", params, luceneDocids, valuesList);
+    assertDocIdsAndValuesAgainstAllClients("sanitycheck", params, luceneDocids, "inplace_updatable_float", valuesList);
 
     // now we're going to overwrite the value for all of our testing docs
     // giving them a value between -5 and +5
@@ -374,7 +421,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
       assert -5.0F <= value && value <= 5.0F;
       valuesList.set(id, value);
     }
-    log.info("inplace_updatable_float: " + valuesList);
+    log.info("inplace_updatable_float: {}", valuesList);
     
     // update doc w/ set
     Collections.shuffle(ids, r); // so updates aren't applied in index order
@@ -389,7 +436,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
                                              "fq", "id:[0 TO *]"),
                                       // existing sort & fl that we want...
                                       params),
-       luceneDocids, valuesList);
+       luceneDocids, "inplace_updatable_float", valuesList);
       
     // update doc, w/increment
     log.info("Updating the documents...");
@@ -400,7 +447,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
       // 0 test docs matching the query inplace_updatable_float:[-10 TO 10]
       final float inc = (r.nextBoolean() ? -1.0F : 1.0F) * (r.nextFloat() + (float)atLeast(20));
       assert 20 < Math.abs(inc);
-      final float value = valuesList.get(id) + inc;
+      final float value = (float)valuesList.get(id) + inc;
       assert value < -10 || 10 < value;
         
       valuesList.set(id, value);
@@ -413,7 +460,22 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
                                              "fq", "id:[0 TO *]"),
                                       // existing sort & fl that we want...
                                       params),
-       luceneDocids, valuesList);
+       luceneDocids, "inplace_updatable_float", valuesList);
+
+    log.info("Updating the documents with new field...");
+    Collections.shuffle(ids, r);
+    for (int id : ids) {
+      final int val = random().nextInt(20);
+      valuesList.set(id, val);
+      index("id", id, "inplace_updatable_int", map((random().nextBoolean()?"inc": "set"), val));
+    }
+    commit();
+
+    assertDocIdsAndValuesAgainstAllClients
+        ("inplace_for_first_field_update", SolrParams.wrapDefaults(params("q", "inplace_updatable_int:[* TO *]",
+            "fq", "id:[0 TO *]"),
+            params),
+            luceneDocids, "inplace_updatable_int", valuesList);
     log.info("docValuesUpdateTest: This test passed fine...");
   }
 
@@ -493,12 +555,14 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
    * @param debug used in log and assertion messages
    * @param req the query to execut, should include rows &amp; sort params such that the results can be compared to luceneDocids and valuesList
    * @param luceneDocids a list of "[docid]" values to be tested against each doc in the req results (in order)
-   * @param valuesList a list of "inplace_updatable_float" values to be tested against each doc in the req results (in order)
+   * @param fieldName used to get value from the doc to validate with valuesList
+   * @param valuesList a list of given fieldName values to be tested against each doc in results (in order)
    */
   private void assertDocIdsAndValuesAgainstAllClients(final String debug,
                                                       final SolrParams req,
                                                       final List<Integer> luceneDocids,
-                                                      final List<Float> valuesList) throws Exception {
+                                                      final String fieldName,
+                                                      final List<Number> valuesList) throws Exception {
     assert luceneDocids.size() == valuesList.size();
     final long numFoundExpected = luceneDocids.size();
     
@@ -523,7 +587,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
         Thread.sleep(WAIT_TIME);          
       }
       
-      assertDocIdsAndValuesInResults(msg, results, luceneDocids, valuesList);
+      assertDocIdsAndValuesInResults(msg, results, luceneDocids, fieldName, valuesList);
     }
   }
   
@@ -534,12 +598,14 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
    * @param msgPre used as a prefix for assertion messages
    * @param results the sorted results of some query, such that all matches are included (ie: rows = numFound)
    * @param luceneDocids a list of "[docid]" values to be tested against each doc in results (in order)
-   * @param valuesList a list of "inplace_updatable_float" values to be tested against each doc in results (in order)
+   * @param fieldName used to get value from the doc to validate with valuesList
+   * @param valuesList a list of given fieldName values to be tested against each doc in results (in order)
    */
   private void assertDocIdsAndValuesInResults(final String msgPre,
                                               final SolrDocumentList results,
                                               final List<Integer> luceneDocids,
-                                              final List<Float> valuesList) {
+                                              final String fieldName,
+                                              final List<Number> valuesList) {
 
     assert luceneDocids.size() == valuesList.size();
     assertEquals(msgPre + ": rows param wasn't big enough, we need to compare all results matching the query",
@@ -549,7 +615,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     
     for (SolrDocument doc : results) {
       final int id = Integer.parseInt(doc.get("id").toString());
-      final Object val = doc.get("inplace_updatable_float");
+      final Object val = doc.get(fieldName);
       final Object docid = doc.get("[docid]");
       assertEquals(msgPre + " wrong val for " + doc.toString(), valuesList.get(id), val);
       assertEquals(msgPre + " wrong [docid] for " + doc.toString(), luceneDocids.get(id), docid);
@@ -586,7 +652,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     assertTrue("Earlier: "+docids+", now: "+getInternalDocIds("100"), docids.equals(getInternalDocIds("100")));
     
     SolrDocument sdoc = LEADER.getById("100");  // RTG straight from the index
-    assertEquals(sdoc.toString(), (float) inplace_updatable_float, sdoc.get("inplace_updatable_float"));
+    assertEquals(sdoc.toString(), inplace_updatable_float, sdoc.get("inplace_updatable_float"));
     assertEquals(sdoc.toString(), title, sdoc.get("title_s"));
     assertEquals(sdoc.toString(), version, sdoc.get("_version_"));
 
@@ -597,7 +663,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
       version = currentVersion;
 
       sdoc = LEADER.getById("100");  // RTG from the tlog
-      assertEquals(sdoc.toString(), (float) inplace_updatable_float, sdoc.get("inplace_updatable_float"));
+      assertEquals(sdoc.toString(), inplace_updatable_float, sdoc.get("inplace_updatable_float"));
       assertEquals(sdoc.toString(), title, sdoc.get("title_s"));
       assertEquals(sdoc.toString(), version, sdoc.get("_version_"));
 
@@ -622,13 +688,25 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     assertTrue(currentVersion > version);
     version = currentVersion;
 
+    // set operation with invalid value for field
+    SolrException e = expectThrows(SolrException.class,
+        () -> addDocAndGetVersion( "id", 100, "inplace_updatable_float", map("set", "NOT_NUMBER")));
+    assertEquals(SolrException.ErrorCode.BAD_REQUEST.code, e.code());
+    MatcherAssert.assertThat(e.getMessage(), containsString("For input string: \"NOT_NUMBER\""));
+
+    // inc operation with invalid inc value
+    e = expectThrows(SolrException.class,
+        () -> addDocAndGetVersion( "id", 100, "inplace_updatable_int", map("inc", "NOT_NUMBER")));
+    assertEquals(SolrException.ErrorCode.BAD_REQUEST.code, e.code());
+    MatcherAssert.assertThat(e.getMessage(), containsString("For input string: \"NOT_NUMBER\""));
+
     // RTG from tlog(s)
     for (SolrClient client : clients) {
       final String clientDebug = client.toString() + (LEADER.equals(client) ? " (leader)" : " (not leader)");
       sdoc = client.getById("100", params("distrib", "false"));
 
-      assertEquals(clientDebug + " => "+ sdoc, (int) 100, sdoc.get("inplace_updatable_int"));
-      assertEquals(clientDebug + " => "+ sdoc, (float) inplace_updatable_float, sdoc.get("inplace_updatable_float"));
+      assertEquals(clientDebug + " => "+ sdoc, 100, sdoc.get("inplace_updatable_int"));
+      assertEquals(clientDebug + " => "+ sdoc, inplace_updatable_float, sdoc.get("inplace_updatable_float"));
       assertEquals(clientDebug + " => "+ sdoc, title, sdoc.get("title_s"));
       assertEquals(clientDebug + " => "+ sdoc, version, sdoc.get("_version_"));
     }
@@ -683,14 +761,16 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
 
     // order the updates correctly for NONLEADER 1
     for (UpdateRequest update : updates) {
-      log.info("Issuing well ordered update: " + update.getDocuments());
+      if (log.isInfoEnabled()) {
+        log.info("Issuing well ordered update: {}", update.getDocuments());
+      }
       NONLEADERS.get(1).request(update);
     }
 
     // Reordering needs to happen using parallel threads, since some of these updates will
     // be blocking calls, waiting for some previous updates to arrive on which it depends.
     ExecutorService threadpool = 
-        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new DefaultSolrThreadFactory(getTestName()));
+        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new SolrNamedThreadFactory(getTestName()));
 
     // re-order the updates for NONLEADER 0
     List<UpdateRequest> reorderedUpdates = new ArrayList<>(updates);
@@ -714,7 +794,9 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
 
     // assert both replicas have same effect
     for (SolrClient client : NONLEADERS) { // 0th is re-ordered replica, 1st is well-ordered replica
-      log.info("Testing client: " + ((HttpSolrClient)client).getBaseURL());
+      if (log.isInfoEnabled()) {
+        log.info("Testing client: {}", ((HttpSolrClient) client).getBaseURL());
+      }
       assertReplicaValue(client, 0, "inplace_updatable_float", (newinplace_updatable_float + (float)(updates.size() - 1)), 
           "inplace_updatable_float didn't match for replica at client: " + ((HttpSolrClient)client).getBaseURL());
       assertReplicaValue(client, 0, "title_s", "title0_new", 
@@ -752,13 +834,15 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
 
     // order the updates correctly for NONLEADER 1
     for (UpdateRequest update : updates) {
-      log.info("Issuing well ordered update: " + update.getDocuments());
+      if (log.isInfoEnabled()) {
+        log.info("Issuing well ordered update: {}", update.getDocuments());
+      }
       NONLEADERS.get(1).request(update);
     }
 
     // Reordering needs to happen using parallel threads
     ExecutorService threadpool = 
-        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new DefaultSolrThreadFactory(getTestName()));
+        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new SolrNamedThreadFactory(getTestName()));
 
     // re-order the updates for NONLEADER 0
     List<UpdateRequest> reorderedUpdates = new ArrayList<>(updates);
@@ -824,13 +908,15 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
 
     // order the updates correctly for NONLEADER 1
     for (UpdateRequest update : updates) {
-      log.info("Issuing well ordered update: " + update.getDocuments());
+      if (log.isInfoEnabled()) {
+        log.info("Issuing well ordered update: {}", update.getDocuments());
+      }
       NONLEADERS.get(1).request(update);
     }
 
     // Reordering needs to happen using parallel threads
     ExecutorService threadpool = 
-        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new DefaultSolrThreadFactory(getTestName()));
+        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new SolrNamedThreadFactory(getTestName()));
     // re-order the last two updates for NONLEADER 0
     List<UpdateRequest> reorderedUpdates = new ArrayList<>(updates);
     Collections.swap(reorderedUpdates, 2, 3);
@@ -872,15 +958,17 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     }
     // All should succeed, i.e. no LIR
     assertEquals(updateResponses.size(), successful);
-    
-    log.info("Non leader 0: "+((HttpSolrClient)NONLEADERS.get(0)).getBaseURL());
-    log.info("Non leader 1: "+((HttpSolrClient)NONLEADERS.get(1)).getBaseURL());
+
+    if (log.isInfoEnabled()) {
+      log.info("Non leader 0: {}", ((HttpSolrClient) NONLEADERS.get(0)).getBaseURL());
+      log.info("Non leader 1: {}", ((HttpSolrClient) NONLEADERS.get(1)).getBaseURL()); // nowarn
+    }
     
     SolrDocument doc0 = NONLEADERS.get(0).getById(String.valueOf(0), params("distrib", "false"));
     SolrDocument doc1 = NONLEADERS.get(1).getById(String.valueOf(0), params("distrib", "false"));
 
-    log.info("Doc in both replica 0: "+doc0);
-    log.info("Doc in both replica 1: "+doc1);
+    log.info("Doc in both replica 0: {}", doc0);
+    log.info("Doc in both replica 1: {}", doc1);
     // assert both replicas have same effect
     for (int i=0; i<NONLEADERS.size(); i++) { // 0th is re-ordered replica, 1st is well-ordered replica
       SolrClient client = NONLEADERS.get(i);
@@ -912,7 +1000,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
         "Waiting for dependant update to timeout", 1, 6000);
 
     ExecutorService threadpool =
-        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new DefaultSolrThreadFactory(getTestName()));
+        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new SolrNamedThreadFactory(getTestName()));
     for (UpdateRequest update : updates) {
       AsyncUpdateWithRandomCommit task = new AsyncUpdateWithRandomCommit(update, cloudClient,
                                                                          random().nextLong());
@@ -961,10 +1049,11 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     }
     
     for (SolrClient client : clients) {
-      log.info("Testing client (Fetch missing test): " + ((HttpSolrClient) client).getBaseURL());
-      log.info(
-          "Version at " + ((HttpSolrClient) client).getBaseURL() + " is: " + getReplicaValue(client, 1, "_version_"));
-
+      if (log.isInfoEnabled()) {
+        log.info("Testing client (Fetch missing test): {}", ((HttpSolrClient) client).getBaseURL());
+        log.info("Version at {} is: {}"
+            , ((HttpSolrClient) client).getBaseURL(), getReplicaValue(client, 1, "_version_")); // nowarn
+      }
       assertReplicaValue(client, 1, "inplace_updatable_float", (newinplace_updatable_float + 2.0f),
           "inplace_updatable_float didn't match for replica at client: " + ((HttpSolrClient) client).getBaseURL());
       assertReplicaValue(client, 1, "title_s", "title1_new",
@@ -985,7 +1074,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
       shardToJetty.get(SHARD1).get(1).jetty.getDebugFilter().addDelay("Waiting for dependant update to timeout", 4, 5998); // the delete update
 
       threadpool =
-          ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new DefaultSolrThreadFactory(getTestName()));
+          ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new SolrNamedThreadFactory(getTestName()));
       for (UpdateRequest update : updates) {
         AsyncUpdateWithRandomCommit task = new AsyncUpdateWithRandomCommit(update, cloudClient,
                                                                            random().nextLong());
@@ -1238,7 +1327,7 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
         "Waiting for dependant update to timeout", 2, 8000);
 
     ExecutorService threadpool =
-        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new DefaultSolrThreadFactory(getTestName()));
+        ExecutorUtil.newMDCAwareFixedThreadPool(updates.size() + 1, new SolrNamedThreadFactory(getTestName()));
     for (UpdateRequest update : updates) {
       AsyncUpdateWithRandomCommit task = new AsyncUpdateWithRandomCommit(update, cloudClient,
                                                                          random().nextLong());
@@ -1270,9 +1359,11 @@ public class TestInPlaceUpdatesDistrib extends AbstractFullDistribZkTestBase {
     }
 
     for (SolrClient client : clients) {
-      log.info("Testing client (testDBQUsingUpdatedFieldFromDroppedUpdate): " + ((HttpSolrClient)client).getBaseURL());
-      log.info("Version at " + ((HttpSolrClient)client).getBaseURL() + " is: " + getReplicaValue(client, 1, "_version_"));
-
+      if (log.isInfoEnabled()) {
+        log.info("Testing client (testDBQUsingUpdatedFieldFromDroppedUpdate): {}", ((HttpSolrClient) client).getBaseURL());
+        log.info("Version at {} is: {}", ((HttpSolrClient) client).getBaseURL(),
+            getReplicaValue(client, 1, "_version_")); // nowarn
+      }
       assertNull(client.getById("1", params("distrib", "false")));
     }
 
