@@ -28,6 +28,7 @@ import java.util.List;
 import java.util.Locale;
 import java.util.Map;
 import java.util.Set;
+import java.util.Objects;
 
 import org.apache.solr.common.MapWriter;
 import org.apache.solr.common.params.AutoScalingParams;
@@ -41,6 +42,8 @@ import static java.util.stream.Collectors.toList;
 /**
  * Bean representation of <code>autoscaling.json</code>, which parses data
  * lazily.
+ *
+ * @deprecated to be removed in Solr 9.0 (see SOLR-14656)
  */
 public class AutoScalingConfig implements MapWriter {
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
@@ -138,6 +141,11 @@ public class AutoScalingConfig implements MapWriter {
       return properties.equals(that.properties);
     }
 
+    @Override
+    public int hashCode() {
+      return Objects.hash(name, trigger, listenerClass);
+    }
+
     @Override
     public String toString() {
       return Utils.toJSONString(this);
@@ -183,6 +191,7 @@ public class AutoScalingConfig implements MapWriter {
       }
       enabled = Boolean.parseBoolean(String.valueOf(this.properties.getOrDefault("enabled", "true")));
 
+      @SuppressWarnings({"unchecked"})
       List<Map<String, Object>> newActions = (List<Map<String, Object>>)this.properties.get("actions");
       if (newActions != null) {
         this.actions = newActions.stream().map(ActionConfig::new).collect(collectingAndThen(toList(), Collections::unmodifiableList));
@@ -226,6 +235,11 @@ public class AutoScalingConfig implements MapWriter {
       return properties.equals(that.properties);
     }
 
+    @Override
+    public int hashCode() {
+      return Objects.hash(name);
+    }
+
     @Override
     public void writeMap(EntryWriter ew) throws IOException {
 //      if (!properties.containsKey(AutoScalingParams.NAME)) {
@@ -252,7 +266,7 @@ public class AutoScalingConfig implements MapWriter {
   public static class ActionConfig implements MapWriter {
     /** Action name. */
     public final String name;
-    /** Class name of action implementtion. */
+    /** Class name of action implementation. */
     public final String actionClass;
     /** Additional action properties. */
     public final Map<String, Object> properties;
@@ -291,6 +305,11 @@ public class AutoScalingConfig implements MapWriter {
       return properties.equals(that.properties);
     }
 
+    @Override
+    public int hashCode() {
+      return Objects.hash(properties);
+    }
+
     @Override
     public String toString() {
       return Utils.toJSONString(this);
@@ -301,6 +320,7 @@ public class AutoScalingConfig implements MapWriter {
    * Construct from bytes that represent a UTF-8 JSON string.
    * @param utf8 config data
    */
+  @SuppressWarnings({"unchecked"})
   public AutoScalingConfig(byte[] utf8) {
     this(utf8 != null && utf8.length > 0 ? (Map<String, Object>)Utils.fromJSON(utf8) : Collections.emptyMap());
   }
@@ -325,7 +345,7 @@ public class AutoScalingConfig implements MapWriter {
     empty = jsonMap.isEmpty();
   }
 
-  private AutoScalingConfig(Policy policy, Map<String, TriggerConfig> triggerConfigs, Map<String,
+  public AutoScalingConfig(Policy policy, Map<String, TriggerConfig> triggerConfigs, Map<String,
       TriggerListenerConfig> listenerConfigs, Map<String, Object> properties, int zkVersion) {
     this.policy = policy;
     this.triggers = triggerConfigs != null ? Collections.unmodifiableMap(new LinkedHashMap<>(triggerConfigs)) : null;
@@ -362,6 +382,7 @@ public class AutoScalingConfig implements MapWriter {
   /**
    * Get trigger configurations.
    */
+  @SuppressWarnings({"unchecked"})
   public Map<String, TriggerConfig> getTriggerConfigs() {
     if (triggers == null) {
       if (jsonMap != null) {
@@ -405,6 +426,7 @@ public class AutoScalingConfig implements MapWriter {
   /**
    * Get listener configurations.
    */
+  @SuppressWarnings({"unchecked"})
   public Map<String, TriggerListenerConfig> getTriggerListenerConfigs() {
     if (listeners == null) {
       if (jsonMap != null) {
@@ -428,6 +450,7 @@ public class AutoScalingConfig implements MapWriter {
   public Map<String, Object> getProperties()  {
     if (properties == null) {
       if (jsonMap != null)  {
+        @SuppressWarnings({"unchecked"})
         Map<String, Object> map = (Map<String, Object>) jsonMap.get("properties");
         if (map == null) {
           this.properties = Collections.emptyMap();
@@ -547,6 +570,11 @@ public class AutoScalingConfig implements MapWriter {
     ew.put("listeners", getTriggerListenerConfigs());
     ew.put("properties", getProperties());
   }
+//  @Override
+//  public int hashCode() {
+//    throw new UnsupportedOperationException("TODO unimplemented");
+//  }
+
 
   public String toString() {
     return Utils.toJSONString(this);
@@ -565,10 +593,16 @@ public class AutoScalingConfig implements MapWriter {
     return getProperties().equals(that.getProperties());
   }
 
+  @Override
+  public int hashCode() {
+    return Objects.hash(getPolicy());
+  }
+
   private static List<Object> getList(String key, Map<String, Object> properties) {
     return getList(key, properties, null);
   }
 
+  @SuppressWarnings({"unchecked", "rawtypes"})
   private static List<Object> getList(String key, Map<String, Object> properties, List<Object> defaultList) {
     if (defaultList == null) {
       defaultList = Collections.emptyList();
