@@ -28,10 +28,12 @@ import java.util.List;
 import java.util.Locale;
 
 import org.apache.lucene.util.LuceneTestCase;
+import org.apache.solr.SolrTestCaseJ4.SuppressPointFields;
 import org.apache.solr.client.solrj.io.SolrClientCache;
 import org.apache.solr.client.solrj.io.Tuple;
 import org.apache.solr.client.solrj.io.comp.ComparatorOrder;
 import org.apache.solr.client.solrj.io.comp.FieldComparator;
+import org.apache.solr.client.solrj.io.stream.expr.Expressible;
 import org.apache.solr.client.solrj.io.stream.expr.StreamFactory;
 import org.apache.solr.client.solrj.io.stream.metrics.CountMetric;
 import org.apache.solr.client.solrj.io.stream.metrics.MaxMetric;
@@ -41,7 +43,6 @@ import org.apache.solr.client.solrj.request.CollectionAdminRequest;
 import org.apache.solr.client.solrj.request.UpdateRequest;
 import org.apache.solr.cloud.AbstractDistribZkTestBase;
 import org.apache.solr.cloud.SolrCloudTestCase;
-import org.apache.solr.SolrTestCaseJ4.SuppressPointFields;
 import org.junit.AfterClass;
 import org.junit.Before;
 import org.junit.BeforeClass;
@@ -73,7 +74,9 @@ public class JDBCStreamTest extends SolrCloudTestCase {
     } else {
       collection = COLLECTIONORALIAS;
     }
-    CollectionAdminRequest.createCollection(collection, "conf", 2, 1).process(cluster.getSolrClient());
+    CollectionAdminRequest.createCollection(collection, "conf", 2, 1)
+        .setPerReplicaState(SolrCloudTestCase.USE_PER_REPLICA_STATE)
+        .process(cluster.getSolrClient());
     AbstractDistribZkTestBase.waitForRecoveriesToFinish(collection, cluster.getSolrClient().getZkStateReader(),
         false, true, TIMEOUT);
     if (useAlias) {
@@ -329,7 +332,7 @@ public class JDBCStreamTest extends SolrCloudTestCase {
               + "    rating_f as rating"
               + "  ),"
               + "  select("
-              + "    jdbc(connection=\"jdbc:hsqldb:mem:.\", sql=\"select PEOPLE.ID, PEOPLE.NAME, COUNTRIES.COUNTRY_NAME from PEOPLE inner join COUNTRIES on PEOPLE.COUNTRY_CODE = COUNTRIES.CODE order by PEOPLE.ID\", sort=\"ID asc\"),"
+              + "    jdbc(fetchSize=300, connection=\"jdbc:hsqldb:mem:.\", sql=\"select PEOPLE.ID, PEOPLE.NAME, COUNTRIES.COUNTRY_NAME from PEOPLE inner join COUNTRIES on PEOPLE.COUNTRY_CODE = COUNTRIES.CODE order by PEOPLE.ID\", sort=\"ID asc\"),"
               + "    ID as personId,"
               + "    NAME as personName,"
               + "    COUNTRY_NAME as country"
@@ -339,6 +342,8 @@ public class JDBCStreamTest extends SolrCloudTestCase {
 
 
       stream = factory.constructStream(expression);
+      String expr = ((Expressible)stream).toExpression(factory).toString();
+      assertTrue(expr.contains("fetchSize=300"));
       stream.setStreamContext(streamContext);
       tuples = getTuples(stream);
 
@@ -551,17 +556,17 @@ public class JDBCStreamTest extends SolrCloudTestCase {
 
       Tuple tuple = tuples.get(0);
       assertEquals("Netherlands", tuple.getString("country"));
-      assertTrue(4.3D == tuple.getDouble("max(rating)"));
-      assertTrue(2.2D == tuple.getDouble("min(rating)"));
-      assertTrue(3.6D == tuple.getDouble("avg(rating)"));
-      assertTrue(6D == tuple.getDouble("count(*)"));
+      assertEquals(4.3D , tuple.getDouble("max(rating)") , 0.0001);
+      assertEquals(2.2D , tuple.getDouble("min(rating)"), 0.0001);
+      assertEquals(3.6D, tuple.getDouble("avg(rating)"), 0.0001);
+      assertEquals(6D , tuple.getDouble("count(*)"), 0.0001);
 
       tuple = tuples.get(1);
       assertEquals("United States", tuple.getString("country"));
-      assertTrue(5D == tuple.getDouble("max(rating)"));
-      assertTrue(3D == tuple.getDouble("min(rating)"));
-      assertTrue(3.95D == tuple.getDouble("avg(rating)"));
-      assertTrue(4D == tuple.getDouble("count(*)"));
+      assertEquals(5D , tuple.getDouble("max(rating)"), 0.0001);
+      assertEquals(3D , tuple.getDouble("min(rating)"), 0.0001);
+      assertEquals(3.95D , tuple.getDouble("avg(rating)"),0.0001);
+      assertEquals(4D , tuple.getDouble("count(*)"), 0.0001);
     } finally {
       solrClientCache.close();
     }
@@ -609,9 +614,7 @@ public class JDBCStreamTest extends SolrCloudTestCase {
     for(double val : values) {
       Tuple t = tuples.get(i);
       double tip = (double)t.get(fieldName);
-      if(tip != val) {
-        throw new Exception("Found value:"+tip+" expecting:"+val);
-      }
+      assertEquals("Found value:"+tip+" expecting:"+val, val, tip, 0.00001);
       ++i;
     }
     return true;
@@ -641,7 +644,7 @@ public class JDBCStreamTest extends SolrCloudTestCase {
   protected boolean assertFields(List<Tuple> tuples, String ... fields) throws Exception{
     for(Tuple tuple : tuples){
       for(String field : fields){
-        if(!tuple.fields.containsKey(field)){
+        if(!tuple.getFields().containsKey(field)){
           throw new Exception(String.format(Locale.ROOT, "Expected field '%s' not found", field));
         }
       }
@@ -652,7 +655,7 @@ public class JDBCStreamTest extends SolrCloudTestCase {
   protected boolean assertNotFields(List<Tuple> tuples, String ... fields) throws Exception{
     for(Tuple tuple : tuples){
       for(String field : fields){
-        if(tuple.fields.containsKey(field)){
+        if(tuple.getFields().containsKey(field)){
           throw new Exception(String.format(Locale.ROOT, "Unexpected field '%s' found", field));
         }
       }
