@@ -63,7 +63,9 @@ public class JsonQueryRequestFacetingIntegrationTest extends SolrCloudTestCase {
     final List<String> solrUrls = new ArrayList<>();
     solrUrls.add(cluster.getJettySolrRunner(0).getBaseUrl().toString());
 
-    CollectionAdminRequest.createCollection(COLLECTION_NAME, CONFIG_NAME, 1, 1).process(cluster.getSolrClient());
+    CollectionAdminRequest.createCollection(COLLECTION_NAME, CONFIG_NAME, 1, 1)
+        .setPerReplicaState(SolrCloudTestCase.USE_PER_REPLICA_STATE)
+        .process(cluster.getSolrClient());
 
     ContentStreamUpdateRequest up = new ContentStreamUpdateRequest("/update");
     up.setParam("collection", COLLECTION_NAME);
@@ -132,7 +134,7 @@ public class JsonQueryRequestFacetingIntegrationTest extends SolrCloudTestCase {
         new FacetBucket("electronics", NUM_ELECTRONICS),
         new FacetBucket("currency", NUM_CURRENCY),
         new FacetBucket("memory", NUM_MEMORY));
-    assertEquals(NUM_CATEGORIES, topLevelFacetData.getBucketBasedFacets("top_cats").getNumBuckets());
+    assertEquals(NUM_CATEGORIES, topLevelFacetData.getBucketBasedFacets("top_cats").getNumBucketsCount());
   }
 
   @Test
@@ -539,9 +541,9 @@ public class JsonQueryRequestFacetingIntegrationTest extends SolrCloudTestCase {
         new FacetBucket(40.0f, 0),
         new FacetBucket(60.0f, 1),
         new FacetBucket(80.0f, 1));
-    assertEquals(0, topLevelFacetData.getBucketBasedFacets("price_range").getBefore());
-    assertEquals(9, topLevelFacetData.getBucketBasedFacets("price_range").getAfter());
-    assertEquals(7, topLevelFacetData.getBucketBasedFacets("price_range").getBetween());
+    assertEquals(0, topLevelFacetData.getBucketBasedFacets("price_range").getBeforeCount());
+    assertEquals(9, topLevelFacetData.getBucketBasedFacets("price_range").getAfterCount());
+    assertEquals(7, topLevelFacetData.getBucketBasedFacets("price_range").getBetweenCount());
   }
 
   private class FacetBucket {
@@ -571,8 +573,8 @@ public class JsonQueryRequestFacetingIntegrationTest extends SolrCloudTestCase {
 
   private void assertHasStatFacetWithValue(NestableJsonFacet response, String expectedFacetName, Double expectedStatValue) {
     assertTrue("Expected response to have stat facet named '" + expectedFacetName + "'",
-        response.getStatFacetValue(expectedFacetName) != null);
-    assertEquals(expectedStatValue, response.getStatFacetValue(expectedFacetName));
+        response.getStatValue(expectedFacetName) != null);
+    assertEquals(expectedStatValue, response.getStatValue(expectedFacetName));
   }
 
   private void assertExpectedDocumentsFoundAndReturned(QueryResponse response, int expectedNumFound, int expectedReturned) {
