@@ -32,6 +32,7 @@ import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.function.Consumer;
 import java.util.function.Predicate;
 
 import org.apache.solr.SolrTestCaseJ4;
@@ -56,12 +57,13 @@ import org.apache.solr.common.params.CollectionAdminParams;
 import org.apache.solr.common.util.NamedList;
 import org.junit.AfterClass;
 import org.junit.Before;
+import org.junit.BeforeClass;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 /**
  * Base class for SolrCloud tests
- *
+ * <p>
  * Derived tests should call {@link #configureCluster(int)} in a {@code BeforeClass}
  * static method or {@code Before} setUp method.  This configures and starts a {@link MiniSolrCloudCluster}, available
  * via the {@code cluster} variable.  Cluster shutdown is handled automatically if using {@code BeforeClass}.
@@ -80,19 +82,27 @@ import org.slf4j.LoggerFactory;
 public class SolrCloudTestCase extends SolrTestCaseJ4 {
 
   private static final Logger log = LoggerFactory.getLogger(MethodHandles.lookup().lookupClass());
-  
+  public static Boolean USE_PER_REPLICA_STATE =  Boolean.FALSE;
+
   public static final int DEFAULT_TIMEOUT = 45; // this is an important timeout for test stability - can't be too short
 
   private static class Config {
     final String name;
     final Path path;
-
     private Config(String name, Path path) {
       this.name = name;
       this.path = path;
     }
   }
 
+  @BeforeClass
+  public static void b4Class() {
+    USE_PER_REPLICA_STATE = Boolean.parseBoolean(System.getProperty("use.perreplica", String.valueOf(random().nextBoolean())));
+    if(USE_PER_REPLICA_STATE) {
+      log.info("Using per-replica state");
+    }
+  }
+
   /**
    * Builder class for a MiniSolrCloudCluster
    */
@@ -101,14 +111,17 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
     private final int nodeCount;
     private final Path baseDir;
     private String solrxml = MiniSolrCloudCluster.DEFAULT_CLOUD_SOLR_XML;
-    private JettyConfig jettyConfig = buildJettyConfig("/solr");
+    private JettyConfig.Builder jettyConfigBuilder = JettyConfig.builder().setContext("/solr").withSSLConfig(sslConfig.buildServerSSLConfig());
     private Optional<String> securityJson = Optional.empty();
 
     private List<Config> configs = new ArrayList<>();
     private Map<String, Object> clusterProperties = new HashMap<>();
 
+    private boolean trackJettyMetrics;
+
     /**
      * Create a builder
+     *
      * @param nodeCount the number of nodes in the cluster
      * @param baseDir   a base directory for the cluster
      */
@@ -118,10 +131,10 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
     }
 
     /**
-     * Use a {@link JettyConfig} to configure the cluster's jetty servers
+     * Use a JettyConfig.Builder to configure the cluster's jetty servers
      */
-    public Builder withJettyConfig(JettyConfig jettyConfig) {
-      this.jettyConfig = jettyConfig;
+    public Builder withJettyConfig(Consumer<JettyConfig.Builder> fun) {
+      fun.accept(jettyConfigBuilder);
       return this;
     }
 
@@ -173,6 +186,7 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
 
     /**
      * Upload a collection config before tests start
+     *
      * @param configName the config name
      * @param configPath the path to the config files
      */
@@ -183,7 +197,8 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
 
     /**
      * Set a cluster property
-     * @param propertyName the property name
+     *
+     * @param propertyName  the property name
      * @param propertyValue the property value
      */
     public Builder withProperty(String propertyName, String propertyValue) {
@@ -191,20 +206,29 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
       return this;
     }
 
+    public Builder withMetrics(boolean trackJettyMetrics) {
+      this.trackJettyMetrics = trackJettyMetrics;
+      return this;
+    }
+
     /**
      * Configure and run the {@link MiniSolrCloudCluster}
+     *
      * @throws Exception if an error occurs on startup
      */
-    public void configure() throws Exception {
-      cluster = build();
+    public MiniSolrCloudCluster configure() throws Exception {
+      return cluster = build();
     }
 
     /**
      * Configure, run and return the {@link MiniSolrCloudCluster}
+     *
      * @throws Exception if an error occurs on startup
      */
     public MiniSolrCloudCluster build() throws Exception {
-      MiniSolrCloudCluster cluster = new MiniSolrCloudCluster(nodeCount, baseDir, solrxml, jettyConfig, null, securityJson);
+      JettyConfig jettyConfig = jettyConfigBuilder.build();
+      MiniSolrCloudCluster cluster = new MiniSolrCloudCluster(nodeCount, baseDir, solrxml, jettyConfig,
+          null, securityJson, trackJettyMetrics);
       CloudSolrClient client = cluster.getSolrClient();
       for (Config config : configs) {
         ((ZkClientClusterStateProvider)client.getClusterStateProvider()).uploadConfig(config.path, config.name);
@@ -220,12 +244,14 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
     }
 
     public Builder withDefaultClusterProperty(String key, String value) {
-      HashMap<String,Object> defaults = (HashMap<String, Object>) this.clusterProperties.get(CollectionAdminParams.DEFAULTS);
+      @SuppressWarnings({"unchecked"})
+      HashMap<String, Object> defaults = (HashMap<String, Object>) this.clusterProperties.get(CollectionAdminParams.DEFAULTS);
       if (defaults == null) {
         defaults = new HashMap<>();
         this.clusterProperties.put(CollectionAdminParams.DEFAULTS, defaults);
       }
-      HashMap<String,Object> cluster = (HashMap<String, Object>) defaults.get(CollectionAdminParams.CLUSTER);
+      @SuppressWarnings({"unchecked"})
+      HashMap<String, Object> cluster = (HashMap<String, Object>) defaults.get(CollectionAdminParams.CLUSTER);
       if (cluster == null) {
         cluster = new HashMap<>();
         defaults.put(CollectionAdminParams.CLUSTER, cluster);
@@ -235,7 +261,9 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
     }
   }
 
-  /** The cluster */
+  /**
+   * The cluster
+   */
   protected static volatile MiniSolrCloudCluster cluster;
 
   protected static SolrZkClient zkClient() {
@@ -247,7 +275,7 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
 
   /**
    * Call this to configure a cluster of n nodes.
-   *
+   * <p>
    * NB you must call {@link Builder#configure()} to start the cluster
    *
    * @param nodeCount the number of nodes
@@ -259,9 +287,12 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
   @AfterClass
   public static void shutdownCluster() throws Exception {
     if (cluster != null) {
-      cluster.shutdown();
+      try {
+        cluster.shutdown();
+      } finally {
+        cluster = null;
+      }
     }
-    cluster = null;
   }
 
   @Before
@@ -281,17 +312,18 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
   protected static void waitForState(String message, String collection, CollectionStatePredicate predicate) {
     waitForState(message, collection, predicate, DEFAULT_TIMEOUT, TimeUnit.SECONDS);
   }
-  
+
   /**
    * Wait for a particular collection state to appear in the cluster client's state reader
-   *
+   * <p>
    * This is a convenience method using the {@link #DEFAULT_TIMEOUT}
    *
-   * @param message     a message to report on failure
-   * @param collection  the collection to watch
-   * @param predicate   a predicate to match against the collection state
+   * @param message    a message to report on failure
+   * @param collection the collection to watch
+   * @param predicate  a predicate to match against the collection state
    */
   protected static void waitForState(String message, String collection, CollectionStatePredicate predicate, int timeout, TimeUnit timeUnit) {
+    log.info("waitForState ({}): {}", collection, message);
     AtomicReference<DocCollection> state = new AtomicReference<>();
     AtomicReference<Set<String>> liveNodesLastSeen = new AtomicReference<>();
     try {
@@ -315,8 +347,7 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
         return false;
       if (collectionState.getSlices().size() != expectedShards)
         return false;
-      if (compareActiveReplicaCountsForShards(expectedReplicas, liveNodes, collectionState)) return true;
-      return false;
+      return compareActiveReplicaCountsForShards(expectedReplicas, liveNodes, collectionState);
     };
   }
 
@@ -328,26 +359,27 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
     return (liveNodes, collectionState) -> {
       if (collectionState == null)
         return false;
-      log.info("active slice count: " + collectionState.getActiveSlices().size() + " expected:" + expectedShards);
+      if (log.isInfoEnabled()) {
+        log.info("active slice count: {} expected: {}", collectionState.getActiveSlices().size(), expectedShards);
+      }
       if (collectionState.getActiveSlices().size() != expectedShards)
         return false;
-      if (compareActiveReplicaCountsForShards(expectedReplicas, liveNodes, collectionState)) return true;
-      return false;
+      return compareActiveReplicaCountsForShards(expectedReplicas, liveNodes, collectionState);
     };
   }
-  
+
   public static LiveNodesPredicate containsLiveNode(String node) {
     return (oldNodes, newNodes) -> {
       return newNodes.contains(node);
     };
   }
-  
+
   public static LiveNodesPredicate missingLiveNode(String node) {
     return (oldNodes, newNodes) -> {
       return !newNodes.contains(node);
     };
   }
-  
+
   public static LiveNodesPredicate missingLiveNodes(List<String> nodes) {
     return (oldNodes, newNodes) -> {
       boolean success = true;
@@ -370,14 +402,11 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
         }
       }
     }
-    
-    log.info("active replica count: " + activeReplicas + " expected replica count: " + expectedReplicas);
-    
-    if (activeReplicas == expectedReplicas) {
-      return true;
-    }
 
-    return false;
+    log.info("active replica count: {} expected replica count: {}", activeReplicas, expectedReplicas);
+
+    return activeReplicas == expectedReplicas;
+
   }
 
   /**
@@ -420,7 +449,7 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
 
   /**
    * Get the {@link CoreStatus} data for a {@link Replica}
-   *
+   * <p>
    * This assumes that the replica is hosted on a live node.
    */
   protected static CoreStatus getCoreStatus(Replica replica) throws IOException, SolrServerException {
@@ -430,7 +459,9 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
     }
   }
 
+  @SuppressWarnings({"rawtypes"})
   protected NamedList waitForResponse(Predicate<NamedList> predicate, SolrRequest request, int intervalInMillis, int numRetries, String messageOnFail) {
+    log.info("waitForResponse: {}", request);
     int i = 0;
     for (; i < numRetries; i++) {
       try {
@@ -450,6 +481,7 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
   /**
    * Ensure that the given number of solr instances are running. If less instances are found then new instances are
    * started. If extra instances are found then they are stopped.
+   *
    * @param nodeCount the number of Solr instances that should be running at the end of this method
    * @throws Exception on error
    */
@@ -458,7 +490,7 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
     List<JettySolrRunner> jettys = cluster.getJettySolrRunners();
     List<JettySolrRunner> copyOfJettys = new ArrayList<>(jettys);
     int numJetties = copyOfJettys.size();
-    for (int i = nodeCount; i < numJetties; i++)  {
+    for (int i = nodeCount; i < numJetties; i++) {
       cluster.stopJettySolrRunner(copyOfJettys.get(i));
     }
     for (int i = copyOfJettys.size(); i < nodeCount; i++) {
@@ -476,4 +508,21 @@ public class SolrCloudTestCase extends SolrTestCaseJ4 {
     cluster.waitForAllNodes(timeoutSeconds);
   }
 
+  public static Map<String, String> mapReplicasToReplicaType(DocCollection collection) {
+    Map<String, String> replicaTypeMap = new HashMap<>();
+    for (Slice slice : collection.getSlices()) {
+      for (Replica replica : slice.getReplicas()) {
+        String coreUrl = replica.getCoreUrl();
+        // It seems replica reports its core URL with a trailing slash while shard
+        // info returned from the query doesn't. Oh well. We will include both, just in case
+        replicaTypeMap.put(coreUrl, replica.getType().toString());
+        if (coreUrl.endsWith("/")) {
+          replicaTypeMap.put(coreUrl.substring(0, coreUrl.length() - 1), replica.getType().toString());
+        }else {
+          replicaTypeMap.put(coreUrl + "/", replica.getType().toString());
+        }
+      }
+    }
+    return replicaTypeMap;
+  }
 }
