@@ -61,6 +61,7 @@ import java.net.UnknownHostException;
 import java.nio.channels.SelectionKey;
 import java.nio.channels.SocketChannel;
 import java.nio.charset.StandardCharsets;
+import java.nio.file.Path;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.Comparator;
@@ -69,6 +70,7 @@ import java.util.List;
 import java.util.Map;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.atomic.AtomicReference;
 
 public class ZkTestServer {
 
@@ -90,7 +92,7 @@ public class ZkTestServer {
   
   protected final ZKServerMain zkServer = new ZKServerMain();
 
-  private volatile String zkDir;
+  private volatile Path zkDir;
 
   private volatile int clientPort;
 
@@ -177,7 +179,9 @@ public class ZkTestServer {
       }
 
       public void updateForFire(WatchedEvent event) {
-        log.debug("Watch fired: {}: {}", desc, event.getPath());
+        if (log.isDebugEnabled()) {
+          log.debug("Watch fired: {}: {}", desc, event.getPath());
+        }
         counters.decrementAndGet(event.getPath());
       }
 
@@ -249,6 +253,12 @@ public class ZkTestServer {
           case NodeChildrenChanged:
             childrenLimit.updateForFire(event);
             break;
+          case ChildWatchRemoved:
+            break;
+          case DataWatchRemoved:
+            break;
+          case PersistentWatchRemoved:
+            break;
         }
       }
     }
@@ -259,7 +269,7 @@ public class ZkTestServer {
 
       public TestServerCnxn(ZooKeeperServer zk, SocketChannel sock, SelectionKey sk,
                             NIOServerCnxnFactory factory, WatchLimiter limiter) throws IOException {
-        super(zk, sock, sk, factory);
+        super(zk, sock, sk, factory, null);
         this.limiter = limiter;
       }
 
@@ -278,11 +288,6 @@ public class ZkTestServer {
         super();
         this.limiter = limiter;
       }
-
-      @Override
-      protected NIOServerCnxn createConnection(SocketChannel sock, SelectionKey sk) throws IOException {
-        return new TestServerCnxn(zkServer, sock, sk, this, limiter);
-      }
     }
 
     private class TestZKDatabase extends ZKDatabase {
@@ -331,11 +336,12 @@ public class ZkTestServer {
         // so rather than spawning another thread, we will just call
         // run() in this thread.
         // create a file logger url from the command line args
-        FileTxnSnapLog ftxn = new FileTxnSnapLog(new File(
-            config.getDataLogDir()), new File(config.getDataDir()));
+        FileTxnSnapLog ftxn = new FileTxnSnapLog(config.getDataLogDir(), config.getDataDir());
+
         zooKeeperServer = new ZooKeeperServer(ftxn, config.getTickTime(),
             config.getMinSessionTimeout(), config.getMaxSessionTimeout(),
-            null /* this is not used */, new TestZKDatabase(ftxn, limiter));
+            config.getClientPortListenBacklog(),
+            new TestZKDatabase(ftxn, limiter), "");
         cnxnFactory = new TestServerCnxnFactory(limiter);
         cnxnFactory.configure(config.getClientPortAddress(),
             config.getMaxClientCnxns());
@@ -384,11 +390,17 @@ public class ZkTestServer {
           zkDb.close();
         }
 
-        if (cnxnFactory != null && cnxnFactory.getLocalPort() != 0) {
-          waitForServerDown(getZkHost(), 30000);
+        if (cnxnFactory != null) {
+          try {
+            int port = cnxnFactory.getLocalPort();
+            if (port > 0) {
+              waitForServerDown(getZkHost(), 30000);
+            }
+          } catch (NullPointerException ignored) {
+            // server never successfully started
+          }
         }
       } finally {
-
         ObjectReleaseTracker.release(this);
       }
     }
@@ -418,11 +430,11 @@ public class ZkTestServer {
     }
   }
 
-  public ZkTestServer(String zkDir) throws Exception {
+  public ZkTestServer(Path zkDir) throws Exception {
     this(zkDir, 0);
   }
 
-  public ZkTestServer(String zkDir, int port) throws KeeperException, InterruptedException {
+  public ZkTestServer(Path zkDir, int port) throws KeeperException, InterruptedException {
     this.zkDir = zkDir;
     this.clientPort = port;
     String reportAction = System.getProperty("tests.zk.violationReportAction");
@@ -456,7 +468,8 @@ public class ZkTestServer {
   }
 
   public String getZkHost() {
-    return "127.0.0.1:" + zkServer.getLocalPort();
+    String hostName = System.getProperty("hostName", "127.0.0.1");
+    return hostName + ":" + zkServer.getLocalPort();
   }
 
   public String getZkAddress() {
@@ -469,9 +482,10 @@ public class ZkTestServer {
    * @return the connection string
    */
   public String getZkAddress(String chroot) {
-    if (!chroot.startsWith("/"))
+    if (!chroot.startsWith("/")) {
       chroot = "/" + chroot;
-    return "127.0.0.1:" + zkServer.getLocalPort() + chroot;
+    }
+    return getZkHost() + chroot;
   }
 
   /**
@@ -526,10 +540,12 @@ public class ZkTestServer {
 
   public void run(boolean solrFormat) throws InterruptedException, IOException {
     log.info("STARTING ZK TEST SERVER");
+    AtomicReference<Throwable> zooError = new AtomicReference<>();
     try {
       if (zooThread != null) {
         throw new IllegalStateException("ZK TEST SERVER IS ALREADY RUNNING");
       }
+      Thread parentThread = Thread.currentThread();
       // we don't call super.distribSetUp
       zooThread = new Thread("ZkTestServer Run Thread") {
 
@@ -539,8 +555,8 @@ public class ZkTestServer {
 
             {
               setClientPort(ZkTestServer.this.clientPort);
-              this.dataDir = zkDir;
-              this.dataLogDir = zkDir;
+              this.dataDir = zkDir.toFile();
+              this.dataLogDir = zkDir.toFile();
               this.tickTime = theTickTime;
               this.maxSessionTimeout = ZkTestServer.this.maxSessionTimeout;
               this.minSessionTimeout = ZkTestServer.this.minSessionTimeout;
@@ -557,13 +573,14 @@ public class ZkTestServer {
               } else {
                 this.clientPortAddress = new InetSocketAddress(clientPort);
               }
-              log.info("client port:" + this.clientPortAddress);
+              log.info("client port: {}", this.clientPortAddress);
             }
           };
           try {
             zkServer.runFromConfig(config);
           } catch (Throwable t) {
-            log.error("zkServer error", t);
+            zooError.set(t);
+            parentThread.interrupt();
           }
         }
       };
@@ -575,29 +592,36 @@ public class ZkTestServer {
       int port = -1;
       try {
         port = getPort();
-      } catch (IllegalStateException e) {
-
+      } catch (IllegalStateException ignored) {
+        // Possibly fix this API to return null instead of throwing
       }
       while (port < 1) {
         Thread.sleep(100);
         try {
           port = getPort();
-        } catch (IllegalStateException e) {
-
+        } catch (IllegalStateException ignored) {
+          // Possibly fix this API to return null instead of throwing
         }
         if (cnt == 500) {
           throw new RuntimeException("Could not get the port for ZooKeeper server");
         }
         cnt++;
       }
-      log.info("start zk server on port:" + port);
+      log.info("start zk server on port: {}", port);
 
       waitForServerUp(getZkHost(), 30000);
 
       init(solrFormat);
     } catch (Exception e) {
-      log.error("Error trying to run ZK Test Server", e);
-      throw new RuntimeException(e);
+      RuntimeException toThrow = new RuntimeException("Could not get ZK port");
+      Throwable t = zooError.get();
+      if (t != null) {
+        toThrow.initCause(t);
+        toThrow.addSuppressed(e);
+      } else {
+        toThrow.initCause(e);
+      }
+      throw toThrow;
     }
   }
 
@@ -637,6 +661,7 @@ public class ZkTestServer {
   }
   
   public static boolean waitForServerDown(String hp, long timeoutMs) {
+    log.info("waitForServerDown: {}", hp);
     final TimeOut timeout = new TimeOut(timeoutMs, TimeUnit.MILLISECONDS, TimeSource.NANO_TIME);
     while (true) {
       try {
@@ -658,6 +683,7 @@ public class ZkTestServer {
   }
   
   public static boolean waitForServerUp(String hp, long timeoutMs) {
+    log.info("waitForServerUp: {}", hp);
     final TimeOut timeout = new TimeOut(timeoutMs, TimeUnit.MILLISECONDS, TimeSource.NANO_TIME);
     while (true) {
       try {
@@ -701,7 +727,7 @@ public class ZkTestServer {
   public static String send4LetterWord(String host, int port, String cmd)
           throws IOException
   {
-    log.info("connecting to " + host + " " + port);
+    log.info("connecting to {} {}", host, port);
     BufferedReader reader = null;
     try (Socket sock = new Socket(host, port)) {
       OutputStream outstream = sock.getOutputStream();
@@ -710,9 +736,8 @@ public class ZkTestServer {
       // this replicates NC - close the output stream before reading
       sock.shutdownOutput();
 
-      reader =
-          new BufferedReader(
-              new InputStreamReader(sock.getInputStream(), "US-ASCII"));
+      reader = new BufferedReader(
+          new InputStreamReader(sock.getInputStream(), StandardCharsets.US_ASCII));
       StringBuilder sb = new StringBuilder();
       String line;
       while ((line = reader.readLine()) != null) {
@@ -727,7 +752,7 @@ public class ZkTestServer {
   }
   
   public static List<HostPort> parseHostPortList(String hplist) {
-    log.info("parse host and port list: " + hplist);
+    log.info("parse host and port list: {}", hplist);
     ArrayList<HostPort> alist = new ArrayList<>();
     for (String hp : hplist.split(",")) {
       int idx = hp.lastIndexOf(':');
@@ -751,7 +776,7 @@ public class ZkTestServer {
     this.theTickTime = theTickTime;
   }
 
-  public String getZkDir() {
+  public Path getZkDir() {
     return zkDir;
   }
 
@@ -786,20 +811,33 @@ public class ZkTestServer {
 
   public static void putConfig(String confName, SolrZkClient zkClient, File solrhome, final String name)
       throws Exception {
-    putConfig(confName, zkClient, solrhome, name, name);
+    putConfig(confName, zkClient, null, solrhome, name, name);
   }
 
-  public static void putConfig(String confName, SolrZkClient zkClient, File solrhome, final String srcName, String destName)
-      throws Exception {
+
+  public static void putConfig(String confName, SolrZkClient zkClient, File solrhome, final String srcName,
+                                 String destName) throws Exception {
+      putConfig(confName, zkClient, null, solrhome, srcName, destName);
+  }
+
+  public static void putConfig(String confName, SolrZkClient zkClient, String zkChroot, File solrhome,
+                               final String srcName, String destName) throws Exception {
     File file = new File(solrhome, "collection1"
         + File.separator + "conf" + File.separator + srcName);
     if (!file.exists()) {
-      log.info("skipping " + file.getAbsolutePath() + " because it doesn't exist");
+      if (log.isInfoEnabled()) {
+        log.info("skipping {} because it doesn't exist", file.getAbsolutePath());
+      }
       return;
     }
 
     String destPath = "/configs/" + confName + "/" + destName;
-    log.info("put " + file.getAbsolutePath() + " to " + destPath);
+    if (zkChroot != null) {
+      destPath = zkChroot + destPath;
+    }
+    if (log.isInfoEnabled()) {
+      log.info("put {} to {}", file.getAbsolutePath(), destPath);
+    }
     zkClient.makePath(destPath, file, false, true);
   }
   
